<?php
defined('BASEPATH') or exit('No direct script access allowed');

class AdvancePaymentController extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('UserModel');
        $this->load->model('AdvancePaymentModel');
        $this->load->library('Middleware');
        $this->load->library("upload");
        // $this->load->library("headerMiddleware");

    }

    function index()
    {
    }


    public function request()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $price = $this->input->post('price');
        $date = $this->input->post('date');
        $goverment = $this->input->post('goverment');
        $projectId = $this->input->post('projectId');
        $userId = $this->input->post("userId");
        $notes = $this->input->post('notes');

        $missingData = [];

        if ($price == null) {
            $missingData[] = 'price';
        }

        if ($date == null) {
            $missingData[] = 'date';
        }

        if ($goverment == null) {
            $missingData[] = 'goverment';
        }

        if ($userId == null) {
            $missingData[] = 'userId';
        }

        if (!empty($missingData)) {
            http_response_code(400);
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'Missing or invalid data for the following field(s): ' . implode(', ', $missingData),
                    'token' => null,
                    'data' => null,
                    'isAuth' => false
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $this->AdvancePaymentModel->createAdvanceReq($price, $date, $goverment, $projectId, $notes, $userId);

        // Return a success response
        $response = array(
            'status' => 200,
            'message' => 'Advance payment request submitted successfully',
            'goverment' => $goverment
        );
        $this->output->set_output(json_encode($response));
    }

    public function deleteRequestPayment()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $id = $this->input->post("id");

        $result = $this->AdvancePaymentModel->deleteRequestPayment($id);

        if ($result) {
            // Successful deletion
            $this->output
                ->set_status_header(200)
                ->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'RequestPayment deleted successfully', 'status' => 200]));
        } else {
            // Error handling if the deletion fails
            $this->output
                ->set_status_header(500)
                ->set_content_type('application/json')
                ->set_output(json_encode(['error' => 'Unable to delete RequestPayment']));
        }

    }




    public function payment()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();


        $requiredFields = [
            'price',
            'categoryId',
            'invoice_num',
            'invoice_date',
            'userId',
            'reqPaymentId',
        ];

        // Check if all required fields are present and not null
        foreach ($requiredFields as $field) {
            if (is_null($this->input->post($field))) {
                // http_response_code(400);
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'Missing or invalid data for field: ' . $field,
                        'token' => null,
                        'data' => null,
                        'isAuth' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }

        // If all required fields are present, proceed with the payment
        $this->processPayment();
        // var_dump($_FILES); // To check the received files
        // var_dump($_POST); // To check the received POST data
    }

    public function processPayment()
    {

        $price = $this->input->post("price");
        $date = $this->input->post("date");
        $categoryId = $this->input->post("categoryId");
        $invoice_num = $this->input->post("invoice_num");
        $invoice_date = $this->input->post("invoice_date");
        $notes = $this->input->post("notes");
        $projectId = $this->input->post("projectId");
        $userId = $this->input->post("userId");
        $reqPaymentId = $this->input->post("reqPaymentId");
        $attachment = $this->input->post("attachment");

        // Define an array of required fields
        $requiredFields = [
            'price',
            'categoryId',
            'invoice_num',
            'invoice_date',
            'userId',
            'reqPaymentId',
        ];

        // Check if all required fields are present and not null
        foreach ($requiredFields as $field) {
            if (is_null($this->input->post($field))) {
                // http_response_code(400);
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'Missing or invalid data for field: ' . $field,
                        'token' => null,
                        'data' => null,
                        'isAuth' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                return; // Return, don't proceed with upload
            }
        }

        // Check if attachment is provided and is not just a string

        if (!empty($_FILES)) {

            if ($_SERVER['SERVER_NAME'] === 'localhost') {
                $config["upload_path"] = $_SERVER['DOCUMENT_ROOT'] . "advance_money_attachment";
            } else {
                $currentPath = $_SERVER['REQUEST_URI'];
                $pathSegments = explode('/', trim($currentPath, '/'));
                $extractedPath = '/' . $pathSegments[0] . '/' . $pathSegments[1];
                
                $config["upload_path"] = '/var/www/html/' . $extractedPath . '/uploads/advance_money_attachment';
            }

            $config['allowed_types'] = 'gif|jpg|jpeg|png|pdf';
            $config['max_size'] = '2048';
            $config['file_name'] = time();

            $this->load->library('upload', $config);
            $this->upload->initialize($config);

            if ($this->upload->do_upload('attachment')) {
                $file_data = $this->upload->data();
                $attachmentPath = $file_data['file_name'];


                // Continue with processing the payment, including the attachment path
                $result = $this->AdvancePaymentModel->paymentAdvance(
                    $price,
                    $date,
                    $categoryId,
                    $invoice_num,
                    $invoice_date,
                    $notes,
                    $projectId,
                    $userId,
                    $reqPaymentId,
                    $attachmentPath
                );


                if ($result === "سعر التسديد اكبر من المتبقي") {

                    echo json_encode(
                        array(
                            'status' => 400,
                            'message' => "سعر التسديد اكبر من المتبقي لسلفة",
                            'token' => null,
                            'data' => null,
                            'isAuth' => false,
                            'action' => false,
                            'result' => $result
                        ),
                        JSON_UNESCAPED_UNICODE
                    );
                    exit();
                }
                if (!$result) {
                    $response = array(
                        'status' => 400,
                        'message' => 'رقم الوصل مستخدم من قبل الرجاء ادخال رقم أخر',
                        'action' => false
                    );
                    return $this->output->set_output(json_encode($response));

                }
                // Send the response
                $response = array(
                    'status' => 200,
                    'message' => 'تم انشاء طلب تسديد السلفه بنجاح',
                    'action' => true
                );
                $this->output->set_output(json_encode($response));
            } else {
                // http_response_code(400);
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'File upload failed: ' . $this->upload->display_errors(),
                        'token' => null,
                        'data' => null,
                        'isAuth' => false,
                        'action' => false,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
            }
        } else {
            // http_response_code(400);
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الصوره مطلوبه',
                    'token' => null,
                    'data' => null,
                    'isAuth' => false,
                    'action' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
        }
    }

    public function updatePayment()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();


        $requiredFields = [
            'price',
            'categoryId',
            'invoice_num',
            'invoice_date',
            'userId',
            'reqPaymentId',
        ];

        // Check if all required fields are present and not null
        foreach ($requiredFields as $field) {
            if (is_null($this->input->post($field))) {
                // http_response_code(400);
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'Missing or invalid data for field: ' . $field,
                        'token' => null,
                        'data' => null,
                        'isAuth' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }

        $this->updateProcessPayment();
    }
    public function updateProcessPayment()
    {

        $id = $this->input->post("id");
        $price = $this->input->post("price");
        $date = $this->input->post("date");
        $categoryId = $this->input->post("categoryId");
        $invoice_num = $this->input->post("invoice_num");
        $invoice_date = $this->input->post("invoice_date");
        $notes = $this->input->post("notes");
        $projectId = $this->input->post("projectId");
        $userId = $this->input->post("userId");
        $reqPaymentId = $this->input->post("reqPaymentId");
        $attachment = $this->input->post("attachment");

        // Define an array of required fields
        $requiredFields = [
            'id',
            'price',
            'categoryId',
            'invoice_num',
            'invoice_date',
            'userId',
            'reqPaymentId',
        ];

        // Check if all required fields are present and not null
        foreach ($requiredFields as $field) {
            if (is_null($this->input->post($field))) {
                // http_response_code(400);
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'Missing or invalid data for field: ' . $field,
                        'token' => null,
                        'data' => null,
                        'isAuth' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                return; // Return, don't proceed with upload
            }
        }

        // Check if attachment is provided and is not just a string

        if (!empty($_FILES)) {

            if ($_SERVER['SERVER_NAME'] === 'localhost') {
                $config["upload_path"] = $_SERVER['DOCUMENT_ROOT'] . "/Hr/uploads/advance_money_attachment";
                // $config["upload_path"] = '/var/www/html/hr/app/uploads/advance_money_attachment';

            } else {
                $config["upload_path"] = '/var/www/html/hr/app/uploads/advance_money_attachment';
            }

            $config['allowed_types'] = 'gif|jpg|jpeg|png|pdf';
            $config['max_size'] = '2048';
            $config['file_name'] = time();

            // $this->load->library('upload', $config);
            $this->upload->initialize($config);


            if ($this->upload->do_upload('attachment')) {
                $file_data = $this->upload->data();
                $attachmentPath = $file_data['file_name'];


                // Continue with processing the payment, including the attachment path
                $this->AdvancePaymentModel->updatePaymentAdvance(
                    $id,
                    $price,
                    $date,
                    $categoryId,
                    $invoice_num,
                    $invoice_date,
                    $notes,
                    $projectId,
                    $userId,
                    $reqPaymentId,
                    $attachmentPath
                );

                // Send the response
                $response = array(
                    'status' => 200,
                    'message' => 'Advance payment submitted successfully',
                );
                $this->output->set_output(json_encode($response));
            } else {
                // http_response_code(400);
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'File upload failed: ' . $this->upload->display_errors(),
                        'token' => null,
                        'data' => null,
                        'isAuth' => false
                    ),
                    JSON_UNESCAPED_UNICODE
                );
            }
        } else {
            // http_response_code(400);
            $this->AdvancePaymentModel->updatePaymentAdvance(
                $id,
                $price,
                $date,
                $categoryId,
                $invoice_num,
                $invoice_date,
                $notes,
                $projectId,
                $userId,
                $reqPaymentId,
                $attachmentPath = null,
            );
            $response = array(
                'status' => 200,
                'message' => 'Advance payment submitted successfully',
            );
            $this->output->set_output(json_encode($response));
            // echo json_encode(
            //     array(
            //         'status' => 400,
            //         'message' => 'Attachment is required',
            //         'token' => null,
            //         'data' => null,
            //         'isAuth' => false
            //     ),
            //     JSON_UNESCAPED_UNICODE
            // );
        }
    }





    public function getPayment()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $hash = $this->input->get('hash');

        $paymentData = $this->AdvancePaymentModel->getPayment($hash);

        if (!empty($paymentData)) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'Payment data found',
                    'data' => $paymentData,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'لا يوجد تسديدات لك',
                    'data' => null,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function getPaymentById()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $id = $this->input->get('id');

        // You can retrieve payment data by ID using a different method or query, depending on your implementation.
        $paymentData = $this->AdvancePaymentModel->getPaymentById($id);

        if (!empty($paymentData)) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'Payment data found',
                    'data' => $paymentData,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'No payment data found for the provided ID',
                    'data' => null,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }


    public function getAdvancePaymentRemaining()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $reqPaymentId = $this->input->get('reqPaymentId');

        $paymentAdvanceData = $this->AdvancePaymentModel->getAdvancePaymentRemaining($reqPaymentId);

        $response = array();

        if (!empty($paymentAdvanceData)) {
            $response['status'] = 200;
            $response['message'] = 'PaymentAdvance data found';
            $response['data'] = $paymentAdvanceData;
            $response['isAuth'] = true;
        } else {
            $response['status'] = 404;
            $response['message'] = 'لم يتم تسديد اي مبلغ لهذا التسديد';
            $response['data'] = null;
            $response['isAuth'] = true;
        }

        header('Content-Type: application/json');
        echo json_encode($response, JSON_UNESCAPED_UNICODE);
        exit();

    }
    public function getAdvancePayment()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $reqPaymentId = $this->input->get('reqPaymentId');

        $paymentAdvanceData = $this->AdvancePaymentModel->getAdvancePayment($reqPaymentId);

        $response = array();

        if (!empty($paymentAdvanceData)) {
            $response['status'] = 200;
            $response['message'] = 'PaymentAdvance data found';
            $response['data'] = $paymentAdvanceData;
            $response['isAuth'] = true;

        } else {
            $response['status'] = 404;
            $response['message'] = 'لم يتم تسديد اي مبلغ لهذا التسديد';
            $response['data'] = null;
            $response['isAuth'] = true;
        }

        header('Content-Type: application/json');
        echo json_encode($response, JSON_UNESCAPED_UNICODE);
        exit();
    }
    public function getAdvancePaymentInfo()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $reqPaymentId = $this->input->get('reqPaymentId');

        $paymentAdvanceData = $this->AdvancePaymentModel->getAdvancePaymentInfo($reqPaymentId);

        $response = array();

        if (!empty($paymentAdvanceData)) {
            $response['status'] = 200;
            $response['message'] = 'PaymentAdvance data found';
            $response['data'] = $paymentAdvanceData;
            $response['isAuth'] = true;
        } else {
            $response['status'] = 404;
            $response['message'] = 'No paymentAdvance data found for the provided user';
            $response['data'] = null;
            $response['isAuth'] = true;
        }

        header('Content-Type: application/json');
        echo json_encode($response, JSON_UNESCAPED_UNICODE);
        exit();
    }
    public function getAdvancePaymentById()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $paymentId = $this->input->get('paymentId');

        $paymentAdvanceData = $this->AdvancePaymentModel->getAdvancePaymentById($paymentId);

        $response = array();

        if (!empty($paymentAdvanceData)) {
            $response['status'] = 200;
            $response['message'] = 'PaymentAdvance data found';
            $response['data'] = $paymentAdvanceData;
            $response['isAuth'] = true;
        } else {
            $response['status'] = 404;
            $response['message'] = 'No paymentAdvance data found for the provided user';
            $response['data'] = null;
            $response['isAuth'] = true;
        }

        header('Content-Type: application/json');
        echo json_encode($response, JSON_UNESCAPED_UNICODE);
    }

    public function deleteAdvancePaymentById()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $paymentId = $this->input->get('paymentId');


        $result = $this->AdvancePaymentModel->deleteAdvancePaymentById($paymentId);

        if ($result) {
            // Successful deletion
            $this->output
                ->set_status_header(200)
                ->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'تم حذف التسديد بنجاح', 'status' => 200, 'action' => 'true']));
        } else {
            // Error handling if the deletion fails
            $this->output
                ->set_status_header(500)
                ->set_content_type('application/json')
                ->set_output(json_encode(['message' => 'فشل حذف التسديد', 'status' => 400, 'action' => 'false']));
        }


    }

    public function getCustomRequestPayments()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $startDate = isset($_GET['startDate']) ? $_GET['startDate'] : null;
        $endDate = isset($_GET['endDate']) ? $_GET['endDate'] : null;
        $hash = $this->input->get('hash');

        if ($startDate === null || $endDate === null) {

            $response = [
                "status" => 400,
                "message" => "Please provide start , end dates and reqPaymentId",
                "data" => null,
                "isAuth" => true,
            ];
            $this->outputResponse($response);
            exit();
        } else {

            $paymentsRequests = $this->AdvancePaymentModel->getCustomRequestPayments($hash, $startDate, $endDate);
            $response = [];

            if (!empty($paymentsRequests)) {
                $response['status'] = 200;
                $response['message'] = 'Payments Requests data found';
                $response['data'] = $paymentsRequests;
                $response['isAuth'] = true;
            } else {
                $response['status'] = 404;
                $response['message'] = 'No Payments Requests data found for the provided Date';
                $response['data'] = null;
                $response['isAuth'] = true;
            }

            $this->outputResponse($response);

            exit();

        }
    }

    public function getCustomAdvancePayments()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $startDate = isset($_GET['startDate']) ? $_GET['startDate'] : null;
        $endDate = isset($_GET['endDate']) ? $_GET['endDate'] : null;
        $reqPaymentId = isset($_GET['reqPaymentId']) ? (int) $_GET['reqPaymentId'] : null;


        if ($startDate === null || $endDate === null) {
            $response = [
                'status' => 400,
                'message' => 'startDate and endDate are required',
                'data' => null,
                'isAuth' => true,
            ];
            $this->outputResponse($response);
            exit();
        } else {


            $paymentAdvanceData = $this->AdvancePaymentModel->getCustomAdvancePayments($startDate, $endDate, $reqPaymentId);

            $response = [];

            if (!empty($paymentAdvanceData)) {
                $response['status'] = 200;
                $response['message'] = 'Payments Requests data found';
                $response['data'] = $paymentAdvanceData;
                $response['isAuth'] = true;
            } else {
                $response['status'] = 404;
                $response['message'] = 'No Payments Requests data found for the provided Date';
                $response['data'] = null;
                $response['isAuth'] = true;
            }

            $this->outputResponse($response);

            exit();

        }


    }

    public function getCustomAdvancePaymentReports()
    {
        $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $startDate = isset($_GET['startDate']) ? $_GET['startDate'] : null;
        $endDate = isset($_GET['endDate']) ? $_GET['endDate'] : null;
        $reqPaymentId = isset($_GET['reqPaymentId']) ? (int) $_GET['reqPaymentId'] : null;


        // Validate start and end date format

        $paymentAdvanceData = $this->AdvancePaymentModel->getCustomAdvancePaymentReports($startDate, $endDate, $reqPaymentId);

        $response = [];

        if (!empty($paymentAdvanceData)) {
            $response['status'] = 200;
            $response['message'] = 'PaymentAdvance data found';
            $response['data'] = $paymentAdvanceData;
            $response['isAuth'] = true;
        } else {
            $response['status'] = 404;
            $response['message'] = 'No paymentAdvance data found for the provided user';
            $response['data'] = null;
            $response['isAuth'] = true;
        }

        $this->outputResponse($response);

        exit();

    }



    private function isValidDateFormat($date)
    {
        return $date === null || (bool) strtotime($date);
    }

    private function outputResponse($response)
    {
        header('Content-Type: application/json');
        echo json_encode($response, JSON_UNESCAPED_UNICODE);
    }


}
