<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Attend extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('AttendModel');
        $this->load->model('UserModel');
        $this->load->model('EmpCalcWorkedHoursConfigModel');
        $this->load->model('Scheduled_tasks_model');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all Groups
    public function getAttends()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $date_start = $this->input->post('date_start');
        $date_end = $this->input->post('date_end');
        $users = $this->AttendModel->getAttends($date_start, $date_end);
        foreach ($users as &$user) {
            $attand = $this->AttendModel->getDayStatus($date_start, $date_end, $user['user_hash']);
            // map attend date => status
            $attand = array_map(function ($item) {
                return array(
                    $item['date'] => $item['status'],
                    "inTime" => $item['inTime'],
                    "outTime" => $item['outTime'],

                );
            }, $attand);
            $user['attand'] = $attand;
        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل البصمات',
                'token' => $token,
                'data' => $users,
                'isAuth' => true,
                'total' => $this->AttendModel->count_records(),
                'start_date' => $date_start,
                'end_date' => $date_end,

            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getAttend()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $day = $this->input->get('day');
        $user_hash = $this->input->get('hash');
        $Attend = $this->AttendModel->getAttend($day, $user_hash);
        // $Attend['user_hash'] = $user_hash;
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بصمة',
                'token' => $token,
                'data' => $Attend,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getTodayAttends()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $page = $this->input->get('page');
        $per_page = $this->input->get('perPage');
        $users = $this->AttendModel->getTodayAttends($page, $per_page);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل البصمات',
                'token' => $token,
                'data' => $users,
                'isAuth' => true,
                //'total' => $this->AttendModel->count_records(),

            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function createAttend()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $user_hash = $this->input->post('user_hash');
        $fingerprint_type = '111';
        $fingerprint_time = $this->input->post('fingerprint_time');
        $in_out = $this->input->post('in_out');

        $data = array(
            'user_hash' => $user_hash,
            'fingerprint_type' => $fingerprint_type,
            'fingerprint_time' => $fingerprint_time,
            'in_out' => $in_out,
            'hash' => $this->jwtmiddleware->createHash(),
        );

        $createAttendRes = $this->AttendModel->createAttend($data);
        $attend = false;
        if ($createAttendRes['status'] == 'success') {
            $attend = $createAttendRes['data'];
        }
        $configData = $this->EmpCalcWorkedHoursConfigModel->getMainConfig();
        $date = (new DateTime($fingerprint_time))->format('Y-d-m');
        $jobData = array(
            'date_start' => $date,
            'date_end' => $date,
            'employees_user_hash_array' => json_encode([$user_hash]),
            'configId' => $configData->id,
        );

        // Check if the attendance was successfully recorded
        if ($attend) {
            if ($date < date('Y-m-d')) {
                $this->Scheduled_tasks_model->insert_task('calculate_specific_employees', $jobData, date('Y-m-d H:i:s'));
            }
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم إضافة البصمة بنجاح',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                    'data' => $attend,
                ),
                JSON_UNESCAPED_UNICODE
            );
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $createAttendRes['message'],
                    'token' => $token,
                    'isAuth' => true,
                    'action' => false,
                    'data' => null,
                ),
                JSON_UNESCAPED_UNICODE
            );
        }

        exit();
    }
    public function updateAttend()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $user_hash = $this->input->post('user_hash');
        $fingerprint_time = $this->input->post('fingerprint_time');
        $in_out = $this->input->post('in_out');
        $hash = $this->input->post('hash');

        $data = array(
            'user_hash' => $user_hash,
            'fingerprint_time' => $fingerprint_time,
            'in_out' => $in_out,
        );

        $getHashAttend = $this->AttendModel->getFingerprint($hash);
        $fingerprint_time = $getHashAttend[0]['fingerprint_time'];
        $date = explode(' ', $fingerprint_time)[0];

        $attend = $this->AttendModel->updateAttend($data, $hash);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل البصمة بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
                'data' => $attend,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function createFlexibleAttend()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $user_hash = $this->input->post('user_hash');
        $fingerprint_type = '111';
        $fingerprint_time = $this->input->post('fingerprint_time');
        $in_out = $this->input->post('in_out');

        $data = array(
            'user_hash' => $user_hash,
            'fingerprint_type' => $fingerprint_type,
            'fingerprint_time' => $fingerprint_time,
            'in_out' => $in_out,
            'hash' => $this->jwtmiddleware->createHash(),
            'is_flexible' => '1',
        );
        $createAttendRes = $this->AttendModel->createFlexibleAttend($data);
        $attend = false;
        if ($createAttendRes['status'] == 'success') {
            $attend = $createAttendRes['data'];
        }

        // Check if the attendance was successfully recorded
        if ($attend) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم إضافة البصمة بنجاح',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                    'data' => $attend,
                ),
                JSON_UNESCAPED_UNICODE
            );
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $createAttendRes['message'],
                    'token' => $token,
                    'isAuth' => true,
                    'action' => false,
                    'data' => null,
                ),
                JSON_UNESCAPED_UNICODE
            );
        }

        exit();

    }
    public function deleteAttend()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $hash = $this->input->post('hash');

        $deleteResponse = $this->AttendModel->deleteAttend($hash);

        if ($deleteResponse) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم حذف البصمة بنجاح',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                    'data' => $deleteResponse,
                ),
                JSON_UNESCAPED_UNICODE
            );
        } else {
            echo json_encode(
                array(
                    'status' => 500,
                    'message' => 'لم يتم حذف البصمة. ',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => false,
                    'data' => null,
                ),
                JSON_UNESCAPED_UNICODE
            );
        }
    }

    public function getFingerprint()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');

        $fingerprint = $this->AttendModel->getFingerprint($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض البصمة',
                'token' => $token,
                'data' => $fingerprint,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}
