<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Auth extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('UserModel');
        $this->load->library('HeaderMiddleware');
        $this->load->library('JwtMiddleware');
    }

    public function index()
    {
    }

    public function login()
    {
        $username = $this->input->post('username');
        $password = $this->input->post('password');

        // Validate username and password
        if ($username == null || $password == null) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'يجب ادخال اسم المستخدم وكلمة المرور',
                    'token' => null,
                    'data' => null,
                    'isAuth' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $user = $this->UserModel->getUser($username, $password);

        if ($user) {
            $hash = $user['hash'];
            $user['username'] = $username;
            $user['password'] = $password;
            $role_ids = $user['role_ids']; // This will be a comma-separated string of role IDs
            $is_mobile = $this->input->post('mobile_token') ? true : false;

            // Convert role_ids to an array for easier manipulation
            $roles = explode(',', $role_ids);

            $cannotLogin = $this->UserModel->isUserHashDeleted($hash);
            if ($user['user_type_name'] !== 'admin') {
                // Check if the user has is_work = 2
                if ($cannotLogin) {
                    echo json_encode(
                        array(
                            'status' => 400,
                            'message' => 'لا يمكن تسجيل الدخول، المستخدم غير نشط',
                            'token' => null,
                            'data' => null,
                            'isAuth' => false,
                        ),
                        JSON_UNESCAPED_UNICODE
                    );
                    exit();
                }

                // Check if the user has a specific role that should not log in via web
                if (!$is_mobile && in_array('7', $roles)) {
                    // Restrict login for specific role on web
                    echo json_encode(
                        array(
                            'status' => 400,
                            'message' => 'الدخول عبر الويب ممنوع لهذه الفئة',
                            'token' => null,
                            'data' => null,
                            'isAuth' => false,
                        ),
                        JSON_UNESCAPED_UNICODE
                    );
                    exit();
                }

            }

            $mobile_token = $this->input->post('mobile_token');
            $userDataToEncode = [
                'name' => $user['name'],
                'hash' => $user['hash'],
                'password' => $password,
                'username' => $username,
                'user_type' => $user['user_type'],
                'image_profile' => $user['image_profile'],
                'user_type_name' => $user['user_type_name'],
                'partment' => $user['partment'],
                'vication_agreement' => $user['vication_agreement'],
                'fingerprint_required' => $user['fingerprint_required'],
                'is_work' => $user['is_work'],
                'role_ids' => $role_ids,
                'is_admin' => $user['is_admin'],
            ];

            if ($mobile_token) {
                $this->UserModel->updateMobileToken($username, $password, $mobile_token);
                $token = $this->jwtmiddleware->encode($userDataToEncode, "99999999999");
                $this->UserModel->updateToken($hash, $token);
            } else {
                $token = $this->jwtmiddleware->encode($userDataToEncode);
            }
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تسجيل الدخول بنجاح',
                    'token' => $token,
                    'data' => $user,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'تأكد من اسم المستخدم وكلمة المرور',
                    'token' => null,
                    'data' => null,
                    'isAuth' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function updateProfile()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $user['hash'];
        $data = $this->input->post();
        $result = $this->UserModel->updateProfile($data, $hash);
        if ($result) {
            $user = $this->UserModel->getUser($data['username'], $data['password']);
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تحديث البيانات بنجاح',
                    'token' => null,
                    'data' => $user,
                    "action" => true,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'نفس البيانات موجودة بالفعل',
                    'token' => null,
                    'data' => null,
                    'isAuth' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    // get all employees
    public function getEmployees()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $employees = $this->UserModel->getEmployees();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تسجيل الدخول بنجاح',
                'token' => $user,
                'data' => $employees,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function error_logs_fingerPrint()
    {
        $error_message = $this->input->post("message");
        $error_code = 400; // HTTP status code for Bad Request

        $this->UserModel->error_logs_fingerPrint($error_message, $error_code);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تسجيل الخطأ بنجاح',
                'token' => null,
                'data' => null,
                'isAuth' => false,
            ),
        );
    }

    public function fingerPrint()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $x = $this->input->post('x');
        $y = $this->input->post('y');
        $type = $this->input->post('type');
        $mobileInfo = $this->input->post('mobileInfo');
        $fingerPrintTime = $this->input->post('fingerprint_time') ?? date('Y-m-d H:i:s');
        $fingerprintType = $this->input->post('fingerprint_type') ?? '111';
        $islos = $this->input->post('isIos') ?? null;
        $app_version = $this->input->post('app_version') ?? null;

        $data = [
            'hash' => $hash,
            'x' => $x,
            'y' => $y,
            'type' => $type,
            'mobileInfo' => $mobileInfo,
            'fingerprint_time' => $fingerPrintTime,
            'fingerprint_type' => $fingerprintType,
        ];

        if ($islos !== null && $app_version !== null) {
            $data['islos'] = $islos;
            $data['app_version'] = $app_version;
        }

        // check if hash , x , y , type , mobileInfo is not null
        if ($hash == null || $x == null || $y == null || $type == null || $mobileInfo == null) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'يجب ادخال جميع البيانات',
                    'data' => null,
                    'isAuth' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $result = $this->UserModel->fingerPrint($data);
        // $this->jwtmiddleware->insertNotToFirebase([
        //     "type" => "fingerPrint",
        //     "status" => $type,
        //     "user" => $hash,
        // ]);

        if (is_string($result)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $result,
                    'data' => null,
                    'isAuth' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تسجيل البصمة بنجاح',
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function fingerPrintsBatch()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $passedData = $this->input->post('data');
        $dataArray = json_decode($passedData, true);
        if (empty($dataArray)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'يجب أن يكون البيانات غير فارغة',
                    'data' => null,
                    'isAuth' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        foreach ($dataArray as $key => $value) {
            $hash = $value['hash'];
            $x = $value['x'];
            $y = $value['y'];
            $type = $value['type'];
            $mobileInfo = $value['mobileInfo'];
            $fingerPrintTime = $value['fingerprint_time'] ?? date('Y-m-d H:i:s');
            $fingerprintType = $value['fingerprint_type'] ?? '111';
            if ($hash == null || $x == null || $y == null || $type == null || $mobileInfo == null) {
                continue;
            }

            $data = [
                'hash' => $hash,
                'x' => $x,
                'y' => $y,
                'type' => $type,
                'mobileInfo' => $mobileInfo,
                'fingerprint_time' => $fingerPrintTime,
                'fingerprint_type' => $fingerprintType,
            ];

            $this->UserModel->fingerPrint($data);

        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم ارسال البصمات',
                'data' => null,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function summary()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $user['hash'];
        $summary = $this->UserModel->summary($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'ملخص الموظف',
                'data' => $summary,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function updateUserPermission()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $senderHash = $user['hash'];
        // get all post data
        $data = $this->input->post();
        // remove hash from data
        $hash = $data['hash'];
        unset($data['hash']);
        $result = $this->UserModel->updateUserPermission($hash, $data, $senderHash);
        echo json_encode($result, JSON_UNESCAPED_UNICODE);
        exit();
    }

    public function getUserPermission()
    {
        $this->load->library('TokenMiddleware');
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $senderHash = $this->jwtmiddleware->decode($token)['hash'];

        $hash = $this->input->get('hash');
        $permission = $this->UserModel->getUserPermission($hash, $senderHash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض الصلاحيات بنجاح',
                'data' => $permission,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getUserActivePermission()
    {
        $token = $this->tokenmiddleware->main();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $user['hash'];
        $permission = $this->UserModel->getUserActivePermission($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض الصلاحيات بنجاح',
                'data' => $permission,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function updateFtthAuthToken()
    {
        $token = $this->input->post('token');
        $id = $this->input->post('id');

        $this->UserModel->updateAuthToken($token, $id);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'information updated succufully',
                'token' => null,
                'data' => null,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getFtthAuthToken()
    {
        $id = $this->input->get('id');

        $authToken = $this->UserModel->getAuthToken($id);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'information retrieved succufully',
                'token' => null,
                'data' => $authToken,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}
