<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Client extends CI_Controller
{

    function __construct()
    {
        parent::__construct();
        $this->load->model('ClientModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all Groups
    public function getALl()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $search = $headers['q'] ?? '';
        $result = $this->ClientModel->get_all($page, $perPage, $search);
        echo json_encode(
            array(
                'status' => 200,
                'message' => ' عرض العملاء بنجاح',
                'token' => $token,
                'data' => $result,
                'isAuth' => true,
                'total' => $this->ClientModel->count_records($search)
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function get()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $result = $this->ClientModel->get($hash);
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض العميل',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء عرض العميل',
                    'token' => $token,
                    'isAuth' => true,
                    'hash' => $hash
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        exit();
    }

    public function create()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // Get data from the user input
        // name, address, id, hash, created_at, updated_at
        $data = array(
            'hash' => $this->jwtmiddleware->createHash(),
            'name' => $this->input->post('name'),
            'address' => $this->input->post('address'),
            'phone' => $this->input->post('phone'),
        );

        // Call the create client function in the model
        $result = $this->ClientModel->add($data);

        // Check if the client was successfully created
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم إنشاء العميل بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'حدث خطأ أثناء إنشاء العميل',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function update()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = array(
            'name' => $this->input->post('name'),
            'address' => $this->input->post('address'),
            'phone' => $this->input->post('phone'),
        );
        $this->ClientModel->update($hash, $data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل العميل بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function delete()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $this->ClientModel->delete($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف العميل بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}