<?php

defined('BASEPATH') or exit('No direct script access allowed');

class CronJob extends CI_Controller
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model('FingerPrintSchedulerModel');
        $this->load->model('EmployeesWorkedHoursModel');
        $this->load->model('SubscribersCustomersFTTHModel');
        $this->load->model('WhatsAppModel');
        $this->load->model('SMSModel');
        $this->load->model('TaskModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function runCronJob()
    {
        $this->FingerPrintSchedulerModel->insertAcutalAbsences();
        $this->FingerPrintSchedulerModel->insertAbsenceFlexible();
        $logData = '';
        $pendingWorkedHoursRecords = $this->FingerPrintSchedulerModel->get_pending_worked_hours_records();

        // this works for scheduled fingerPrints fine
        foreach ($pendingWorkedHoursRecords as $record) {
            $date = $record->date;
            $userHash = $record->user_hash;

            if ($record->is_flexible == '1') {
                $this->FingerPrintSchedulerModel->update_flexible_worked_hours($userHash, $date, $record->id);
            }

            $this->FingerPrintSchedulerModel->update_worked_hours($userHash, $date, $record->id);
            // $this->EmployeesWorkedHoursModel->update_worked_hours($date, $date, $userHash, $record->id);
            $logData .= "Processed worked hours for record ID: " . $record->user_hash . "\n";
        }

        // Retrieve pending missed out fingerprints
        $pendingMissedOutFingerprints = $this->FingerPrintSchedulerModel->get_pending_missed_out_fingerprint();

        // Process pending missed out fingerprints

        foreach ($pendingMissedOutFingerprints as $record) {
            $date = $record->date;
            $userHash = $record->user_hash;
            $fingerPrintInId = $record->fingerprint_in_id;
            $schedulerId = $record->id;

            if ($record->is_flexible == '1') {
                $this->FingerPrintSchedulerModel->fixFlexibleForgoutToOut($fingerPrintInId, $schedulerId);
                $logData .= "should works now cuz is flexible";
            }

            if ($record->is_flexible == '0') {
                $this->FingerPrintSchedulerModel->fixForgoutToOut($fingerPrintInId, $schedulerId);
                $logData .= "should works now cuz is not flexible";
            }

            $logData .= "Processing missed out fingerprint for record ID: " . $record->fingerprint_in_id . "\n";
        }

        $logData .= "Cron job executed successfully!\n";

        echo $logData;
    }

    public function automatedFingerOutForDutyVacations()
    {
        $now = date('Y-m-d H:i:s');


        $pendingMissedOutFingerprints = $this->FingerPrintSchedulerModel->get_partial_fingerprints();

        // Process pending missed out fingerprints

        foreach ($pendingMissedOutFingerprints as $record) {
            $date = $record->date;
            $userHash = $record->user_hash;
            $fingerPrintInId = $record->fingerprint_in_id;
            $schedulerId = $record->id;

            if ($record->is_flexible == '1') {
                $this->FingerPrintSchedulerModel->fixFlexibleForgoutToOut($fingerPrintInId, $schedulerId);
                $logData .= "should works now cuz is flexible";
            }

            if ($record->is_flexible == '0') {
                $this->FingerPrintSchedulerModel->fixForgoutToOut($fingerPrintInId, $schedulerId);
                $logData .= "should works now cuz is not flexible";
            }

            $logData .= "Processing missed out fingerprint for record ID: " . $record->fingerprint_in_id . "\n";
        }

        $logData .= "Cron job executed successfully!\n";

        echo $logData;
    }

    public function automatedFingerPrintSchedulerModel()
    {
        $handleOutFingerPrints = $this->FingerPrintSchedulerModel->handle_partial_fingerprintOuts();
        echo $handleOutFingerPrints;

        $handleInFingerPrints = $this->FingerPrintSchedulerModel->handle_partial_fingerprintInIds();
        echo $handleInFingerPrints['message'];
    }

    public function remindTasks()
    {

        $tasks = $this->TaskModel->getTasksDueWithin24Hours();

        foreach ($tasks as $task) {
            $this->sendReminder($task);

            $this->TaskModel->markTaskAsNotified($task['hash']);
        }
    }

    private function sendReminder($task)
    {
        $message = "تذكير: لديك مهمة {$task['title']} ستبدأ غداً. تأكد من التحضير لها.";

        $data = json_encode(array('type' => 3));

        $receiver = $task['accept_user'];
        $mobileToken = $this->UserModel->getMobileToken($receiver);

        $this->jwtmiddleware->sendNot(
            "تذكير: الواجب سوف يبدأ غداً",
            $data,
            $mobileToken,
            $message
        );
    }


    public function sendWhatsAppMesssageDynamic()
    {
        $whatsAppTemplates = $this->WhatsAppModel->getAllTemplatesWithNotifiy();
        foreach ($whatsAppTemplates as $template) {

            // 24 hours
            if ($template['hash_type'] == '111') {
                $customers = $this->SubscribersCustomersFTTHModel->getExpiringCustomerDataWithoutRecentMessage();


                foreach ($customers as $customer) {
                    $formattedMobile = '+964' . ltrim($customer->mobile, '0');  // Remove leading '0' and prepend '+964'

                    // Send WhatsApp message
                    $resJson = $this->jwtmiddleware->sendWhatsMessageForMultiUsers(
                        [
                            'body' => $template['message'],
                            'numbers' => [$formattedMobile]
                        ]
                    );

                    $res = json_decode($resJson, true);


                    if ($res && $res['sent'] == "true" && $res['message'] == "ok") {
                        // Message sent successfully
                        echo "Message sent successfully. Updating log.\n";

                        // Update the log in the database
                        $logData = [
                            'customer_id' => intval($customer->customer_id),
                            'send_date' => date('Y-m-d H:i:s'),
                            'notify_type' => $template['notify_type_id'],
                            'expires' => $customer->expires,
                            'status' => 'success',
                            'details' => $res['message']
                        ];
                        $this->WhatsAppModel->insertWhatsAppMessageLog($logData);
                    } else if ($res && $res['sent'] == "true" && $res['message'] != "ok") {
                        // Flag as pending
                        echo "Message sent but flagged as pending.\n";
                        $logData = [
                            'customer_id' => intval($customer->customer_id),
                            'send_date' => date('Y-m-d H:i:s'),
                            'notify_type' => $template['notify_type_id'],
                            'expires' => $customer->expires,
                            'status' => 'pending',
                            'details' => $res['message']
                        ];
                        $this->WhatsAppModel->insertWhatsAppMessageLog($logData);
                    } else if ($res && $res['sent'] != "true") {
                        // Flag as failed
                        echo "Message failed to send.\n";
                        $logData = [
                            'customer_id' => intval($customer->customer_id),
                            'send_date' => date('Y-m-d H:i:s'),
                            'notify_type' => $template['notify_type_id'],
                            'expires' => $customer->expires,
                            'status' => 'failed',
                            'details' => $res['message']
                        ];
                        $this->WhatsAppModel->insertWhatsAppMessageLog($logData);
                    } else {
                        // Handle unknown failure
                        echo "Unknown error occurred.\n";
                        $logData = [
                            'customer_id' => intval($customer->customer_id),
                            'send_date' => date('Y-m-d H:i:s'),
                            'notify_type' => $template['notify_type_id'],
                            'expires' => $customer->expires,
                            'status' => 'unknown_error',
                            'details' => $res['message']
                        ];
                        $this->WhatsAppModel->insertWhatsAppMessageLog($logData);
                    }
                }
            }

            // 4 hours
            if ($template['hash_type'] == '222') {
                $customers = $this->SubscribersCustomersFTTHModel->getExpiringWithinFourHoursCustomerDataWithoutRecentMessage();


                foreach ($customers as $customer) {

                    $formattedMobile = '+964' . ltrim($customer->mobile, '0');

                    // Send WhatsApp message
                    $resJson = $this->jwtmiddleware->sendWhatsMessageForMultiUsers(
                        [
                            'body' => $template['message'],
                            'numbers' => [$formattedMobile]
                        ]
                    );

                    $res = json_decode($resJson, true);


                    if ($res && $res['sent'] == "true" && $res['message'] == "ok") {
                        // Message sent successfully
                        echo "Message sent successfully. Updating log.\n";

                        // Update the log in the database
                        $logData = [
                            'customer_id' => intval($customer->customer_id),
                            'send_date' => date('Y-m-d H:i:s'),
                            'notify_type' => $template['notify_type_id'],
                            'expires' => $customer->expires,
                            'status' => 'success',
                            'details' => $res['message']
                        ];
                        $this->WhatsAppModel->insertWhatsAppMessageLog($logData);
                    } else if ($res && $res['sent'] == "true" && $res['message'] != "ok") {
                        // Flag as pending
                        echo "Message sent but flagged as pending.\n";
                        $logData = [
                            'customer_id' => intval($customer->customer_id),
                            'send_date' => date('Y-m-d H:i:s'),
                            'notify_type' => $template['notify_type_id'],
                            'expires' => $customer->expires,
                            'status' => 'pending',
                            'details' => $res['message']
                        ];
                        $this->WhatsAppModel->insertWhatsAppMessageLog($logData);
                    } else if ($res && $res['sent'] != "true") {
                        // Flag as failed
                        echo "Message failed to send.\n";
                        $logData = [
                            'customer_id' => intval($customer->customer_id),
                            'send_date' => date('Y-m-d H:i:s'),
                            'notify_type' => $template['notify_type_id'],
                            'expires' => $customer->expires,
                            'status' => 'failed',
                            'details' => $res['message']
                        ];
                        $this->WhatsAppModel->insertWhatsAppMessageLog($logData);
                    } else {
                        // Handle unknown failure
                        echo "Unknown error occurred.\n";
                        $logData = [
                            'customer_id' => intval($customer->customer_id),
                            'send_date' => date('Y-m-d H:i:s'),
                            'notify_type' => $template['notify_type_id'],
                            'expires' => $customer->expires,
                            'status' => 'unknown_error',
                            'details' => $res['message']
                        ];
                        $this->WhatsAppModel->insertWhatsAppMessageLog($logData);
                    }
                }
            }
        }
        $this->jwtmiddleware->insertMutedNotification();

    }


    public function sendSMSForFailedWhatsAppCustomers()
    {
        // Get all customers who failed to send WhatsApp message
        $customersExpiringIn24h = $this->SubscribersCustomersFTTHModel->get24hExpiringCustomersWithFailedMessages();
        $customersExpiringIn4h = $this->SubscribersCustomersFTTHModel->get4hExpiringCustomersWithFailedMessages();
        $token = "fPcZ2weKRNmTDZPHEKwH1A:APA91bEmcMtTpQELg1IQ0BdYdZVmvIS4MTG40v_uLQlH3wSH_2_nPIGOHQIi68fZ3ox4FWMw_OGAx2-vlC7rJg-QY1iJW8hNAuCCh293FeBtBccpEjyDjZo";

        foreach ($customersExpiringIn24h as $customer) {
            $body = "";
            $title = "";
            $phone = $customer->mobile;

            $message = "ايرث بغداد للانترنت اشتراكك سينتهي خلال 24ساعة للتفعيل اتصل07821205889";
            $res = $this->jwtmiddleware->sendSMSNot($token, $title, $body, $phone, $message);
            if ($res['status'] == 200) {
                $notifiy_type = $this->SMSModel->getNotifiyFor24hrsMessage();
                $logData = [
                    'customer_id' => intval($customer->customer_id),
                    'notify_type' => $notifiy_type['id'],
                    'send_date' => date('Y-m-d H:i:s'),
                    'expires' => $customer->expires,
                ];
                $this->SMSModel->insertSMSAppMessageLog($logData);
            }
        }

        foreach ($customersExpiringIn4h as $customer) {
            $body = "";
            $title = "";
            $phone = $customer->mobile;

            $message = "ايرث بغداد للانترنت اشتراكك سينتهي خلال 4ساعة للتفعيل اتصل07821205889";
            $res = $this->jwtmiddleware->sendSMSNot($token, $title, $body, $phone, $message);
            if ($res['status'] == 200) {
                $notifiy_type = $this->SMSModel->getNotifiyFor4hrsMessage();
                $logData = [
                    'customer_id' => intval($customer->customer_id),
                    'notify_type' => $notifiy_type['id'],
                    'send_date' => date('Y-m-d H:i:s'),
                    'expires' => $customer->expires,
                ];
                $this->SMSModel->insertSMSAppMessageLog($logData);
            }
        }

        $this->jwtmiddleware->insertMutedNotification();

    }



    // here on mobile if user not open application I need to send hidden notification to make user always send his location


    // I will run this every minute
    public function insertNotToMobileToSendLocation()
    {
        $this->jwtmiddleware->sendToMapNot();
    }

    public function notifyUsIfFTTHNotWorking()
    {

        $checks = [
            ['table' => 'ftth_auth', 'id' => 1, 'timeLimit' => 1800, 'column' => 'modefilDate', 'message' => 'Token not updated in 30 minutes'],
            ['table' => 'subscribers_customers_ftth', 'id' => 1, 'timeLimit' => 3600, 'column' => 'updated_at', 'message' => 'Subscription not updated in 1 hour'],
            ['table' => 'subscribers_customers_information_ftth', 'id' => 1, 'timeLimit' => 9000, 'column' => 'updated_at', 'message' => 'Customer info not updated in 2.5 hours'],
            ['table' => 'subscribers_customers_ftth_all_expired', 'id' => 1, 'timeLimit' => 12600, 'column' => 'updated_at', 'message' => 'Expired customer data not updated in 3.5 hours'],
        ];

        foreach ($checks as $check) {
            if (!$this->SubscribersCustomersFTTHModel->checkUpdatedWithinTime($check['table'], $check['id'], $check['timeLimit'], $check['column'])) {
                $this->sendSupportMessage($check['message']);
            }
        }

    }

    private function sendSupportMessage($message)
    {
        $this->jwtmiddleware->sendWhatsMessageForMultiUsers(
            [
                'body' => $message,
                'numbers' => ['+9647703460461']
            ]
        );
    }


}
