<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Employees extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('EmployeesModel');
        $this->load->model('RolesPermissionsModel');
        $this->load->model('UserModel');
        $this->load->model('AttendModel');
        $this->load->model('GroupScheduleModel');
        $this->load->model('ScheduleModel');
        $this->load->model('UserAttachmentModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all employees
    public function getEmployees()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $employees = $this->EmployeesModel->getEmployees();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الموظفين',
                'token' => $token,
                'data' => $employees,
                'isAuth' => true,
                'total' => $this->EmployeesModel->count_records(),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function getTaskMessages()
    {
        $taskHash = $this->input->get('taskHash');
        $userhash = $this->input->get('user_hash');

        $result = $this->EmployeesModel->getTaskMessages($taskHash, $userhash);

        echo json_encode(
            array('status' => 200, 'message' => 'Success', 'token' => null, 'data' => $result, 'isAuth' => false),
            JSON_UNESCAPED_UNICODE
        );
    }

    public function getEmployeesWithTaskHash()
    {

        $taskHash = $this->input->get('taskHash');

        $result = $this->EmployeesModel->getEmployeesWithTaskHash($taskHash);

        echo json_encode(
            array('status' => 200, 'message' => 'Success', 'token' => null, 'data' => $result, 'isAuth' => false),
            JSON_UNESCAPED_UNICODE
        );
    }
    public function getSalaryTypes()
    {
        $this->UserModel->checkPermission();
        $salaryTypes = $this->EmployeesModel->getSalaryTypes();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'Success',
                'token' => null,
                'data' => $salaryTypes,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeesWithPayments()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $employees = $this->EmployeesModel->getEmployeesWithPayments();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الموظفين',
                'data' => $employees,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeesWithSpecifications()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $employees = $this->EmployeesModel->getEmployeesWithSpecifications();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الموظفين',
                'data' => $employees,
                'isAuth' => true,
                'total' => $this->EmployeesModel->count_employees_withSpecification(),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function getEmployeesWithPayroll()
    {
        $this->UserModel->checkPermission();
        $from = $this->input->get("from");
        $to = $this->input->get("to");

        $employees = $this->EmployeesModel->getEmployeesWithPayroll($from, $to);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الموظفين',
                'data' => $employees,
                'isAuth' => true,
                'total' => $this->EmployeesModel->countEmployeesWithPayroll($from, $to),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeeDetailWithPayroll()
    {
        $this->UserModel->checkPermission();
        $hash = $this->input->get('emp_id');

        if (empty($hash)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'الرجاء  تقديم البيانات الصحيحة',
                    'data' => null,
                    'isAuth' => true,
                    'action' => false,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        $employee = $this->EmployeesModel->getEmployeeDetailWithPayroll($hash);
        if (empty($employee)) {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'لم يتم ايجاد موظف',
                    'data' => [],
                    'isAuth' => true,
                    'action' => false,

                ),
                JSON_UNESCAPED_UNICODE,
            );
            exit();
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الموظفين',
                'data' => $employee,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function timeStringToSeconds($timeStr)
    {
        list($hours, $minutes, $seconds) = explode(':', $timeStr);
        return $hours * 3600 + $minutes * 60 + $seconds;
    }

    public function secondsToTimeString($seconds)
    {
        $hours = floor($seconds / 3600);
        $minutes = floor(($seconds % 3600) / 60);
        $remainingSeconds = $seconds % 60;
        return sprintf('%02d:%02d:%02d', $hours, $minutes, $remainingSeconds);
    }

    // public function getEmployeesPayrollData()
    // {
    //     $token = $this->tokenmiddleware->main();
    //     $this->UserModel->checkPermission();
    //     $year_months = $this->input->get('year_months');
    //     $start_date = $this->input->get('startDate');
    //     $end_date = $this->input->get('endDate');
    //     if (empty($year_months)) {
    //         echo json_encode(
    //             array(
    //                 'status' => 400,
    //                 'message' => 'الرجاء  تقديم البيانات الصحيحة',
    //                 'data' => null,
    //                 'isAuth' => true,
    //                 'action' => false,
    //             ),
    //             JSON_UNESCAPED_UNICODE
    //         );
    //         exit();
    //     }

    //     $employee = $this->EmployeesModel->getEmployeesPayrollData($year_months, $start_date, $end_date);
    //     foreach ($employee as &$i) {
    //         // Correctly accessing the current element in the loop
    //         $i['price_group'] = json_decode($i['price_group'], true);
    //     }
    //     $employees = $employee;
    //     foreach ($employees as &$employee) {
    //         $qualificationsArray = [];
    //         $positionsArray = [];
    //         $othersArray = [];

    //         $discountsArray = [];

    //         $totalDiscount = 0;
    //         $totalPositions = 0;
    //         $totalSpecification = 0;
    //         $totalOthers = 0;

    //         $uniqueMap = [];
    //         $uniqueArray = [];

    //         foreach ($employee['price_group'] as $element) {
    //             $innerKey = array_key_first($element); // Get the key of the inner array
    //             $nameSpecification = $element[$innerKey]['name_specifications'];

    //             $combinedKey = $innerKey . ':' . $nameSpecification; // Generate a unique key for comparison

    //             if (!isset($uniqueMap[$combinedKey])) {
    //                 $uniqueMap[$combinedKey] = true; // Mark it as seen
    //                 $uniqueArray[] = $element; // Add to the unique array
    //             }
    //         }
    //         // $uniqueArray now contains the array with unique name_specifications values

    //         $employee['price_group'] = $uniqueArray;

    //         // Process the price_group of each employee
    //         foreach ($employee['price_group'] as $item) {
    //             if (isset($item['specifications_qualifictions'])) {
    //                 $nameSpecifications = $item['specifications_qualifictions']['name_specifications'] ?? null;
    //                 $price = min($item['specifications_qualifictions']['total_price'], $item['specifications_qualifictions']['maximumPrice']);

    //                 if ($nameSpecifications !== null && !isset($qualificationsArray[$nameSpecifications])) {

    //                     $qualificationsArray[] = ['nameSpecifications' => $nameSpecifications, 'price' => $price];
    //                     $totalSpecification += $price;
    //                 }
    //             } elseif (isset($item['positions_qualifictions'])) {
    //                 $price = min($item['positions_qualifictions']['total_price'], $item['positions_qualifictions']['maximumPrice']);
    //                 $positions_qualifictions = $item['positions_qualifictions']['name_specifications'] ?? null;

    //                 if ($positions_qualifictions !== null) {
    //                     $positionsArray[] = ['positions_qualifictions' => $positions_qualifictions, 'price' => $price];
    //                     $totalPositions += $price;
    //                 }
    //             } elseif (isset($item['specifications_discounts'])) {
    //                 $price = min($item['specifications_discounts']['total_price'], $item['specifications_discounts']['maximumPrice']);
    //                 $specifications_discounts = $item['specifications_discounts']['name_specifications'] ?? null;

    //                 if ($specifications_discounts !== null) {
    //                     $discountsArray[] = ['specifications_discounts' => $specifications_discounts, 'price' => $price];
    //                     $totalDiscount += $price;
    //                 }
    //             } elseif (isset($item['specifications_others'])) {
    //                 $price = min($item['specifications_others']['total_price'], $item['specifications_others']['maximumPrice']);
    //                 $specifications_others = $item['specifications_others']['name_specifications'] ?? null;

    //                 if ($specifications_others !== null) {
    //                     $othersArray[] = ['specifications_others' => $specifications_others, 'price' => $price];
    //                     $totalOthers += $price;
    //                 }
    //             }
    //         }

    //         $employee['qualifcations'] = $qualificationsArray;
    //         $employee['positions'] = $positionsArray;
    //         $employee['discounts'] = $discountsArray;
    //         $employee['others'] = $othersArray;
    //         $employee['totalDiscount'] = round($totalDiscount, -3);
    //         $employee['totalPositions'] = round($totalPositions, -3);
    //         $employee['totalSpecification'] = round($totalSpecification, -3);
    //         $employee['totalOthers'] = round($totalOthers, -3);
    //         // $employee['total_salary'] = round($employee['primary_salary'] + $employee['total_Yearly_salary'] + $totalSpecification + $totalPositions + $totalPositions, -3);
    //         $employee['total_salary'] = round($employee['primary_salary'] + $employee['total_Yearly_salary'] + $totalSpecification + $totalPositions + $totalOthers + $totalDiscount, -3);
    //         // Split worked_hours and target_hours into hours, minutes, and seconds
    //         $workHourParts = explode(":", $employee['worked_hours']);
    //         $workHourTargetParts = explode(":", $employee['target_hours']);

    //         // Check if the explode function returns an array with at least three elements
    //         if (count($workHourParts) >= 3 && count($workHourTargetParts) >= 3) {
    //             // Convert hours, minutes, and seconds to integers for worked_hours
    //             $hours1 = (int) $workHourParts[0];
    //             $minutes1 = (int) $workHourParts[1];
    //             $seconds1 = (int) $workHourParts[2];

    //             // Calculate the total number of hours for worked_hours
    //             $totalHoursWorked = $hours1 + $minutes1 / 60 + $seconds1 / 3600;

    //             // Convert hours, minutes, and seconds to integers for target_hours
    //             $hours2 = (int) $workHourTargetParts[0];
    //             $minutes2 = (int) $workHourTargetParts[1];
    //             $seconds2 = (int) $workHourTargetParts[2];

    //             // Calculate the total number of hours for target_hours
    //             $totalHoursTarget = $hours2 + $minutes2 / 60 + $seconds2 / 3600;

    //             // Calculate the hourly rate
    //             if ($totalHoursTarget != 0) {
    //                 $hourlyRate = (float) $employee['total_salary'] / $totalHoursTarget;
    //             } else {
    //                 $hourlyRate = 0;
    //             }
    //             $employee['hourlyRate'] = round($hourlyRate, -3);
    //             // echo round($totalHoursTarget, -2);
    //             // echo $totalHoursWorked;
    //             $employee['missedHours'] = round($totalHoursTarget - $totalHoursWorked);
    //         } else {
    //             // Handle the case where the explode function does not return an array with at least three elements
    //             // For example, you might want to set default values or log an error
    //             $totalHoursWorked = 0;
    //             $totalHoursTarget = 0;
    //             $hourlyRate = 0; // or any default value

    //         }

    //         // Check if total hours worked is less than total hours target
    //         if ($totalHoursWorked < $totalHoursTarget) {
    //             $discountAmount = round($hourlyRate * ($totalHoursTarget - $totalHoursWorked), -3);
    //             $employee['discountAmount'] = $discountAmount;
    //         } else {
    //             $employee['discountAmount'] = $discountAmount = 0;
    //         }
    //         // بدون الاستقطاعات
    //         $employee['salaryAfterDiscount'] = round($employee['primary_salary'] +
    //             $employee['total_Yearly_salary'] +
    //             ($employee['differences_salary'] ?? 0) +
    //             $employee['totalSpecification'] -
    //             $discountAmount, -3);

    //         $sum_rewards = isset($employee['sum_rewards']) ? round((int) $employee['sum_rewards'], -3) : 0;
    //         $sum_penalties = isset($employee['sum_penalties']) ? round((int) $employee['sum_penalties'], -3) : 0;

    //         $employee['sum_penalties'] = $sum_penalties;
    //         $employee['sum_rewards'] = $sum_rewards;
    //         // بل الاستقطاعات
    //         $employee['flat_salary'] = round($employee['total_salary'] - $discountAmount + $sum_rewards + $sum_penalties, -3);
    //         $employee['total_years'] = (int) $employee['number_of_year'] + (int) $employee['number_of_year_exp'];

    //     }

    //     // $employee[0]['price_group'] = json_decode($employee[0]['price_group'], true);
    //     if (empty($employee)) {
    //         echo json_encode(
    //             array(
    //                 'status' => 404,
    //                 'message' => 'لم يتم ايجاد موظف',
    //                 'data' => [],
    //                 'isAuth' => true,
    //                 'action' => false,

    //             ),
    //             JSON_UNESCAPED_UNICODE,
    //         );
    //         exit();
    //     }
    //     echo json_encode(
    //         array(
    //             'status' => 200,
    //             'message' => 'عرض كل الموظفين',
    //             'data' => $employees,
    //             'isAuth' => true,
    //             'total' => $this->EmployeesModel->countEmployeesPayrollData($year_months),
    //         ),
    //         JSON_UNESCAPED_UNICODE
    //     );
    //     exit();
    // }

    // create new employee
    public function createEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // get all data from post
        $data = $this->input->post();
        $data['hash'] = $this->jwtmiddleware->createHash();
        $result = $this->EmployeesModel->createEmployee($data);

        if ($result['status']) {
            $this->EmployeesModel->updateEmpInfo($data);
            $this->EmployeesModel->updateUserVacations($data['vacationData'], $data['hash']);
            $this->EmployeesModel->createLocations($data);
            $this->GroupScheduleModel->updateEmployeeActualSchedule($data['hash']);
            $userPermissions = str_replace("'", '"', $data['user_permissions']);
            $userPermissions = json_decode($data['user_permissions'], true);

            $this->RolesPermissionsModel->insertUserRole($data['hash'], $userPermissions);
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم اضافة الموظف بنجاح',
                    'token' => $token,
                    'data' => $data,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            // Handle the error message
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $result['message'],
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    // get employee by hash
    public function getEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $today = date('Y-m-d');
        $employee = $this->EmployeesModel->getEmployee($hash);
        if (empty($employee)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لا يوجد موظف بهذا الرقم',
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $employeeVacations = $this->EmployeesModel->getEmployeeVacations($hash);
        $employeeUserVacations = $this->EmployeesModel->getUserVacations($hash);
        $employeeSchedule = $this->GroupScheduleModel->getScheudleToday($hash);
        $employeeData['scheduleName'] = $this->GroupScheduleModel->getScheduleName($hash);

        $data['finger_print'] = $this->AttendModel->getCurrentMonthAttendForEmp($hash);
        $data['finger_print']['attend'] = $data['finger_print']['attend'] ?? "";
        $data['finger_print']['name'] = $data['finger_print']['name'] ?? $employee[0]['name'];
        $data['finger_print']['user_hash'] = $data['finger_print']['user_hash'] ?? $hash;
        $employeeAttachments = $this->UserAttachmentModel->get_all($hash);
        $employeeRules = $this->EmployeesModel->getEmployeeRules($hash);

        if (count($employeeVacations) > 0) {
            $data['vacations'] = $employeeVacations;
        } else {
            $data['vacations'] = [];
        }

        if (count($employee) > 0) {
            $data['employee'] = $employee[0];
        }

        if (count($employeeAttachments) > 0) {
            $data['attachments'] = $employeeAttachments;
        } else {
            $data['attachments'] = [];
        }
        $data['user_permissions'] = $employeeRules;
        $data['vacationDays'] = $employeeUserVacations;
        $data['scheduleData'] = $employeeSchedule;
        $data['scheduleName'] = $employeeData['scheduleName'];

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بيانات الموظف',
                'token' => $token,
                'data' => $data ?? [],
                //array('employee'=>$employee,'employeeVacations'=>$employeeVacations, 'employeeFingerPrint'=>$employeeFingerPrint ),
                'isAuth' => true,
                'action' => false
            ),
            JSON_UNESCAPED_UNICODE,
        );
        exit();
    }

    public function getEmployeeForUpdateInfo()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $today = date('Y-m-d');
        $employee = $this->EmployeesModel->getEmployeeForUpdateInfo($hash);
        if (empty($employee)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لا يوجد موظف بهذا الرقم',
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $employeeVacations = $this->EmployeesModel->getEmployeeVacations($hash);
        $employeeUserVacations = $this->EmployeesModel->getUserVacations($hash);
        $employeeSchedule = $this->GroupScheduleModel->getScheudleToday($hash);
        $data['finger_print'] = $this->AttendModel->getCurrentMonthAttendForEmp($hash);
        $data['finger_print']['attend'] = $data['finger_print']['attend'] ?? "";
        $data['finger_print']['name'] = $data['finger_print']['name'] ?? $employee[0]['name'];
        $data['finger_print']['user_hash'] = $data['finger_print']['user_hash'] ?? $hash;
        $employeeAttachments = $this->UserAttachmentModel->get_all($hash);
        $employeeRules = $this->EmployeesModel->getEmployeeRules($hash);

        if (count($employeeVacations) > 0) {
            $data['vacations'] = $employeeVacations;
        } else {
            $data['vacations'] = [];
        }

        if (count($employee) > 0) {
            $data['employee'] = $employee[0];
        }

        if (count($employeeAttachments) > 0) {
            $data['attachments'] = $employeeAttachments;
        } else {
            $data['attachments'] = [];
        }
        $data['user_permissions'] = $employeeRules;
        $data['vacationDays'] = $employeeUserVacations;
        $data['scheduleData'] = $employeeSchedule;
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بيانات الموظف',
                'token' => $token,
                'data' => $data ?? [],
                //array('employee'=>$employee,'employeeVacations'=>$employeeVacations, 'employeeFingerPrint'=>$employeeFingerPrint ),
                'isAuth' => true,
                'action' => false
            ),
            JSON_UNESCAPED_UNICODE,
        );
        exit();
    }

    public function getEmployeeForPrint()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $hash = $this->input->get('hash');

        $employee = $this->EmployeesModel->getEmployeeForPrint($hash);
        if (empty($employee)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لا يوجد موظف بهذا الرقم',
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بيانات الموظف',
                'token' => $token,
                'data' => $employee ?? [],
                //array('employee'=>$employee,'employeeVacations'=>$employeeVacations, 'employeeFingerPrint'=>$employeeFingerPrint ),
                'isAuth' => true,
                'action' => false
            ),
            JSON_UNESCAPED_UNICODE,
        );
        exit();
    }

    public function getEmployeesDetailsTEst()
    {
        $this->UserModel->checkPermission();

        $employee = $this->EmployeesModel->getEmployeesDetailsTEst();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بيانات الموظف',
                'data' => $employee ?? [],
                //array('employee'=>$employee,'employeeVacations'=>$employeeVacations, 'employeeFingerPrint'=>$employeeFingerPrint ),
                'isAuth' => true,
                'action' => false
            ),
            JSON_UNESCAPED_UNICODE,
        );
        exit();
    }

    public function updateFingerPrint()
    {
        $this->UserModel->checkPermission();
        $start_date = $this->input->get('start_date');
        $end_date = $this->input->get('end_date');
        $this->EmployeesModel->updateFingerPrintRecord($start_date, $end_date);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تحديث البيانات',
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function getEmployeeDetailsByHash()
    {
        $this->UserModel->checkPermission();

        $hash = $this->input->get('hash');
        $start_date = $this->input->get('startDate');
        $end_date = $this->input->get('endDate');
        $year_months = $this->input->get('year_months');
        // $employee = $this->EmployeesModel->getEmployeeDetailsByHash($hash);

        $employee = $this->EmployeesModel->getEmployeesPayrollDataWithHash($year_months, $start_date, $end_date, $hash);

        foreach ($employee as &$i) {
            // Correctly accessing the current element in the loop
            $i['price_group'] = json_decode($i['price_group'], true);
        }
        $employees = $employee;
        foreach ($employees as &$employee) {
            $qualificationsArray = [];
            $positionsArray = [];
            $othersArray = [];

            $discountsArray = [];

            $totalDiscount = 0;
            $totalPositions = 0;
            $totalSpecification = 0;
            $totalOthers = 0;

            $uniqueMap = [];
            $uniqueArray = [];

            foreach ($employee['price_group'] as $element) {
                $innerKey = array_key_first($element); // Get the key of the inner array
                $nameSpecification = $element[$innerKey]['name_specifications'];

                $combinedKey = $innerKey . ':' . $nameSpecification; // Generate a unique key for comparison

                if (!isset($uniqueMap[$combinedKey])) {
                    $uniqueMap[$combinedKey] = true; // Mark it as seen
                    $uniqueArray[] = $element; // Add to the unique array
                }
            }
            // $uniqueArray now contains the array with unique name_specifications values

            $employee['price_group'] = $uniqueArray;

            // Process the price_group of each employee
            foreach ($employee['price_group'] as $item) {
                if (isset($item['specifications_qualifictions'])) {
                    $nameSpecifications = $item['specifications_qualifictions']['name_specifications'] ?? null;
                    $price = min($item['specifications_qualifictions']['total_price'], $item['specifications_qualifictions']['maximumPrice']);

                    if ($nameSpecifications !== null && !isset($qualificationsArray[$nameSpecifications])) {

                        $qualificationsArray[] = ['nameSpecifications' => $nameSpecifications, 'price' => $price];
                        $totalSpecification += $price;
                    }
                } elseif (isset($item['positions_qualifictions'])) {
                    $price = min($item['positions_qualifictions']['total_price'], $item['positions_qualifictions']['maximumPrice']);
                    $positions_qualifictions = $item['positions_qualifictions']['name_specifications'] ?? null;

                    if ($positions_qualifictions !== null) {
                        $positionsArray[] = ['positions_qualifictions' => $positions_qualifictions, 'price' => $price];
                        $totalPositions += $price;
                    }
                } elseif (isset($item['specifications_discounts'])) {
                    $price = min($item['specifications_discounts']['total_price'], $item['specifications_discounts']['maximumPrice']);
                    $specifications_discounts = $item['specifications_discounts']['name_specifications'] ?? null;

                    if ($specifications_discounts !== null) {
                        $discountsArray[] = ['specifications_discounts' => $specifications_discounts, 'price' => $price];
                        $totalDiscount += $price;
                    }
                } elseif (isset($item['specifications_others'])) {
                    $price = min($item['specifications_others']['total_price'], $item['specifications_others']['maximumPrice']);
                    $specifications_others = $item['specifications_others']['name_specifications'] ?? null;

                    if ($specifications_others !== null) {
                        $othersArray[] = ['specifications_others' => $specifications_others, 'price' => $price];
                        $totalOthers += $price;
                    }
                }
            }

            $employee['qualifcations'] = $qualificationsArray;
            $employee['positions'] = $positionsArray;
            $employee['discounts'] = $discountsArray;
            $employee['others'] = $othersArray;
            $employee['totalDiscount'] = round($totalDiscount, -3);
            $employee['totalPositions'] = round($totalPositions, -3);
            $employee['totalSpecification'] = round($totalSpecification, -3);
            $employee['totalOthers'] = round($totalOthers, -3);
            // $employee['total_salary'] = round($employee['primary_salary'] + $employee['total_Yearly_salary'] + $totalSpecification + $totalPositions + $totalPositions, -3);
            $employee['total_salary'] = round($employee['primary_salary'] + $employee['total_Yearly_salary'] + $totalSpecification + $totalPositions + $totalOthers + $totalDiscount, -3);
            // Split worked_hours and target_hours into hours, minutes, and seconds
            $workHourParts = explode(":", $employee['worked_hours']);
            $workHourTargetParts = explode(":", $employee['target_hours']);

            // Check if the explode function returns an array with at least three elements
            if (count($workHourParts) >= 3 && count($workHourTargetParts) >= 3) {
                // Convert hours, minutes, and seconds to integers for worked_hours
                $hours1 = (int) $workHourParts[0];
                $minutes1 = (int) $workHourParts[1];
                $seconds1 = (int) $workHourParts[2];

                // Calculate the total number of hours for worked_hours
                $totalHoursWorked = $hours1 + $minutes1 / 60 + $seconds1 / 3600;

                // Convert hours, minutes, and seconds to integers for target_hours
                $hours2 = (int) $workHourTargetParts[0];
                $minutes2 = (int) $workHourTargetParts[1];
                $seconds2 = (int) $workHourTargetParts[2];

                // Calculate the total number of hours for target_hours
                $totalHoursTarget = $hours2 + $minutes2 / 60 + $seconds2 / 3600;

                // Calculate the hourly rate
                if ($totalHoursTarget != 0) {
                    $hourlyRate = (float) $employee['total_salary'] / $totalHoursTarget;
                } else {
                    $hourlyRate = 0;
                }
                $employee['hourlyRate'] = round($hourlyRate, -3);
                // echo round($totalHoursTarget, -2);
                // echo $totalHoursWorked;
                $employee['missedHours'] = round($totalHoursTarget - $totalHoursWorked);
            } else {
                // Handle the case where the explode function does not return an array with at least three elements
                // For example, you might want to set default values or log an error
                $totalHoursWorked = 0;
                $totalHoursTarget = 0;
                $hourlyRate = 0; // or any default value

            }

            // Check if total hours worked is less than total hours target
            if ($totalHoursWorked < $totalHoursTarget) {
                $discountAmount = round($hourlyRate * ($totalHoursTarget - $totalHoursWorked), -3);
                $employee['discountAmount'] = $discountAmount;
            } else {
                $employee['discountAmount'] = $discountAmount = 0;
            }
            // بدون الاستقطاعات
            $employee['salaryAfterDiscount'] = round($employee['primary_salary'] +
                $employee['total_Yearly_salary'] +
                ($employee['differences_salary'] ?? 0) +
                $employee['totalSpecification'] -
                $discountAmount, -3);

            $sum_rewards = isset($employee['sum_rewards']) ? round((int) $employee['sum_rewards'], -3) : 0;
            $sum_penalties = isset($employee['sum_penalties']) ? round((int) $employee['sum_penalties'], -3) : 0;

            $employee['sum_penalties'] = $sum_penalties;
            $employee['sum_rewards'] = $sum_rewards;
            // بل الاستقطاعات
            $employee['flat_salary'] = round($employee['total_salary'] - $discountAmount + $sum_rewards + $sum_penalties, -3);
            $employee['total_years'] = (int) $employee['number_of_year'] + (int) $employee['number_of_year_exp'];

            $vacationData = $employee['vacationData'];
            foreach ($vacationData as $vacation) {
                if ($vacation['vacation_type_hash'] === '222') {
                    $employee['normalVacation'] = $vacation;
                }
                if ($vacation['vacation_type_hash'] === '333') {
                    $employee['timeVacation'] = $vacation;
                }
            }
        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بيانات الموظف',
                'data' => $employee ?? [],
                //array('employee'=>$employee,'employeeVacations'=>$employeeVacations, 'employeeFingerPrint'=>$employeeFingerPrint ),
                'isAuth' => true,
                'action' => false
            ),
            JSON_UNESCAPED_UNICODE,
        );
        exit();
    }

    // update employee
    public function updateEmployee()
    {
        $this->load->model("SpecifactionModel");
        $this->load->model("FinancialDiscountsModel");

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $data = $this->input->post();

        if (empty($data['user_specification_id'])) {
            $this->SpecifactionModel->deletAllSpecificationForUser($data['hash']);
        }

        if (!empty($data['user_specification_id'])) {
            $this->db->trans_start();
            $currentUserSpecifications = $this->SpecifactionModel->getCurrentUserSpecifications($data['hash']);

            $currentUserSpecifications = is_array($currentUserSpecifications) ? $currentUserSpecifications : [];

            $specifications_array = is_array($data['user_specification_id']) ? $data['user_specification_id'] : [];
            $toDelete = array_diff($currentUserSpecifications, $specifications_array);
            foreach ($toDelete as $specification_id) {
                $this->SpecifactionModel->deleteUserSpecification($data['hash'], $specification_id);
            }
            $toInsert = array_diff($specifications_array, $currentUserSpecifications);

            foreach ($toInsert as $specification_id) {
                $user_specificationsData = [
                    'emp_info_id' => $data['hash'],
                    'specification_id' => $specification_id,
                ];
                $this->SpecifactionModel->insertUserSpecification($user_specificationsData);
            }
            $this->db->trans_complete();

            if ($this->db->trans_status() === false) {
                $this->sendResponse(500, 'تعديل المخصصات فشل الرجاء مراجعه البيانات', false);
            } else {
                $this->sendResponse(200, 'تم تعديل المخصصات بنجاح', false);
            }
        }

        if (empty($data['user_financial_discounts_id'])) {

            $this->FinancialDiscountsModel->deletAllFinancialDiscountsForUser($data['hash']);
        }

        if (!empty($data['user_financial_discounts_id'])) {
            $this->db->trans_start();
            //
            $currentUserFinancialDiscounts = $this->FinancialDiscountsModel->getCurrentUserFinancialDiscounts($data['hash']);

            $currentUserFinancialDiscounts = is_array($currentUserFinancialDiscounts) ? $currentUserFinancialDiscounts : [];

            $financialDiscounts_array = is_array($data['user_financial_discounts_id']) ? $data['user_financial_discounts_id'] : [];
            $toDelete = array_diff($currentUserFinancialDiscounts, $financialDiscounts_array);
            foreach ($toDelete as $financialDiscount_id) {
                $this->FinancialDiscountsModel->deleteUserFinancialDiscount($data['hash'], $financialDiscount_id);
            }
            $toInsert = array_diff($financialDiscounts_array, $currentUserFinancialDiscounts);
            foreach ($toInsert as $specification_id) {
                $user_specificationsData = [
                    'emp_info_id' => $data['hash'],
                    'financial_discounts_id' => $specification_id,
                ];
                $this->FinancialDiscountsModel->insertUserFinancialDiscount($user_specificationsData);
            }
            $this->db->trans_complete();
            if ($this->db->trans_status() === false) {
                $this->sendResponse(500, 'تعديل المقطوعات فشل الرجاء مراجعه البيانات', false);
            } else {
                $this->sendResponse(200, 'تم تعديل المقطوعات بنجاح', false);
            }
        }

        $res1 = $this->EmployeesModel->updateEmp($data);
        $res2 = $this->EmployeesModel->updateEmpInfo($data);
        $res3 = $this->EmployeesModel->updateUserVacations($data['vacationData'], $data['hash']);
        $this->GroupScheduleModel->updateEmployeeActualSchedule($data['hash']);

        $userPermissions = str_replace("'", '"', $data['user_permissions']);
        $userPermissions = json_decode($data['user_permissions'], true);

        $this->RolesPermissionsModel->updateUserRole($data['hash'], $userPermissions);
        $this->EmployeesModel->createLocations($data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل بيانات الموظف بنجاح',
                'token' => $token,
                'data' => array(
                    'user' => $res1,
                    'emp' => $res2,
                    'vacation' => $res3,
                ),

                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    // update permissions
    public function updatePermission()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $userHash = $this->jwtmiddleware->decode($token)['hash'];

        $data = $this->input->post();
        $res = $this->EmployeesModel->updatePermission($data, $userHash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل صلاحيات الموظف بنجاح',
                'token' => $token,
                'data' => $res,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    // delete employee
    public function deleteEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $this->EmployeesModel->deleteEmployee($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف الموظف بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function deleteEmployeeFromDB()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $userHash = $this->input->post('hash');
        $res = $this->EmployeesModel->deleteEmployeeFromDB($userHash);

        if ($res['status'] == true) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم حذف الموظف بنجاح',
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else if ($res['status'] == false) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $res['message'],
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function employeesCanAcceptVacations()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        $user_type = $user['user_type'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $employees = $this->EmployeesModel->employeesCanAcceptVacations($hash, $user_type);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الموظفين الذين يمكنهم قبول الاجازات',
                'token' => $token,
                'data' => $employees,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getCurrentVacationsOfUser()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $vacations = $this->EmployeesModel->getCurrentVacationsOfUser($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الاجازات الحالية للموظف',
                'token' => $token,
                'data' => $vacations,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getHistoryOfVacation()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $page = $this->input->get("page") ?? 1;
        $perPage = $this->input->get("perPage") ?? 10;
        $vacations = $this->EmployeesModel->getHistoryOfVacation($hash, $page, $perPage);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الاجازات السابقة للموظف',
                'token' => $token,
                'data' => $vacations,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function receivedVAcationsOfUser()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $vacations = $this->EmployeesModel->receivedVAcationsOfUser($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الاجازات المستلمة للموظف',
                'token' => $token,
                'data' => $vacations,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeeDetails()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        //
        // hash from user to get his data
        $hash = $user['hash'];
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        // Model is here
        $details = $this->EmployeesModel->getEmployeeDetails($hash);
        $scheduleData = $this->GroupScheduleModel->getScheudleToday($hash, date('Y-m-d'));
        $details[0]['scheduleData'] = $scheduleData;
        // Add the scheduleData to the details array
        // $details['scheduleData'] = $scheduleData;
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تفاصيل الموظف',
                'token' => $token,
                'data' => $details,
                'isAuth' => true,
                'hash' => $hash,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeeShecduleDetail()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $userHash = $this->input->get('hash') ?? '';
        $enteredDate = $this->input->get('date') ?? '';

        if (!isset($userHash) || trim($userHash) === '' || !isset($enteredDate) || trim($enteredDate) === '') {
            return $this->sendResponse(400, 'الرجاء ادخال البيانات المطلوبة', false);
        }
        $schedule = $this->EmployeesModel->getEmployeeShecduleDetail($userHash, $enteredDate);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض جميع البيانات',
                'token' => $token,
                'data' => $schedule,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeesPayrollData()
    {

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        // Fetch request data
        $payrollRequestData = (object) [
            'start_date' => $this->input->get('startDate'),
            'end_date' => $this->input->get('endDate'),
            'year_months' => $this->input->get('year_months'),
        ];

        // Get payroll data
        $result = $this->EmployeesModel->getEmployeesPayrollData($payrollRequestData);
        $data = $result['result'];
        $total = $result['total'];
        // Output data as JSON
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض جميع البيانات',
                'token' => $token,
                'data' => $data,
                'total' => $total,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function retrieveEmployeeAttendanceData()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $userHash = $user['hash'];

        $month = $this->input->get('month');
        $year = $this->input->get('year');

        if (empty($month) || empty($year) || empty($userHash)) {
            return $this->sendResponse(400, 'الرجاء ادخال البيانات المطلوبة', false);
        }

        $employeeAttendanceData = (object) [
            'month' => $month,
            'year' => $year,
            'userHash' => $userHash,
        ];

        $data = $this->EmployeesModel->retrieveEmployeeAttendanceData($employeeAttendanceData);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض جميع البيانات',
                'token' => $token,
                'data' => $data,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeeSchedule()
    {
        $this->UserModel->checkPermission();
        $user_hash = $this->input->get('user_hash');
        $date = $this->input->get('date');
        $result = $this->ScheduleModel->getEmployeeSchedule($user_hash, $date);
        echo json_encode($result);
        exit();
    }

    public function getEmployeeFlexible()
    {
        $this->UserModel->checkPermission();
        $user_hash = $this->input->get('user_hash');
        $date = $this->input->get('date');
        $result = $this->ScheduleModel->getEmployeeFlexible($user_hash, $date);
        echo json_encode($result);
        exit();
    }

    public function insertMissingCalacultions()
    {
        $this->UserModel->checkPermission();
        $this->EmployeesModel->insertMissingCalacultions();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم اضافة البيانات بنجاح',
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function sendLocationFlageEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $userHash = $user['hash'];
        $flage = $this->input->post('flage');

        $this->EmployeesModel->sendLocationFlageEmployee($userHash, $flage);

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تغير البيانات بنجاح',
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    private function sendResponse($status, $message, $action = true, $total = null)
    {
        $response = array('status' => $status, 'message' => $message, 'action' => $action, 'total' => $total);
        $this->output->set_output(json_encode($response));
    }
}
