<?php
class GroupSchedulyController extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('UserModel');
        $this->load->library('Middleware');
        $this->load->model('GroupScheduleModel');
    }

    public function index()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token)['user'];

        $result = $this->GroupScheduleModel->get_all_groups();
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض كل الجدوال',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                    // 'total' => count($result)
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'لا يوجد بيانات',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function getGroupSchedulyTable()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token)['hash'];

        $result = $this->GroupScheduleModel->getGroupSchedulyTable();
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'عرض كل الجدوال',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                    'total' => $this->GroupScheduleModel->getGroupScheduleCount(),
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'لا يوجد بيانات',
                    'data' => [],
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function create()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $data = array(
            'name' => $this->input->post('name'),
            'schedule_data' => $this->input->post('schedule_data'),
        );
        $result = $this->GroupScheduleModel->create_group($data);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم انشاء الجدول بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'حدث خطأ أثناء انشاء الجدول',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function getById()
    {

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $id = $this->input->get('id');

        // Edit group
        $result = $this->GroupScheduleModel->get_group_by_id($id);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تحميل الجدول بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'لا يوجد بيانات',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function addScheduleToGroup()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $id = $this->input->post('id');
        $data = array(
            'schedule_data' => $this->input->post('schedule_data'),
            'allowance_time' => $this->input->post('allowance_time'),
        );

        // Edit group
        $result = $this->GroupScheduleModel->addScheduleToGroup($id, $data);
        $this->GroupScheduleModel->updateActualSchedules();
        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تحديث الجدول بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'حدث خطأ أثناء تحديث الجدول',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function getGroupDataById()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $id = $this->input->get('id');

        // Edit group
        $result = $this->GroupScheduleModel->getGroupDataById($id);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تحميل الجدول بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'لا يوجد بيانات',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function getworkSchedulesOptions()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $result = $this->GroupScheduleModel->getworkSchedulesOptions();

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تحميل الجدول بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'لا يوجد بيانات',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
    public function update()
    {

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $id = $this->input->post('id');
        // Update group
        $data = array(
            'name' => $this->input->post('name'),
            'schedule_data' => $this->input->post('schedule_data'),
        );
        $result = $this->GroupScheduleModel->update_group($id, $data);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم تحديث الجدول بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'حدث خطأ أثناء تحديث الجدول',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function delete()
    {

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $id = $this->input->post('id');

        // Delete group
        $result = $this->GroupScheduleModel->delete_group($id);

        if ($result) {
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم حذف الجدول بنجاح',
                    'token' => $token,
                    'data' => $result,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            echo json_encode(
                array(
                    'status' => 404,
                    'message' => 'حدث خطأ أثناء حذف الجدول',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }
}
