<?php defined('BASEPATH') or exit('No direct script access allowed');
class MovementsController extends CI_Controller
{
    private $server2BaseUrl = "https://location.native-code-iq.com";

    public function __construct()
    {
        parent::__construct();
        $this->load->model('UserModel');
        $this->load->model('MovementsModel');
        $this->load->library('Middleware');

    }

    public function index()
    {
    }
    public function insertMovements()
    {
        $data = $this->input->post();

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        if (!empty($data)) {
            $movementData = array(
                'user_hash' => $data['userHash'],
                'lat' => $data['lat'],
                'lng' => $data['lng'],
            );
            $inserted_id = $this->MovementsModel->insertMovement($movementData);
            if ($inserted_id) {
                $response = array(
                    'status' => 200,
                    'message' => 'Movements inserted successfully',
                    'data' => $inserted_id,
                );

                echo json_encode($response);
                // $this->output->set_output(json_encode($response));
                exit();
            } else {
                $response = array(
                    'status' => 500,
                    'message' => 'some thing went wrong',
                    'data' => $inserted_id,
                );
                echo json_encode($response);
                exit();
                // $this->output->set_output(json_encode($response));
            }
            // $this->response->sendResponse(200, 'Movement inserted successfully');
        } else {
            $response = array(
                'status' => 500,
                'message' => 'some thing went wrong',
                'data' => 'hellow test',
            );
            echo json_encode($response);
            // $this->response->sendResponse(400, 'Bad request');
        }
    }

    // for test now
    public function insertDummyNotification()
    {
        $user = $this->input->post('hash');
        $data = array(
            "condition" => "'test' in topics",
            "priority" => "high",
            "user_admin" => $user,
        );

        $this->jwtmiddleware->sendToMapNot();
    }

    public function sendNotificationToUserNotConnected()
    {
        $this->UserModel->checkPermission();

        $user_hash = $this->input->post('user_hash');
        $message = $this->input->post('message');
        $receiverJson = $this->input->post('receiver');
        $receiverArray = json_decode($receiverJson, true);

        foreach ($receiverArray as $receiver) {
            $data = array(
                "type" => "not_connected",
                "user" => $user_hash,
                "message" => $message,
                "receiver" => $receiver,
            );
            $this->jwtmiddleware->insertNotToFirebase($data);

            $this->jwtmiddleware->sendNot(
                "غير متصل",
                json_encode(array('type' => 10)),
                $this->UserModel->getMobileToken($receiver),
                $message
            );
        }
    }



    public function getMovements()
    {
        $this->UserModel->checkPermission();
        $data = ['companyHash' => $this->input->get('companyHash')];
        $movements = $this->fetchDataFromServer("last-locations", $data);
        $this->sendResponse($movements, 'Movements retrieved successfully');
    }





    public function getMovementsReport()
    {
        $this->UserModel->checkPermission();
        $userHash = $this->input->get('userHash');
        $eventName = $this->input->get('eventName') ?? NULL;
        $userHashs = json_decode($userHash);
        $eventNames = json_decode($eventName) == [] ? NULL : json_decode($eventName);
        
        $data = [
            'startTime' => $this->input->get('start_date') ?? '',
            'endTime' => $this->input->get('end_date') ?? '',
            'userHash' => $userHashs,
            'companyHash' => $this->input->get('companyHash'),
            'eventName' => $eventNames,
            'isMoving' => $this->input->get('isMoving') == "true" ? true : false,
            'accuracy' => $this->input->get('accuracy') ?? NULL,
            'limit' => 100000,
        ];


        $movements = $this->fetchDataFromServer("user-locations", $data);

        // if (!empty($movements)) {
        //     $movements = $this->filterAndSortMovements($movements);
        // }


        $this->sendResponse($movements, 'Movements retrieved successfully');

    }


    private function filterAndSortMovements($movements)
    {
        usort($movements, fn($a, $b) => $a['user_hash'] === $b['user_hash']
            ? strtotime($a['timestamp']) - strtotime($b['timestamp'])
            : $a['user_hash'] <=> $b['user_hash']);

        $filteredMovements = [];
        $lastTimestamps = [];

        foreach ($movements as $movement) {
            $userHash = $movement['user_hash'];
            $currentTimestamp = strtotime($movement['timestamp']);
            $day = date('Y-m-d', $currentTimestamp);

            if (!isset($lastTimestamps[$userHash][$day])) {
                $lastTimestamps[$userHash][$day] = $currentTimestamp - 300;
            }

            if ($currentTimestamp - $lastTimestamps[$userHash][$day] >= 300) {
                $filteredMovements[] = $movement;
                $lastTimestamps[$userHash][$day] = $currentTimestamp;
            }
        }
        return $filteredMovements;
    }

    public function getMovement($id)
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $movement = $this->MovementsModel->getMovement($id);
        $response = array(
            'status' => 200,
            'message' => 'Movement retrieved successfully',
            'data' => $movement,
        );
        $this->output->set_output(json_encode($response));
    }

    public function updateMovement($id)
    {
        $data = $this->input->post();

        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        if (!empty($data)) {
            $movementData = array(
                'user_hash' => $data['hash'],
                'lat' => $data['lat'],
                'lng' => $data['lng'],
            );
            $this->MovementsModel->updateMovement($movementData, $id);
            $response = array(
                'status' => 200,
                'message' => 'Movement updated successfully',
                'data' => [],
            );
            $this->output->set_output(json_encode($response));
        } else {
            $response = array(
                'status' => 400,
                'message' => 'Bad request',
                'data' => [],
            );
            $this->output->set_output(json_encode($response));
        }
    }

    public function deleteMovement($id)
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $this->MovementsModel->deleteMovement($id);
        $response = array(
            'status' => 200,
            'message' => 'Movement deleted successfully',
            'data' => [],
        );
        $this->output->set_output(json_encode($response));
    }

    public function getEmployeeMovementsConfig()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $config = $this->MovementsModel->getEmployeeMovementsConfig();
        $this->sendResponse($config, "Employee movements config retrieved successfully");
    }

    public function updateEmployeeMovementsConfig()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();

        $meters = $this->input->post('meters') ?? NULL;

        if (empty($meters)) {
            $this->output->set_output(json_encode(array(
                'status' => 400,
                'message' => 'Bad request',
            )));

            return;
        }

        $data = array('meters' => $meters);
        $this->MovementsModel->updateEmployeeMovementsConfig($data);

        $res = array(
            'status' => 200,
            'message' => 'Employee movements config updated successfully',
        );
        $this->sendResponse($res, "Employee movements config updated successfully");
    }

    private function fetchDataFromServer($endpoint, $data)
    {
        $url = "$this->server2BaseUrl/$endpoint";
        $response = $this->sendHttpRequest($url, $data);
        $decodedResponse = json_decode($response, true);
        return $decodedResponse['locations'] ?? [];
    }
    private function sendHttpRequest($url, $data)
    {
        $ch = curl_init($url);

        curl_setopt_array($ch, [
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => json_encode($data),
            CURLOPT_HTTPHEADER => ["Content-Type: application/json"],
        ]);

        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        return ($httpCode === 200) ? $response : json_encode(["error" => "Failed to fetch data from Server 2"]);
    }
    private function sendResponse($data, $successMessage)
    {
        $response = !empty($data) ? [
            'status' => 200,
            'message' => $successMessage,
            'data' => $data,
        ] : [
            'status' => 404,
            'message' => 'No movements found',
            'data' => [],
        ];

        $this->output->set_output(json_encode($response));
    }

}
