<?php
defined('BASEPATH') or exit('No direct script access allowed');
class Permission extends CI_Controller
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model('PermissionModel');
        $this->load->model('UserModel');
        $this->load->library('Middleware');
    }

    public function getAll()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $group_hash = $this->input->get('group_hash');
        $data = $this->PermissionModel->getPermissions($group_hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الصلاحيات',
                'data' => $data,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function create()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $data = array(
            'group_hash' => $this->input->post('group_hash'),
            'table_name' => $this->input->post('table_name'),
            'select_op' => $this->input->post('select_op'),
            'insert_op' => $this->input->post('insert_op'),
            'update_op' => $this->input->post('update_op'),
            'delete_op' => $this->input->post('delete_op'),
            'hash' => $this->jwtmiddleware->createHash(),
        );
        $data = $this->PermissionModel->createPermission($data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'انشاء صلاحية جديدة',
                'data' => $data,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function update()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $userHash = $this->jwtmiddleware->decode($token)['hash'];

        $data = array(
            'group_hash' => $this->input->post('group_hash'),
            'table_name' => $this->input->post('table_name'),
            'select_op' => $this->input->post('select_op'),
            'insert_op' => $this->input->post('insert_op'),
            'update_op' => $this->input->post('update_op'),
            'delete_op' => $this->input->post('delete_op'),
            'hash' => $this->input->post('hash'),
        );
        $data = $this->PermissionModel->updatePermission($data, $userHash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الاوامر',
                'data' => $data,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function delete()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = $this->PermissionModel->deletePermission($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'حذف صلاحية',
                'data' => $data,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}
