<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Vacation extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('VacationModel');
        $this->load->model('UserModel');
        $this->load->model('ScheduleModel');
        $this->load->library('Middleware');
    }

    public function index()
    {
    }

    // get all Vacation
    public function getVacations()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $start_date = $this->input->get('start_date');
        $end_date = $this->input->get('end_date');
        $vacations = $this->VacationModel->getVacations($start_date, $end_date);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الموظفين',
                'token' => $token,
                'data' => $vacations,
                'isAuth' => true,
                'total' => $this->VacationModel->count_records($start_date, $end_date),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getVacation()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $vacation = $this->VacationModel->getVacation($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض اجازة معينة',
                'token' => $token,
                'data' => $vacation,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getVacationType()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $vacations_type = $this->VacationModel->getVacationType();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض انواع الاجازات',
                'token' => $token,
                'data' => $vacations_type,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getVacationStatus()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $vacations_type = $this->VacationModel->getVacationStatus();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض حالات الاجازات',
                'token' => $token,
                'data' => $vacations_type,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function isVacationTypeByTime($hash)
    {
        if ($hash == '333' || $hash == '444') {
            return true;
        }
        return false;
    }

    public function createVacation()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        $user_hash = $user['hash'];
        // check if vacation created

        $data = $this->prepareVacationData($user_hash);

        if ($this->VacationModel->checkIfVacationExist($data['user_hash'], $data['date_start'], $data['date_end'])) {
            return $this->jsonResponse(400, 'هذه الاجازة موجودة بالفعل ومازالت قيد الموافقة', $token);
        }
        $createVacationResponse = $this->VacationModel->createVacation($data);

        if (is_string($createVacationResponse)) {
            return $this->jsonResponse(400, $createVacationResponse, $token);
        }

        if (!$createVacationResponse) {
            return $this->jsonResponse(400, 'حدث خطأ اثناء اضافة الاجازة', $token);
        }

        if (!$this->createVacationAgreement($data)) {
            return $this->jsonResponse(400, 'حدث خطأ اثناء اضافة الاجازة', $token);
        }

        $this->sendCreateNotifications($data);

        return $this->jsonResponse(200, 'تم اضافة الاجازة بنجاح', $token);

    }

    private function prepareVacationData($user_hash)
    {
        return array(
            'user_hash' => $this->input->post('user_hash'),
            'vication_type_hash' => $this->input->post('vication_type_hash'),
            'date_start' => $this->input->post('date_start'),
            'date_end' => $this->input->post('date_end'),
            'note' => $this->input->post('note'),
            'admin' => $this->input->post('admin') ?? $user_hash,
            'hash' => $this->jwtmiddleware->createHash(),
        );
    }

    private function createVacationAgreement($data)
    {
        return $this->VacationModel->createVacationAgreement(
            array(
                'vacations_hash' => $data['hash'],
                'status' => 444,
                'note' => $data['note'],
                'receiver' => $data['admin'] ?? $data['user_hash'],
                'sender' => $data['user_hash'],
                'hash' => $this->jwtmiddleware->createHash(),
            )
        );
    }

    private function sendCreateNotifications($data)
    {

        $this->jwtmiddleware->insertNotToFirebase([
            "type" => "vacation",
            "status" => "request",
            "user" => $data['user_hash'],
            "receiver" => $data['admin'],
        ]);

        $notificationSuccess = true;

        $response = $this->jwtmiddleware->sendNot(
            "تم ارسال اليك طلب اجازة",
            json_encode(array('type' => 2)),
            $this->UserModel->getMobileToken($data['admin']),
            'تم ارسال طلب اجازة اليك'
        );

        if (isset($response['error'])) {
            $notificationSuccess = false;
        }

        return $notificationSuccess;

    }

    private function jsonResponse($status, $message, $token)
    {
        echo json_encode(
            array(
                'status' => $status,
                'message' => $message,
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function updateVacation()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $data = array(
            'user_hash' => $this->input->post('user_hash'),
            'vication_type_hash' => $this->input->post('vication_type_hash'),
            'date_start' => $this->input->post('date_start'),
            'date_end' => $this->input->post('date_end'),
            'note' => $this->input->post('note'),
        );
        $vacationResponse = $this->VacationModel->updateVacation($hash, $data);

        if (is_string($vacationResponse)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $vacationResponse,
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل الاجازة بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function deleteVacation()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $vacationResponse = $this->VacationModel->deleteVacation($hash);
        if (is_string($vacationResponse)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $vacationResponse,
                    'token' => $token,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف الاجازة بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function changeVacationStatus()
    {
        $this->validateAndAuthorizeRequest();
        $vacationData = $this->getVacationDataFromRequest();

        // this get vacation_type_hash column from vacation table
        $vacationType = $this->VacationModel->getVacationTypeData($vacationData['hash']);
        
        if ($vacationData['status'] == '111') {

            if ($this->isVacationTypeByTime($vacationType->hash)) {
                $response = $this->handleTimeBasedVacation($vacationType);
                if ($response['status'] === 'error') {
                    $this->sendErrorResponse($response['message']);
                    return; // Prevent further execution
                }
            } else {
                $result = $this->handleDaysVacation($vacationType);
                if ($result['status'] === 'error') {
                    $this->sendErrorResponse($result['message']);
                    return;
                }
            }
        }

        $response = $this->updateVacationStatus($vacationData);
        $message = '';
        if ($response['status'] == 'error') {
            $message = $response['message'];
            $this->notifyRelevantParties($vacationData);
            $this->sendNotifications($vacationData['receiver'], $vacationData['status']);
            $this->sendSuccessResponse($message);
        } else {
            $this->notifyRelevantParties($vacationData);
            $message = $this->sendNotifications($vacationData['receiver'], $vacationData['status'])['message'];
            $this->sendSuccessResponse($message);
        }
    }

    private function validateAndAuthorizeRequest()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $this->jwtmiddleware->decode($token);
    }

    private function getVacationDataFromRequest()
    {
        return [
            'hash' => $this->input->post('hash'),
            'status' => $this->input->post('status'),
            'sender' => $this->input->post('sender'),
            'receiver' => $this->input->post('receiver'),
            'note' => $this->input->post('note'),
            'agreeHash' => $this->input->post('agreeHash'),
        ];
    }

    private function notifyRelevantParties($data)
    {
        $senderName = $this->UserModel->getUserName($data['sender']);
        $receiverName = $this->UserModel->getUserName($data['receiver']);

        $this->sendFirebaseNotification($data);
        $this->sendPushNotification($data, $senderName, $receiverName);
    }

    private function sendNotifications($receiver, $status)
    {
        // Assume sending notification may fail but ensure no exception is thrown
        $notificationSuccess = true;
        $message = '';

        switch ($status) {
            case '111':
                $message = 'تم الموافقة على طلب الاجازة';
                $response = $this->jwtmiddleware->sendNot(
                    "تم الموافقة على طلب الاجازة",
                    json_encode(['type' => 1]),
                    $this->UserModel->getMobileToken($receiver),
                    'تم الموافقة على طلب الاجازة'
                );
                break;
            case '222':
                $message = 'تم رفض طلب الاجازة';
                $response = $this->jwtmiddleware->sendNot(
                    "تم رفض طلب الاجازة",
                    json_encode(['type' => 1]),
                    $this->UserModel->getMobileToken($receiver),
                    'تم رفض طلب الاجازة'
                );
                break;
            case '333':
                $message = 'تم تمرير طلب اجازة اليك';
                $response = $this->jwtmiddleware->sendNot(
                    "تم تمرير طلب اجازة اليك",
                    json_encode(['type' => 2]),
                    $this->UserModel->getMobileToken($receiver),
                    'تم تمرير طلب اجازة اليك'
                );
                break;
            default:
                $response = true; // Default success for undefined cases
                $message = 'Notification status undefined';
                break;
        }

        // If the response contains an error, log it and return false
        if (isset($response['error'])) {
            $notificationSuccess = false;
        }

        // Return true if successful, false if something failed
        return [
            'success' => $notificationSuccess,
            'message' => $message,
        ];
    }

    private function sendFirebaseNotification($data)
    {
        $this->jwtmiddleware->insertNotToFirebase([
            "type" => "vacation",
            "status" => $data['status'],
            "user" => $data['sender'],
            "receiver" => $data['receiver'],
        ]);
    }

    private function sendPushNotification($data, $senderName, $receiverName)
    {
        $notificationData = $this->getNotificationData($data['status'], $senderName, $receiverName);
        $this->jwtmiddleware->sendNot(
            $notificationData['title'],
            json_encode(['type' => 2]),
            $this->UserModel->getMobileToken($data['receiver']),
            $notificationData['body']
        );
    }

    private function getNotificationData($status, $senderName, $receiverName)
    {
        $notifications = [
            '111' => [
                'title' => "تمت الموافقة علي اجازة",
                'body' => "تمت الموافقة علي اجازة $receiverName من قبل $senderName",
            ],
            '222' => [
                'title' => "تم رفض اجازة",
                'body' => "تم رفض اجازة $receiverName من قبل $senderName",
            ],
            '333' => [
                'title' => "تم تمرير اجازة",
                'body' => "تم تمرير اجازة الي $receiverName من قبل $senderName",
            ],
        ];

        return $notifications[$status] ?? ['title' => '', 'body' => ''];

        return $notifications[$status] ?? ["", ""];
    }

    private function updateVacationStatus($data)
    {
        return $this->VacationModel->changeVacationStatus($data['hash'], [
            'status' => $data['status'],
            'sender' => $data['sender'],
            'receiver' => $data['receiver'],
            'note' => $data['note'],
            'agreeHash' => $data['agreeHash'],
        ]);
    }

    private function handleTimeBasedVacation($vacationType)
    {
        return $this->ScheduleModel->updateScheduleForTimeVacation(
            $vacationType->user_hash,
            $vacationType->date_start,
            $vacationType->date_end
        );
    }

    private function handleDaysVacation($vacationType)
    {
        return $this->ScheduleModel->handleNormalVacation(
            $vacationType->user_hash,
            $vacationType->date_start,
            $vacationType->date_end
        );
    }

    private function sendSuccessResponse($message)
    {
        echo json_encode([
            'status' => 200,
            'message' => $message,
            'token' => $this->tokenmiddleware->main(),
            'isAuth' => true,
            'action' => true,
        ], JSON_UNESCAPED_UNICODE);
        exit();
    }
    private function sendErrorResponse($message)
    {
        echo json_encode([
            'status' => 400,
            'message' => $message,
            'token' => $this->tokenmiddleware->main(),
            'isAuth' => true,
            'action' => true,
        ], JSON_UNESCAPED_UNICODE);
        exit();
    }

    public function yearlyDetails()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // hash from user to get his data
        // $hash = $user['hash'];
        $hash = $this->input->get('hash');
        $details = $this->VacationModel->yearlyDetails($hash);
        echo json_encode(
            array(
                'status' => 200,
                'data' => $details,
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function monthlyDetails()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // hash from user to get his data
        $hash = $this->input->get('hash');
        $details = $this->VacationModel->monthlyDetails($hash);
        echo json_encode(
            array(
                'status' => 200,
                'data' => $details,
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function Details()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // hash from user to get his data
        $hash = $this->input->get('hash');
        $details = $this->VacationModel->Details($hash);
        echo json_encode(
            array(
                'status' => 200,
                'data' => $details,
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
    public function getUserVacationReport()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');
        $monthVacations = $this->VacationModel->getUserVacationReport($hash, "month");
        $yearVacations = $this->VacationModel->getUserVacationReport($hash, "year");
        $vacationLimit = $this->VacationModel->getUserVacaitionLimit($hash);
        echo json_encode(
            array(
                'status' => 200,
                'data' => array(
                    'month' => $monthVacations,
                    'year' => $yearVacations,
                    'limit' => $vacationLimit[0],
                ),
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }
}
