<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Zk_fingerprintsLogsController extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('Zk_fingerprintsLogsModel', 'Log_model');
        $this->load->model('UserModel', 'user');
        $this->load->helper('url');
        $this->load->library('form_validation');
        $this->output->set_content_type('application/json');
    }

    /**
     * Helper method to send JSON responses.
     */
    protected function jsonResponse($status, $message, $data = [], $httpCode = 200, $total = 100)
    {
        $this->output
            ->set_status_header($httpCode)
            ->set_output(json_encode([
                'status' => $status,
                'message' => $message,
                'data' => $data,
                'total' => $total
            ]));
    }
    /**
     * List all records.
     */
    public function index()
    {
        try {
            $records = $this->Log_model->getAll();
            $this->jsonResponse('success', 'Records retrieved successfully', $records);
        } catch (Exception $e) {
            $this->jsonResponse('error', $e->getMessage(), [], 500);
        }
    }


    public function getFormattedDataForPage()
    {
        try {
            $res = $this->Log_model->get_fingerprint_logs();
            $data = $res['data'];
            $total = $res['total'];

            $this->jsonResponse(200, 'Records retrieved successfully', $data, 200, $total);
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }
    }


    /**
     * Get a single record by ID.
     */
    public function show($id)
    {
        try {
            $record = $this->Log_model->getById($id);
            if ($record) {
                $this->jsonResponse('success', 'Record retrieved successfully', $record);
            } else {
                $this->jsonResponse('error', 'Record not found', [], 404);
            }
        } catch (Exception $e) {
            $this->jsonResponse('error', $e->getMessage(), [], 500);
        }
    }

    // API endpoint to handle JSON data
    public function create()
    {
        $json_data = $this->input->post('data');

        // Decode the JSON data
        $data = json_decode($json_data, true);

        // Validate the input
        if (empty($data) || !is_array($data)) {
            $this->output
                ->set_content_type('application/json')
                ->set_status_header(400)
                ->set_output(json_encode(['error' => 'Invalid input: Expected an array of JSON objects.']));
            return;
        }

        // Insert data into the database
        $inserted_ids = [];
        foreach ($data as $log) {
            if (isset($log['indRegId'], $log['dateTime'], $log['user_hash'])) {
                $insert_data = [
                    'indRegId' => $log['indRegId'],
                    'dateTime' => $log['dateTime'],
                    'user_hash' => $log['user_hash'],
                    'device_info' => $log['device_hash'],
                ];

                $inserted_id = $this->Log_model->insert_log($insert_data);
                if ($inserted_id) {
                    $inserted_ids[] = $inserted_id;
                }
            }
        }

        // Return success response
        $this->jsonResponse('success', 'Record created successfully', ["insertedIds" => $inserted_ids], 201);
    }

    public function delete($id)
    {
        try {
            $this->Log_model->delete($id);
            $this->jsonResponse(200, 'Record deleted successfully', [], 200);
        } catch (Exception $e) {
            $this->jsonResponse(500, $e->getMessage(), [], 500);
        }

    }
}