<?php
class AccountingTreeModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    // get all Groups
    public function count_records()
    {
        $headers = apache_request_headers();
        $search = $this->input->get('q') ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('accounting_tree');
        if ($search) {
            $this->db->like('name', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getAccountings()
    {
        // get page and limit and search from header
        $headers = apache_request_headers();
        $page = $headers['page'] ?? $this->input->get('page') ?? 1;
        $perPage = $headers['perPage'] ?? $this->input->get('perPage') ?? 10;
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        $this->db->select(
            '*'
        );
        $this->db->from('accounting_tree');

        // search
        if ($search) {
            $this->db->like('name', $search);
        }

        // order by
        $this->db->order_by('id', 'DESC');
        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();

        return $query->result_array();
    }

    public function getNamesForSelect()
    {
        $this->db->select('name AS label, id AS value');
        $this->db->from('accounting_tree');
        $query = $this->db->get();
        return $query->result_array();
    }

    function getAccount($hash)
    {
        $this->db->select('*');
        $this->db->from('accounting_tree');
        $this->db->where('accounting_tree.hash =', $hash);
        $query = $this->db->get();

        $result = $query->result_array();
        return $result;
    }

    function createAccount($data)
    {     
        $this->db->insert('accounting_tree', $data);
        return $this->db->insert_id();
    }

    function updateAccount($data)
    {
        $this->db->where('hash', $data['hash']);
        $this->db->update('accounting_tree', $data);
        return $this->db->affected_rows();
    }

    function deleteAccount($hash)
    {
        $this->db->where('hash', $hash);
        $this->db->delete('accounting_tree');
        return $this->db->affected_rows();
    }
}
