<?php
class AttendModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    // get all Groups
    public function count_records()
    {
        $headers = apache_request_headers();
        $search = $headers['q'] ?? '';
        $department = $this->input->post('department');
        $this->db->select('count(*) as count');
        $this->db->from('system_users');
        // left join with emp_fingerprint
        $this->db->join('emp_information', 'emp_information.user_hash = system_users.hash', 'inner');
        // where is_work = 1
        $this->db->where('is_work', 1);
        if ($search) {
            $this->db->like('system_users.hash', $search);
        }
        if (isset($department) && $department != "") {
            $this->db->where('emp_information.partment', $department);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getAttends($date_start, $date_end)
    {
        // get page and limit and search from header
        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $search = $headers['q'] ?? '';
        $offset = ($page - 1) * $perPage;
        $department = $this->input->post('department');

        $this->db->select('
            system_users.name,
            system_users.hash as user_hash,
            image_profile as image,
            (select name from system_group_name where system_group_name.hash=user_type) as job_name,
            partments.name as department
         ');
        $this->db->from('system_users');
        // inner join emp_information
        $this->db->join('emp_information', 'emp_information.user_hash = system_users.hash', 'inner');
        $this->db->join('partments', 'emp_information.partment = partments.hash', 'left');
        // where is_work = 1
        $this->db->where('is_work', 1);
        // search
        if (isset($search) && $search != "") {
            $this->db->where('system_users.hash', $search);
        }
        if (isset($department) && $department != "") {
            $this->db->like('emp_information.partment', $department);
        }
        // order by
        $this->db->order_by("partments.name, system_users.id", "asc");
        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getFingerPrints($hash, $start_time, $end_time)
    {
        $this->db->select('day_print');
        $this->db->from('emp_fingerprint');
        $this->db->where('user_hash', $hash);
        $this->db->where('date(fingerprint_time) >=', $start_time);
        $this->db->where('date(fingerprint_time) <=', $end_time);
        $query = $this->db->get();
        return $query->result_array();
    }


    function getAttend($day, $user_hash)
    {
        $query = $this->db->query(
            "
            SELECT 
                DATE(fingerprint_time) AS date,
                TIME_FORMAT(MIN(fingerprint_time), '%h:%i %p') AS inTime,
                case 
                    when count(*) = 1 then 'لم يخرج بعد'
                    else TIME_FORMAT(MAX(fingerprint_time), '%h:%i %p')
                end as outTime,
                TIMESTAMPDIFF(minute,
                    MIN(fingerprint_time),
                    MAX(fingerprint_time))/60 AS time_difference,
                hash,
                CASE 
                    WHEN COUNT(*) = 0 THEN 'غياب'
                    WHEN COUNT(*) = 1 AND MIN(emp_fingerprint.fingerprint_time) IS NULL THEN 'غياب'
                    WHEN COUNT(*) = 1 AND in_out = 'in' AND allow_time < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
                    WHEN COUNT(*) = 1 AND in_out = 'out' THEN 'بصم خروج بدون دخول'
                    WHEN COUNT(*) = 1 AND allow_time <= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
                    WHEN COUNT(*) = 1 AND allow_time >= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'مازال يداوم'
                    WHEN COUNT(*) = 2 AND allow_time < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
                    ELSE 'مازال في الدوام'
                end as status,
                ST_X(location) AS lat,
                ST_Y(location) AS lng,
                (select ST_X(location) from emp_fingerprint where in_out='out' and date(fingerprint_time) = '$day' and emp_fingerprint.user_hash = '$user_hash' limit 1) as out_lat,
                (select ST_Y(location) from emp_fingerprint where in_out='out' and date(fingerprint_time) = '$day' and emp_fingerprint.user_hash = '$user_hash' limit 1) as out_lng,
                (select hash from emp_fingerprint where in_out='out' and date(fingerprint_time) = '$day' and emp_fingerprint.user_hash = '$user_hash' limit 1) as out_hash
            FROM
                emp_fingerprint
            left join emp_information on emp_information.user_hash= emp_fingerprint.user_hash
            where date(fingerprint_time) = '$day' and emp_fingerprint.user_hash = '$user_hash'
            GROUP BY emp_fingerprint.user_hash , DATE(fingerprint_time);
            "
        );
        // one row
        return $query->result_array()[0];
    }

    public function getDayStatus($start_date, $end_date, $user_hash)
    {
        $query = $this->db->query(
            "
            SELECT
                case 
                WHEN COUNT(*) = 0 THEN 'غياب'
                WHEN COUNT(*) = 1 AND MIN(emp_fingerprint.fingerprint_time) IS NULL THEN 'غياب'
                WHEN COUNT(*) = 1 AND in_out = 'in' AND allow_time < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
                WHEN COUNT(*) = 1 AND in_out = 'out' THEN 'بصم خروج بدون دخول'
                WHEN COUNT(*) = 1 AND allow_time <= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
                WHEN COUNT(*) = 1 AND allow_time >= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'مازال يداوم'
                WHEN COUNT(*) = 2 AND user_create_record = 5555 THEN 'السيستم بصم عنه بصمة الخروج'
                WHEN COUNT(*) = 2 AND allow_time < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل صباحا وخرج من الدوام'
                ELSE 'حضر وانتهي دوامه'
                end as status,
                DATE(fingerprint_time) AS date,
                TIME_FORMAT(MIN(fingerprint_time), '%h:%i %p') AS inTime,
                case 
                    when count(*) = 1 then 'لم يخرج بعد'
                    else TIME_FORMAT(MAX(fingerprint_time), '%h:%i %p')
                end as outTime
            FROM
                emp_fingerprint
            left join emp_information on emp_information.user_hash= emp_fingerprint.user_hash
            where date(fingerprint_time) >= '$start_date' and date(fingerprint_time) <= '$end_date'  and emp_fingerprint.user_hash = '$user_hash'
            GROUP BY emp_fingerprint.user_hash , DATE(fingerprint_time);
            "
        );
        return $query->result_array();
    }

    public function getCurrentMonthAttendForEmp($user_hash)
    {
        $firstDayOfMonth = date('Y-m-01 00:00:00');
        $firstDayOfNextMonth = date('Y-m-d H:i:s', strtotime('+1 month', strtotime($firstDayOfMonth)));
        $this->db->select('(select name from system_users where hash=emp_fingerprint.user_hash) as name,user_hash');
        $this->db->select("GROUP_CONCAT( DATE(fingerprint_time)) AS  attend");
        $this->db->from('emp_fingerprint');
        // check user
        $this->db->where('user_hash', $user_hash);
        // check data span
        $this->db->where('fingerprint_time >=', $firstDayOfMonth);
        $this->db->where('fingerprint_time <', $firstDayOfNextMonth);

        // return result
        $query = $this->db->get();
        $result = $query->result_array();
        // add start and end date as date(YYYY-MM-DD)
        $result[0]['start'] = date('Y-m-01', strtotime($firstDayOfMonth));
        $result[0]['end'] = date('Y-m-d', strtotime($firstDayOfNextMonth . ' -1 day'));
        return $result[0];
    }

    //    public function saveCurrentLocation($hash, $note, $lat, $lng)
    //    {
    //        $query = $this->db->query(
    //            'insert into check_in(user_hash,note,location) values($hash,$note,POINT($lat, $lng))'
    //        );
    //        return $query->result_array();
    //    }

    public function getTodayAttends($page, $perPage)
    {
        $offset = ($page - 1) * $perPage;
        $query = $this->db->query(" 
        SELECT 
        name,
        system_users.hash as hash,
        (select name from system_group_name where system_group_name.hash=user_type) as job_name,
        (select name from partments where partments.hash=partment) as department_name,
        image_profile,
        CASE
            WHEN (SELECT type FROM holiday WHERE date = CURDATE()) IS NULL THEN 'يوم عمل'
            ELSE (SELECT type FROM holiday WHERE date = CURDATE())
        END AS status,
        CASE 
            WHEN date_start THEN 'مجاز'
            WHEN COUNT(*) = 0 THEN 'غياب'
            WHEN COUNT(*) = 1 AND MIN(emp_fingerprint.fingerprint_time) IS NULL THEN 'غياب'
            WHEN COUNT(*) = 1 AND in_out = 'in' AND allow_time < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
            WHEN COUNT(*) = 1 AND in_out = 'out' THEN 'بصم خروج بدون دخول'
            WHEN COUNT(*) = 1 AND allow_time <= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
            WHEN COUNT(*) = 1 AND allow_time >= TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'مازال يداوم'
            WHEN COUNT(*) = 2 AND allow_time < TIME(MIN(emp_fingerprint.fingerprint_time)) THEN 'متاخر عن العمل'
            ELSE ''
        END AS title,
        CASE 
            WHEN COUNT(*)  < 2 THEN ''
            ELSE TIME_FORMAT(MAX(emp_fingerprint.fingerprint_time), '%h:%i %p')
        END AS outTime,
        CASE 
            WHEN COUNT(*) > 0   THEN TIME_FORMAT(min(emp_fingerprint.fingerprint_time), '%h:%i %p') 
            ELSE ''
        END AS inTime,
        case 
            when count(*) = 1 then TIMESTAMPDIFF(minute,MIN(fingerprint_time),curtime())/60
            when count(*) = 2 then TIMESTAMPDIFF(minute,MIN(fingerprint_time),MAX(fingerprint_time))/60
            else ''
        end as time_difference
    FROM
        system_users
            LEFT JOIN
        emp_fingerprint ON user_hash = system_users.hash
            AND DATE(emp_fingerprint.fingerprint_time) = DATE(CURDATE())
            JOIN
        emp_information ON system_users.hash = emp_information.user_hash
            AND is_work = 1
        LEFT JOIN vacations ON vacations.user_hash = system_users.hash
            AND vacations.date_start <= CURDATE()
            AND vacations.date_end >= CURDATE()
    GROUP BY system_users.hash
    order by inTime desc 
    limit $perPage offset $offset;");


        $result = $query->result_array();
        // total emp_informations where is_work = 1
        $this->db->select('count(*) as count');
        $this->db->from('emp_information');
        $this->db->where('is_work', 1);
        $query = $this->db->get();
        $total = $query->result_array();

        return array(
            'data' => $result,
            'total' => $total[0]['count']
        );
    }

    public function createAttend($data)
    {
        $this->db->insert('emp_fingerprint', $data);
        return $this->db->insert_id();
    }
    public function updateAttend($data, $hash)
    {
        $this->db->where('hash', $hash);
        $this->db->update('emp_fingerprint', $data);
        return $this->db->affected_rows();
    }

    public function getFingerprint($hash)
    {
        $this->db->select('fingerprint_time, in_out, system_users.name AS user_name, system_users.hash AS user_hash');
        $this->db->from('emp_fingerprint');
        $this->db->join('system_users', 'emp_fingerprint.user_hash = system_users.hash ');

        $this->db->where('emp_fingerprint.hash', $hash);

        $query = $this->db->get();
        $result = $query->result_array();
        return $result;
    }

}