<?php

class CheckInModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    // get all Groups
    public function count_records()
    {
        $headers = apache_request_headers();
        $search = $headers['q'] ?? '';
        $today = date('Y-m-d');
        $start_date = $this->input->get('start_date') ?? $today;
        $end_date = $this->input->get('end_date') ?? $today;
        $user_hash = $this->input->get('user_hash') ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('check_in');
        $this->db->join('system_users', 'check_in.user_hash = system_users.hash');
        $this->db->where("note LIKE '%{$search}%' and date(data_entry) <= '$end_date' and date(data_entry) >= '$start_date' and system_users.hash LIKE '%$user_hash%'");
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getChecksIn()
    {

        // Get page and limit and search from header
        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $search = $headers['q'] ?? '';
        $offset = ($page - 1) * $perPage;
        $today = date('Y-m-d');
        $start_date = $this->input->get('start_date') ?? $today;
        $end_date = $this->input->get('end_date') ?? $today;
        $user_hash = $this->input->get('user_hash') ?? '';
        $departmant = $this->input->get('department');

        $this->db->select("system_users.name as emp, image_profile, check_in.user_hash,
    (select name from system_group_name where system_group_name.hash=system_users.user_type) as jop,
    DATE(fingerprint_time) AS date,
    TIME_FORMAT(TIME(fingerprint_time), '%l:%i %p') AS time,
    ST_X(location) AS x,
    ST_Y(location) AS y,
    check_in.id, check_in.in_out, check_in.hash, note, partments.name as department");
        $this->db->from("check_in");
        $this->db->join("system_users", "check_in.user_hash = system_users.hash");
        $this->db->join("emp_information", "check_in.user_hash = emp_information.user_hash");
        $this->db->join("partments", "emp_information.partment = partments.hash", 'left');
        $this->db->like("note", $search);
        $this->db->where("DATE(fingerprint_time) <=", $end_date);
        $this->db->where("DATE(fingerprint_time) >=", $start_date);
        if (isset($departmant) && $departmant != "null") {
            $this->db->where("emp_information.partment", $departmant);
        }
        $this->db->like("system_users.hash", $user_hash);
        $this->db->order_by("fingerprint_time", "desc");
        // $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }

    function deleteCheckIn($hash)
    {
        $this->db->where('hash', $hash);
        $this->db->delete('check_in');
        return $this->db->affected_rows();
    }

    public function createCheckIn($data)
    {
        $user = $this->db->escape($data['user_hash']); // Escaping values to prevent SQL injection
        $hash = $this->db->escape($data['hash']);
        $lng = $data['lng'];
        $lat = $data['lat'];
        $note = $this->db->escape($data['note']);
        $mobile_info = isset($data['mobile_info']) ? $this->db->escape($data['mobile_info']) : 'NULL';

        $query = "INSERT INTO check_in (user_hash, note, location, hash, mobile_info) 
                  VALUES ($user, $note, ST_GeomFromText('POINT($lat $lng)'), $hash, $mobile_info)";

        $this->db->query($query);

        return $this->db->affected_rows(); // Return number of affected rows
    }


    public function saveCurrentLocation($user_hash, $note, $lat, $lng, $hash, $in_out, $fingerprint_time, $mobile_info, $time = null)
    {
        $data = [
            'user_hash' => $user_hash,
            'note' => $note,
            'hash' => $hash,
            'in_out' => $in_out,
            'fingerprint_time' => $fingerprint_time,
            'mobile_info' => $mobile_info ?: null, // Handle NULL correctly
        ];

        // Handle spatial data (POINT)
        $this->db->set('location', "POINT($lat, $lng)", false);

        // Handle timestamp field correctly
        if (!empty($time)) {
            $data['time'] = $time;
        }

        // Insert data
        $query = $this->db->insert('check_in', $data);

        // Check for errors
        if (!$query) {
            return $this->db->error()['message']; // Return error message if insertion fails
        }

        return 1; // Success
    }


    public function getHistoryLocations($hash)
    {
        $query = $this->db->query(
            "
           select 
                (select name from system_users where hash=check_in.user_hash) as 'emp',
                DATE(data_entry) as 'date',
                ST_X(location) as x,
                ST_Y(location) as y,
                note,
                in_out,
                fingerprint_time
           from 
                check_in
           where
                user_hash='$hash' 
           "
        );
        return $query->result_array();
    }
}