<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Employees extends CI_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('EmployeesModel');
        $this->load->model('UserModel');
        $this->load->model('AttendModel');
        $this->load->model('UserAttachmentModel');
        $this->load->library('Middleware');
        $this->load->model('GroupScheduleModel');
    }

    public function index()
    {
    }

    // get all employees
    public function getEmployees()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $employees = $this->EmployeesModel->getEmployees();
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض كل الموظفين',
                'token' => $token,
                'data' => $employees,
                'isAuth' => true,
                'total' => $this->EmployeesModel->count_records(),
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    // create new employee

    public function createEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        // get all data from post
        $data = $this->input->post();
        $data['hash'] = $this->jwtmiddleware->createHash();
        $result = $this->EmployeesModel->createEmployee($data);

        if ($result['status']) {
            $this->EmployeesModel->updateEmpInfo($data);
            $this->EmployeesModel->createLocations($data);
            echo json_encode(
                array(
                    'status' => 200,
                    'message' => 'تم اضافة الموظف بنجاح',
                    'token' => $token,
                    'data' => $data,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            // Handle the error message
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => $result['message'],
                    'token' => $token,
                    'data' => null,
                    'isAuth' => true,
                    'action' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }

    }

    // get employee by hash
    public function getEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');

        $employee = $this->EmployeesModel->getEmployee($hash);
        if (empty($employee)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لا يوجد موظف بهذا الرقم',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $employeeVacations = $this->EmployeesModel->getEmployeeVacations($hash);
        $data['finger_print'] = $this->AttendModel->getCurrentMonthAttendForEmp($hash);
        $data['finger_print']['attend'] = $data['finger_print']['attend'] ?? "";
        $data['finger_print']['name'] = $data['finger_print']['name'] ?? $employee[0]['name'];
        $data['finger_print']['user_hash'] = $data['finger_print']['user_hash'] ?? $hash;
        $employeeAttachments = $this->UserAttachmentModel->get_all($hash);

        if (count($employeeVacations) > 0) {
            $data['vacations'] = $employeeVacations;
        } else {
            $data['vacations'] = [];
        }

        if (count($employee) > 0) {
            $data['employee'] = $employee[0];
        }

        if (count($employeeAttachments) > 0) {
            $data['attachments'] = $employeeAttachments;
        } else {
            $data['attachments'] = [];
        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بيانات الموظف',
                'token' => $token,
                'data' => $data ?? [],
                //array('employee'=>$employee,'employeeVacations'=>$employeeVacations, 'employeeFingerPrint'=>$employeeFingerPrint ),
                'isAuth' => true,
                'action' => false
            ),
            JSON_UNESCAPED_UNICODE,
        );
        exit();
    }
    public function getEmployeeForUpdateInfo()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->get('hash');

        $employee = $this->EmployeesModel->getEmployeeForUpdateInfo($hash);
        if (empty($employee)) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لا يوجد موظف بهذا الرقم',
                    'token' => $token,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        $employeeVacations = $this->EmployeesModel->getEmployeeVacations($hash);
        $data['finger_print'] = $this->AttendModel->getCurrentMonthAttendForEmp($hash);
        $data['finger_print']['attend'] = $data['finger_print']['attend'] ?? "";
        $data['finger_print']['name'] = $data['finger_print']['name'] ?? $employee[0]['name'];
        $data['finger_print']['user_hash'] = $data['finger_print']['user_hash'] ?? $hash;
        $employeeAttachments = $this->UserAttachmentModel->get_all($hash);

        if (count($employeeVacations) > 0) {
            $data['vacations'] = $employeeVacations;
        } else {
            $data['vacations'] = [];
        }

        if (count($employee) > 0) {
            $data['employee'] = $employee[0];
        }

        if (count($employeeAttachments) > 0) {
            $data['attachments'] = $employeeAttachments;
        } else {
            $data['attachments'] = [];
        }

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'عرض بيانات الموظف',
                'token' => $token,
                'data' => $data ?? [],
                //array('employee'=>$employee,'employeeVacations'=>$employeeVacations, 'employeeFingerPrint'=>$employeeFingerPrint ),
                'isAuth' => true,
                'action' => false
            ),
            JSON_UNESCAPED_UNICODE,
        );
        exit();
    }

    // update employee
    public function updateEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $data = $this->input->post();
        $res1 = $this->EmployeesModel->updateEmp($data);
        $res2 = $this->EmployeesModel->updateEmpInfo($data);
        $this->EmployeesModel->createLocations($data);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل بيانات الموظف بنجاح',
                'token' => $token,
                'data' => array(
                    'user' => $res1,
                    'emp' => $res2,
                ),
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    // update permissions
    public function updatePermission()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $userHash = $this->jwtmiddleware->decode($token)['hash'];

        $data = $this->input->post();
        $res = $this->EmployeesModel->updatePermission($data, $userHash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم تعديل صلاحيات الموظف بنجاح',
                'token' => $token,
                'data' => $res,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    // delete employee
    public function deleteEmployee()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);

        $hash = $this->input->post('hash');
        $this->EmployeesModel->deleteEmployee($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تم حذف الموظف بنجاح',
                'token' => $token,
                'isAuth' => true,
                'action' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function employeesCanAcceptVacations()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        $user_type = $user['user_type'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $employees = $this->EmployeesModel->employeesCanAcceptVacations($hash, $user_type);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الموظفين الذين يمكنهم قبول الاجازات',
                'token' => $token,
                'data' => $employees,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getCurrentVacationsOfUser()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $vacations = $this->EmployeesModel->getCurrentVacationsOfUser($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الاجازات الحالية للموظف',
                'token' => $token,
                'data' => $vacations,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getHistoryOfVacation()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $page = $this->input->get("page") ?? 1;
        $perPage = $this->input->get("perPage") ?? 10;
        $vacations = $this->EmployeesModel->getHistoryOfVacation($hash, $page, $perPage);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الاجازات السابقة للموظف',
                'token' => $token,
                'data' => $vacations,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function receivedVAcationsOfUser()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        // hash from user to get his data
        $hash = $user['hash'];
        // check hash is exist
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        $vacations = $this->EmployeesModel->receivedVAcationsOfUser($hash);
        echo json_encode(
            array(
                'status' => 200,
                'message' => 'الاجازات المستلمة للموظف',
                'token' => $token,
                'data' => $vacations,
                'isAuth' => true,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

    public function getEmployeeDetails()
    {
        $token = $this->tokenmiddleware->main();
        $this->UserModel->checkPermission();
        $user = $this->jwtmiddleware->decode($token);
        //
        // hash from user to get his data
        $hash = $user['hash'];
        if (!empty($hash)) {
            $employee = $this->EmployeesModel->getEmployee($hash);
            if (empty($employee)) {
                echo json_encode(
                    array(
                        'status' => 400,
                        'message' => 'لا يوجد موظف بهذا الرقم',
                        'token' => $token,
                        'data' => null,
                        'isAuth' => true,
                    ),
                    JSON_UNESCAPED_UNICODE
                );
                exit();
            }
        }
        // Model is here
        $details = $this->EmployeesModel->getEmployeeDetails($hash);
        $details['scheduleData'] = $this->GroupScheduleModel->getScheudleToday($hash, date('Y-m-d'));

        echo json_encode(
            array(
                'status' => 200,
                'message' => 'تفاصيل الموظف',
                'token' => $token,
                'data' => $details,
                'isAuth' => true,
                'hash' => $hash,
            ),
            JSON_UNESCAPED_UNICODE
        );
        exit();
    }

}
