<?php
class EmployeesModel extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->library('Middleware');
        $this->load->model('EmployeesWorkedHoursModel');
    }

    // get all employees
    public function count_records()
    {
        $search = $this->input->get('q') ?? '';
        $isWork = $this->input->get('iswork');
        $department = $this->input->get('department') ?? '';
        $positionId = $this->input->get('positionId') ?? '';

        if (!$isWork) {
            if ($isWork == 2 || $isWork == '2') {
                $isWork = 2;
            } else {
                $isWork = 1;
            }
        }
        $this->db->select('count(*) as count');
        $this->db->from('system_users');
        // inner join
        $this->db->join('emp_information', 'system_users.hash=emp_information.user_hash');
        // where
        $this->db->where('is_work', $isWork);
        if (isset($search) && $search != 'null' && $search != '') {
            $this->db->like('system_users.name', $search);
            $this->db->or_like('username', $search);
        }
        if (isset($department) && $department != 'null' && $department != '') {
            $this->db->where('emp_information.partment', $department); // if it not work, use table.field. not return name
        }

        if (!empty($positionId) && $positionId != '') {
            $this->db->where('emp_information.position', $positionId);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function count_employees_withSpecification()
    {
        $this->db->select('count(*) as count');
        $this->db->from('user_specifications');
        $this->db->join('emp_information', 'user_specifications.emp_info_id = emp_information.user_hash');
        $this->db->where("emp_information.salary_type = 1");
        $result = $this->db->get();
        return $result->result_array()[0]['count'];
    }

    public function getEmployees()
    {
        $isWork = $this->input->get('iswork');
        $department = $this->input->get('department') ?? '';
        $page = $this->input->get('page') ?? '1';
        $perPage = $this->input->get('perPage') ?? '10';
        $positionId = $this->input->get('positionId') ?? '';
        $officeHash = $this->input->get('officeHash') ?? '';

        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        $this->db->select('system_users.name,username,employment_code,password,image_profile,(select name from system_group_name where system_group_name.hash=system_users.user_type) as jop,emp_id as id,(select name from partments where partments.hash=emp_information.partment) as partment, (select Name from Positions where Positions.ID=emp_information.position) as position , (select ID from Positions where Positions.ID=emp_information.position) as positionValue, emp_information.partment as part_hash,jop_type,user_type,system_users.hash');
        $this->db->select('(SELECT GROUP_CONCAT(office_hash) as office_array FROM office_to_user WHERE user_hash = system_users.hash) as location_of_work');
        $this->db->select("vication_agreement,task_agreement,work_in_check_distance,work_out_check_distance, advance_agreement, advance_request");
        $this->db->select("(select name from work_schedules where id = emp_information.jop_type) as job_type_label");
        $this->db->select("emp_information.is_work");
        $this->db->select("(select name from job_title where job_title.id = emp_information.job_title_id) as job_title");
        $this->db->select("(select Roles.name from UserRoles
        LEFT JOIN Roles ON UserRoles.role_id = Roles.id WHERE user_hash = emp_information.user_hash
        ORDER BY Roles.priority asc LIMIT 1) as role_name");

        $this->db->from('system_users');

        // inner join
        $this->db->join('emp_information', 'system_users.hash=emp_information.user_hash');
        $this->db->join('partments', 'emp_information.partment = partments.hash', 'left');
        if ($officeHash != '') {
            $this->db->join('office_to_user otu', 'emp_information.user_hash = otu.user_hash', 'left');
            $this->db->join('office_location ol', 'otu.office_hash = ol.hash', 'left');
            $this->db->where('ol.hash IS NOT NULL'); // This ensures only valid office_hash records are included
        }
        if ($isWork == '1') {
            $this->db->where('emp_information.is_work', '1');
        } else if ($isWork == '2') {
            $this->db->group_start() // Start grouping conditions
                ->where('is_work', '2')
                ->group_start() // Start subgrouping conditions
                ->where('stop_work_date IS NULL', null, false)
                ->or_where('stop_work_date <=', date('Y-m-d'))
                ->group_end() // End subgroup
                ->group_end(); // End main group
        }
        // where
        // $this->db->where('emp_information.is_work !=', $isWork);

        // search
        if (isset($department) && $department != 'null' && $department != '') {
            $this->db->where('emp_information.position', $department);
        }

        if (isset($search) && $search != 'null' && $search != '') {
            $this->db->group_start();
            $this->db->like('system_users.name', $search);
            $this->db->or_like('username', $search);
            $this->db->group_end();
        }

        if (!empty($positionId) && $positionId != '') {
            $this->db->where('emp_information.position', $positionId);
        }

        if (!empty($officeHash) && $officeHash != '') {
            $this->db->where('ol.hash', $officeHash);
        }


        $this->db->order_by('system_users.insert_record_date', 'asc');
        $this->db->group_by('system_users.hash');

        // limit
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }

    public function getEmployeesWithSpecifications()
    {

        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $searchTerm = $headers['q'] ?? ''; // Get the search term from headers

        $offset = ($page - 1) * $perPage;

        $this->db->distinct();

        $this->db->select('emp_info_id as user_hash, image_profile');
        $this->db->select("(select name from system_users WHERE user_specifications.emp_info_id = system_users.hash) as name");
        $this->db->from('user_specifications');

        $this->db->join('emp_information', 'user_specifications.emp_info_id = emp_information.user_hash');
        $this->db->join('system_users', 'user_specifications.emp_info_id = system_users.hash');
        $this->db->where("emp_information.salary_type = 1");
        $this->db->limit($perPage, $offset);

        if (!empty($searchTerm)) {
            $this->db->like('system_users.name', urldecode($searchTerm));
            // Add more conditions if needed
        }
        $query = $this->db->get();
        return $query->result();
    }

    public function count_payroll_records()
    {
        $this->db->select('count(*) as count');
        $this->db->from('emp_payroll');
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }
    public function countEmployeesWithPayroll($from, $to, $job_type = null, $searchTerm = '')
    {
        $job_type = $this->input->get('jobType') ?? null;
        $searchTerm = $this->input->get('q') ?? '';

        $this->db->distinct();
        $this->db->select('count(*) as count');
        $this->db->where('ep.from', $from);
        $this->db->where('ep.to', $to);

        $this->db->from('emp_payroll ep');
        $this->db->join('system_users su', 'ep.emp_id = su.hash');
        $this->db->join('emp_information ei', 'ei.user_hash = su.hash');

        // Add the search condition if a search term is provided
        if (!empty($searchTerm)) {
            $this->db->like('su.name', $searchTerm);
            // Add more conditions if needed
        }
        if (!empty($job_type)) {
            $this->db->where('ei.jop_type', $job_type);
        }

        // Instead of getting the results, count the rows
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getEmployeesWithPayroll($from, $to)
    {
        $headers = apache_request_headers();
        //        $page = $headers['page'] ?? 1;
        $page = $this->input->get('page') ?? 1;
        $perPage = $this->input->get('perPage') ?? 10;
        $job_type = $this->input->get('jobType') ?? null;
        $searchTerm = $this->input->get('q') ?? ''; // Get the search term from headers
        $salaryType = $this->input->get('salaryType') ?? '';
        $position = $this->input->get('position') ?? '';
        $offset = ($page - 1) * $perPage;

        $this->db->distinct();
        $this->db->select('ep.id as id, ep.emp_id as user_hash, su.image_profile, su.name, ep.*');
        $this->db->where('ep.from', $from);
        $this->db->where('ep.to', $to);
        $this->db->where('ep.approved', '0');

        $this->db->from('emp_payroll ep');
        $this->db->join('system_users su', 'ep.emp_id = su.hash');
        $this->db->join('emp_information ei', 'ei.user_hash = su.hash');

        // Add the search condition if a search term is provided
        if (!empty($searchTerm)) {
            $this->db->like('su.name', $searchTerm);
            // Add more conditions if needed
        }
        if (!empty($job_type)) {
            $this->db->where('ei.jop_type', $job_type);
        }

        if (!empty($salaryType)) {
            $this->db->where('ei.salary_type', $salaryType);
        }

        if (!empty($position)) {
            $this->db->where('ei.position', $position);
        }

        $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        return $query->result();
    }

    public function getEmployeeDetailWithPayroll($hash)
    {
        $this->db->select('*');
        $this->db->from('emp_payroll');
        $this->db->where("emp_id", $hash);

        $query = $this->db->get();
        return $query->row();
    }

    public function getSalaryTypes()
    {
        $this->db->select('id as value , name as label ')->from('salary_types');
        return $this->db->get()->result_array();
    }

    public function getEmployeesWithPayments()
    {

        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $isWork = $this->input->get('iswork');
        $department = $this->input->get('department') ?? '';
        // check iswork is undefined or !== 0

        if (!$isWork) {
            if ($isWork == 2 || $isWork == '2') {
                $isWork = 2;
            } else {
                $isWork = 1;
            }
        }
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        // JOINS
        $this->db->distinct('system_users.hash');
        $this->db->select('system_users.hash , system_users.name,username,employment_code,image_profile,(select name from system_group_name where system_group_name.hash=system_users.user_type) as jop,emp_id as id,(select name from partments where partments.hash=emp_information.partment) as partment,emp_information.partment as part_hash,jop_type,user_type, ');
        $this->db->select('(SELECT GROUP_CONCAT(office_hash) as office_array FROM office_to_user WHERE user_hash = system_users.hash) as location_of_work');
        $this->db->select("vication_agreement,task_agreement,work_in_check_distance,work_out_check_distance");
        $this->db->from('system_users');
        $this->db->join('advance', 'system_users.hash = advance.user_hash AND advance.is_deleted = 0', 'inner');
        $this->db->join('emp_information', 'system_users.hash = emp_information.user_hash');
        $this->db->join('partments', 'emp_information.partment = partments.hash', 'left');

        // where
        $this->db->where('is_work', $isWork);
        $this->db->where('advance.status_id', '1');
        // search
        if (isset($department) && $department != 'null' && $department != '') {
            $this->db->where('emp_information.partment', $department); // if it not work, use table.field. not return name
        }
        if (isset($search) && $search != 'null' && $search != '') {
            $this->db->like('system_users.name', $search);
            $this->db->or_like('username', $search);
        }

        // order by
        $this->db->order_by('emp_information.partment');

        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();

        return $query->result_array();
    }

    public function getEmployeeDetailsByHash($userHash)
    {

        $query = $this->db->query("SELECT
        emp_name,
        user_hash,
        start_work_date,
        primary_salary,
        primary_salary * POWER(1 + (annual_percentage_increase / 100), TRUNCATE(DATEDIFF(CURDATE(), start_work_date) / 365.25, 0))  + annual_price_increase AS total_Yearly_salary,

        (select cut_off_amount from finger_print_report where emp_hash=user_hash ) as salary_after_cut_off,
        (select work_hour from finger_print_report where emp_hash=user_hash ) as worked_hours,
                (select work_hour_target from finger_print_report where emp_hash=user_hash ) as target_hours,
        (primary_salary-(select cut_off_amount from finger_print_report where emp_hash=user_hash )) as primary_salary_after_cut_off ,
        annual_percentage_increase,
        annual_price_increase,
        TRUNCATE(DATEDIFF(CURDATE(), start_work_date) / 365.25, 0) AS number_of_year,
        TRUNCATE(DATEDIFF(CURDATE(), start_work_date) / 365.25, 0) * ((annual_percentage_increase/100)*primary_salary+annual_price_increase) AS price_increase_per_year,

        JSON_ARRAYAGG(
           JSON_OBJECT(
               specifications_type,
               JSON_OBJECT(
                   \"primary_salary\", primary_salary,
                   \"name_specifications\", name,
                  \"amount\", price,
                   \"percentage\", percentage,
                   \"total_price\", ((primary_salary)*(percentage/100)+price),
                   \"maximumPrice\", maximumPrice
               )
           )
       ) AS price_group


    FROM (
        SELECT
            user_hash,
            (
                SELECT name
                FROM system_users
                WHERE hash = emp_information.user_hash
            ) AS emp_name,
            primary_salary,
            annual_percentage_increase,
            annual_price_increase,
            start_work_date,
            specification_view.*
        FROM emp_information
        INNER JOIN (
            SELECT
                emp_info_id,
                specification_id,
                CASE
                     WHEN specifications_type = 1 THEN 'specifications_qualifictions'
                 WHEN specifications_type = 2 THEN 'positions_qualifictions'
                 WHEN specifications_type = 3 THEN 'specifications_others'
                 WHEN specifications_type = '-' THEN 'specifications_discounts'
                        END AS specifications_type,
                name,
                percentage,
                price,
                maximumPrice,
                nameValue
            FROM user_specifications
            INNER JOIN specifications ON user_specifications.specification_id = specifications.id
        ) AS specification_view ON emp_information.user_hash = specification_view.emp_info_id
        WHERE emp_information.user_hash = ?

        UNION

        SELECT
            user_hash,
            (
                SELECT name
                FROM system_users
                WHERE hash = emp_information.user_hash
            ) AS emp_name,
            primary_salary,
            annual_percentage_increase,
            annual_price_increase,
            start_work_date,
            discount_view.*
        FROM emp_information
        INNER JOIN (
            SELECT
                emp_info_id,
                financial_discounts.id,
                'specifications_discounts',
                name,
                percentage * - 1,
                price * - 1,
                maximumPrice,
                ''
            FROM user_financial_discounts
            INNER JOIN financial_discounts ON user_financial_discounts.financial_discounts_id = financial_discounts.id
        ) AS discount_view ON emp_information.user_hash = discount_view.emp_info_id
        WHERE emp_information.user_hash = ?

        ORDER BY user_hash
    ) AS payroll_view
    GROUP BY emp_name;", array($userHash, $userHash));

        return $query->row();
    }

    public function countEmployeesPayrollData($year_months)
    {
        $search = $this->input->get('q') ?? '';
        $countQuery = "
        select COUNT(*) as total from (    SELECT
        (
            SELECT image_profile
            FROM system_users
            WHERE emp_information.user_hash = system_users.hash
        ) as image_profile,
        (
            SELECT name
            FROM system_users
            WHERE emp_information.user_hash = system_users.hash
        ) as name,
        emp_name,
        emp_information.user_hash,
        emp_information.start_work_date,
        emp_information.primary_salary,
        ROUND(emp_information.primary_salary * POWER(1 + (emp_information.annual_percentage_increase / 100), TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0)) + emp_information.annual_price_increase
    ,-3)
        AS total_Yearly_salary,
        (
            SELECT cut_off_amount
            FROM finger_print_report
            WHERE emp_hash=emp_information.user_hash
        ) as salary_after_cut_off,
        (
            SELECT work_hour
            FROM finger_print_report
            WHERE emp_hash=emp_information.user_hash
        ) as worked_hours,
        (
            SELECT work_hour_target
            FROM finger_print_report
            WHERE emp_hash=emp_information.user_hash
        ) as target_hours,
        ROUND((emp_information.primary_salary - (
            SELECT cut_off_amount
            FROM finger_print_report
            WHERE emp_hash=emp_information.user_hash
        )) ,-3)
        as primary_salary_after_cut_off ,
        emp_information.annual_percentage_increase,
        emp_information.annual_price_increase,
        TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0) AS number_of_year,
    ROUND(    TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0) * ((emp_information.annual_percentage_increase/100)*emp_information.primary_salary+emp_information.annual_price_increase)
    ,-3)
        AS price_increase_per_year,
    (
        SELECT approved
        FROM emp_payroll
        WHERE emp_id = emp_information.user_hash AND year_months = '$year_months'
    ) as is_approved,
    (
        SELECT COUNT(*)
        FROM emp_payroll
        WHERE emp_id = emp_information.user_hash AND year_months = '$year_months'
    ) as is_exist,
      JSON_ARRAYAGG(
           JSON_OBJECT(
               specifications_type,
               JSON_OBJECT(
                   \"primary_salary\", emp_information.primary_salary,
                   \"name_specifications\", name,
                  \"amount\", price,
                   \"percentage\", percentage,
                   \"total_price\", ((emp_information.primary_salary)*(percentage/100)+price),
                   \"maximumPrice\", maximumPrice
               )
           )
       ) AS price_group

, user_specifications.emp_info_id
FROM (
    SELECT
        emp_information.user_hash,
        (
            SELECT name
            FROM system_users
            WHERE hash = emp_information.user_hash
        ) AS emp_name,
        emp_information.primary_salary,
        annual_percentage_increase,
        emp_information.annual_price_increase,
        emp_information.start_work_date,
        specification_view.*
    FROM emp_information
    INNER JOIN (
        SELECT
            emp_info_id,
            specification_id,
            CASE
                WHEN specifications_type = 1 THEN 'specifications_qualifictions'
                WHEN specifications_type = 2 THEN 'positions_qualifictions'
                WHEN specifications_type = 3 THEN 'specifications_others'
                WHEN specifications_type = '-' THEN 'specifications_discounts'
            END AS specifications_type,
            name,
            percentage,
            price,
            maximumPrice,
            nameValue
        FROM user_specifications
        INNER JOIN specifications ON user_specifications.specification_id = specifications.id
    ) AS specification_view ON emp_information.user_hash = specification_view.emp_info_id
    UNION
    SELECT
        emp_information.user_hash,
        (
            SELECT name
            FROM system_users
            WHERE hash = emp_information.user_hash
        ) AS emp_name,
        emp_information.primary_salary,
        annual_percentage_increase,
        emp_information.annual_price_increase,
        emp_information.start_work_date,
        discount_view.*
    FROM emp_information
    LEFT JOIN (
        SELECT
            emp_info_id,
            financial_discounts.id,
            'specifications_discounts',
            name,
            percentage * - 1,
            price * - 1,
            maximumPrice,
            ''
        FROM user_financial_discounts
        LEFT JOIN financial_discounts ON user_financial_discounts.financial_discounts_id = financial_discounts.id
    ) AS discount_view ON emp_information.user_hash = discount_view.emp_info_id
    ORDER BY user_hash
) AS payroll_view

JOIN user_specifications  ON payroll_view.user_hash = user_specifications.emp_info_id
JOIN emp_information ON  payroll_view.user_hash = emp_information.user_hash

  WHERE emp_name = (SELECT name FROM system_users WHERE user_specifications.emp_info_id = system_users.hash)
 AND emp_information.salary_type = 1
    AND  emp_name LIKE '%$search%'

    GROUP BY emp_name
    ORDER BY emp_information.id
    -- LIMIT 10 OFFSET 0
) as total_rows";

        $countResult = $this->db->query($countQuery)->row();

        return $countResult->total;
    }

    //     public function getEmployeesPayrollData($year_months, $start_date = null, $end_date = null)
    //     {

    //         $page = $this->input->get('page') ?? 1;
    //         $perPage = $this->input->get('perPage') ?? 10;

    //         $search = $this->input->get('q') ?? '';
    //         $positionValue = $this->input->get('positionValue') ?? '';
    //         $offset = ($page - 1) * $perPage;

    //         $positionSubQuery = "
    //             SELECT ID
    //             FROM hr.Positions
    //             WHERE ID = '$positionValue'
    //             UNION ALL
    //             SELECT ID
    //             FROM hr.Positions
    //             WHERE ParentID = '$positionValue'
    //             LIMIT 1

    //         ";
    //         $sql = "SELECT
    // (
    //     SELECT image_profile
    //     FROM system_users
    //     WHERE emp_information.user_hash = system_users.hash
    // ) as image_profile,
    // (
    //     SELECT Name
    //     FROM Positions
    //     WHERE Positions.ID = emp_information.position
    // ) as position,
    // (
    //     SELECT name
    //     FROM system_users
    //     WHERE emp_information.user_hash = system_users.hash
    // ) as name,
    // (
    //     SELECT SUM(
    //         CASE
    //             WHEN pre.penalties_rewards_types_id = 1 THEN pre.amount
    //             ELSE 0
    //         END
    //     )
    //     FROM penalties_rewards_employees pre
    //     WHERE pre.emp_id = emp_information.user_hash
    //  AND pre.date_added >= '$start_date'
    //             AND pre.date_added <= '$end_date'
    // ) AS sum_rewards,
    // (
    //     SELECT SUM(
    //         CASE
    //             WHEN pre.penalties_rewards_types_id = 2 THEN -pre.amount
    //             ELSE 0
    //         END
    //     )
    //     FROM penalties_rewards_employees pre
    //     WHERE pre.emp_id = emp_information.user_hash
    //    AND pre.date_added >= '$start_date'
    //             AND pre.date_added <= '$end_date'
    // ) AS sum_penalties,
    // emp_name,
    // emp_information.user_hash,
    // emp_information.start_work_date,
    // emp_information.primary_salary,
    // emp_information.number_of_year_exp,
    // emp_information.salary_type,
    // CASE
    //         WHEN emp_information.salary_type = 2 THEN 0
    //         ELSE ROUND(
    //             emp_information.primary_salary * POWER(1 + (emp_information.annual_percentage_increase / 100), (TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0))+emp_information.number_of_year_exp) + emp_information.annual_price_increase,
    //             -3
    //         )
    //     END AS total_Yearly_salary,
    // (
    //             SELECT cut_off_amount
    //             FROM finger_print_report
    //             WHERE emp_hash=emp_information.user_hash
    //         ) as salary_after_cut_off,
    //     (
    //         SELECT CASE
    //             WHEN emp_information.salary_type = 2 THEN
    //                 0
    //             WHEN emp_information.fingerprint_required = 1 THEN
    //                 (SELECT work_hour  FROM finger_print_report WHERE emp_hash = emp_information.user_hash)
    //             ELSE
    //                 0
    //         END
    //     ) AS worked_hours,

    //         (
    //              SELECT CASE
    //                 WHEN emp_information.salary_type = 2 THEN
    //                 0
    //                 WHEN emp_information.fingerprint_required = 1 THEN
    //            ( SELECT work_hour_target
    //             FROM finger_print_report
    //             WHERE emp_hash=emp_information.user_hash)
    //             ELSE
    //                 0
    //                 END
    //         ) as target_hours,

    //         ROUND((emp_information.primary_salary - (
    //             SELECT cut_off_amount
    //             FROM finger_print_report
    //             WHERE emp_hash=emp_information.user_hash
    //         )) ,-3)
    //         as primary_salary_after_cut_off ,
    // emp_information.annual_percentage_increase,
    // emp_information.annual_price_increase,
    // TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0) AS number_of_year,
    // ROUND((emp_information.primary_salary * POWER(1 + (emp_information.annual_percentage_increase / 100), TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0)) - emp_information.primary_salary) ,-3) AS price_increase_per_year,

    // (
    //     SELECT approved
    //     FROM emp_payroll
    //     WHERE emp_id = emp_information.user_hash AND year_months = '$year_months'
    // ) as is_approved,

    // (
    //     SELECT COUNT(*)
    //     FROM emp_payroll
    //     WHERE emp_id = emp_information.user_hash AND year_months = '$year_months'
    // ) as is_exist,

    // COALESCE(JSON_ARRAYAGG(
    //     JSON_OBJECT(
    //         COALESCE(specifications_type, 'default_spec_type'),
    //         JSON_OBJECT(
    //             \"primary_salary\", COALESCE(emp_information.primary_salary, 0),
    //             \"name_specifications\", COALESCE(name, ''),
    //             \"amount\", COALESCE(price, 0),
    //             \"percentage\", COALESCE(percentage, 0),
    //             \"total_price\", IFNULL(
    //                 (CAST(emp_information.primary_salary * POWER(1 + (emp_information.annual_percentage_increase / 100), (TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0))+emp_information.number_of_year_exp) + emp_information.annual_price_increase AS UNSIGNED) * (COALESCE(percentage, 0) / 100) + COALESCE(price, 0)),
    //                 0
    //             ),
    //             \"maximumPrice\", COALESCE(maximumPrice, 0)
    //         )

    //     )
    // ), '[]') AS price_group,

    // user_specifications.emp_info_id
    // FROM (
    // SELECT
    //     emp_information.user_hash,
    //     (
    //         SELECT name
    //         FROM system_users
    //         WHERE hash = emp_information.user_hash
    //     ) AS emp_name,
    //     emp_information.primary_salary,
    //     annual_percentage_increase,
    //     emp_information.annual_price_increase,
    //     emp_information.start_work_date,
    //     specification_view.*
    // FROM emp_information
    // LEFT JOIN (
    //     SELECT
    //         emp_info_id,
    //         specification_id,
    //         CASE
    //             WHEN specifications_type = 1 THEN 'specifications_qualifictions'
    //             WHEN specifications_type = 2 THEN 'positions_qualifictions'
    //             WHEN specifications_type = 3 THEN 'specifications_others'
    //             WHEN specifications_type = '-' THEN 'specifications_discounts'
    //         END AS specifications_type,
    //         name,
    //         percentage,
    //         price,
    //         maximumPrice,
    //         nameValue
    //             FROM user_specifications
    //     LEFT JOIN specifications ON user_specifications.specification_id = specifications.id
    // ) AS specification_view ON emp_information.user_hash = specification_view.emp_info_id
    // UNION
    // SELECT
    //     emp_information.user_hash,
    //     (
    //         SELECT name
    //         FROM system_users
    //         WHERE hash = emp_information.user_hash
    //     ) AS emp_name,
    //     emp_information.primary_salary,
    //     annual_percentage_increase,
    //     emp_information.annual_price_increase,
    //     emp_information.start_work_date,
    //     discount_view.*
    //     FROM emp_information
    // LEFT JOIN (
    //     SELECT
    //         emp_info_id,
    //         financial_discounts.id,
    //         'specifications_discounts',
    //         name,
    //         percentage * - 1,
    //         price * - 1,
    //         maximumPrice,
    //         ''
    //     FROM user_financial_discounts
    //     LEFT JOIN financial_discounts ON user_financial_discounts.financial_discounts_id = financial_discounts.id
    // ) AS discount_view ON emp_information.user_hash = discount_view.emp_info_id
    // ORDER BY user_hash
    // ) AS payroll_view

    // LEFT JOIN user_specifications  ON payroll_view.user_hash = user_specifications.emp_info_id
    // JOIN emp_information ON  payroll_view.user_hash = emp_information.user_hash

    // -- WHERE emp_name = (SELECT name FROM system_users WHERE user_specifications.emp_info_id = system_users.hash)
    // -- AND emp_information.salary_type = 1
    // WHERE  emp_name LIKE '%$search%'
    //  " . ($positionValue !== '' ? "AND emp_information.position IN ($positionSubQuery)" : "") . "
    // AND emp_information.is_work = 1
    // GROUP BY emp_name
    // ORDER BY emp_information.id
    //     LIMIT $perPage OFFSET $offset";
    //         $query = $this->db->query($sql);

    //         return $query->result_array();
    //     }

    public function getEmployeesPayrollData($payrollRequestData)
    {
        $page = $this->input->get('page') ?? 1;
        $perPage = $this->input->get('perPage') ?? 10;
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        $startDate = $this->db->escape($payrollRequestData->start_date);
        $endDate = $this->db->escape($payrollRequestData->end_date);
        $year_months = $this->db->escape($payrollRequestData->year_months);
        $search = $this->db->escape('%' . $search . '%');
        $curDate = date('Y-m-d');

        $endDateString = trim($endDate, "'");
        $curDateObject = new DateTime($curDate);
        $endDateObject = new DateTime($endDateString);
        $penaltyMutiply = $this->EmployeesWorkedHoursModel->getMainPenalty();

        // if ($endDateObject > $curDateObject) {
        //     $result = $this->getPenaltyForMissingDate($curDate, $endDateString);
        //     var_dump($result);
        // }

        // SQL query for the paginated results
        $sql = "SELECT
            EAS.user_hash,
            su.image_profile,
            su.name,
            SEC_TO_TIME(ROUND(TRUNCATE(SUM(work_hour * 3600), 2))) AS total_work_hour,
            SEC_TO_TIME(ROUND(TRUNCATE(SUM(work_hour_target * 3600), 2))) AS total_work_hour_target,
            SEC_TO_TIME(ROUND(TRUNCATE(SUM(over_time * 3600), 2))) AS total_over_time,
            SEC_TO_TIME(ROUND(TRUNCATE(SUM(cut_off_hours * 3600), 2))) AS total_cut_off_hours,
            SEC_TO_TIME(ROUND(TRUNCATE(SUM(finger_limit_out * 60), 2))) AS total_finger_limit_out,
            SEC_TO_TIME(ROUND(TRUNCATE(SUM(finger_limit_out_leave * 60), 2))) AS total_finger_limit_out_leave,
            finger_limit_out,
            finger_limit_out_leave,
     ROUND(
        SUM(
            CASE
                WHEN work_hour = 0 AND finger_limit_out = 0 AND finger_limit_out_leave = 0 AND v.user_hash IS NULL
                THEN (work_hour_target - work_hour) * $penaltyMutiply
                WHEN work_hour = 0 AND finger_limit_out = 0 AND finger_limit_out_leave = 0 AND v.user_hash IS NOT NULL THEN (work_hour_target - work_hour)
                ELSE (finger_limit_out / 60 + finger_limit_out_leave / 60) * $penaltyMutiply
            END
        ),
    2) AS total_penalty_hours,


            ei.primary_salary,
            ROUND(ei.primary_salary / NULLIF(SUM(work_hour_target), 0), 2) AS hourly_rate,

         ROUND(
        SUM(
            CASE
                WHEN work_hour = 0 AND finger_limit_out = 0 AND finger_limit_out_leave = 0 AND v.user_hash IS NULL THEN
                    (work_hour_target - work_hour) * $penaltyMutiply
                WHEN work_hour = 0 AND finger_limit_out = 0 AND finger_limit_out_leave = 0 AND v.user_hash IS NOT NULL THEN
                    (work_hour_target - work_hour)
                ELSE
                    (COALESCE(finger_limit_out, 0) / 60 + COALESCE(finger_limit_out_leave, 0) / 60) * $penaltyMutiply
            END
        ) * (ei.primary_salary / NULLIF(SUM(work_hour_target), 0)),
    2) AS cut_off_amount,

            ep.approved AS is_approved,
            (
                SELECT SUM(CASE WHEN penalties_rewards_types_id = 1 THEN amount ELSE 0 END)
                FROM penalties_rewards_employees
                WHERE date_added BETWEEN $startDate AND $endDate
                AND emp_id = EAS.user_hash
            ) AS sum_rewards,
            (
                SELECT SUM(CASE WHEN penalties_rewards_types_id = 2 THEN -amount ELSE 0 END)
                FROM penalties_rewards_employees
                WHERE date_added BETWEEN $startDate AND $endDate
                AND emp_id = EAS.user_hash
            ) AS sum_penalties,
            ROUND(
        ei.primary_salary
        -   ROUND(
        SUM(
            CASE
                WHEN work_hour = 0 AND finger_limit_out = 0 AND finger_limit_out_leave = 0 AND v.user_hash IS NULL THEN
                    (work_hour_target - work_hour) * $penaltyMutiply
                WHEN work_hour = 0 AND finger_limit_out = 0 AND finger_limit_out_leave = 0 AND v.user_hash IS NOT NULL THEN
                    (work_hour_target - work_hour)
                    ELSE
                    (COALESCE(finger_limit_out, 0) / 60 + COALESCE(finger_limit_out_leave, 0) / 60) * $penaltyMutiply
            END
        ) * (ei.primary_salary / NULLIF(SUM(work_hour_target), 0)),
    2)
        + (
            SELECT COALESCE(SUM(CASE WHEN penalties_rewards_types_id = 1 THEN amount ELSE 0 END), 0)
            FROM penalties_rewards_employees
            WHERE date_added BETWEEN $startDate AND $endDate
            AND emp_id = EAS.user_hash
        )
        + (
            SELECT COALESCE(SUM(CASE WHEN penalties_rewards_types_id = 2 THEN -amount ELSE 0 END), 0)
            FROM penalties_rewards_employees
            WHERE date_added BETWEEN $startDate AND $endDate
            AND emp_id = EAS.user_hash
        ),
        2
    ) AS salary_after_cutoffs
        FROM
            EmployeeAttendanceSummary EAS
        LEFT JOIN
            emp_information ei ON EAS.user_hash = ei.user_hash
        LEFT JOIN
            system_users su ON EAS.user_hash = su.hash
        LEFT JOIN
            emp_payroll ep ON EAS.user_hash = ep.emp_id AND ep.year_months = $year_months
        LEFT JOIN vacations v ON EAS.user_hash = v.user_hash
        AND (
        (EAS.date  BETWEEN DATE(v.date_start) AND DATE_SUB(DATE(v.date_end), INTERVAL 1 DAY) AND v.date_start <> v.date_end)
        OR (EAS.date = DATE(v.date_start) AND v.date_start = v.date_end)
            )
        AND v.status = '111'
        WHERE
            EAS.date BETWEEN $startDate AND $endDate
            AND su.name LIKE $search
            -- AND ei.is_work = 1
        GROUP BY
            EAS.user_hash, su.image_profile, su.name, ei.primary_salary, ep.approved
        ORDER BY ei.id
        LIMIT $offset, $perPage;";

        // SQL query for the total count
        $countSql = "SELECT COUNT(DISTINCT EAS.user_hash) AS total
        FROM
            EmployeeAttendanceSummary EAS
        LEFT JOIN
            emp_information ei ON EAS.user_hash = ei.user_hash
        LEFT JOIN
            system_users su ON EAS.user_hash = su.hash
        LEFT JOIN
            emp_payroll ep ON EAS.user_hash = ep.emp_id AND ep.year_months = $year_months
        WHERE
            EAS.date BETWEEN $startDate AND $endDate
            AND su.name LIKE $search";

        // Execute the queries
        $result = $this->db->query($sql)->result_array();
        if ($endDateObject > $curDateObject) {

            $curDateObject = new DateTime($curDate);
            $curDateObject->modify('+1 day');
            $curDatePlusOne = $curDateObject->format('Y-m-d');

            $penalties = $this->getPenaltyForMissingDate($curDatePlusOne, $endDateString);
            // Step 3: Merge penalties into payroll data
            foreach ($result as &$record) {
                $userHash = $record['user_hash'];

                // Get penalty hours
                $penaltyHours = $penalties[$userHash] ?? 0; // Default to 0 if not found
                // Ensure total_work_hour_target is treated as numeric
                $totalWorkHourTargetNumeric = $this->convertHHMMSSToHours($record['total_work_hour_target']);

                // Add penalty hours to the total work hour target (in numeric format)
                $totalWorkHourTargetNumeric += $penaltyHours;

                // Convert back to HH:MM:SS format for display
                $record['total_work_hour_target'] = $this->formatHoursToHHMMSS($totalWorkHourTargetNumeric);

                // Calculate new hourly rate based on updated work_hour_target
                $newWorkHourTarget = $totalWorkHourTargetNumeric; // Use the updated value
                $record['hourly_rate'] = round($record['primary_salary'] / ($newWorkHourTarget > 0 ? $newWorkHourTarget : 1), 2);

                // Store the penalty for this specific record in formatted form for display
                $record['penalty_for_missing_date'] = $this->formatHoursToHHMMSS($penaltyHours);

                $record["from_penalty_missing_date"] = $curDate;
                $record["to_penalty_missing_date"] = $endDateString;

                // Initialize totals before accumulating
                $record['total_penalty_hours'] += $penaltyHours; // Keep the original numeric value for calculations
                $record['new_cut_off_amount'] = $record['total_penalty_hours'] * $record['hourly_rate'];

                // Update salary after cutoffs

                // Round values
                $record['cut_off_amount'] = $record['new_cut_off_amount'];
                $record['cut_off_amount'] = round($record['cut_off_amount'], 2);
                $record['salary_after_cutoffs'] = $record['primary_salary'] - $record['cut_off_amount'];
                $record['salary_after_cutoffs'] = round($record['salary_after_cutoffs'], 2);
            }
        }

        $total = $this->db->query($countSql)->row()->total;

        // Return the result and total count
        return [
            'result' => $result,
            'total' => $total,
        ];
    }

    private function convertHHMMSSToHours($timeString)
    {
        list($hours, $minutes, $seconds) = explode(':', $timeString);
        return $hours + ($minutes / 60) + ($seconds / 3600);
    }

    private function formatHoursToHHMMSS($hours)
    {
        // Calculate hours, minutes, and seconds
        $hoursInt = floor($hours);
        $minutes = floor(($hours - $hoursInt) * 60);
        $seconds = 0; // Assuming seconds are 0 since we typically deal with whole hours

        // Format the result to HH:MM:SS
        return sprintf('%02d:%02d:%02d', $hoursInt, $minutes, $seconds);
    }

    private function getPenaltyForMissingDate($curDate, $endDate)
    {
        // Convert the dates to DateTime objects for easy manipulation
        $curDateObj = new DateTime($curDate);
        $endDateObj = new DateTime($endDate);

        // Fetch all employees and their schedules in one query
        $sql = "SELECT
                    ei.user_hash,
                    ei.target_hours,
                    ei.flexible_schedule AS is_flexible,
                    sg.schedule_data
                FROM
                    emp_information ei
                LEFT JOIN
                    schedule_groups sg ON ei.work_schedule = sg.id";

        $employees = $this->db->query($sql)->result_array();
        $penalties = [];

        // calc non flexible employees
        // Loop through each employee to calculate penalties
        foreach ($employees as $employee) {
            $userHash = $employee['user_hash'];
            $isFlexible = $employee['is_flexible'];
            $totalTargetHours = 0;

            // Proceed only for non-flexible employees
            if (!$isFlexible) {
                // Decode the schedule data once
                $scheduleData = json_decode($employee['schedule_data'], true);

                // Fetch vacation days for the current employee

                $vacationSql = "SELECT dv.en_label
                FROM user_vacations uv
                JOIN days_of_week dv ON uv.day_of_week_id = dv.id
                WHERE uv.user_hash = ?";
                $vacationDays = $this->db->query($vacationSql, [$userHash])->result_array();
                $vacationDaysArray = array_column($vacationDays, 'en_label');

                // Get the current day of the week
                $dayOfWeek = $curDateObj->format('l'); // 'l' returns the full textual representation of the day

                // Loop through the date range
                while ($curDateObj <= $endDateObj) {
                    // Get the current day of the week (e.g., Monday, Tuesday, etc.)
                    $dayOfWeek = $curDateObj->format('l');

                    // Skip if today is a vacation day

                    if (in_array($dayOfWeek, $vacationDaysArray)) {
                        // Move to the next day
                        $curDateObj->modify('+1 day');
                        // Ensure this is within a loop to use continue
                        continue; // Skip penalty calculation
                    }

                    // Check if there is a schedule for this day
                    if (isset($scheduleData[$dayOfWeek])) {
                        foreach ($scheduleData[$dayOfWeek] as $schedule) {
                            // Calculate the duration of each schedule (end - start)
                            $start = $schedule['start'];
                            $end = $schedule['end'];
                            $duration = $end - $start;
                            $totalTargetHours += $duration;
                        }
                    }

                    // Move to the next day
                    $curDateObj->modify('+1 day');
                }

                // Store total target hours for the employee
                $penalties[$userHash] = $totalTargetHours;
            }

            // Reset curDateObj for the next employee
            $curDateObj = new DateTime($curDate);
        }

        // calc flexible employees

        $flexbilbe_sql = "SELECT
                    ei.user_hash,
                    ei.target_hours,
                    flexible_schedule AS is_flexible
                FROM
                    emp_information ei
                WHERE flexible_schedule = '1'";

        $flexibleEmployees = $this->db->query($flexbilbe_sql)->result_array();

        foreach ($flexibleEmployees as $employee) {
            $userHash = $employee['user_hash'];
            $isFlexible = $employee['is_flexible'];
            $totalTargetHours = 0;

            // Proceed only for non-flexible employees
            if ($isFlexible) {

                // Fetch vacation days for the current employee
                $vacationSql = "SELECT dv.en_label
                FROM user_vacations uv
                JOIN days_of_week dv ON uv.day_of_week_id = dv.id
                WHERE uv.user_hash = ?";
                $vacationDays = $this->db->query($vacationSql, [$userHash])->result_array();
                $vacationDaysArray = array_column($vacationDays, 'en_label');

                // Get the current day of the week
                $dayOfWeek = $curDateObj->format('l');

                // Loop through the date range
                while ($curDateObj <= $endDateObj) {
                    // Get the current day of the week (e.g., Monday, Tuesday, etc.)
                    $dayOfWeek = $curDateObj->format('l');

                    // Skip if today is a vacation day
                    if (in_array($dayOfWeek, $vacationDaysArray)) {
                        // Move to the next day
                        $curDateObj->modify('+1 day');
                        // Ensure this is within a loop to use continue
                        continue; // Skip penalty calculation
                    }

                    // Check if there is a schedule for this day

                    $duration = $employee['target_hours'];

                    $totalTargetHours += $duration;

                    // Move to the next day
                    $curDateObj->modify('+1 day');
                }

                // Store total target hours for the employee
                $penalties[$userHash] = $totalTargetHours;
            }

            // Reset curDateObj for the next employee
            $curDateObj = new DateTime($curDate);
        }

        return $penalties; // Return penalties or use this array for further calculations
    }

    // private function getPenaltyForMissingDate($curDate, $endDate)
    // {
    //     // Convert the dates to DateTime objects for easy manipulation
    //     $curDateObj = new DateTime($curDate);
    //     $endDateObj = new DateTime($endDate);

    //     // Get all employees
    //     $employees = $this->db->query("SELECT user_hash, target_hours, flexible_schedule as is_flexible FROM emp_information")->result_array();

    //     $penalties = [];

    //     // Loop through each employee to calculate penalties
    //     foreach ($employees as $employee) {
    //         $userHash = $employee['user_hash'];
    //         $targetHours = $employee['target_hours'];
    //         $isFlexible = $employee['is_flexible'];
    //         $totalTargetHours = 0;
    //         // Get the scheduled work hours for non-flexible employees
    //         if (!$isFlexible) {
    //             $sql = "SELECT
    //                         ei.user_hash,
    //                         ei.primary_salary,
    //                         ei.flexible_schedule,
    //                         ei.target_hours,
    //                         sg.schedule_data
    //                     FROM
    //                         emp_information ei
    //                      JOIN
    //                         schedule_groups sg ON ei.work_schedule = sg.id
    //                     WHERE
    //                          ei.flexible_schedule = 0
    //                          AND ei.user_hash = ?";

    //             $result = $this->db->query($sql, [$userHash])->row();
    //             if (!$result) {
    //                 continue; // Skip if no result found
    //             }

    //             $scheduleData = json_decode($result->schedule_data, true);

    //             // Loop through the date range
    //             while ($curDateObj <= $endDateObj) {
    //                 // Get the current day of the week (e.g., Monday, Tuesday, etc.)
    //                 $dayOfWeek = $curDateObj->format('l');

    //                 // Check if there is a schedule for this day
    //                 if (isset($scheduleData[$dayOfWeek])) {
    //                     foreach ($scheduleData[$dayOfWeek] as $schedule) {
    //                         // Calculate the duration of each schedule (end - start)
    //                         $start = $schedule['start'];
    //                         $end = $schedule['end'];
    //                         $duration = $end - $start;
    //                         $totalTargetHours += $duration;
    //                     }
    //                 }

    //                 // Move to the next day
    //                 $curDateObj->modify('+1 day');
    //             }

    //             // Calculate penalty or return total target hours
    //             $penalties[$userHash] = $totalTargetHours;

    //         }
    //         // else {

    //         // $penalties[$userHash] = $targetHours + $this->getFlexiblePenalty($curDate, $endDateObj, $employee);
    //         // }

    //         // Reset curDateObj for the next employee
    //         $curDateObj = new DateTime($curDate);
    //     }

    //     return $penalties; // Return penalties or use this array for further calculations
    // }

    // Sample function for calculating penalties for flexible employees
    // private function getFlexiblePenalty($curDateObj, $endDateObj, $employee)
    // {
    //     $userHash = $employee['user_hash'];
    //     $targetHours = $employee['target_hours'];
    //     $isFlexible = $employee['is_flexible'];
    //     $totalTargetHours = 0;

    //     // Get the scheduled work hours for flexible employees
    //     if ($isFlexible) {

    //         // Loop through the date range
    //         while ($curDateObj <= $endDateObj) {
    //             // Get the current day of the week (e.g., Monday, Tuesday, etc.)
    //             $dayOfWeek = $curDateObj->format('l');

    //             // Check if there is a schedule for this day
    //             if (isset($scheduleData[$dayOfWeek])) {
    //                 foreach ($scheduleData[$dayOfWeek] as $schedule) {
    //                     // Calculate the duration of each schedule (end - start)
    //                     $start = $schedule['start'];
    //                     $end = $schedule['end'];
    //                     $duration = $end - $start;
    //                     $totalTargetHours += $duration;
    //                 }
    //             }

    //             // Move to the next day
    //             $curDateObj->modify('+1 day');
    //         }

    //         // Calculate penalty or return total target hours
    //         return $totalTargetHours;
    //     }

    //     return 0;
    // }

    public function getEmployeesPayrollDataWithHash($year_months, $start_date = null, $end_date = null, $userHash = null)
    {
        $page = $this->input->get('page') ?? 1;
        $perPage = $this->input->get('perPage') ?? 10;

        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        // Prepare the base query
        $query = "
        SELECT
        (
            SELECT image_profile
            FROM system_users
            WHERE emp_information.user_hash = system_users.hash
        ) as image_profile,
        (
            SELECT name
            FROM system_users
            WHERE emp_information.user_hash = system_users.hash
        ) as name,
        (
            SELECT SUM(
                CASE
                    WHEN pre.penalties_rewards_types_id = 1 THEN pre.amount
                    ELSE 0
                END
            )
            FROM penalties_rewards_employees pre
            WHERE pre.emp_id = emp_information.user_hash
             AND pre.date_added >= '$start_date'
            AND pre.date_added <= '$end_date'
        ) AS sum_rewards,
         (
            SELECT SUM(
                CASE
                    WHEN pre.penalties_rewards_types_id = 2 THEN -pre.amount
                    ELSE 0
                END
            )
            FROM penalties_rewards_employees pre
            WHERE pre.emp_id = emp_information.user_hash
            AND pre.date_added >= '$start_date'
            AND pre.date_added <= '$end_date'
        ) AS sum_penalties,
        emp_name,
        emp_information.user_hash,
        emp_information.start_work_date,
        emp_information.primary_salary,
        emp_information.number_of_year_exp,
        emp_information.salary_type,
        CASE
        WHEN emp_information.salary_type = 2 THEN emp_information.primary_salary
        ELSE ROUND(
            emp_information.primary_salary * POWER(1 + (emp_information.annual_percentage_increase / 100),
            (TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0)) +
            (emp_information.number_of_year_exp)) +
            emp_information.annual_price_increase
        )
    END AS total_Yearly_salary,
        (
            SELECT cut_off_amount
            FROM finger_print_report
            WHERE emp_hash=emp_information.user_hash
        ) as salary_after_cut_off,
    (
        SELECT
    CASE
        WHEN emp_information.salary_type = 2 THEN
            0
        WHEN emp_information.fingerprint_required = 1 THEN
            COALESCE(
                (SELECT work_hour
                 FROM finger_print_report
                 WHERE emp_hash = emp_information.user_hash),
                0
            )
        ELSE
            0
    END AS worked_hours
    ) AS worked_hours,

    COALESCE(
        (SELECT work_hour_target
         FROM finger_print_report
         WHERE emp_hash = emp_information.user_hash),
        0
    ) AS target_hours,

        ROUND((emp_information.primary_salary - (
            SELECT cut_off_amount
            FROM finger_print_report
            WHERE emp_hash=emp_information.user_hash
        )) ,-3)
        as primary_salary_after_cut_off ,
        emp_information.annual_percentage_increase,
        emp_information.annual_price_increase,
        TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0) AS number_of_year,
        ROUND((emp_information.primary_salary * POWER(1 + (emp_information.annual_percentage_increase / 100), TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0)) - emp_information.primary_salary) ,-3) AS price_increase_per_year,

    (
        SELECT approved
        FROM emp_payroll
        WHERE emp_id = emp_information.user_hash AND year_months = '$year_months'
    ) as is_approved,
    (
        SELECT COUNT(*)
        FROM emp_payroll
        WHERE emp_id = emp_information.user_hash AND year_months = '$year_months'
    ) as is_exist,

    COALESCE(JSON_ARRAYAGG(
        JSON_OBJECT(
            COALESCE(specifications_type, 'default_spec_type'),
            JSON_OBJECT(
                \"primary_salary\", COALESCE(emp_information.primary_salary, 0),
                \"name_specifications\", COALESCE(name, ''),
                \"amount\", COALESCE(price, 0),
                \"percentage\", COALESCE(percentage, 0),
                \"total_price\", IFNULL(
                    (CAST(emp_information.primary_salary * POWER(1 + (emp_information.annual_percentage_increase / 100), (TRUNCATE(DATEDIFF(CURDATE(), emp_information.start_work_date) / 365.25, 0))+emp_information.number_of_year_exp) + emp_information.annual_price_increase AS UNSIGNED) * (COALESCE(percentage, 0) / 100) + COALESCE(price, 0)),
                    0
                ),
                \"maximumPrice\", COALESCE(maximumPrice, 0)
            )

        )
    ), '[]') AS price_group,

    user_specifications.emp_info_id
FROM (
    SELECT
        emp_information.user_hash,
        (
            SELECT name
            FROM system_users
            WHERE hash = emp_information.user_hash
        ) AS emp_name,
        emp_information.primary_salary,
        annual_percentage_increase,
        emp_information.annual_price_increase,
        emp_information.start_work_date,
        specification_view.*
    FROM emp_information
    LEFT JOIN (
        SELECT
            emp_info_id,
            specification_id,
            CASE
                WHEN specifications_type = 1 THEN 'specifications_qualifictions'
                WHEN specifications_type = 2 THEN 'positions_qualifictions'
                WHEN specifications_type = 3 THEN 'specifications_others'
                WHEN specifications_type = '-' THEN 'specifications_discounts'
            END AS specifications_type,
            name,
            percentage,
            price,
            maximumPrice,
            nameValue
                        FROM user_specifications
        LEFT JOIN specifications ON user_specifications.specification_id = specifications.id
    ) AS specification_view ON emp_information.user_hash = specification_view.emp_info_id
    UNION
    SELECT
        emp_information.user_hash,
        (
            SELECT name
            FROM system_users
            WHERE hash = emp_information.user_hash
        ) AS emp_name,
        emp_information.primary_salary,
        annual_percentage_increase,
        emp_information.annual_price_increase,
        emp_information.start_work_date,
        discount_view.*
       FROM emp_information
    LEFT JOIN (
        SELECT
            emp_info_id,
            financial_discounts.id,
            'specifications_discounts',
            name,
            percentage * - 1,
            price * - 1,
            maximumPrice,
            ''
        FROM user_financial_discounts
        LEFT JOIN financial_discounts ON user_financial_discounts.financial_discounts_id = financial_discounts.id
    ) AS discount_view ON emp_information.user_hash = discount_view.emp_info_id
    ORDER BY user_hash
) AS payroll_view
    JOIN user_specifications ON payroll_view.user_hash = user_specifications.emp_info_id
    JOIN emp_information ON payroll_view.user_hash = emp_information.user_hash
    WHERE emp_name = (SELECT name FROM system_users WHERE user_specifications.emp_info_id = system_users.hash)
    AND emp_name LIKE '%$search%'";

        // Add condition to filter by user hash if provided
        if ($userHash) {
            $query .= " AND emp_information.user_hash = '$userHash'";
        }

        $query .= " GROUP BY emp_name ORDER BY emp_information.id LIMIT $perPage OFFSET $offset";

        $result = $this->db->query($query);

        $data = $result->result_array();

        $vacationQueryResult = $this->db->query("CALL CalculateMonthlyVacationDays('$userHash', '$start_date','$end_date')");
        $vacationData = $vacationQueryResult->result_array();

        $data[0]['vacationData'] = $vacationData;

        return $data;
    }

    public function getVacationData($userHash, $start_date, $end_date)
    {
        $vacationQueryResult = $this->db->query("CALL CalculateMonthlyVacationDays('$userHash', '$start_date','$end_date')");
        $vacationData = $vacationQueryResult->result_array();

        return $vacationData[0]['vacationData'] = $vacationData;
    }

    public function updateFingerPrintRecord($start_date, $end_date)
    {
        $this->db->query("call finger_print_report('$start_date', '$end_date')");
        return true;
    }

    public function getTargetByUser($hash)
    {
        $query = $this->db->query("SELECT
        finger_print_report.*,
        partments.name AS 'department',
        image_profile
    FROM
        finger_print_report
    LEFT JOIN system_users ON system_users.hash = finger_print_report.emp_hash
    LEFT JOIN
        emp_information ON emp_information.user_hash = finger_print_report.emp_hash
    LEFT JOIN
        partments ON partments.hash = emp_information.partment
    WHERE
        emp_information.user_hash = ?", array($hash));

        return $query->row();
    }

    // get employee rules by hash
    public function getEmployeeRules($hash)
    {
        $this->db->select('role_id as value , (select name from Roles where role_id = id ) as label')->where('user_hash', $hash)->from('UserRoles');
        return $this->db->get()->result_array();
    }
    // get employee by hash
    public function getEmployee($hash)
    {
        $this->db->select(
            '
            emp_information.user_hash as user_hash,
            name,username,password,employment_code,
            user_type,vication_agreement,task_agreement,
            advance_agreement,advance_request,
            work_in_check_distance,work_out_check_distance,
            primary_salary,vacations_limit,sick_leave,status,
            work_from,work_to,allow_time,birthday,address,phone1,
            phone2,partment, position ,start_work_date,jop_type,is_work,image_profile,annual_percentage_increase,annual_price_increase,
            (select Name from Positions where Positions.ID=emp_information.position) as positionLabel,
            (select name from system_group_name where system_group_name.hash=system_users.user_type) as jop,
            emp_information.position,
            emp_information.salary_type,
            emp_information.position_other,
            emp_information.IBN,
            emp_information.number_of_year_exp,
            emp_information.fingerprint_required,
            emp_information.target_hours,
            emp_information.flexible_schedule,
            emp_information.work_schedule,
            emp_information.stop_work_date,
            emp_information.send_myLocation,
            emp_information.can_change_flag_to_send_myLocation,

            (select name from positions_other  where positions_other.id=emp_information.position_other) as position_other_label
            ,
            (select id from job_title where  emp_information.job_title_id = job_title.id) as job_title_id'
        );
        $this->db->select('
            (SELECT GROUP_CONCAT((select name from office_location where hash=office_hash)) as office_array FROM office_to_user WHERE user_hash = system_users.hash) as name_location_of_work,
            (SELECT GROUP_CONCAT(office_hash) as office_array FROM office_to_user WHERE user_hash = system_users.hash) as location_of_work
        ');
        $this->db->from('system_users');
        // inner join
        $this->db->join('emp_information', 'system_users.hash=emp_information.user_hash');
        // where
        $this->db->where('system_users.hash', $hash);

        $query = $this->db->get();
        return $query->result_array();
    }

    public function getEmployeeForUpdateInfo($hash)
    {
        $this->db->select(
            '
                name,username,password,employment_code,
                user_type,
                work_in_check_distance,
                primary_salary,vacations_limit,sick_leave,status,
                work_from,work_to,allow_time,birthday,address,phone1,
                phone2,partment, position ,start_work_date,jop_type,is_work,image_profile,annual_percentage_increase,annual_price_increase,
                (select Name from Positions where Positions.ID=emp_information.position) as positionLabel,
                (select name from system_group_name where system_group_name.hash=system_users.user_type) as jop,
                emp_information.position,
                emp_information.salary_type,
                emp_information.position_other,
                emp_information.IBN,
                emp_information.number_of_year_exp,
                emp_information.fingerprint_required,
                emp_information.target_hours,
                emp_information.flexible_schedule,
                emp_information.work_schedule,
                emp_information.stop_work_date,
                (select name from positions_other  where positions_other.id=emp_information.position_other) as position_other_label
                ,
                (select id from job_title where  emp_information.job_title_id = job_title.id) as job_title_id'
        );
        $this->db->select('
                (SELECT GROUP_CONCAT((select name from office_location where hash=office_hash)) as office_array FROM office_to_user WHERE user_hash = system_users.hash) as name_location_of_work,
                (SELECT GROUP_CONCAT(office_hash) as office_array FROM office_to_user WHERE user_hash = system_users.hash) as location_of_work
            ');
        $this->db->from('system_users');
        // inner join
        $this->db->join('emp_information', 'system_users.hash=emp_information.user_hash');
        // where
        $this->db->where('system_users.hash', $hash);

        $query = $this->db->get();
        return $query->result_array();
    }

    public function getEmployeeForPrint($hash)
    {
        $this->db->select('
        us.name, ei.phone1 , ei.phone2, ei.address, ei.status , ei.birthday,
        us.username,
        us.employment_code,
        ei.primary_salary,
        ei.start_work_date,
        pa.name AS partment,
        ei.work_from,
        ei.work_to,
        ei.allow_time,
        ei.vacations_limit,
        ei.sick_leave AS sick_leave,
        ei.is_work AS is_work,
        (select name from system_group_name sgn where sgn.hash=us.user_type) as jop,
        ei.jop_type,
        (select name from office_location ofl WHERE ofl.hash=ei.location_of_work) as location_of_work
        ,us.image_profile
        ');
        $this->db->from("system_users us");
        $this->db->join("emp_information ei", "us.hash=ei.user_hash");
        $this->db->join("partments pa", "pa.hash=ei.partment", "left");
        $this->db->where('us.hash', $hash);

        $query = $this->db->get();
        return $query->row();
    }

    public function getEmployeesWithAssignItem()
    {
        $headers = apache_request_headers();
        $page = $headers['page'] ?? 1;
        $perPage = $headers['perPage'] ?? 10;
        $isWork = $this->input->get('iswork');
        $department = $this->input->get('department') ?? '';
        // check iswork is undefined or !== 0

        if (!$isWork) {
            if ($isWork == 2 || $isWork == '2') {
                $isWork = 2;
            } else {
                $isWork = 1;
            }
        }
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        // JOINS
        $this->db->distinct('system_users.hash');
        $this->db->select('system_users.hash , system_users.name,username,employment_code,password,image_profile,(select name from system_group_name where system_group_name.hash=system_users.user_type) as jop,emp_id as id,(select name from partments where partments.hash=emp_information.partment) as partment,emp_information.partment as part_hash,jop_type,user_type, ');
        $this->db->select('(SELECT GROUP_CONCAT(office_hash) as office_array FROM office_to_user WHERE user_hash = system_users.hash) as location_of_work');
        $this->db->select("vication_agreement,task_agreement,work_in_check_distance,work_out_check_distance");
        $this->db->from('system_users');
        $this->db->join('item_assignments', 'system_users.hash = item_assignments.user_hash', 'inner'); // Inner join with requestPayment
        $this->db->join('emp_information', 'system_users.hash = emp_information.user_hash');
        $this->db->join('partments', 'emp_information.partment = partments.hash', 'left');

        // where
        $this->db->where('is_work', $isWork);
        // search
        if (isset($department) && $department != 'null' && $department != '') {
            $this->db->where('emp_information.partment', $department); // if it not work, use table.field. not return name
        }
        if (isset($search) && $search != 'null' && $search != '') {
            $this->db->like('system_users.name', $search);
            $this->db->or_like('username', $search);
        }

        // order by
        $this->db->order_by('emp_information.partment');

        // limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();

        return $query->result_array();
    }

    public function getEmployeeForAccount($hash)
    {
        $this->db->select('id,name');
        $this->db->from('system_users');
        $this->db->where('system_users.hash', $hash);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getEmployeeVacations($hash)
    {
        $this->db->select("
            date(date_start) as start,
            date(date_end) as end,
            hash,
            note,
            (select type from vacations_type where hash=vacations.vication_type_hash) as type,
            (select title from vacations_status where hash=vacations.status) as status,
            (select name from system_users where hash=admin) as admin_name
        ");
        $this->db->from('vacations');
        $this->db->where('user_hash', $hash);
        $this->db->where('vacations.vication_type_hash !=', '666');
        $this->db->where('vacations.hash !=', null);

        // order by
        $this->db->order_by('vacations.id', 'DESC');
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getEmployeeFingerPrint($hash)
    {
        $this->db->select('*');
        $this->db->from('finger_print_report');
        $this->db->where('emp_hash', $hash);

        $query = $this->db->get();
        return $query->result_array();
    }

    public function createEmployee($data)
    {
        $this->db->where('name', $data['name']);
        $query = $this->db->get('system_users');
        if ($query->num_rows() > 0) {
            return ['status' => false, 'message' => 'الاسم موجود بالفعل'];
        }

        // Check if the username already exists
        $this->db->where('username', $data['username']);
        $query = $this->db->get('system_users');
        if ($query->num_rows() > 0) {
            return ['status' => false, 'message' => 'اسم المستخدم موجود بالفعل'];
        }

        // Prepare user data
        $userData = array(
            'name' => $data['name'],
            'username' => $data['username'],
            'password' => $data['password'],
            // 'user_type' => $data['user_type'],
            'hash' => $data['hash'],
            'employment_code' => $data['employment_code'],
        );

        // Insert user data
        $this->db->insert('system_users', $userData);

        // Prepare employee data
        $empData = array(
            'user_hash' => $data['hash'],
        );

        // Insert employee data
        $this->db->insert('emp_information', $empData);

        // Check if rows were affected
        if ($this->db->affected_rows() > 0) {
            return ['status' => true, 'message' => 'تم إنشاء الموظف بنجاح'];
        } else {
            return ['status' => false, 'message' => 'فشل في إنشاء الموظف'];
        }
    }

    public function deleteEmployeeFromDB($userHash)
    {
        // Check if the employee has any related records
        $hasRecords = false;

        // Check fingerprints
        $query = $this->db->select('id')
            ->from('emp_fingerprint')
            ->where('user_hash', $userHash)
            ->limit(1)
            ->get();
        if ($query->num_rows() > 0) {
            return [
                'status' => false,
                'message' => 'لا يمكن المسح الموظف لديه بصمات'
            ];
        }

        // Check vacations
        $query = $this->db->select('id')
            ->from('vacations')
            ->where('user_hash', $userHash)
            ->limit(1)
            ->get();
        if ($query->num_rows() > 0) {
            return [
                'status' => false,
                'message' => 'لا يمكن المسح الموظف لديه اجازات.'
            ];
        }

        // Check item assignments
        $query = $this->db->select('user_hash')
            ->from('item_assignments')
            ->where('user_hash', $userHash)
            ->limit(1)
            ->get();
        if ($query->num_rows() > 0) {
            return [
                'status' => false,
                'message' => 'لا يمكن المسح الموظف لديه ذمم.'
            ];
        }

        // Delete employee information
        $this->db->where('user_hash', $userHash)
            ->delete('emp_information');

        // Delete user from the system
        $this->db->where('hash', $userHash)
            ->delete('system_users');

        return [
            'status' => true,
            'message' => 'تم مسح الموظف بنجاح.'
        ];
    }


    public function updatePermission($data, $changed_by)
    {
        // Get current values
        $current_emp_info = $this->db->query("
            SELECT work_in_check_distance, work_out_check_distance, advance_agreement, advance_request
            FROM emp_information WHERE user_hash = '{$data['hash']}'
        ")->row_array();

        $current_sys_users = $this->db->query("
            SELECT vication_agreement, task_agreement
            FROM system_users WHERE hash = '{$data['hash']}'
        ")->row_array();

        // Function to log changes
        function log_change($field, $old_value, $new_value, $user_hash, $changed_by, $db)
        {
            if ($old_value != $new_value) {
                $db->query("
                    INSERT INTO permission_change_log (user_hash, field_name, old_value, new_value, changed_by)
                    VALUES ('{$user_hash}', '{$field}', '{$old_value}', '{$new_value}', '{$changed_by}')
                ");
            }
        }

        // Log changes for emp_information
        log_change('work_in_check_distance', $current_emp_info['work_in_check_distance'], $data['work_in_check_distance'], $data['hash'], $changed_by, $this->db);
        log_change('work_out_check_distance', $current_emp_info['work_out_check_distance'], $data['work_out_check_distance'], $data['hash'], $changed_by, $this->db);
        log_change('advance_agreement', $current_emp_info['advance_agreement'], $data['advance_agreement'], $data['hash'], $changed_by, $this->db);
        log_change('advance_request', $current_emp_info['advance_request'], $data['advance_request'], $data['hash'], $changed_by, $this->db);

        // Update emp_information
        $this->db->query("
            UPDATE emp_information SET
                work_in_check_distance = '{$data['work_in_check_distance']}',
                work_out_check_distance = '{$data['work_out_check_distance']}',
                advance_agreement = '{$data['advance_agreement']}',
                advance_request = '{$data['advance_request']}',
                can_change_flag_to_send_myLocation = '{$data['can_change_flag_to_send_myLocation']}'
            WHERE user_hash = '{$data['hash']}'
        ");

        // Log changes for system_users
        log_change('vication_agreement', $current_sys_users['vication_agreement'], $data['vication_agreement'], $data['hash'], $changed_by, $this->db);
        log_change('task_agreement', $current_sys_users['task_agreement'], $data['task_agreement'], $data['hash'], $changed_by, $this->db);

        // Update system_users
        $this->db->query("
            UPDATE system_users SET
                vication_agreement = '{$data['vication_agreement']}',
                task_agreement = '{$data['task_agreement']}'
            WHERE hash = '{$data['hash']}'
        ");

        return $this->db->affected_rows();
    }

    public function updateEmpInfo($data)
    {

        $allowedKeys = [
            'primary_salary' => 'primary_salary',
            'vacations_limit' => 'vacations_limit',
            'sick_leave' => 'sick_leave',
            'status' => 'status',
            'work_from' => 'work_from',
            'work_to' => 'work_to',
            'allow_time' => 'allow_time',
            'work_in_check_distance' => 'work_in_check_distance',
            'work_out_check_distance' => 'work_out_check_distance',
            'birthday' => 'birthday',
            'address' => 'address',
            'phone1' => 'phone1',
            'phone2' => 'phone2',
            'position' => 'position',
            'annual_percentage_increase' => 'annual_percentage_increase',
            'annual_price_increase' => 'annual_price_increase',
            'start_work_date' => 'start_work_date',
            'jop_type' => 'jop_type',
            'is_work' => 'is_work',
            'salary_type' => 'salary_type',
            'position_other' => 'position_other',
            'IBN' => 'IBN',
            'number_of_year_exp' => 'number_of_year_exp',
            'fingerprint_required' => 'fingerprint_required',
            'work_schedule' => 'work_schedule',
            'job_title' => 'job_title_id',
            // TODO: on future
            'flexible_schedule' => 'flexible_schedule',
            'target_hours' => 'target_hours',
            'stop_work_date' => 'stop_work_date',

        ];

        $userHash = $data['hash'];

        // Fetch current values from the database
        $this->db->select('flexible_schedule, target_hours');
        $this->db->from('emp_information');
        $this->db->where('user_hash', $userHash);
        $query = $this->db->get();
        $currentValues = $query->row_array();

        // Initialize the query parts
        $updateParts = [];
        $params = [];

        // Loop through the allowed keys and build the query dynamically
        foreach ($data as $key => $value) {
            if (isset($allowedKeys[$key])) {
                if ($value === 'null') {
                    $updateParts[] = "{$allowedKeys[$key]} = NULL";
                } else {
                    $updateParts[] = "{$allowedKeys[$key]} = ?";
                    $params[] = $value;
                }
            }
        }

        // Ensure there are parts to update and the user_hash key exists
        if (!empty($updateParts) && isset($data['hash'])) {
            $query = "UPDATE emp_information SET " . implode(', ', $updateParts) . " WHERE user_hash = ?";
            $params[] = $data['hash']; // Add user_hash to the parameters

            // Execute the query
            $this->db->query($query, $params);

            // Handle flexible_schedule updates
            if (isset($data['flexible_schedule']) || isset($data['target_hours'])) {
                $wasFlexibleScheduleSet = isset($currentValues['flexible_schedule']) && $currentValues['flexible_schedule'] != 0;
                $isFlexibleScheduleUpdated = isset($data['flexible_schedule']) && $data['flexible_schedule'] != 0;
                $isFlexibleScheduleRemoved = isset($data['flexible_schedule']) && $data['flexible_schedule'] == 0;

                $wasTargetHoursSet = isset($currentValues['target_hours']) && $currentValues['target_hours'] != 0;
                $isTargetHoursUpdated = isset($data['target_hours']) && $data['target_hours'] != 0;
                $isTargetHoursRemoved = isset($data['target_hours']) && $data['target_hours'] == 0;

                // Determine if we need to insert or remove schedules
                $shouldInsertSchedules = (!$wasFlexibleScheduleSet && $isFlexibleScheduleUpdated) || (!$wasTargetHoursSet && $isTargetHoursUpdated);
                $shouldRemoveSchedules = ($wasFlexibleScheduleSet && $isFlexibleScheduleRemoved) || ($wasTargetHoursSet && $isTargetHoursRemoved);

                if ($shouldRemoveSchedules) {
                    $this->removeDailySchedules($userHash);
                } elseif ($shouldInsertSchedules) {
                    $this->insertDailySchedules($userHash);
                }
            }

            return $this->db->affected_rows();
        }
    }

    private function insertDailySchedules($userHash)
    {
        $formatted_date = date('Y-m-d');

        $sql = $this->db->query("
            SELECT ei.user_hash, '$formatted_date' AS date, ei.target_hours
            FROM emp_information ei
            LEFT JOIN user_vacations uv ON ei.user_hash = uv.user_hash
                                        AND uv.day_of_week_id = DAYOFWEEK('$formatted_date')
            LEFT JOIN vacations v ON ei.user_hash = v.user_hash
                                   AND (
                                       '$formatted_date' BETWEEN v.date_start AND DATE(v.date_end - INTERVAL 1 DAY)
                                       OR (v.date_start = '$formatted_date' AND v.date_end = '$formatted_date')
                                   )
                                   AND v.status = '111'
            LEFT JOIN holiday h ON '$formatted_date' = h.date
            WHERE
                ei.user_hash = '$userHash'
                AND uv.user_hash IS NULL
                AND v.user_hash IS NULL
                AND ei.flexible_schedule = '1'
                AND ei.target_hours IS NOT NULL
                AND h.date IS NULL
                AND NOT EXISTS (
                    SELECT 1
                    FROM actual_schedules
                    WHERE user_hash = ei.user_hash
                    AND date = '$formatted_date'
                )
                AND NOT EXISTS (
                    SELECT 1
                    FROM daily_flexible_schedule_targets
                    WHERE user_hash = ei.user_hash
                    AND date = '$formatted_date'
                )
        ");

        $query = $sql;

        if ($query->num_rows() > 0) {
            $result = $query->row();
            $data = ['user_hash' => $userHash, 'date' => $result->date, 'target_hours' => $result->target_hours];

            // Check if a record with the same user_hash, date, and is_deleted = '1' exists
            $this->db->where('user_hash', $userHash);
            $this->db->where('date', $result->date);
            $this->db->where('is_deleted', '1');
            $existingQuery = $this->db->get('daily_flexible_schedule_targets');

            if ($existingQuery->num_rows() > 0) {
                // Delete the existing record
                $this->db->where('user_hash', $userHash);
                $this->db->where('date', $result->date);
                $this->db->where('is_deleted', '1');
                $this->db->delete('daily_flexible_schedule_targets');
            }

            // Insert the new record
            $this->db->insert('daily_flexible_schedule_targets', $data);
        }
    }

    private function removeDailySchedules($userHash)
    {
        $this->db->where('user_hash', $userHash);
        $this->db->where('date', 'CURDATE()', false); // Use CURDATE() function in SQL
        $this->db->update('daily_flexible_schedule_targets', ['is_deleted' => '1']);
    }

    private function InsertDailyFlexibleTarget($isFlexible, $userHash)
    {
        $this->db->select('user_hash ,  date as CURDATE() ,target_hours')->from('emp_information')->where('user_hash', $userHash);
    }

    public function updateUserVacations($vacationData, $user_hash)
    {
        $decodedData = json_decode($vacationData, true);

        if (is_null($decodedData) || empty($decodedData)) {
            $this->db->where('user_hash', $user_hash);
            $this->db->delete('user_vacations');
        } else if (is_array($decodedData) && count($decodedData) > 0) {
            $this->db->where('user_hash', $user_hash);
            $this->db->delete('user_vacations');
            foreach ($decodedData as $dayOfWeekId) {
                $data = [
                    'user_hash' => $user_hash,
                    'day_of_week_id' => $dayOfWeekId,
                ];
                $this->db->insert('user_vacations', $data);
            }
        } else {
            return null;
        }
        return $this->db->affected_rows();
    }

    public function getUserVacations($hash)
    {
        $this->db->select('day_of_week_id as value , days_of_week.label')->from('user_vacations')->where('user_hash', $hash);
        $this->db->join('days_of_week', 'days_of_week.id = user_vacations.day_of_week_id');
        return $this->db->get()->result_array();
    }

    public function createLocations($data)
    {
        // delete all locations
        $this->db->where('user_hash', $data['hash']);
        $this->db->delete('office_to_user');
        // insert new locations from location_of_work like 16678156717559213,16648815219977578
        $locations = explode(',', $data['location_of_work']);
        foreach ($locations as $location) {
            $this->db->query("INSERT INTO office_to_user (user_hash,office_hash) VALUES ('{$data['hash']}','{$location}')");
        }
        return $this->db->affected_rows();
    }

    public function updateEmp($data)
    {
        $this->db->query("
        UPDATE system_users SET
            name = '{$data['name']}',
            username = '{$data['username']}',
            password = '{$data['password']}',
            user_type = '{$data['user_type']}',

            image_profile = '{$data['image_profile']}',
            employment_code = '{$data['employment_code']}'
        WHERE hash = '{$data['hash']}'
        ");
        return $this->db->affected_rows();
    }

    public function deleteEmployee($hash)
    {
        $this->db->where('user_hash', $hash);
        $this->db->update('emp_information', ['is_work' => '2']);
        return $this->db->affected_rows();
    }

    public function HavePosition($user_hash)
    {
        $query = $this->db->select('position')
            ->from('emp_information')
            ->where('user_hash', $user_hash)
            ->where('position !=', '') // Check for non-empty
            ->where('position IS NOT NULL') // Correct way to check for non-null
            ->get(); // Execute the query

        return $query->num_rows() > 0;
    }

    public function employeesCanAcceptVacations($hash, $user_type)
    {

        // if ($this->HavePosition($hash)) {
        //     $this->db->select("
        //     CASE
        //         WHEN su_emp.hash = su_lead.hash THEN su_parent_lead.name
        //         ELSE COALESCE(su_lead.name, su_parent_lead.name)
        //     END AS name,
        //     CASE
        //         WHEN su_emp.hash = su_lead.hash THEN su_parent_lead.hash
        //         ELSE COALESCE(su_lead.hash, su_parent_lead.hash)
        //     END AS hash
        // ", false);

        //     $this->db->from('emp_information ei');
        //     $this->db->join('Positions p', 'ei.position = p.id');
        //     $this->db->join('system_users su_emp', 'su_emp.hash = ei.user_hash');
        //     $this->db->join('system_users su_lead', 'su_lead.hash = p.lead_hash', 'left');
        //     $this->db->join('Positions parent_p', 'p.ParentID = parent_p.ID', 'left');
        //     $this->db->join('system_users su_parent_lead', 'su_parent_lead.hash = parent_p.lead_hash', 'left');

        //     $this->db->where('ei.is_work', 1);
        //     $this->db->where('su_emp.hash', $hash);

        //     $query = $this->db->get();
        //     $result = $query->result_array();
        //     return $result;
        // }



        // SELECT * FROM system_users inner join emp_information on  system_users.hash=emp_information.user_hash where is_work='1' and vication_agreement='1' and system_users.hash<>'${CacheService.hash}
        $this->db->select('
            name,hash
        ')->from('system_users');
        $this->db->join('emp_information', 'system_users.hash=emp_information.user_hash');
        $this->db->join('UserRoles', 'UserRoles.user_hash = system_users.hash');

        $this->db->where("is_work = 1 and  system_users.hash != '$hash' ");
        // where roles for موارد بشريه
        $this->db->where('UserRoles.role_id = 1');
        // and (user_type = $user_type or partment = '1337866435131594') vication_agreement = 1 and
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getCurrentVacationsOfUser($hash)
    {
        //
        $query = $this->db->query("
            SELECT
                vacations.hash AS 'vication_number',
                vacations.user_hash AS 'user_hash',
                vacations.datetime AS 'create_date',
                (SELECT
                        name
                    FROM
                        system_users
                    WHERE
                        system_users.hash = vacations.user_hash
                    LIMIT 1) AS 'user_name',
                (SELECT
                        type
                    FROM
                        vacations_type
                    WHERE
                        vacations_type.hash = vacations.vication_type_hash) AS 'type',
                (SELECT
                        title
                    FROM
                        vacations_status
                    WHERE
                        vacations_status.hash = vacations_agreement.status) AS 'agreement_status',
                (SELECT
                        title
                    FROM
                        vacations_status
                    WHERE
                        vacations_status.hash = vacations.status) AS 'final_status',
                DATEDIFF(date_end, date_start) AS 'diff_day',
                TIMESTAMPDIFF(HOUR,
                    date_start,
                    date_end) AS 'diff_hours',
                date_start AS 'start',
                date_end AS 'end',
                vacations_agreement.hash AS 'agreement_hash',
                vacations_agreement.note AS 'final_note'
            FROM
                vacations
                    INNER JOIN
                    vacations_agreement ON vacations.hash = vacations_agreement.vacations_hash and vacations_agreement.id = (select max(id) from vacations_agreement where vacations_hash=vacations.hash)
            WHERE
                vacations_agreement.id IN (SELECT
                        MAX(id) AS last_agrement
                    FROM
                        vacations_agreement
                    GROUP BY vacations_agreement.vacations_hash) and user_hash = '{$hash}'
            ORDER BY create_date DESC
        ");
        return $query->result_array();
    }

    public function getHistoryOfVacation($hash, $page = 1, $perPage = 10)
    {

        $offset = ($page - 1) * $perPage;

        $query = $this->db->query("
        SELECT
        distinct vacations.hash AS 'vication_number',
        vacations.user_hash AS 'user_hash',
        vacations.datetime AS 'create_date',
        (SELECT
                name
            FROM
                system_users
            WHERE
                system_users.hash = vacations.user_hash
            LIMIT 1) AS 'user_name',
        (SELECT
                type
            FROM
                vacations_type
            WHERE
                vacations_type.hash = vacations.vication_type_hash) AS 'type',
        (SELECT
                title
            FROM
                vacations_status
            WHERE
                vacations_status.hash = vacations_agreement.status) AS 'agreement_status',
        (SELECT
                title
            FROM
                vacations_status
            WHERE
                vacations_status.hash = vacations.status) AS 'final_status',
        DATEDIFF(date_end, date_start) AS 'diff_day',
        TIMESTAMPDIFF(HOUR,
            date_start,
            date_end) AS 'diff_hours',
        date_start AS 'start',
        date_end AS 'end'
    FROM
        vacations
            INNER JOIN
            vacations_agreement FORCE INDEX (index3,index4,index5) ON vacations.hash = vacations_agreement.vacations_hash and vacations_agreement.id = (select max(id) from vacations_agreement where vacations_hash=vacations.hash)
    WHERE
         vacations_agreement.sender = '$hash'
              AND (vacations_agreement.status != '444'
            )
    ORDER BY vacations.id DESC
LIMIT {$offset}, {$perPage};
");

        return $query->result_array();
    }

    public function receivedVAcationsOfUser($hash)
    {
        $query = $this->db->query("
            SELECT
                vacations.hash AS 'vication_number',
                vacations.user_hash AS 'user_hash',
                vacations.datetime AS 'create_date',
                (SELECT
                        name
                    FROM
                        system_users
                    WHERE
                        system_users.hash = vacations.user_hash
                    LIMIT 1) AS 'user_name',
                (SELECT
                        type
                    FROM
                        vacations_type
                    WHERE
                        vacations_type.hash = vacations.vication_type_hash) AS 'type',
                (SELECT
                        title
                    FROM
                        vacations_status
                    WHERE
                        vacations_status.hash = vacations_agreement.status) AS 'agreement_status',
                (SELECT
                        title
                    FROM
                        vacations_status
                    WHERE
                        vacations_status.hash = vacations.status) AS 'final_status',
                DATEDIFF(date_end, date_start) AS 'diff_day',
                TIMESTAMPDIFF(HOUR,
                    date_start,
                    date_end) AS 'diff_hours',
                date_start AS 'start',
                date_end AS 'end',
                vacations_agreement.hash AS 'agreement_hash',
                vacations.note AS 'main_note',
                vacations_agreement.note AS 'comment_note'
            FROM
                vacations
                    INNER JOIN
                    vacations_agreement ON vacations.hash = vacations_agreement.vacations_hash and vacations_agreement.id = (select max(id) from vacations_agreement where vacations_hash=vacations.hash)
            WHERE
                vacations_agreement.receiver = '{$hash}'
                    AND vacations_agreement.sender <> '{$hash}'
                    AND (vacations.status = '444' OR vacations.status = '333')
                    AND flage = '0'
            ORDER BY vacations.id DESC
        ");
        return $query->result_array();
    }

    public function getEmployeeDetails($hash)
    {
        $today = date('Y-m-d');
        $canCreate = $this->checkCanCreateSimpleTasks($hash);

        $this->db->select('system_users.*, emp_information.*, emp_information.can_change_flag_to_send_myLocation AS ChangeLocationPermission');
        // $this->db->select("(SELECT TIME_FORMAT(ADDTIME('00:00:00', SEC_TO_TIME(3600 * JSON_EXTRACT(schedule_data, '$[0].start'))), '%r') FROM actual_schedules WHERE user_hash =  '$hash' AND date = '$today' LIMIT 1) AS from_work_time");
        // $this->db->select('(SELECT TIME_FORMAT(ADDTIME("00:00:00", SEC_TO_TIME(3600 * JSON_EXTRACT(schedule_data, "$[0].end"))), "%r") FROM actual_schedules WHERE user_hash = "' . $hash . '" AND date = "' . $today . '" LIMIT 1) AS to_work_time');
        $this->db->select("
            CASE
                WHEN EXISTS (
                    SELECT 1
                    FROM daily_flexible_schedule_targets
                    WHERE user_hash = '$hash' AND date = '$today'
                )
                THEN 'ليس هناك وقت محدد'
                ELSE COALESCE(
                    (SELECT TIME_FORMAT(ADDTIME('00:00:00', SEC_TO_TIME(3600 * JSON_EXTRACT(schedule_data, '$[0].start'))), '%r')
                    FROM actual_schedules
                    WHERE user_hash = '$hash' AND date = '$today'
                    LIMIT 1), 'لا توجد خطه')
            END AS from_work_time
        ");

        $this->db->select("
            CASE
                WHEN EXISTS (
                    SELECT 1
                    FROM daily_flexible_schedule_targets
                    WHERE user_hash = '$hash' AND date = '$today'
                )
                THEN 'ليس هناك وقت محدد'
                ELSE COALESCE(
                    (SELECT TIME_FORMAT(ADDTIME('00:00:00', SEC_TO_TIME(3600 * JSON_EXTRACT(schedule_data, '$[0].end'))), '%r')
                    FROM actual_schedules
                    WHERE user_hash = '$hash' AND date = '$today'
                    LIMIT 1), 'لا توجد خطه')
            END AS to_work_time
        ");

        $this->db->select('GROUP_CONCAT(office_location.name) AS location');
        $this->db->select('(SELECT Name FROM Positions WHERE Positions.ID = emp_information.Position) AS department');
        $this->db->select('(SELECT name FROM office_location WHERE office_location.hash = emp_information.location_of_work LIMIT 1) AS place');
        // $this->db->select('(SELECT name FROM UserRoles WHERE system_group_name.hash = system_users.user_type) AS emp_title');
        $this->db->select('(SELECT r.name FROM Roles r JOIN UserRoles ur ON r.id = ur.role_id WHERE ur.user_hash = emp_information.user_hash ORDER BY r.priority ASC LIMIT 1) AS emp_title');
        $this->db->from('system_users');
        $this->db->join('emp_information', 'system_users.hash = emp_information.user_hash', 'left');
        $this->db->join('office_to_user', 'office_to_user.user_hash = system_users.hash', 'left');
        $this->db->join('office_location', 'office_to_user.office_hash = office_location.hash', 'left');
        $this->db->join('system_put_role_to_users', 'system_users.hash = system_put_role_to_users.user_hash', 'left');
        $this->db->where('system_users.hash', $hash);
        $query = $this->db->get();
        // return $query->result_array();

        $data = $query->result_array();
        foreach ($data as &$row) {
            $row['canCreateSimpleTask'] = $canCreate;
        }

        return $data;
    }

    public function checkCanCreateSimpleTasks($senderHash)
    {
        $roleCheckPassed = $this->checkHaveRoleProject($senderHash);
        return $roleCheckPassed;

    }

    private function checkHaveRoleProject($senderHash, $role_id = 2)
    {
        $this->db->select('*');
        $this->db->from('project_employees');
        $this->db->where('employee_id', $senderHash);
        $this->db->where("(role_id = $role_id OR role_id = 1 OR role_id = 4 OR role_id = 3)");
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return true; // Found at least one project with matching role
        }
        return false; // No matching roles found across any projects
    }

    public function sendLocationFlageEmployee($user_hash, $flage)
    {
        $data = array('send_myLocation' => $flage);
        $this->db->where('user_hash', $user_hash);
        $this->db->update('emp_information', $data);
    }

    public function getEmployeeShecduleDetail($user_hash, $date)
    {
        $sql = "    SELECT
    ei.user_hash,
    JSON_UNQUOTE(JSON_EXTRACT(sg.schedule_data, CONCAT('$.', DAYNAME('$date')))) AS schedule_for_day,
    MIN(CASE WHEN ef.in_out = 'in' THEN ef.fingerprint_time END) AS in_fingerprint_time,
    MIN(CASE WHEN ef.in_out = 'out' THEN ef.fingerprint_time END) AS out_fingerprint_time,
    MIN(CASE WHEN ef.in_out = 'in' THEN ef.in_out END) AS in_in_out,
    MIN(CASE WHEN ef.in_out = 'out' THEN ef.in_out END) AS out_in_out,
    MIN(v.id) AS vacation_id,
    MIN(vt.hash) AS vacation_type_hash,
    MIN(v.date_start) AS startDate,
    MIN(v.date_end) AS endDate,
    DATEDIFF(MIN(v.date_end), MIN(v.date_start)) AS vacation_duration_days,
    (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE emp_information.user_hash = ei.user_hash) AS number_of_hours_per_day,
    MIN(vt.type) AS vacation_type,
    CASE
        WHEN MIN(vt.hash) IN ('111', '222', '555', '333', '444') THEN
            CASE
                WHEN MIN(v.date_start) < CONCAT(MIN(DATE(v.date_start)), ' ', ei.work_from) THEN
                    DATEDIFF(MIN(v.date_end), MIN(v.date_start)) *
                        (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE emp_information.user_hash = ei.user_hash)
                WHEN MAX(v.date_end) > CONCAT(MAX(DATE(v.date_start)), ' ', ei.work_to) THEN
                    DATEDIFF(MAX(v.date_end), MIN(v.date_start)) *
                        (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE emp_information.user_hash = ei.user_hash)
                ELSE
                    ROUND(TIMESTAMPDIFF(MINUTE, MIN(v.date_start), MAX(v.date_end)) / 60.0)
            END
        ELSE 0
    END AS number_of_hours_v,
    CASE
        WHEN DATEDIFF(MAX(v.date_end), MIN(v.date_start)) = 0 AND MIN(vt.hash) IN ('111','333','444','222', '555') THEN
            IF(TIMESTAMPDIFF(HOUR, MIN(v.date_start), MAX(v.date_end)) >= (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE emp_information.user_hash = ei.user_hash),
                1,
                ROUND(DATEDIFF(MAX(v.date_end), MIN(v.date_start)) *
                    (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE emp_information.user_hash = ei.user_hash) /
                    TIMESTAMPDIFF(HOUR, work_from, work_to))
            )
        WHEN MIN(vt.hash) IN ('111','333','444','222', '555') THEN
            ROUND(DATEDIFF(MAX(v.date_end), MIN(v.date_start)) *
                (SELECT TIMESTAMPDIFF(HOUR, work_from, work_to) FROM emp_information WHERE emp_information.user_hash = ei.user_hash) /
                TIMESTAMPDIFF(HOUR, work_from, work_to))
        ELSE 0
    END AS number_of_days
FROM
    emp_information ei
JOIN
    schedule_groups sg ON ei.work_schedule = sg.id
LEFT JOIN
    emp_fingerprint ef ON ei.user_hash = ef.user_hash
    AND DATE(ef.fingerprint_time) = '$date'
LEFT JOIN
    vacations v ON v.user_hash = ei.user_hash
    AND v.status = '111'
    AND DATE(v.date_start) = '$date'
LEFT JOIN
    vacations_type vt ON vt.hash = v.vication_type_hash
WHERE
    ei.user_hash = '$user_hash'
GROUP BY
    ei.user_hash
ORDER BY
    in_fingerprint_time;";

        return $this->db->query($sql)->row();
    }

    public function retrieveEmployeeAttendanceData($employeeAttendanceData)
    {
        $month = $this->db->escape($employeeAttendanceData->month);
        $year = $this->db->escape($employeeAttendanceData->year);
        $userHash = $this->db->escape($employeeAttendanceData->userHash);

        $sql = "
        SELECT
            d.user_hash,
            su.name,
            su.image_profile,
            d.date,
            d.type,
            d.vacation_type,
            d.details
        FROM (
            -- Scenario 1: User worked on this day with multiple in and out times
            SELECT
                em_in.user_hash,
                DATE(em_in.fingerprint_time) AS date,
                'worked' AS type,
                NULL AS vacation_type,
                JSON_OBJECT(
                    'fingerprint_in_time', em_in.fingerprint_time,
                    'fingerprint_out_time', em_out.fingerprint_time,
                    'location_in', ST_AsText(em_in.location),
                    'location_out', ST_AsText(em_out.location)
                ) AS details
            FROM emp_fingerprint em_in
            JOIN emp_fingerprint em_out
                ON em_in.user_hash = em_out.user_hash
                AND DATE(em_in.fingerprint_time) = DATE(em_out.fingerprint_time)
                AND em_in.fingerprint_time < em_out.fingerprint_time
                AND em_in.in_out = 'in'
                AND em_out.in_out = 'out'
            JOIN employees_worked_hours ew
                ON em_in.user_hash = ew.user_hash
                AND DATE(em_in.fingerprint_time) = ew.date
            WHERE MONTH(em_in.fingerprint_time) = $month
              AND YEAR(em_in.fingerprint_time) = $year
              AND em_in.user_hash = $userHash

            UNION ALL

         SELECT
            ewh.user_hash,
            ewh.date,
            'absence' AS type,
            NULL AS vacation_type,
            NULL AS details
            FROM employees_worked_hours ewh
            WHERE ewh.total_worked_hours = 0
            AND ewh.user_hash = $userHash
            AND MONTH(ewh.date) = $month
            AND YEAR(ewh.date) = $year

            UNION ALL

            -- Scenario 3: User was on vacation
            SELECT
                v.user_hash,
                v.date_start AS date,
                CASE
                    WHEN vt.hash != '666' THEN 'user_vacation'
                    ELSE 'vacation'
                END AS type,
                vt.type AS vacation_type,
                JSON_OBJECT(
                    'vacation_type', vt.type,
                    'date_start', v.date_start,
                    'date_end', v.date_end
                ) AS details
            FROM vacations v
            LEFT JOIN vacations_type vt ON v.vication_type_hash = vt.hash
            WHERE (YEAR(v.date_start) = $year AND MONTH(v.date_start) = $month)
               OR (YEAR(v.date_end) = $year AND MONTH(v.date_end) = $month)
               OR (YEAR(v.date_start) < $year AND YEAR(v.date_end) > $year)
               AND v.user_hash = $userHash
        ) d
        JOIN system_users su ON d.user_hash = su.hash
        WHERE d.user_hash = $userHash
        ORDER BY d.user_hash, d.date, JSON_UNQUOTE(JSON_EXTRACT(d.details, '$.fingerprint_in_time'))
    ";

        $query = $this->db->query($sql);
        $result = $query->result_array();

        // Decode the 'details' JSON field
        foreach ($result as &$row) {
            if ($row['details']) {
                $row['details'] = json_decode($row['details'], true);
            }
        }

        return $result;
    }

    public function decimalToTime($decimal)
    {
        $hours = floor($decimal);
        $minutes = ($decimal - $hours) * 60;
        return sprintf('%02d:%02d', $hours, $minutes);
    }
    public function insertMissingCalacultions()
    {
        $employees = $this->getEmployeesNotCalculatedEmployeesWorkedHours();

        foreach ($employees as $employee) {
            $employee_data = json_decode($employee['employee_data'], true);
            $calculatedData = $this->calculateLateArrivalAndEarlyDeparture($employee_data);
            if ($calculatedData['status'] == 'success') {
                $this->insertCalculatedData($calculatedData);
            }
        }
    }
    public function getEmployeesWithTaskHash($taskHash)
    {
        // Get the task ID and sender information based on the project hash
        $taskData = $this->db->select('id, sender, project_hash')
            ->from('task')
            ->where('hash', $taskHash)
            ->get()
            ->row_array();

        // Make sure the task ID was found
        if (empty($taskData)) {
            return null; // Or handle the case where the task is not found
        }

        $taskId = $taskData['id'];
        $senderHash = $taskData['sender'];
        $projectHash = $taskData['project_hash'];

        // Get the sender's information using su.hash
        $senderInfo = $this->db->select('su.name, su.image_profile, ei.user_hash, jt.name as job_title, su.mobile_token as token')
            ->from('system_users su')
            ->join('emp_information ei', 'ei.user_hash = su.hash', 'left')
            ->join('job_title jt', 'ei.job_title_id = jt.id', 'left')
            ->where('su.hash', $senderHash)
            ->get()
            ->row_array();

        $projectData = $this->db->select('id')->from('project')->where('hash', $projectHash)->get()->row_array();
        $projectId = $projectData['id'] ?? null;
        $projectEmployees = [];

        if ($projectId) {
            $projectEmployees = $this->db->select('su.name, su.image_profile, ei.user_hash, jt.name as job_title, su.mobile_token as token')
                ->from('project_employees pe')
                ->join('emp_information ei', 'ei.user_hash = pe.employee_id', 'left')
                ->join('system_users su', 'su.hash = ei.user_hash', 'left')
                ->join('job_title jt', 'ei.job_title_id = jt.id', 'left')
                ->where('pe.project_id', $projectId)
                ->get()
                ->result_array();
        }

        // Get the employees for the task
        $taskEmployees = $this->db->select('su.name, su.image_profile, ei.user_hash, jt.name as job_title, su.mobile_token as token')
            ->from('task_users tu')
            ->join('emp_information ei', 'ei.user_hash = tu.user_hash', 'left')
            ->join('system_users su', 'su.hash = ei.user_hash', 'left')
            ->join('job_title jt', 'ei.job_title_id = jt.id', 'left')
            ->where('tu.task_id', $taskId)
            ->get()
            ->result_array();

        // Combine sender with employees
        if ($senderInfo) {
            array_unshift($taskEmployees, $senderInfo); // Add sender as the first "employee" in the array
        }

        $combinedEmployees = array_merge($taskEmployees, $projectEmployees);

        return $combinedEmployees;
    }

    public function getTaskMessages($taskHash, $userHash)
    {
        // Step 1: Retrieve the messages for the given task hash
        $this->db->select('id, user_hash, text, type, created_at, updated_at')
            ->from('messages')
            ->where('task_hash', $taskHash);

        $messages = $this->db->get()->result_array();
        // Step 2: Mark all messages as "seen" for the given user
        $messageIds = array_column($messages, 'id'); // Get all message IDs from the result

        if (!empty($messageIds)) {
            $this->db->where_in('message_id', $messageIds)
                ->where('user_hash', $userHash)
                ->where('seen_at IS NULL') // Only update unseen messages
                ->update('message_reads', ['seen_at' => date('Y-m-d H:i:s')]);
        }

        return $messages;
    }

    private function getEmployeesNotCalculatedEmployeesWorkeidHours()
    {
        $sql = "SELECT
    JSON_OBJECT(
        'user_hash', ei.user_hash,
        'schedule_for_day', asch.schedule_data,
        'allowance_time', asch.allowance_time,
        'in_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE WHEN ef.in_out = 'in' THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"') END ORDER BY ef.fingerprint_time SEPARATOR ','), ']'),
        'out_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE WHEN ef.in_out = 'out' THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"') END ORDER BY ef.fingerprint_time DESC SEPARATOR ','), ']'),
        'date', asch.date
    ) AS employee_data
FROM
    emp_information ei
JOIN
    actual_schedules asch ON ei.user_hash = asch.user_hash
    AND asch.date < CURDATE()
LEFT JOIN
    emp_fingerprint ef ON asch.user_hash = ef.user_hash
LEFT JOIN
    employees_worked_hours ewh ON ei.user_hash = ewh.user_hash
    AND asch.date = ewh.date
WHERE
    asch.schedule_data IS NOT NULL
    AND TRIM(asch.schedule_data) <> ''
    AND ewh.user_hash IS NULL
    AND asch.date < CURDATE()
    AND JSON_LENGTH(asch.schedule_data) > 0
GROUP BY
    ei.user_hash, asch.date;
";
        return $this->db->query($sql)->result_array();
    }

    private function calculateLateArrivalAndEarlyDeparture($data)
    {
        $totalLateArrivalMinutes = 0;
        $totalEarlyDepartureMinutes = 0;
        $totalScheduledMinutes = 0;
        $totalWorkedMinutes = 0;
        $totalTargetMinutes = 0;

        // Check if schedules are provided
        if (!isset($data['schedule_for_day']) || !is_array($data['schedule_for_day']) || empty($data['schedule_for_day'])) {
            return [
                'error' => 'Missing or invalid schedule_for_day data.',
                'status' => 'error',
            ];
        }

        $allowanceTime = isset($data['allowance_time']) ? (int) $data['allowance_time'] : null;

        foreach ($data['schedule_for_day'] as $index => $schedule) {
            $startWork = $schedule['start'];
            $endWork = $schedule['end'];
            // $allowanceTime = isset($schedule['allowance_time']) ? (int) $schedule['allowance_time'] : null;

            // Calculate scheduled work duration
            $startWorkFormatted = $this->decimalToTime($startWork);
            $endWorkFormatted = $this->decimalToTime($endWork);
            $scheduledStartTime = new DateTime($data['date'] . ' ' . $startWorkFormatted);
            $scheduledEndTime = new DateTime($data['date'] . ' ' . $endWorkFormatted);
            $scheduledDuration = $scheduledStartTime->diff($scheduledEndTime);
            $scheduledMinutes = $scheduledDuration->h * 60 + $scheduledDuration->i;

            $totalScheduledMinutes += $scheduledMinutes;
            $totalTargetMinutes += $scheduledMinutes;

            // Check if in_fingerprint_time exists for this schedule index
            if (isset($data['in_fingerprint_time'])) {
                $inFingerprintTimes = json_decode($data['in_fingerprint_time'], true);
                if (isset($inFingerprintTimes[$index])) {
                    $inFingerprintTime = new DateTime($inFingerprintTimes[$index]);

                    // Calculate late arrival
                    if ($inFingerprintTime > $scheduledStartTime) {
                        $lateArrival = $scheduledStartTime->diff($inFingerprintTime);
                        $lateArrivalMinutes = $lateArrival->h * 60 + $lateArrival->i;
                        if (is_numeric($allowanceTime)) {
                            if ($lateArrivalMinutes <= $allowanceTime) {
                                $lateArrivalMinutes = 0;
                            }
                        }
                        $totalLateArrivalMinutes += $lateArrivalMinutes;
                    }
                }
            }

            // Check if out_fingerprint_time exists for this schedule index
            if (isset($data['out_fingerprint_time'])) {
                $outFingerprintTimes = json_decode($data['out_fingerprint_time'], true);
                if (isset($outFingerprintTimes[$index])) {
                    $outFingerprintTime = new DateTime($outFingerprintTimes[$index]);

                    // Calculate early departure
                    if ($outFingerprintTime < $scheduledEndTime) {
                        $earlyDeparture = $outFingerprintTime->diff($scheduledEndTime);
                        $earlyDepartureMinutes = $earlyDeparture->h * 60 + $earlyDeparture->i;
                        $totalEarlyDepartureMinutes += $earlyDepartureMinutes;
                    }
                }
            }

            // Check if both in_fingerprint_time and out_fingerprint_time exist for this schedule index
            if (isset($data['in_fingerprint_time'], $data['out_fingerprint_time'])) {
                $inFingerprintTimes = json_decode($data['in_fingerprint_time'], true);
                $outFingerprintTimes = json_decode($data['out_fingerprint_time'], true);
                if (isset($inFingerprintTimes[$index], $outFingerprintTimes[$index])) {
                    $inFingerprintTime = new DateTime($inFingerprintTimes[$index]);
                    $outFingerprintTime = new DateTime($outFingerprintTimes[$index]);

                    // Calculate worked duration
                    $workedDuration = $inFingerprintTime->diff($outFingerprintTime);
                    $workedMinutes = $workedDuration->h * 60 + $workedDuration->i;
                    if ($workedMinutes >= 15) {
                        $totalWorkedMinutes += $workedMinutes;
                    }
                    // $totalWorkedMinutes += $workedMinutes;
                }
            }
        }

        $totalMissedMinutes = max(round($totalScheduledMinutes - $totalWorkedMinutes), 0);
        $overtimeMinutes = $totalWorkedMinutes - $totalScheduledMinutes;
        $overtimeHours = max(round($overtimeMinutes / 60, 2), 0);
        $targetHours = round($totalTargetMinutes / 60, 2);
        $workedHours = round($totalWorkedMinutes / 60, 2);

        return [
            'totalLateArrivalMinutes' => $totalLateArrivalMinutes,
            'totalEarlyDepartureMinutes' => $totalEarlyDepartureMinutes,
            'totalMissedHours' => round($totalMissedMinutes / 60, 2),
            'totalWorkedHours' => $workedHours,
            'targetHours' => $targetHours,
            'overtimeHours' => $overtimeHours,
            'date' => $data['date'],
            'status' => "success",
            'user_hash' => $data['user_hash'],
        ];
    }

    private function insertCalculatedData($data)
    {
        $insert_data = array(
            'total_late_arrival_minutes' => $data['totalLateArrivalMinutes'],
            'total_early_departure_minutes' => $data['totalEarlyDepartureMinutes'],
            'total_missed_hours' => $data['totalMissedHours'],
            'total_worked_hours' => $data['totalWorkedHours'],
            'target_hours' => $data['targetHours'],
            'overtime_hours' => $data['overtimeHours'],
            'user_hash' => $data['user_hash'],
            'date' => $data['date'],
        );
        $this->db->insert('employees_worked_hours', $insert_data);
    }
}
