<?php
defined('BASEPATH') or exit('No direct script access allowed');

class EmployeesWorkedHoursModel extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }
    public function getMainPenalty()
    {
        $query = $this->db->get_where('emp_worked_hours_flags', ['is_main' => 1]);
        if ($query->num_rows() > 0) {
            $row = $query->row();
            return $row->penalty_multiplier;
        }
    }

    public function getAllowanceTimeFlag()
    {
        $query = $this->db->get_where('emp_worked_hours_flags', ['is_main' => 1]);
        if ($query->num_rows() > 0) {
            $row = $query->row();
            return $row->calculate_full_late == 1 ? true : false;
        }
        return false;
    }

    public function getAllowanceOvertime()
    {
        $query = $this->db->get_where('emp_worked_hours_flags', ['is_main' => 1]);
        if ($query->num_rows() > 0) {
            $row = $query->row();
            return $row->over_time_allowance;
        }
        return 0;
    }

    public function getIsPenaltyBasedOnMissingFlag()
    {
        $query = $this->db->get_where('emp_worked_hours_flags', ['is_main' => 1]);
        if ($query->num_rows() > 0) {
            $row = $query->row();
            return $row->calc_by_missing == 1 ? true : false;
        }
        return false;
    }

    public function getConfigDataById($configId)
    {
        $query = $this->db->get_where('emp_worked_hours_flags', ['id' => $configId]);

        if ($query->num_rows() > 0) {
            return $query->row_array();
        } else {
            return null;
        }
    }

    public function insertCalculatedData($data)
    {
        $inserted_data = array(
            'user_hash' => $data['user_hash'],
            'date' => $data['date'],
            'total_late_arrival_minutes' => $data['totalLateArrivalMinutes'],
            'total_early_departure_minutes' => $data['totalEarlyDepartureMinutes'],
            'total_missed_hours' => $data['totalMissedHours'],
            'total_worked_hours' => $data['totalWorkedHours'],
            'target_hours' => $data['targetHours'],
            'overtime_hours' => $data['overtimeHours'],
            'is_flexible' => $data['is_flexible'] ?? '0',
            'updated_at' => date('Y-m-d H:i:s'),
            'details' => $data['details']
        );

        // Perform the database operation with the filtered data
        $this->db->replace('employees_worked_hours', $inserted_data);

    }

    public function getEmployeeData($date_start, $date_end, $user_hash)
    {
        $query = "
            SELECT
                JSON_OBJECT(
                    'user_hash', ei.user_hash,
                    'schedule_for_day', asch.schedule_data,
                    'allowance_time', asch.allowance_time,
                    'in_fingerprint_time', CONCAT(
                        '[',
                        GROUP_CONCAT(
                            CASE
                                WHEN ef.in_out = 'in' AND DATE(ef.fingerprint_time) = asch.date
                                THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"')
                            END
                            ORDER BY ef.fingerprint_time ASC
                            SEPARATOR ','
                        ),
                        ']'
                    ),
                    'out_fingerprint_time', CONCAT(
                        '[',
                        GROUP_CONCAT(
                            CASE
                                WHEN ef.in_out = 'out' AND ef.in_out_reference IS NOT NULL AND DATE(ef.fingerprint_time) = asch.date
                                THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"')
                            END
                            ORDER BY ef.fingerprint_time ASC
                            SEPARATOR ','
                        ),
                        ']'
                    ),
                        'in_fingerprint_time_schedule_id', CONCAT(
                        '[',
                        GROUP_CONCAT(
                            CASE
                                WHEN ef.in_out = 'in' AND DATE(ef.fingerprint_time) = asch.date
                                THEN CONCAT('\"', ef.schedule_id, '\"')
                            END
                            ORDER BY ef.fingerprint_time ASC
                            SEPARATOR ','
                        ),
                        ']'
                    ),
                    'out_fingerprint_time_schedule_id', CONCAT(
                        '[',
                        GROUP_CONCAT(
                            CASE
                                WHEN ef.in_out = 'out' AND ef.in_out_reference IS NOT NULL AND DATE(ef.fingerprint_time) = asch.date
                                THEN CONCAT('\"', ef.schedule_id, '\"')
                            END
                            ORDER BY ef.fingerprint_time ASC
                            SEPARATOR ','
                        ),
                        ']'
                    ),
                    'date', asch.date
                ) AS employee_data
            FROM
                emp_information ei
            JOIN
                actual_schedules asch ON ei.user_hash = asch.user_hash
            LEFT JOIN
                emp_fingerprint ef ON ei.user_hash = ef.user_hash AND DATE(ef.fingerprint_time) = asch.date
            WHERE
                asch.date >= '$date_start'
                AND asch.date <= '$date_end'
                AND ei.user_hash = '$user_hash'
            GROUP BY
                ei.user_hash, asch.date
        ";

        $result = $this->db->query($query);

        $results = [];
        foreach ($result->result_array() as $row) {
            $results[] = json_decode($row['employee_data'], true);
        }

        return $results;
    }

    // for flexible

    public function getFlexibleWorkedHours($user_hash, $date)
    {
        $this->db->select('
    emp_fingerprint_in.id AS in_id,
    emp_fingerprint_in.user_hash,
    DATE(emp_fingerprint_in.fingerprint_time) AS date_fingerprint,

    emp_fingerprint_in.fingerprint_time AS in_fingerprint_time,
    emp_fingerprint_out.fingerprint_time AS out_fingerprint_time,

    IFNULL(TIMESTAMPDIFF(HOUR, emp_fingerprint_in.fingerprint_time, emp_fingerprint_out.fingerprint_time), 0) AS worked_hours,
    IF(emp_fingerprint_out.fingerprint_time IS NULL, 0, TIMESTAMPDIFF(HOUR, emp_fingerprint_in.fingerprint_time, emp_fingerprint_out.fingerprint_time)) AS missed_hours,

    dft.target_hours
    ');

        $this->db->from('emp_fingerprint emp_fingerprint_in');
        $this->db->join('emp_fingerprint emp_fingerprint_out', 'emp_fingerprint_out.in_out_reference = emp_fingerprint_in.id AND emp_fingerprint_out.in_out = "out"', 'left');
        $this->db->join('daily_flexible_schedule_targets dft', 'dft.user_hash = emp_fingerprint_in.user_hash AND dft.date = DATE(emp_fingerprint_in.fingerprint_time)', 'left');

        $this->db->where('emp_fingerprint_in.in_out', 'in');
        $this->db->where('emp_fingerprint_in.is_flexible', '1');
        $this->db->where('DATE(emp_fingerprint_in.fingerprint_time)', $date);
        $this->db->where('emp_fingerprint_in.user_hash', $user_hash);

        $this->db->order_by('emp_fingerprint_in.fingerprint_time', 'DESC');

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            $result = $query->result();
            $calculatedData = $this->calculateWorkedHours($result);
            return $calculatedData;
        } else {
            return 0; // No records found
        }
    }
    public function getWorkedHoursAndInsert($user_hash, $date)
    {
        $this->db->select('
            emp_fingerprint_out.id AS out_id,
            emp_fingerprint_out.user_hash,
            DATE(emp_fingerprint_out.fingerprint_time) AS date_fingerprint,

            emp_fingerprint_in.fingerprint_time AS in_fingerprint_time,
            emp_fingerprint_out.fingerprint_time AS out_fingerprint_time,
            dft.target_hours
        ');

        $this->db->from('emp_fingerprint emp_fingerprint_out');
        $this->db->join('emp_fingerprint emp_fingerprint_in', 'emp_fingerprint_out.in_out_reference = emp_fingerprint_in.id', 'left');
        $this->db->join('daily_flexible_schedule_targets dft', 'dft.user_hash = emp_fingerprint_out.user_hash AND dft.date = DATE(emp_fingerprint_in.fingerprint_time)', 'left');

        $this->db->where('emp_fingerprint_out.in_out', 'out');
        $this->db->where('emp_fingerprint_in.in_out', 'in');
        $this->db->where('emp_fingerprint_in.is_flexible', '1');
        $this->db->where('DATE(emp_fingerprint_in.fingerprint_time)', $date);
        $this->db->where('emp_fingerprint_out.is_flexible', '1');
        $this->db->where('emp_fingerprint_out.user_hash', $user_hash);

        $this->db->order_by('emp_fingerprint_out.fingerprint_time', 'DESC');

        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            $result = $query->result();
            $calculatedData = $this->calculateWorkedHours($result);
            $insertFlexibleWorkedHoursData = array_merge($calculatedData, ['date' => $date, 'user_hash' => $user_hash]);
            $this->insertFlexibleWorkedHours($insertFlexibleWorkedHoursData);
            return ['status' => 'ok'];
        } else {
            return 0; // No records found
        }
    }

    public function getWorkedHours($user_hash, $date)
    {
        $this->db->select('total_worked_hours')->from('employees_worked_hours')->where('user_hash', $user_hash)->where('date', $date);

        return $this->db->get()->row_array();
    }
    private function calculateWorkedHours($records)
    {
        $worked_hours = 0;
        $target_hours = 0;

        foreach ($records as $record) {

            $target_hours = $record->target_hours;
            $in_time = strtotime($record->in_fingerprint_time);
            $out_time = strtotime($record->out_fingerprint_time);
            if ($in_time && $out_time) {
                $worked_hours += ($out_time - $in_time) / 3600; // Convert seconds to hours
            }

        }

        // Calculate missed and overtime hours
        $missed_hours = max(0, $target_hours - $worked_hours);
        $overTimeDuration = max(0, $worked_hours - $target_hours);
        $overTimeHours = 0;
        if ($overTimeDuration >= $this->getAllowanceOvertime() ?? 0) {
            $overTimeHours = $overTimeDuration;
        }
        // Round all values to 2 decimal places
        return [
            'worked_hours' => round($worked_hours, 2),
            'missed_hours' => round($missed_hours, 2),
            'overTimeHours' => round($overTimeHours, 2),
            'target_hours' => round($target_hours, 2),
        ];
    }

    private function insertFlexibleWorkedHours($data)
    {
        $data = array(
            'user_hash' => $data['user_hash'],
            'totalLateArrivalMinutes' => 0,
            'totalEarlyDepartureMinutes' => 0,
            'totalMissedHours' => $data['missed_hours'],
            'totalWorkedHours' => $data['worked_hours'],
            'targetHours' => $data['target_hours'],
            'overtimeHours' => $data['overTimeHours'],
            'date' => $data['date'],
            'is_flexible' => 1,

        );

        $this->insertCalculatedData($data);

    }

    public function calculateLateArrivalAndEarlyDeparture($data, $configData = null)
    {
        $totalLateArrivalMinutes = 0;
        $totalEarlyDepartureMinutes = 0;
        $totalScheduledMinutes = 0;
        $totalWorkedMinutes = 0;
        $totalOvertimeMinutes = 0;
        if ($configData == null) {
            $calc_full_late = $this->getAllowanceTimeFlag();
            $is_calc_by_missing = $this->getIsPenaltyBasedOnMissingFlag();
            $over_time_allowance = $this->getAllowanceOvertime() ?? 0;
        } else {
            $calc_full_late = $configData['calculate_full_late'];
            $is_calc_by_missing = $configData['calc_by_missing'];
            $over_time_allowance = $configData['over_time_allowance'] ?? 0;
        }
        $over_time_allowance = ($over_time_allowance ?? 0) * 60;

        if (!isset($data['schedule_for_day']) || !is_array($data['schedule_for_day']) || empty($data['schedule_for_day'])) {
            return [
                'error' => 'Missing or invalid schedule_for_day data.',
                'status' => 'error',
            ];
        }

        $allowanceTime = isset($data['allowance_time']) ? (int) $data['allowance_time'] : null;

        $inFingerprintTimes = isset($data['in_fingerprint_time']) ? json_decode($data['in_fingerprint_time'], true) : [];
        $outFingerprintTimes = isset($data['out_fingerprint_time']) ? json_decode($data['out_fingerprint_time'], true) : [];
        $ins = isset($data['in_fingerprint_time']) ? json_decode($data['in_fingerprint_time'], true) : [];
        $outs = isset($data['out_fingerprint_time']) ? json_decode($data['out_fingerprint_time'], true) : [];

        $in_fingerprint_time_schedule_id = isset($data['in_fingerprint_time_schedule_id']) ? json_decode($data['in_fingerprint_time_schedule_id'], true) : [];
        $out_fingerprint_time_schedule_id = isset($data['out_fingerprint_time_schedule_id']) ? json_decode($data['out_fingerprint_time_schedule_id'], true) : [];

        sort($inFingerprintTimes);
        sort($outFingerprintTimes);

        sort($in_fingerprint_time_schedule_id);
        sort($out_fingerprint_time_schedule_id);

        // Combine in_fingerprint_time with schedule_id
        $inFingerprintWithSchedule = [];
        foreach ($inFingerprintTimes as $index => $time) {
            $inFingerprintWithSchedule[] = [
                'time' => $time,
                'schedule_id' => $in_fingerprint_time_schedule_id[$index] ?? null // Use null if schedule_id is missing
            ];
        }

        // Combine out_fingerprint_time with schedule_id
        $outFingerprintWithSchedule = [];
        foreach ($outFingerprintTimes as $index => $time) {
            $outFingerprintWithSchedule[] = [
                'time' => $time,
                'schedule_id' => $out_fingerprint_time_schedule_id[$index] ?? null // Use null if schedule_id is missing
            ];
        }

        // Format the final object
        $formattedObject = [
            'in_fingerprint_time' => $inFingerprintWithSchedule,
            'out_fingerprint_time' => $outFingerprintWithSchedule
        ];




        $firstScheduleStartTime = null;
        $lastScheduleEndTime = null;
        // Get the first in_fingerprint_time and last out_fingerprint_time
        $firstScheduleStartTime = new DateTime($data['date'] . ' ' . $this->decimalToTime($data['schedule_for_day'][0]['start']));
        $lastScheduleEndTime = new DateTime($data['date'] . ' ' . $this->decimalToTime(end($data['schedule_for_day'])['end']));

        // Create DateTime objects using $data['date'] and the formatted times
        // $firstScheduleStartTime = new DateTime($data['date'] . ' ' . $firstInTime);
        // $lastScheduleEndTime = new DateTime($data['date'] . ' ' . $lastOutTime);


        $details = [];

        foreach ($data['schedule_for_day'] as $schedule) {
            $scheduleId = $schedule['id'];
            $startWork = $schedule['start'];
            $endWork = $schedule['end'];
            $earlyDepartureMinutes = 0;
            $lateArrivalMinutes = 0;


            $startWorkFormatted = $this->decimalToTime($startWork);
            $endWorkFormatted = $this->decimalToTime($endWork);
            $scheduledStartTime = new DateTime($data['date'] . ' ' . $startWorkFormatted);
            $scheduledEndTime = new DateTime($data['date'] . ' ' . $endWorkFormatted);


            $scheduledDuration = $scheduledStartTime->diff($scheduledEndTime);
            $scheduledMinutes = $scheduledDuration->h * 60 + $scheduledDuration->i;
            $totalScheduledMinutes += $scheduledMinutes;

            $closestInTime = null;
            $closestOutTime = null;
            $inTime = null;
            $outTime = null;
            // Search in_fingerprint_time for the current schedule_id
            foreach ($formattedObject['in_fingerprint_time'] as $inEntry) {


                if ($inEntry['schedule_id'] == $scheduleId) {
                    $inTime = (new DateTime($inEntry['time']))->format('H:i:s');
                    $closestInTime = new DateTime($data['date'] . ' ' . $inTime);
                    break;
                }
            }

            // Search out_fingerprint_time for the current schedule_id
            foreach ($formattedObject['out_fingerprint_time'] as $outEntry) {
                if ($outEntry['schedule_id'] == $scheduleId) {
                    $outTime = (new DateTime($outEntry['time']))->format('H:i:s');
                    $closestOutTime = new DateTime($data['date'] . ' ' . $outTime);
                    break;
                }
            }

            // this means absence
            if ($closestInTime === null && $closestOutTime === null) {
                $penaltyDuration = $endWork - $startWork;
                $shouldInAt = $this->decimalToTime($startWork);
                $shouldOutAt = $this->decimalToTime($endWork);
                $details[] = [
                    'id' => $scheduleId,
                    'shouldInAt' => $shouldInAt,
                    'shouldOutAt' => $shouldOutAt,
                    'actual' => null,
                    'penalty_duration_with_hours' => $penaltyDuration,
                    'allowance_with_minutes' => $data['allowance_time'],
                    'status' => 'absent',
                ];
                continue;
            }

            if ($closestInTime !== null && $closestInTime > $scheduledStartTime) {
                $lateArrival = $closestInTime->diff($scheduledStartTime);
                $lateArrivalMinutes = $lateArrival->h * 60 + $lateArrival->i;
                if (is_numeric($allowanceTime)) {
                    if ($lateArrivalMinutes <= $allowanceTime) {
                        $lateArrivalMinutes = 0;
                    } else if (!$calc_full_late) {
                        $lateArrivalMinutes -= $allowanceTime;
                    }
                }
                $totalLateArrivalMinutes += $lateArrivalMinutes;
            }

            // Calculate early departure
            if ($closestOutTime !== null && $closestOutTime < $scheduledEndTime) {
                $earlyDeparture = $scheduledEndTime->diff($closestOutTime);
                $earlyDepartureMinutes = $earlyDeparture->h * 60 + $earlyDeparture->i;
                $totalEarlyDepartureMinutes += $earlyDepartureMinutes;
            }

            // Calculate actual worked time within this scheduled time
            if ($closestInTime !== null && $closestOutTime !== null) {
                if ($is_calc_by_missing) {
                    // Count all worked hours, ignoring the scheduled times
                    $workedDuration = $closestInTime->diff($closestOutTime);
                    $workedMinutes = $workedDuration->h * 60 + $workedDuration->i;
                    $totalWorkedMinutes += $workedMinutes;
                } else {
                    // Ensure that the actual worked time is within the scheduled time
                    $actualStartTime = max($closestInTime, $scheduledStartTime);
                    $actualEndTime = min($closestOutTime, $scheduledEndTime);

                    // Calculate worked time if there is a valid period
                    if ($actualStartTime < $actualEndTime) {
                        $workedDuration = $actualStartTime->diff($actualEndTime);
                        $workedMinutes = $workedDuration->h * 60 + $workedDuration->i;
                        $totalWorkedMinutes += $workedMinutes;
                    }
                }
            }

            $shouldInAt = $this->decimalToTime($startWork);
            $shouldOutAt = $this->decimalToTime($endWork);
            $details[] = [
                'id' => $scheduleId,
                'shouldInAt' => $shouldInAt,
                'shouldOutAt' => $shouldOutAt,
                'actualIn' => $inTime,
                'actualOut' => $outTime,
                'penatlyLateInMinutes' => $lateArrivalMinutes,
                'penatlyDepartureOutMinutes' => $earlyDepartureMinutes,
                'penatlyLateInHours' => $lateArrivalMinutes / 60,
                'penatlyDepartureOutHours' => $earlyDepartureMinutes / 60,
                'allowanceWithMinutes' => $data['allowance_time'],
                'status' => 'attended',
            ];
        }

        // Calculate overtime outside scheduled hours
        if (!empty($ins)) {
            $earliestInTime = new DateTime(min($ins));
            if ($earliestInTime < $firstScheduleStartTime) {
                $earlyOvertimeDuration = $earliestInTime->diff($firstScheduleStartTime);
                $earlyOvertimeMinutes = $earlyOvertimeDuration->h * 60 + $earlyOvertimeDuration->i;

                // Apply the overtime allowance
                if ($earlyOvertimeMinutes > $over_time_allowance) {
                    // Only add the minutes that exceed the allowance
                    $totalOvertimeMinutes += $earlyOvertimeMinutes;
                }
            }
        }

        // Late overtime outside scheduled hours (after the schedule ends)
        if (!empty($outs)) {
            $latestOutTime = new DateTime(max($outs));
            if ($latestOutTime > $lastScheduleEndTime) {
                $lateOvertimeDuration = $latestOutTime->diff($lastScheduleEndTime);
                $lateOvertimeMinutes = $lateOvertimeDuration->h * 60 + $lateOvertimeDuration->i;

                // Apply the overtime allowance
                if ($lateOvertimeMinutes > $over_time_allowance) {
                    $totalOvertimeMinutes += $lateOvertimeMinutes;
                }
            }
        }

        $totalMissedMinutes = max($totalScheduledMinutes - $totalWorkedMinutes, 0);
        $overtimeHours = round($totalOvertimeMinutes / 60, 2);

        if ($is_calc_by_missing) {
            $overtimeHours = round(($totalWorkedMinutes / 60) - $totalScheduledMinutes / 60, 2);
            if ($overtimeHours < 0) {
                $overtimeHours = 0;
            }
        } else {
            $overtimeHours = round($totalOvertimeMinutes / 60, 2);
        }

        $targetHours = round($totalScheduledMinutes / 60, 2);
        $workedHours = round($totalWorkedMinutes / 60, 2);

        return [
            'totalLateArrivalMinutes' => $totalLateArrivalMinutes,
            'totalEarlyDepartureMinutes' => $totalEarlyDepartureMinutes,
            'totalMissedHours' => round($totalMissedMinutes / 60, 2),
            'totalWorkedHours' => $workedHours,
            'targetHours' => $targetHours,
            'overtimeHours' => $overtimeHours,
            'date' => $data['date'],
            'status' => "success",
            'details' => json_encode($details)
        ];
    }

    public function decimalToTime($decimal)
    {
        $hours = floor($decimal);
        $minutes = ($decimal - $hours) * 60;
        return sprintf('%02d:%02d', $hours, $minutes);
    }

    public function processEmployeesData($date_start, $date_end, $user_hashes_array, $configId = null)
    {
        foreach ($user_hashes_array as $user_hash) {
            $results = $this->getEmployeeData($date_start, $date_end, $user_hash);
            if ($configId) {
                $configData = $this->getConfigDataById($configId);
                foreach ($results as &$result) {
                    $calculation = $this->calculateLateArrivalAndEarlyDeparture($result, $configData);
                    $result = array_merge($result, $calculation);
                    $this->insertCalculatedData($result);
                }
            } else {
                foreach ($results as &$result) {
                    $calculation = $this->calculateLateArrivalAndEarlyDeparture($result);
                    $result = array_merge($result, $calculation);
                    $this->insertCalculatedData($result);
                }
            }
        }

        return ['status' => 'success'];
    }

    public function processEmployeeData($date_start, $date_end, $user_hash, $configId = null)
    {
        // get schedule on this date and , fingerprints too on this day on object
        $results = $this->getEmployeeData($date_start, $date_end, $user_hash);

        if ($configId) {
            $configData = $this->getConfigDataById($configId);
            foreach ($results as &$result) {
                $calculation = $this->calculateLateArrivalAndEarlyDeparture($result, $configData);
                $result = array_merge($result, $calculation);
                $this->insertCalculatedData($result);
            }
        } else {
            foreach ($results as &$result) {
                $calculation = $this->calculateLateArrivalAndEarlyDeparture($result);
                $result = array_merge($result, $calculation);
                $this->insertCalculatedData($result);
            }
        }

        return ['status' => 'success'];
    }

    public function processFlexbileEmployeeData($date_start, $date_end, $user_hash, $configId = null)
    {
        $results = $this->getEmployeeData($date_start, $date_end, $user_hash);
        if ($configId) {
            $configData = $this->getConfigDataById($configId);
            foreach ($results as &$result) {
                $calculation = $this->calculateLateArrivalAndEarlyDeparture($result, $configData);
                $result = array_merge($result, $calculation);
                $this->insertCalculatedData($result);
            }
        } else {
            foreach ($results as &$result) {
                $calculation = $this->calculateLateArrivalAndEarlyDeparture($result);
                $result = array_merge($result, $calculation);
                $this->insertCalculatedData($result);
            }
        }

        return ['status' => 'success'];
    }

    public function getAllEmployeeUserHashes()
    {
        $this->db->select('user_hash');
        // $this->db->where('is_work', '1');

        $query = $this->db->get('emp_information');
        if ($query->num_rows() > 0) {
            return $query->result_array();
        } else {
            return [];
        }
    }

    // shcedule
    public function handleForgotPrintOut($user_hash, $date)
    {
    }
}
