<?php

class FinancialDiscountsModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }
    public function count_records()
    {
        $search = $this->input->get('q') ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('financial_discounts');
        if ($search) {
            $this->db->like('name', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }
    public function getFinancialDiscounts()
    {
        $page = $this->input->get('page') ?? '1';

        $perPage = $this->input->get('perPage') ?? '10';

        $search = $this->input->get('q') ?? '';

        $offset = ($page - 1) * $perPage;


        $this->db->select('*');
        $this->db->from('financial_discounts');

        if ($search) {
            $this->db->like('name', $search);
        }
        $this->db->order_by('id', 'ASC');
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }


    // public function getUserSpecifications($userId)
    // {
    //     $this->db->select('specifications.name AS label, specifications.id AS value');
    //     $this->db->from('user_specifications');
    //     $this->db->join('specifications', 'user_specifications.id = specifications.id');
    //     $this->db->where('user_specifications.emp_info_id', $userId);
    //     $query = $this->db->get();

    //     if ($query->num_rows() > 0) {
    //         $result = $query->result();
    //         $jsonArray = array();
    //         foreach ($result as $specification) {
    //             $jsonArray[] = array(
    //                 'label' => $specification->label,
    //                 'value' => $specification->value
    //             );
    //         }
    //         return json_encode($jsonArray);
    //     } else {
    //         return json_encode(array()); // Return an empty JSON array if no results are found
    //     }
    // }

    public function insertUserFinancialDiscount($data)
    {
        $this->db->insert('user_financial_discounts', $data);
        return true;
    }

    public function deleteUserFinancialDiscount($user_id, $financial_discounts_id)
    {
        $this->db->where('emp_info_id', $user_id);
        $this->db->where('financial_discounts_id', $financial_discounts_id);
        $this->db->delete('user_financial_discounts');
    }

    public function getFinancialDiscount($id)
    {
        $this->db->select('*');
        $this->db->from('financial_discounts');
        $this->db->where('id', $id);
        $query = $this->db->get();
        return $query->row();
    }

    public function getCurrentUserFinancialDiscounts($user_id)
    {
        $this->db->select("financial_discounts_id");
        $this->db->from("user_financial_discounts");
        $this->db->where('emp_info_id', $user_id);
        $query = $this->db->get();
        if ($query->num_rows() > 0) {
            $result = $query->result_array();
            $relations = array_column($result, 'financial_discounts_id');
            return $relations;
        } else {
            return [];
        }

    }

    public function getCurrentUserFinancialDiscountsSelect($user_id)
    {
        $this->db->select("user_financial_discounts.financial_discounts_id, financial_discounts.id AS value, financial_discounts.name AS label");
        $this->db->from("user_financial_discounts");
        $this->db->join("financial_discounts", "user_financial_discounts.financial_discounts_id = financial_discounts.id");
        $this->db->where('emp_info_id', $user_id);

        return $this->db->get()->result_array();
    }

    public function insertFinancialDiscounts($data)
    {
        if (empty($data)) {
            return array(
                'status' => 500,
                'message' => 'فشل في إدراج المقطوعه',
                'action' => 'false'
            );
        }

        $validData = array_intersect_key($data, array_flip(['name', 'price', 'percentage']));

        $is_unique = $this->checkUniqueness($validData);

        if (!$is_unique) {
            // Inform the user about the uniqueness constraint violation (Arabic error message)
            return array(
                'status' => 400,
                'message' => 'هذه المقطوعه موجوده بالفعل',
                'action' => 'false'
            );
        }

        $this->db->insert('financial_discounts', $validData);

        // Return a success message
        return array(
            'status' => 200,
            'message' => 'تم إدراج المقطوعه بنجاح',
            'action' => 'true'
        );
    }

    private function checkUniqueness($data)
    {
        // Assuming 'name' is the field that needs to be unique
        $existingRecord = $this->db->get_where('financial_discounts', ['name' => $data['name']])->row();

        return !$existingRecord; // Returns true if the record is unique, false otherwise
    }

    public function updateFinancialDiscounts($data)
    {
        $validData = array_intersect_key($data, array_flip(['name', 'price', 'percentage']));
        $id = $data['id'];
        $this->db->where('id', $id);
        $this->db->update('financial_discounts', $validData);
        return true;
    }


    public function deleteFinancialDiscounts($id)
    {
        $existing_record = $this->db->get_where('financial_discounts', ['id' => $id])->row();
        if (!$existing_record) {
            return false;
        }
        $this->db->where('id', $id);
        $this->db->delete('financial_discounts');
        return true;
    }



    public function deletAllFinancialDiscountsForUser($id)
    {
        // Check if there are any records with the given emp_info_id
        $existing_records = $this->db->get_where('user_financial_discounts', ['emp_info_id' => $id])->result();

        if (empty($existing_records)) {
            return false; // No records found for the given emp_info_id
        }

        // Loop through the existing records and delete each one
        foreach ($existing_records as $record) {
            $this->db->delete('user_financial_discounts', ['id' => $record->id]);
        }
        return true;

    }

}