<?php
defined('BASEPATH') or exit('No direct script access allowed');
date_default_timezone_set('Asia/Baghdad');

class FingerPrintSchedulerModel extends CI_Model
{

    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->model('EmployeesWorkedHoursModel');
        $this->load->library('Middleware');
    }

    public function insertAcutalAbsences()
    {
        // Query to get the data
        // Query to get the data
        $this->db->select('ach.user_hash, ach.date, ach.schedule_data, ach.allowance_time');
        $this->db->from('actual_schedules ach');
        $this->db->join('employees_worked_hours ewh', 'ach.date = ewh.date AND ach.user_hash = ewh.user_hash', 'left');
        $this->db->where('ewh.user_hash IS NULL');
        $query = $this->db->get();

        // Check if rows are returned
        if ($query->num_rows() > 0) {
            $data = $query->result_array();
            foreach ($data as $row) {

                // Decode the JSON data from the 'schedule_data' column
                $scheduleData = json_decode($row['schedule_data'], true);
                $allowance_with_minutes = $row['allowance_time'] ?? 0;
                // Generate details and calculate total target hours
                $details = $this->generateAbsenceDetails($scheduleData, $allowance_with_minutes);
                $totalTargetHours = array_sum(array_column($details, 'penalty_duration_with_minutes'));

                // Prepare data for insertion into employees_worked_hours table
                $data = [
                    'user_hash' => $row['user_hash'],
                    'total_late_arrival_minutes' => 0,
                    'total_early_departure_minutes' => 0,
                    'total_missed_hours' => $totalTargetHours,
                    'total_worked_hours' => 0,
                    'target_hours' => $totalTargetHours,
                    'overtime_hours' => 0,
                    'date' => $row['date'],
                    'is_flexible' => 0,
                    'details' => json_encode($details),
                ];

                $this->db->insert('employees_worked_hours', $data);
            }

            return true;
        }

        return false;

    }


    private function generateAbsenceDetails($scheduleData, $allowance_with_minutes)
    {
        $details = [];

        foreach ($scheduleData as $schedule) {
            $shouldBe = $schedule['start'];
            $penaltyDuration = $schedule['end'] - $schedule['start'];

            $details[] = [
                'id' => $schedule['id'],
                'shouldInAt' => $schedule['start'],
                'shouldOutAt' => $schedule['end'],
                'actual' => null,
                'penalty_duration_with_hours' => $penaltyDuration,
                'allowance_with_minutes' => $allowance_with_minutes,
                'status' => 'absent',
            ];
        }

        return $details;
    }

    // Insert a new fingerprint scheduler record (for 'in' fingerprint)
    public function insert_in_fingerprint($data)
    {
        $this->db->insert('finger_print_scheduler', $data);
        return $this->db->insert_id(); // return the ID of the inserted row
    }

    // Update the scheduler to set the 'out' fingerprint ID
    public function update_out_fingerprint($user_hash, $out_fingerprint_id, $fingerprintIn = null)
    {
        // Start a transaction to ensure data consistency
        $this->db->trans_start();

        if ($fingerprintIn !== null) {
            $this->db->where('fingerprint_in_id', $fingerprintIn);
        }
        // Step 1: Update the fingerprint_out_id in the scheduler table
        $this->db->where('user_hash', $user_hash);
        $this->db->where('fingerprint_out_id IS NULL'); // Ensure we're updating the correct record
        $this->db->order_by('created_at', 'desc');
        $this->db->limit(1);
        $this->db->update('finger_print_scheduler', ['fingerprint_out_id' => $out_fingerprint_id]);

        // Step 2: Retrieve the corresponding fingerprint_in_id
        $this->db->select('fingerprint_in_id, is_flexible');
        $this->db->from('finger_print_scheduler');
        $this->db->where('user_hash', $user_hash);
        $this->db->where('fingerprint_out_id', $out_fingerprint_id); // Ensure we're retrieving the correct record
        $query = $this->db->get();
        $in_fingerprint_id = $query->row()->fingerprint_in_id;
        $is_flexible = $query->row()->is_flexible;

        $this->db->where('id', $out_fingerprint_id);
        $data = [
            'in_out_reference' => $in_fingerprint_id,
            'is_flexible' => $is_flexible,
        ];
        $this->db->update('emp_fingerprint', $data);

        $this->db->where('id', $in_fingerprint_id);
        $fingerprint_query = $this->db->get('emp_fingerprint');
        $schedule_id = $fingerprint_query->row()->schedule_id;

        if ($schedule_id !== null) {
            $this->db->where('id', $out_fingerprint_id);
            $this->db->update('emp_fingerprint', ['schedule_id' => $schedule_id]);
        }

        // Complete the transaction
        $this->db->trans_complete();

        if ($this->db->trans_status() === false) {
            return false; // Transaction failed
        } else {
            return true; // Success
        }
    }

    // Check if there's already a scheduler record for the given user_hash
    public function check_existing_scheduler($user_hash)
    {
        $this->db->where('user_hash', $user_hash);
        $this->db->group_start();
        $this->db->where('fingerprint_out_id IS NULL'); // Only incomplete schedules (no 'out' yet)
        $this->db->or_group_start();
        $this->db->where('fingerprint_out_id IS NOT NULL');
        $this->db->where('scheduler_status', 'pending'); // Include schedules where 'out' is set but scheduler_status is pending
        $this->db->group_end();
        $this->db->group_end();
        $query = $this->db->get('finger_print_scheduler');
        return $query->row();
    }

    public function get_pending_worked_hours_records()
    {

        $this->db->where('fingerprint_in_id IS NOT NULL');
        $this->db->where('fingerprint_out_id IS NOT NULL');
        $this->db->where('scheduler_status', 'pending');
        $query = $this->db->get('finger_print_scheduler');
        return $query->result();
    }
    public function get_pending_worked_hours_records_TEST()
    {

        $this->db->where('fingerprint_in_id IS NOT NULL');
        $this->db->where('fingerprint_out_id IS NOT NULL');
        $this->db->where('scheduler_status', 'pend');
        $query = $this->db->get('finger_print_scheduler');
        return $query->result();
    }

    public function get_pending_missed_out_fingerprint()
    {
        $current_time = date('Y-m-d H:i:s');
        $this->db->where('fingerprint_out_id IS NULL');
        $this->db->where('dead_line <', $current_time);
        $this->db->where('scheduler_status', 'pending');
        $query = $this->db->get('finger_print_scheduler');
        return $query->result();
    }

    // just clarify not here

    public function handle_partial_fingerprintOuts()
    {

        $query = $this->db->query("
SELECT 
     JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.start_time')) AS sche_date,
    JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id')) AS sche_id,
    JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.user_id')) AS sche_user_id,
      finger_sch.id AS fingerprint_scheduler_id,
     finger_sch.date AS finger_date,
     finger_sch.user_hash AS finger_user_id,
    finger_sch.fingerprint_in_id,
   
     emp_fingerprint.schedule_id AS emp_schedule_id,
     sche_task.execute_at
FROM 
    scheduled_tasks AS sche_task
 JOIN 
     emp_fingerprint 
     ON emp_fingerprint.schedule_id =   JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id'))
     AND emp_fingerprint.user_hash = JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.user_id'))
 JOIN 
     finger_print_scheduler AS finger_sch 
     ON JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.user_id')) = finger_sch.user_hash 
     AND JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id')) = emp_fingerprint.schedule_id 
     -- this later maybe occur some problems be aware of it 
     AND DATE(JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.start_time'))) = DATE(emp_fingerprint.fingerprint_time)
 WHERE 
		JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id')) IS NOT NULL
      AND finger_sch.fingerprint_out_id IS NULL 
     AND finger_sch.scheduler_status = 'pending'
      AND sche_task.execute_at <= NOW()
      AND finger_sch.dead_line >= NOW() - INTERVAL 1 MINUTE
        AND sche_task.status = 'pending'
      AND sche_task.executed_at IS NULL
      AND DATE(execute_at) = DATE(NOW())
      GROUP BY fingerprint_scheduler_id

     ;
    ");

        $data = $query->result_array();
        if (!empty($data)) {
            foreach ($data as $row) {
                $fingerprint_scheduler_id = $row['fingerprint_scheduler_id'];

                // Update the deadline for the corresponding fingerprint scheduler
                $this->db->set('dead_line', 'NOW()', false); // Use raw SQL for NOW()

                $this->db->where('id', $fingerprint_scheduler_id);
                $this->db->update('finger_print_scheduler');
            }
            // Check if any rows were affected
            if ($this->db->affected_rows() > 0) {
                return "Fingerprints out were successfully updated.";
            } else {
                return "No fingerprints out were updated.";
            }
        } else {

            return "No data provided for update.";
        }
    }

    public function handle_partial_fingerprintInIds()
    { {
            // Start a transaction
            $this->db->trans_start();

            // Step 1: Run the query to get the required data
            $query = $this->db->query("
                SELECT 
    emp_fingerprint.id AS idfingerprint_for_previous,
    JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.end_time')) AS sche_date,
    JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id')) AS sche_id,
    JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.user_id')) AS sche_user_id,
    sche_task.execute_at,
    emp_fingerprint.user_hash,
    emp_fingerprint.location,
    emp_fingerprint.fingerprint_type,
    emp_fingerprint.mobile_info,
    JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.end_time')) AS fingerprint_time,
    emp_fingerprint.in_out,
    JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id')) AS schedule_id,
    0 AS is_flexible
FROM
    scheduled_tasks AS sche_task
        JOIN
    emp_fingerprint ON emp_fingerprint.schedule_id = JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.firstScheduleId'))
        AND emp_fingerprint.schedule_id != JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id'))
        AND emp_fingerprint.user_hash = JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.user_id'))
        AND DATE(emp_fingerprint.fingerprint_time) = DATE(JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.end_time')))
        JOIN
    finger_print_scheduler AS finger_sch ON JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.user_id')) = finger_sch.user_hash
        AND JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.firstScheduleId')) = emp_fingerprint.schedule_id
        AND DATE(JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.end_time'))) = DATE(emp_fingerprint.fingerprint_time)
        AND (finger_sch.fingerprint_in_id IS NOT NULL
        AND fingerprint_out_id IS NOT NULL)
WHERE
    JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id')) IS NOT NULL
        AND JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id')) = JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data,
                    '$.secondScheduleId'))
        AND sche_task.execute_at <= NOW()
        AND emp_fingerprint.in_out = 'in'
        
        AND NOT EXISTS( SELECT 
            1
        FROM
            emp_fingerprint ef
        WHERE
            ef.schedule_id = JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.schedule_id'))
                AND ef.user_hash = JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.user_id'))
                AND DATE(ef.fingerprint_time) = DATE(JSON_UNQUOTE(JSON_EXTRACT(sche_task.task_data, '$.end_time')))
                AND ef.in_out = 'in')
                
                GROUP BY emp_fingerprint.id
                ;
                
            ");

            $results = $query->result_array();

            // Step 2: Insert into emp_fingerprint and finger_print_scheduler
            foreach ($results as $row) {
                // Insert into emp_fingerprint
                $fingerprintData = [
                    'user_hash' => $row['user_hash'],
                    'hash' => rand(1000000000000000, 9999999999999999),
                    'fingerprint_type' => $row['fingerprint_type'],
                    'fingerprint_time' => $row['fingerprint_time'],
                    'in_out' => 'in',
                    'location' => $row['location'],
                    'mobile_info' => $row['mobile_info'],
                    'schedule_id' => $row['schedule_id'],
                    'is_flexible' => $row['is_flexible'],
                ];

                $this->db->insert('emp_fingerprint', $fingerprintData);
                $insertedFingerprintId = $this->db->insert_id(); // Get the inserted ID

                // Insert into finger_print_scheduler
                $schedulerData = [
                    'fingerprint_in_id' => $insertedFingerprintId,
                    'user_hash' => $row['user_hash'],
                    'dead_line' => date('Y-m-d H:i:s', strtotime($row['fingerprint_time'] . ' +24 hours')),
                    'date' => date('Y-m-d', strtotime($row['fingerprint_time'])),
                    'is_flexible' => 0,
                ];

                $this->db->insert('finger_print_scheduler', $schedulerData);
            }

            // Commit the transaction
            $this->db->trans_complete();

            // Check if the transaction was successful
            if ($this->db->trans_status() === FALSE) {
                // Handle transaction failure
                return ['status' => 'error', 'message' => 'Transaction failed.'];
            } else {
                // Transaction successful
                return ['status' => 'success', 'message' => 'Records inserted successfully.'];
            }
        }
    }

    public function update_worked_hours($userHash, $date, $schedulerId)
    {
        $result = $this->EmployeesWorkedHoursModel->processEmployeeData($date, $date, $userHash);
        if ($result['status'] == 'success') {
            $this->db->where('id', $schedulerId);
            $this->db->update('finger_print_scheduler', ['scheduler_status' => 'processed']);
        }
    }
    public function update_test_worked_hours($userHash, $date, $schedulerId)
    {
        $result = $this->EmployeesWorkedHoursModel->processEmployeeData($date, $date, $userHash);
        if ($result['status'] == 'success') {
            $this->db->where('id', $schedulerId);
            $this->db->update('finger_print_scheduler', ['scheduler_status' => 'processed']);
        }
    }

    public function update_flexible_worked_hours($userHash, $date, $schedulerId)
    {
        $result = $this->EmployeesWorkedHoursModel->getWorkedHoursAndInsert($userHash, $date);
        if ($result['status'] == 'ok') {
            $this->db->where('id', $schedulerId);
            $this->db->update('finger_print_scheduler', ['scheduler_status' => 'processed']);
        }
    }

    public function insertAbsenceFlexible()
    {
        // First, get all the records that need to be inserted

        $query = $this->db->query("SELECT
                    dft.user_hash,
                    dft.date,
					dft.target_hours
                FROM
                    daily_flexible_schedule_targets dft
                    LEFT JOIN employees_worked_hours ewh ON ewh.date = dft.date AND ewh.user_hash = dft.user_hash
                WHERE
                    ewh.user_hash IS NULL
                    AND dft.is_deleted = '0'");

        $records = $query->result();

        foreach ($records as $record) {
            $targetHours = $record->target_hours;

            $data = array(
                'user_hash' => $record->user_hash,
                'date' => $record->date,
                'total_late_arrival_minutes' => 0,
                'total_early_departure_minutes' => 0,
                'total_missed_hours' => $targetHours,
                'total_worked_hours' => 0,
                'target_hours' => $targetHours,
                'overtime_hours' => 0,
                'is_flexible' => 1,
                'created_at' => date('Y-m-d H:i:s'),
                'updated_at' => date('Y-m-d H:i:s'),
            );

            $this->db->insert('employees_worked_hours', $data);
        }

        return $this->db->affected_rows();

    }

    public function fixFlexibleForgoutToOut($fingerPrintInId, $shcedulerId)
    {

        $fingerPrintInData = $this->db->select('*, DATE(fingerprint_time) as date')
            ->from('emp_fingerprint')
            ->where('id', $fingerPrintInId)
            ->get()->row_array();

        if (!$fingerPrintInData) {
            return false;
        }

        $result = $this->EmployeesWorkedHoursModel->getFlexibleWorkedHours($fingerPrintInData['user_hash'], $fingerPrintInData['date']);
        if (!is_array($result)) {
            return false;
        }
        $missingHours = $result['missed_hours'];
        // If missing hours is positive, create the out time
        if ($missingHours > 0) {
            // Convert missing hours into H:i:s format
            $outTimeInterval = $this->convertHoursToHIS($missingHours);

            // Add the missing hours to the last fingerprint_in time
            $outTime = $this->addIntervalToTime($fingerPrintInData['fingerprint_time'], $outTimeInterval);

            // Prepare data for insertion into emp_fingerprint
            $data = [
                'user_hash' => $fingerPrintInData['user_hash'],
                'fingerprint_type' => '111', // Assuming 111 is for 'out'
                'fingerprint_time' => $outTime,
                'in_out' => 'out',
                'hash' => $this->jwtmiddleware->createHash(),
                'user_create_record' => "12345", // Replace with the actual user ID or relevant value
                'in_out_reference' => $fingerPrintInId,
                'is_flexible' => 1, // Assuming this is for flexible out
            ];

            // Insert the new out record
            $this->db->insert('emp_fingerprint', $data);
            $insertedId = $this->db->insert_id();
            $this->update_out_fingerprint($fingerPrintInData['user_hash'], $insertedId, $fingerPrintInId);
            $this->update_flexible_worked_hours($fingerPrintInData['user_hash'], $fingerPrintInData['date'], $shcedulerId);
        } else {
            // If no missing hours, set the out time to 5 minutes after the last fingerprint_in time
            $outTime = $this->addMinutesToTime($fingerPrintInData['fingerprint_time'], 5);

            // Prepare data for insertion into emp_fingerprint
            $data = [
                'user_hash' => $fingerPrintInData['user_hash'],
                'fingerprint_type' => '111', // Assuming 111 is for 'out'
                'fingerprint_time' => $outTime,
                'in_out' => 'out',
                'hash' => $this->jwtmiddleware->createHash(),
                'user_create_record' => "12345", // Replace with the actual user ID or relevant value
                'in_out_reference' => $fingerPrintInId,
                'is_flexible' => 1, // Assuming this is for flexible out
            ];

            // Insert the new out record
            $this->db->insert('emp_fingerprint', $data);
            $insertedId = $this->db->insert_id();
            $this->update_out_fingerprint($fingerPrintInData['user_hash'], $insertedId, $fingerPrintInId);
            $this->update_flexible_worked_hours($fingerPrintInData['user_hash'], $fingerPrintInData['date'], $shcedulerId);

        }
    }

    private function addIntervalToTime($time, $interval)
    {
        // Convert the fingerprint time to a DateTime object
        $dateTime = new DateTime($time);

        // Extract hours and minutes from the interval
        list($h, $m, $s) = explode(':', $interval);

        // Add the interval to the DateTime object
        $dateTime->modify("+{$h} hours +{$m} minutes +{$s} seconds");

        // Return the updated time in the original format
        return $dateTime->format('Y-m-d H:i:s');
    }

    private function convertHoursToHIS($hours)
    {
        $h = floor($hours); // Get the whole hours
        $m = ($hours - $h) * 60; // Convert the fractional part to minutes

        // Return the formatted time as H:i:s
        return sprintf('%02d:%02d:00', $h, $m);
    }

    private function addMinutesToTime($time, $minutes)
    {
        // Convert the fingerprint time to a DateTime object
        $dateTime = new DateTime($time);

        // Add minutes to the DateTime object
        $dateTime->modify("+{$minutes} minutes");

        // Return the updated time in the original format
        return $dateTime->format('Y-m-d H:i:s');
    }

    public function fixForgoutToOut($fingerPrintInId, $shcedulerId)
    {
        $fingerPrintInData = $this->db->select('*, DATE(fingerprint_time) as date')
            ->from('emp_fingerprint')
            ->where('id', $fingerPrintInId)
            ->get()->row_array();

        if (!$fingerPrintInData) {
            return false; // If no fingerprint data is found
        }

        $scheduleId = $fingerPrintInData['schedule_id'];
        $userHash = $fingerPrintInData['user_hash'];
        $date = $fingerPrintInData['date'];
        $fingerprint_time = $fingerPrintInData['fingerprint_time'];
        $schedule = $this->getScheduleById($userHash, $date, $scheduleId);
        if (empty($schedule)) {
            $schedule = $this->getScheduleUserHashAndDate($userHash, $date);
        }
        if (!empty($schedule) && isset($schedule['end'])) {
            $end = $schedule['end'];
            $outFingerPrintTime = $this->getFingerprintOutTime($fingerprint_time, $end);
            $data = array(
                'user_hash' => $userHash,
                'fingerprint_type' => '111',
                'fingerprint_time' => $outFingerPrintTime,
                'in_out' => 'out',
                'hash' => $this->jwtmiddleware->createHash(),
                'user_create_record' => "12345",
                'in_out_reference' => $fingerPrintInId,
                'is_flexible' => 0,
                'schedule_id' => $scheduleId,
            );
            $this->db->insert('emp_fingerprint', $data);
            $insertedId = $this->db->insert_id();
            $this->update_out_fingerprint($fingerPrintInData['user_hash'], $insertedId, $fingerPrintInId);
            $this->update_worked_hours($fingerPrintInData['user_hash'], $fingerPrintInData['date'], $shcedulerId);

        } else {
            var_dump("EMPTY YA => , ", $fingerPrintInId);

        }
    }

    private function getScheduleById($user_hash, $date, $scheduleId)
    {
        // Step 1: Retrieve the schedule_data from actual_schedules table
        $this->db->select('schedule_data');
        $this->db->from('actual_schedules');
        $this->db->where('user_hash', $user_hash);
        $this->db->where('date', $date);
        $query = $this->db->get();

        // Check if any result is returned
        if ($query->num_rows() > 0) {
            $row = $query->row_array();
            $scheduleData = json_decode($row['schedule_data'], true); // Step 2: Decode JSON data

            // Step 3: Search for the schedule by ID
            foreach ($scheduleData as $schedule) {
                if ($schedule['id'] == $scheduleId) {
                    // Step 4: Return start and end times if ID matches
                    return [
                        'start' => $schedule['start'],
                        'end' => $schedule['end'],
                    ];
                }
            }
        }

        // Return null if no match is found
        return null;
    }

    private function getScheduleUserHashAndDate($user_hash, $date)
    {
        // Step 1: Retrieve the schedule_data from actual_schedules table
        $this->db->select('schedule_data');
        $this->db->from('actual_schedules');
        $this->db->where('user_hash', $user_hash);
        $this->db->where('date', $date);
        $query = $this->db->get();

        // Check if any result is returned
        if ($query->num_rows() > 0) {
            $row = $query->row_array();
            $scheduleData = json_decode($row['schedule_data'], true); // Step 2: Decode JSON data

            if (count($scheduleData) == 1) {
                return $scheduleData[0];
            }
            return null;
        }
        return null;
    }

    private function getFingerprintOutTime($fingerprintInTime, $outDate)
    {
        // Step 1: Convert outDate (0 to 24) to H:i:s format
        $hours = floor($outDate); // The integer part of outDate is the hour
        $minutes = ($outDate - $hours) * 60; // The fractional part of outDate gives us the minutes

        // Convert to H:i:s format
        $outTime = sprintf('%02d:%02d:00', $hours, $minutes);

        // Step 2: Extract the date part from fingerprintInTime
        $date = date('Y-m-d', strtotime($fingerprintInTime));

        // Step 3: Combine the date and outTime to form the fingerprintOutTime
        $fingerprintOutTime = $date . ' ' . $outTime;

        return $fingerprintOutTime;
    }


    public function handleAutomaticFingerPrintOutForPartialVacation($fingerPrintInId, $shcedulerId)
    {
        $fingerPrintInData = $this->db->select('*, DATE(fingerprint_time) as date')
            ->from('emp_fingerprint')
            ->where('id', $fingerPrintInId)
            ->get()->row_array();

        if (!$fingerPrintInData) {
            return false; // If no fingerprint data is found
        }

        $scheduleId = $fingerPrintInData['schedule_id'];
        $userHash = $fingerPrintInData['user_hash'];
        $date = $fingerPrintInData['date'];
        $fingerprint_time = $fingerPrintInData['fingerprint_time'];
        $schedule = $this->getScheduleById($userHash, $date, $scheduleId);
        if (empty($schedule)) {
            $schedule = $this->getScheduleUserHashAndDate($userHash, $date);
        }
        if (!empty($schedule) && isset($schedule['end'])) {
            $end = $schedule['end'];
            $outFingerPrintTime = $this->getFingerprintOutTime($fingerprint_time, $end);
            $data = array(
                'user_hash' => $userHash,
                'fingerprint_type' => '111',
                'fingerprint_time' => $outFingerPrintTime,
                'in_out' => 'out',
                'hash' => $this->jwtmiddleware->createHash(),
                'user_create_record' => "12345",
                'in_out_reference' => $fingerPrintInId,
                'is_flexible' => 0,
                'schedule_id' => $scheduleId,
            );
            $this->db->insert('emp_fingerprint', $data);
            $insertedId = $this->db->insert_id();
            $this->update_out_fingerprint($fingerPrintInData['user_hash'], $insertedId, $fingerPrintInId);
            $this->update_worked_hours($fingerPrintInData['user_hash'], $fingerPrintInData['date'], $shcedulerId);

        } else {
            var_dump("EMPTY YA => , ", $fingerPrintInId);

        }
    }
}
