<?php
class ItemsModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        $this->load->library('Middleware');
    }

    public function getCount()
    {
        $headers = apache_request_headers();
        $search = $headers['q'] ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('inventory_items');
        if (isset ($search) && $search != "" && $search != 'null') {
            $this->db->where('inventory_items.name', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }
    public function getCountTypes()
    {
        $headers = apache_request_headers();
        $search = $headers['q'] ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('inventory_items_types');
        if (isset ($search) && $search != "" && $search != 'null') {
            $this->db->where('inventory_items_types.name', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getAll()
    {
        $headers = apache_request_headers();

        $page = $this->input->get('page' ?? 1);
        $perPage = $this->input->get('perPage' ?? 10);
        $search = $this->input->get('q' ?? '');
        // $page = $headers['page'] ?? 1;
        // $perPage = $headers['perPage'] ?? 10;
        // $search = $headers['q'] ?? '';
        $offset = ($page - 1) * $perPage;
        $this->db->select("*");

        $this->db->from('inventory_items');
        // Search
        if (isset ($search) && $search != "" && $search != 'null') {
            $this->db->or_like('inventory_items.item_name', $search);
        }
        $this->db->order_by('inventory_items.item_id DESC');
        // Limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getItemsTypes()
    {
        $headers = apache_request_headers();
        $page = $this->input->get('page');
        $perPage = $this->input->get('perPage');
        $search = $this->input->get('q');

        $offset = ($page - 1) * $perPage;
        $this->db->select("*");

        $this->db->from('inventory_items_types');
        // Search
        if (isset ($search) && $search != "" && $search != 'null') {
            $this->db->or_like('inventory_items_types.name', $search);
        }
        $this->db->order_by('inventory_items_types.id DESC');
        // Limit
        $this->db->limit($perPage, $offset);
        $query = $this->db->get();
        return $query->result_array();
    }

    public function get($id)
    {
        $this->db->select('it.item_name, it.item_id, iit.id, iit.name');
        $this->db->from('inventory_items AS it');
        $this->db->where('it.item_id', $id);
        $this->db->join('inventory_items_types AS iit', 'it.item_type = iit.id');
        $query = $this->db->get();
        return $query->result_array();
    }

    public function getItemType($id)
    {
        $this->db->select("*");
        $this->db->from("inventory_items_types");
        $this->db->where('id', $id);
        $query = $this->db->get();
        return $query->result_array();
    }



    public function create($data)
    {
        $this->db->insert('inventory_items', $data);
        return $this->db->insert_id();
    }

    public function createItemType($data)
    {
        $this->db->insert('inventory_items_types', $data);
        return $this->db->insert_id();
    }

    public function update($id)
    {
        $data = $this->input->post();
        // delete id in array
        unset($data['id']);
        $this->db->where('item_id', $id);
        $this->db->update('inventory_items', $data);
        return $this->db->affected_rows();
    }

    public function updateItemType($id)
    {
        $data = $this->input->post();
        // delete id in array
        unset($data['id']);
        $this->db->where('id', $id);
        $this->db->update('inventory_items_types', $data);
        return ($this->db->affected_rows() != 1) ? false : true;
    }




    public function getItemsTypeSelect()
    {
        $this->db->select('id, name'); // Assuming 'id' and 'name' are columns in your table
        $query = $this->db->get('inventory_items_types');

        $itemTypes = [];
        foreach ($query->result() as $row) {
            $itemTypes[$row->id] = $row->name;
        }

        return $itemTypes;
    }


    public function delete($id)
    {

        $query = $this->db->select("item_id")
            ->from('item_assignments')
            ->where('item_id', $id)
            ->get();

        if ($query->num_rows() > 0) {
            // item_id exists
            return "لا يمكن مسح الذمه بسبب لانها مع شخص او أكثر";
        } else {
            $this->db->where('item_id', $id);
            $this->db->delete('inventory_items');
            return $this->db->affected_rows();
        }
    }

    public function deleteItemType($id)
    {
        $query = $this->db->select('*')->from('inventory_items')->where('item_type', $id)->get();
        if ($query->num_rows() > 0) {
            return "لا يمكن مسح نوع الذمه  لانها مع شخص او أكثر";
        } else {
            $this->db->where('id', $id);
            $this->db->delete('inventory_items_types');
            return $this->db->affected_rows();
        }
    }


}