<?php
class RewardsAndPenaltiesModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    public function get($id)
    {
        $this->db->select('*');
        $this->db->select('(select name from penalties_rewards_types where penalties_rewards_types.id = penalties_rewards_types_id) as type_name');
        $this->db->from('penalties_rewards');
        $this->db->where("id", $id);


        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->row();
        } else {
            return "لم يتم ايجاد";
        }
    }
    public function getAll()
    {
        $page = $this->input->get('page') ? $this->input->get('page') : 1;
        $perPage = $this->input->get('perPage') ? $this->input->get('perPage') : 10; // Default to 10 if not provided
        $searchTerm = $this->input->get('q');
        // Calculate the offset
        $offset = ($page - 1) * $perPage;

        $this->db->select('*');
        $this->db->from('penalties_rewards');
        $this->db->limit($perPage, $offset);

        if (isset ($searchTerm)) {
            $this->db->like('description', $searchTerm);
            $this->db->or_like('notes', $searchTerm);
        }

        return $this->db->get()->result_array();
    }

    public function insert($data)
    {
        try {
            $this->db->insert('penalties_rewards', $data);
            if ($this->db->affected_rows() > 0) {
                return true; // Success
            } else {
                return "لم يتم إدخال البيانات"; // Data not inserted
            }
        } catch (Exception $e) {
            return "حدث خطأ أثناء إدخال البيانات"; // An error occurred while inserting data
        }
    }

    public function update($id, $data)
    {
        try {
            $this->db->where('id', $id);
            $this->db->update('penalties_rewards', $data);
            if ($this->db->affected_rows() > 0) {
                return true; // Success
            } else {
                return "لم يتم تحديث البيانات"; // Data not updated
            }
        } catch (Exception $e) {
            // Log the error message for debugging
            return "حدث خطأ أثناء تحديث البيانات"; // An error occurred while updating data
        }
    }

    public function delete($id)
    {
        try {
            $this->db->where('id', $id);
            $this->db->delete('penalties_rewards');
            if ($this->db->affected_rows() > 0) {
                return true; // Success
            } else {
                return "لم يتم حذف البيانات"; // Data not deleted
            }
        } catch (Exception $e) {
            // Log the error message for debugging
            return "حدث خطأ أثناء حذف البيانات"; // An error occurred while deleting data
        }
    }

    // get Employees who have rewards_penalties 

    public function getEmployee($id)
    {
        $this->db->select('p.*,su.name as user_name, su.image_profile, prt.id as type_id , prt.name as type_name , an.hash as announce_id');

        $this->db->from('penalties_rewards_employees p');
        $this->db->join('system_users su', 'p.emp_id = su.hash');
        $this->db->join('penalties_rewards_types prt', 'prt.id = p.penalties_rewards_types_id');
        $this->db->join('announce an' , 'an.reward_id= p.id', 'left');

        $this->db->where('p.id', $id);
        $query = $this->db->get();


        if ($query->num_rows() > 0) {
            return $query->row();
        } else {
            return "لم يتم ايجاد الموظف";
        }
    }


    public function countRewardsEmployees()
    {
        $this->db->select('COUNT(*) as total');

        $this->db->from('penalties_rewards_employees p');
        return $this->db->get()->row();
    }

    public function getEmployees()
    {
        // Get the page number and number of records per page from the query parameters
        $page = $this->input->get('page') ? $this->input->get('page') : 1;
        $perPage = $this->input->get('perPage') ? $this->input->get('perPage') : 10; // Default to 10 if not provided

        // Calculate the offset
        $offset = ($page - 1) * $perPage;
        $searchTerm = $this->input->get('q');

        $this->db->select('p.*,su.name as user_name, su.image_profile, prt.id as type_id , prt.name as type_name');

        $this->db->from('penalties_rewards_employees p');
        $this->db->join('system_users su', 'p.emp_id = su.hash');
        $this->db->join('announce an', 'an.reward_id = p.id', 'left');
        $this->db->join('penalties_rewards_types prt', 'prt.id = p.penalties_rewards_types_id');
        
        $this->db->limit($perPage, $offset);

        if (isset ($searchTerm)) {
            $this->db->like('su.name', $searchTerm);
        }
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->result_array();
        } else {
            return array(); // Return an empty array if no employees are found
        }
    }

    // get types of rewaredAndPentailes

    public function getTypes()
    {
        $this->db->select('*');
        $this->db->from('penalties_rewards_types');
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return $query->result_array();
        } else {
            return array(); // Return an empty array if no types are found
        }
    }


    public function addNewEmployeeRewardOrPenalty($data)
    {
        $this->db->insert('penalties_rewards_employees', $data);
        if ($this->db->affected_rows() > 0) {
            return $this->db->insert_id();
            return true; // Success
        } else {
            return "لم يتم إدخال البيانات"; // Data not inserted
        }
    }
    public function updateEmployeeRewardOrPenalty($id, $data)
    {
        $this->db->where('id', $id);
        $this->db->update('penalties_rewards_employees', $data);

        if ($this->db->affected_rows() > 0) {
            return true; // Success
        } else {
            return "لم يتم تحديث البيانات"; // Data not updated
        }
    }

    public function deleteEmployeeRewardOrPenalty($id)
    {
        $this->db->where('id', $id);
        $this->db->delete('penalties_rewards_employees');

        if ($this->db->affected_rows() > 0) {
            return true; // Success
        } else {
            return "لم يتم حذف البيانات"; // Data not deleted
        }
    }


}