<?php

class RolesPermissionsModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }



    public function insert_permission($data)
    {
        if (empty($data['permissionIds'])) {
            // Delete all rows for the role
            $this->db->where('role_id', $data['role_id']);
            $this->db->delete('RolesPermissions');
            return true;
        } else {
            // First, delete any existing permissions for the role
            $this->db->where('role_id', $data['role_id']);
            $this->db->delete('RolesPermissions');

            // Then, insert the new permissions
            $result = array();
            foreach ($data['permissionIds'] as $permissionId) {
                $insertData = array(
                    'role_id' => $data['role_id'],
                    'permission_id' => $permissionId,
                    'is_allowed' => $data['is_allowed']
                );
                $result[] = $this->db->insert('RolesPermissions', $insertData);
            }
            return $result;
        }
    }

    // This for select
    public function getAllRoles()
    {
        $this->db->select('id AS value, name AS label');
        $this->db->from("Roles");
        $query = $this->db->get();
        return $query->result_array();
    }


    public function getRoles()
    {
        $page = $this->input->get('page') ?? 1;
        $perPage = $this->input->get('perPage') ?? 10;
        $offset = ($page - 1) * $perPage;


        $this->db->select('id, name , priority');
        $this->db->from("Roles");
        $this->db->order_by('priority', 'ASC');
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }


    public function get_permission_labels()
    {
        $this->db->select('id as value, name as label');
        $this->db->from('permissions_for_roles');
        $query = $this->db->get();
        $results = $query->result();
        return $results;
    }
    public function get_permissions_label_by_role($role_id)
    {
        $this->db->select('permissions_for_roles.name as label, permissions_for_roles.id as value');
        $this->db->from('RolesPermissions');
        $this->db->join('permissions_for_roles', 'RolesPermissions.permission_id = permissions_for_roles.id');
        $this->db->where('RolesPermissions.role_id', $role_id);
        $query = $this->db->get();
        $results = $query->result();
        return $results;
    }
    public function countAllRoles()
    {
        $this->db->select('COUNT(*) as total');
        $this->db->from("Roles");
        $query = $this->db->get();
        return $query->row()->total;
    }



    public function getUserRoles($user_id)
    {

        $this->db->select("ur.role_id, r.id AS value, r.name AS label")
            ->from("UserRoles ur")
            ->join("Roles r", "ur.role_id = r.id")
            ->where('ur.user_hash', $user_id);
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            // User ID exists and has associated roles
            return $query->result_array();
        } else {
            // User ID does not exist or does not have any associated roles
            return [];
        }
    }

    public function insertUserRole($user_hash, $rolesArr)
    {
        // Start the transaction
        $this->db->trans_begin();

        try {
            $existingRoles = $this->db->from('UserRoles')->where(['user_hash' => $user_hash])->get()->result_array();
            $existingRoleIds = array_column($existingRoles, 'role_id');

            $insertedRoles = [];
            foreach ($rolesArr as $role) {
                // Correctly check if the role does not exist in the existing roles
                if (!in_array($role, $existingRoleIds)) {
                    $data = [
                        'user_hash' => $user_hash,
                        'role_id' => $role, // Ensure this is the correct format for your database
                    ];

                    $this->db->insert('UserRoles', $data);

                    if ($this->db->affected_rows() > 0) {
                        $insertedRoles[] = $role;
                    }
                }
            }
            $this->handlePermissions($user_hash, $rolesArr);

            // Commit the transaction if all inserts were successful
            if ($this->db->trans_status() === false) {
                throw new Exception('Transaction failed');
            } else {
                $this->db->trans_commit();
                return $insertedRoles;
            }
        } catch (Exception $e) {
            // Rollback the transaction on failure
            $this->db->trans_rollback();
            // Optionally, log the error or handle it as needed
            // For example:
            // log_message('error', $e->getMessage());
            return []; // Return an empty array or handle the error as appropriate
        }
    }

    public function updateUserRole($user_hash, $rolesArr)
    {
        // Start the transaction
        $this->db->trans_begin();

        try {
            // Get existing roles before the update
            $existingRoles = $this->db->from('UserRoles')->where(['user_hash' => $user_hash])->get()->result_array();
            $this->db->where('user_hash', $user_hash)->delete('UserRoles');

            $existingRoleIds = array_column($existingRoles, 'role_id');

            // Calculate roles to remove (existing roles not present in the updated roles)
            $rolesToRemove = array_diff($existingRoleIds, $rolesArr);

            // Delete user roles to remove
            if (!empty($rolesToRemove)) {
                $this->updateRolePermissions($user_hash, $rolesToRemove, 0);
            }

            // Insert updated user roles
            if (!empty($rolesArr)) {
                foreach ($rolesArr as $role) {
                    $data = [
                        'user_hash' => $user_hash,
                        'role_id' => $role, // Ensure this is the correct format for your database
                    ];

                    $this->db->insert('UserRoles', $data);
                }
                // Update permissions based on updated roles
                $this->handlePermissions($user_hash, $rolesArr);

            }

            // Commit the transaction if all updates were successful
            if ($this->db->trans_status() === false) {
                throw new Exception('Transaction failed');
            } else {
                $this->db->trans_commit();
                return true; // Return true for success
            }
        } catch (Exception $e) {
            // Rollback the transaction on failure
            $this->db->trans_rollback();
            // Optionally, log the error or handle it as needed
            // For example:
            // log_message('error', $e->getMessage());
            return false; // Return false for failure
        }
    }

    // private function handlePermissions($user_hash, $rolesArr)
    // {
    //     $pages = $this->db->select('page_id')->from('RolesPages')->where_in('role_id', $rolesArr)->get()->result_array();
    //     $pageIds = array_column($pages, 'page_id');

    //     $permissions = $this->db->from('permission')->where(['user_hash' => $user_hash])->get()->result_array();

    //     // Update existing permissions
    //     foreach ($permissions as $permission) {
    //         if (in_array($permission['page_id'], $pageIds) && $permission['access'] == 0) {
    //             $data = ['access' => 1];
    //             $this->db->where(['user_hash' => $user_hash, 'page_id' => $permission['page_id']])->update('permission', $data);
    //         }
    //     }

    //     // Insert new permissions for pages related to roles where access is not granted
    //     $newPermissions = array_diff($pageIds, array_column($permissions, 'page_id'));
    //     foreach ($newPermissions as $newPageId) {
    //         // Check if the permission does not already exist
    //         $existingPermission = $this->db->where(['user_hash' => $user_hash, 'page_id' => $newPageId])->get('permission')->row();
    //         if (!$existingPermission) {
    //             $data = [
    //                 'user_hash' => $user_hash,
    //                 'page_id' => $newPageId,
    //                 'access' => 1,
    //             ];
    //             $this->db->insert('permission', $data);
    //         }
    //     }
    // }


    private function handlePermissions($user_hash, $rolesArr)
    {
        // Check if $rolesArr is not empty before executing the query
        if (!empty($rolesArr)) {
            $pages = $this->db->select('page_id')->from('RolesPages')->where_in('role_id', $rolesArr)->get()->result_array();
            $pageIds = array_column($pages, 'page_id');

            $permissions = $this->db->from('permission')->where(['user_hash' => $user_hash])->get()->result_array();

            // Update existing permissions
            foreach ($permissions as $permission) {
                if (in_array($permission['page_id'], $pageIds) && $permission['access'] == 0) {
                    $data = ['access' => 1];
                    $this->db->where(['user_hash' => $user_hash, 'page_id' => $permission['page_id']])->update('permission', $data);
                }
            }

            // Insert new permissions for pages related to roles where access is not granted
            $newPermissions = array_diff($pageIds, array_column($permissions, 'page_id'));
            foreach ($newPermissions as $newPageId) {
                // Check if the permission does not already exist
                $existingPermission = $this->db->where(['user_hash' => $user_hash, 'page_id' => $newPageId])->get('permission')->row();
                if (!$existingPermission) {
                    $data = [
                        'user_hash' => $user_hash,
                        'page_id' => $newPageId,
                        'access' => 1,
                    ];
                    $this->db->insert('permission', $data);
                }
            }
        }
    }


    private function updateRolePermissions($user_hash, $rolesToRemove, $access)
    {
        $pagesToRemove = $this->db->select('page_id')
            ->from('RolesPages')
            ->where_in('role_id', $rolesToRemove)
            ->get()
            ->result_array();

        $pageIdsToRemove = array_column($pagesToRemove, 'page_id');

        if (!empty($pageIdsToRemove)) {
            $this->db->where('user_hash', $user_hash)
                ->where_in('page_id', $pageIdsToRemove)
                ->update('permission', ['access' => $access]);
        }
    }

}
