<?php

class SpecifactionModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }
    public function count_records()
    {
        $search = $this->input->get('q') ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('specifications');
        if ($search) {
            $this->db->like('name', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function  count_other_specifications() {
        $search = $this->input->get('q') ?? '';
        $this->db->select('count(*) as count');
        $this->db->from('others_specifications');
        if ($search) {
            $this->db->like('name', $search);
        }
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }
    
    public function getSpecifactions()
    {
        $page = $this->input->get('page') ?? '1';

        $perPage = $this->input->get('perPage') ?? '10';

        $search = $this->input->get('q') ?? '';

        $offset = ($page - 1) * $perPage;


        $this->db->select('specifications.*, specifications_types.name AS typeName, specifications_types.table, specifications.name AS name');
        $this->db->from('specifications');
        $this->db->join('specifications_types', 'specifications.specifications_type = specifications_types.id', 'left');

        if ($search) {
            $this->db->like('specifications.name', $search);
        }
        $this->db->order_by('id', 'ASC');
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        return $query->result_array();
    }

    public function getOtherSpecifications()
    {
        $page = $this->input->get('page') ?? '1';

        $perPage = $this->input->get('perPage') ?? '10';

        $search = $this->input->get('q') ?? '';

        $offset = ($page - 1) * $perPage;


        $this->db->select('*');
        $this->db->from('others_specifications');

        if ($search) {
            $this->db->like('name', $search);
        }
        $this->db->order_by('id', 'ASC');
        $this->db->limit($perPage, $offset);

        $query = $this->db->get();
        return $query->result_array();

    }



    public function getUserSpecifications($userId)
    {
        $this->db->select('specifications.name AS label, specifications.id AS value');
        $this->db->from('user_specifications');
        $this->db->join('specifications', 'user_specifications.id = specifications.id');
        $this->db->where('user_specifications.emp_info_id', $userId);
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            $result = $query->result();
            $jsonArray = array();
            foreach ($result as $specification) {
                $jsonArray[] = array(
                    'label' => $specification->label,
                    'value' => $specification->value
                );
            }
            return json_encode($jsonArray);
        } else {
            return json_encode(array()); // Return an empty JSON array if no results are found
        }
    }

    public function insertUserSpecification($data)
    {
        $this->db->insert('user_specifications', $data);
        return true;
    }

    public function deleteUserSpecification($user_id, $specification_id)
    {
        $this->db->where('emp_info_id', $user_id);
        $this->db->where('specification_id', $specification_id);
        $this->db->delete('user_specifications');
    }

    public function getSpecifaction($id)
    {
        $this->db->select('specifications.*, specifications_types.name as typeName, specifications_types.table');
        $this->db->from('specifications');
        $this->db->join('specifications_types', 'specifications.specifications_type = specifications_types.id', 'left');
        $this->db->where('specifications.id', $id);
        $query = $this->db->get();
        return $query->row_array();
    }

    public function getCurrentUserSpecifications($user_id)
    {
        $this->db->select("specification_id");
        $this->db->from("user_specifications");
        $this->db->where('emp_info_id', $user_id);
        $query = $this->db->get();
        if ($query->num_rows() > 0) {
            $result = $query->result_array();
            $relations = array_column($result, 'specification_id');

            return $relations;
        } else {
            // No relations found for the given position
            return [];
        }
    }

    // This will get but with types like qualifcations: , others: , and so more
    public function getCurrentUserTypesSpecifications($user_id)
    {
        $this->db->select("user_specifications.specification_id, 
        specifications.id AS value, 
        specifications.name AS label, 
        specifications.specifications_type,
        CASE specifications.specifications_type 
            WHEN 1 THEN 'specifications_qualifications' 
            WHEN 2 THEN 'positions_qualifications' 
            WHEN 3 THEN 'specifications_others'
        END AS specification_type_name");
        $this->db->from("user_specifications");
        $this->db->join("specifications", "user_specifications.specification_id = specifications.id");
        $this->db->where('emp_info_id', $user_id);

        $query = $this->db->get();
        if ($query->num_rows() > 0) {
            $result = $query->result_array();
            $groupedResult = array(
                'specifications_qualifications' => array(),
                'positions_qualifications' => array(),
                'specifications_others' => array(),
            );

            foreach ($result as $row) {
                $type = $row['specification_type_name'];
                unset($row['specification_type_name']); // Remove the specification_type_name from the row
                $groupedResult[$type][] = $row;
            }

            return $groupedResult;
        } else {
            // No relations found for the given position
            return [];
        }
    }
    public function getCurrentUserSpecificationsSelect($user_id)
    {
        $this->db->select("user_specifications.specification_id, specifications.id AS value, specifications.name AS label");
        $this->db->from("user_specifications");
        $this->db->join("specifications", "user_specifications.specification_id = specifications.id");
        $this->db->where('emp_info_id', $user_id);
        return $this->db->get()->result_array();
    }

    public function getSpecifactionsSelect()
    {
        $this->db->select('id AS value, name AS label');
        $this->db->from("specifications");
        $query = $this->db->get();
        return $query->result_array();
    }
    public function getSpecifactionsTypes()
    {
        $this->db->select('id AS value, name AS label');
        $this->db->from("specifications_types");
        $query = $this->db->get();
        return $query->result_array();
    }



    public function getDataByType($tableName)
    {
        // Safety check to prevent SQL injection
        if (!$this->db->table_exists($tableName)) {
            return []; // Return an empty array if table doesn't exist
        }

        if ($tableName === "Positions") {
            $this->db->select("ID AS value,Name AS label");
        } else {
            $this->db->select('id AS value, name AS label');
        }

        $query = $this->db->get($tableName);
        return $query->result_array();
    }

    // FOr test now
    public function getDataBySpecificationsId($specificationsId)
    {
        // Safety check to prevent SQL injection
        // Safety check to prevent SQL injection
        if (!is_numeric($specificationsId) || $specificationsId <= 0) {
            return []; // Invalid ID, return an empty array
        }



        // Add a WHERE clause to filter by the provided ID

        $array = array(1 => 'id', 2 => 'ID', 3 => 'id');
        $this->db->where($array[$specificationsId], $specificationsId);

        // Execute the query on the "specifications" table
        $query = $this->db->get('specifications_types');
        $test = $query->row_array();
        // $builder = $this->db->table();


        // Return the result as an associative array
        $table = $test['table'];
        $this->db->select('*');
        $this->db->from($table);

        return $this->db->get()->result_array();
        // return $test;
    }
    public function getOtherSpecificationById($otherSpecificationsId)
    {

        $this->db->where('id', $otherSpecificationsId);
        $query = $this->db->get('others_specifications');

        if ($query->num_rows() > 0) {
            $result = $query->row_array();
            return $result;
        } else {
            return null;
        }
    }

    public function insertSpecifactions($data)
    {
        if (empty ($data)) {
            return array(
                'status' => 500,
                'message' => 'Failed to insert specifications',
                'action' => 'false'
            );
        }

        $validData = array_intersect_key($data, array_flip(['specifications_type', 'name', 'percentage', 'price', 'maximumPrice', 'nameValue']));

        $is_unique = $this->checkUniqueness($validData);

        if (!$is_unique) {
            // Inform the user about the uniqueness constraint violation (Arabic error message)
            return array(
                'status' => 400,
                'message' => 'هذا المخصص موجود بالفعل',
                'action' => 'false'
            );
        }

        $this->db->insert('specifications', $validData);

        // Return a success message
        return array(
            'status' => 200,
            'message' => 'تم إدراج المخصصات بنجاح',
            'action' => 'true'
        );
    }

    private function checkUniqueness($data)
    {
        // Assuming 'name' is the field that needs to be unique
        $existingRecord = $this->db->get_where('specifications', ['name' => $data['name']])->row();

        return !$existingRecord; // Returns true if the record is unique, false otherwise
    }

    public function insertOtherSpecifications($data)
    {
        if (empty ($data)) {
            return;
        }
        $this->db->select('name');
        $this->db->from('others_specifications');
        $this->db->where('name', $data['name']);
        $countName = $this->db->get()->num_rows();

        if ($countName > 0) {
            return "هذ المخصص موجود بل الفعل";
        }

        $validData = array_intersect_key($data, array_flip(['name']));

        $this->db->insert('others_specifications', $validData);


        // Return the ID of the inserted record
        return $this->db->insert_id();
    }

    public function updateSpecifactions($data)
    {



        $validData = array_intersect_key($data, array_flip(['specifications_type', 'name', 'percentage', 'price', 'maximumPrice', 'nameValue']));
        $id = $data['id'];


        $this->db->where('id', $id);
        $this->db->update('specifications', $validData);
        return true;
    }
    public function updateOtherSpecifications($data)
    {
        $validData = array_intersect_key($data, array_flip(['name']));
        $id = $data['id'];


        $this->db->where('id', $id);
        $this->db->update('others_specifications', $validData);
        return true;
    }

    // public function deleteSpecification($id)
    // {
    //     $table_name = 'specifications';

    //     $existing_record = $this->db->get_where($table_name, ['id' => $id])->row();
    //     if (!$existing_record) {
    //         return false;
    //     }

    //     $this->db->where('id', $id);
    //     $this->db->delete($table_name);

    //     return true;
    // }

    public function deleteSpecification($id)
    {
        $specifications_table = 'specifications';
        $user_specifications_table = 'user_specifications';

        // Check if the ID exists in user_specifications table
        $user_specification_exists = $this->db->get_where($user_specifications_table, ['specification_id' => $id])->row();
        if ($user_specification_exists) {
            // If ID exists in user_specifications table, provide a user message
            return "هذا المخصص موجود لموظف بل الفعل";
        }

        // Check if the ID exists in specifications table
        $existing_record = $this->db->get_where($specifications_table, ['id' => $id])->row();
        if (!$existing_record) {
            return "Specification with ID $id not found.";
        }

        // If ID exists in specifications table, proceed with deletion
        $this->db->where('id', $id);
        $this->db->delete($specifications_table);

        return true;
    }

    public function deleteOtherSpecifications($id)
    {
        $table_name = 'others_specifications';

        $existing_record = $this->db->get_where($table_name, ['id' => $id])->row();
        if (!$existing_record) {
            return false;
        }

        $this->db->where('id', $id);
        $this->db->delete($table_name);

        return true;
    }

    public function deletAllSpecificationForUser($id)
    {
        // Check if there are any records with the given emp_info_id
        $existing_records = $this->db->get_where('user_specifications', ['emp_info_id' => $id])->result();

        if (empty ($existing_records)) {
            return false; // No records found for the given emp_info_id
        }

        // Loop through the existing records and delete each one
        foreach ($existing_records as $record) {
            $this->db->delete('user_specifications', ['id' => $record->id]);
            // Assuming 'id' is the primary key of the 'specifications' table
        }
        return true;

    }

















}