<?php
class SuggestionModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    // get all Groups
    public function count_records($user_type, $user_hash)
    {
        $headers = apache_request_headers();
        $search = $this->input->get('q') ?? '';
        $type = $this->input->get('type') ?? '1';
        $this->db->select('count(*) as count');
        $this->db->from('suggestion');
        if ($search) {
            $this->db->like('text', $search);
        }
        if ($user_type != "9876533") {
            $this->db->where('suggestion.user_hash', $user_hash);
        }
        $this->db->where('type', $type);
        $query = $this->db->get();
        return $query->result_array()[0]['count'];
    }

    public function getSuggestions($user_type, $user_hash)
    {

        if ($this->isAdmin($user_hash)) {
            $headers = apache_request_headers();
            $page = $headers['page'] ?? 1;
            $perPage = $headers['perPage'] ?? 10;
            $offset = ($page - 1) * $perPage;
            $search = $this->input->get('q') ?? '';
            $type = $this->input->get('type') ?? '1';

            $this->db->select(
                'text,
            system_users.name as user,
            image_profile,
            suggestion.user_hash,
            (SELECT name FROM partments WHERE emp_information.partment = partments.hash) AS department,
            (SELECT name FROM system_group_name WHERE hash = system_users.user_type) AS job,
            suggestion.hash,
            suggestion.data_entry'
            );
            $this->db->from('suggestion');
            // Join system_users
            $this->db->join('system_users', 'suggestion.user_hash = system_users.hash', 'left');
            $this->db->join('emp_information', 'suggestion.user_hash = emp_information.user_hash', 'left');

            // Add search condition
            if ($search) {
                $this->db->like('suggestion.text', $search);
            }

            // Add condition to retrieve both sender and receiver suggestions
            // $this->db->group_start(); // Start grouping conditions
            // $this->db->where('suggestion.user_hash', $user_hash); // Suggestions sent by the user
            // $this->db->or_where('suggestion.sender_hash', $user_hash); // Suggestions received by the user
            // $this->db->group_end(); // End grouping conditions

            // Filter by type
            $this->db->where('suggestion.type', $type);

            // Order by
            $this->db->order_by('suggestion.id', 'DESC');

            // Limit the results
            $this->db->limit($perPage, $offset);

            $query = $this->db->get();
            return $query->result_array();

        }
        return [];
    }

    public function isAdmin($userHash)
    {
        $this->db->select('role_id')
            ->from('UserRoles')
            ->where('user_hash', $userHash)
            ->where('role_id', 4);
        $query = $this->db->get();
        return $query->num_rows() > 0;
    }

    public function getSuggestion($hash)
    {
        $this->db->select(
            'text,
            (select name from system_users where hash = user_hash) as user,
            user_hash,data_entry,type,
            hash'
        );
        $this->db->from('suggestion');
        $this->db->where('hash =', $hash);
        $query = $this->db->get();

        return $query->result_array();
    }

    public function getEmployeeShouldBeNotifiedWhenAddingSuggestion() {
       $this->db->select('user_hash')->from('UserRoles')
       ->where_in('role_id', [4 , 1]); 

       $query = $this->db->get();
       $users = $query->result_array();
       return $users;
    }

    public function addSuggestion($data)
    {
        $this->db->insert('suggestion', $data);
        return $this->db->insert_id();
    }

    public function updateSuggestion($hash, $data)
    {
        $this->db->where('hash', $hash);
        $this->db->update('suggestion', $data);
        return $this->db->affected_rows();
    }

    public function deleteSuggestion($hash)
    {
        $this->db->where('hash', $hash);
        $this->db->delete('suggestion');
        return $this->db->affected_rows();
    }
}
