<?php
class UserModel extends CI_Model
{

    const FINGERPRINT_TYPE = '111';
    const IN_TYPE = 'in';
    const OUT_TYPE = 'out';

    public function __construct()
    {
        parent::__construct();
        $this->load->database();
        // load Middleware
        $this->load->library('TokenMiddleware');
        $this->load->model('GroupScheduleModel');
        $this->load->model('FingerPrintSchedulerModel');
    }

    public function isUserHashDeleted($user_hash)
    {
        if ($user_hash == '1562195310922321') {
            return false;
        }
        $this->db->select("is_work, stop_work_date")->from("emp_information")->where("user_hash", $user_hash);
        // $this->db->where("is_work", "2");

        $this->db->group_start() // Start grouping conditions
            ->where('is_work', '2')
            ->group_start() // Start subgrouping conditions
            ->where('stop_work_date IS NULL', null, false)
            ->or_where('stop_work_date <=', date('Y-m-d'))
            ->group_end() // End subgroup
            ->group_end(); // End main group

        $query = $this->db->get();
        return $query->num_rows() > 0;
    }
    public function getUserName($hash)
    {
        $this->db->select('name');
        $this->db->from('system_users');
        $this->db->where('hash', $hash);
        $query = $this->db->get();
        // return username
        return $query->row_array()['name'];
    }

    public function getMobileToken($hash)
    {
        $this->db->select('mobile_token');
        $this->db->from('system_users');
        $this->db->where('hash', $hash);
        $query = $this->db->get();
        // return token
        return $query->row_array()['mobile_token'];
    }

    public function updateMobileToken($username, $password, $token)
    {
        $this->db->set('mobile_token', $token);
        $this->db->where('username', $username);
        $this->db->where('password', $password);
        $this->db->update('system_users');
        return $this->db->affected_rows();
    }

    public function updateToken($user_hash, $token)
    {
        $this->db->set('token', $token);
        $this->db->where('hash', $user_hash);
        $this->db->update('system_users');
        return $this->db->affected_rows();
    }

    public function getUser($username, $password)
    {
        $this->db->select('
        system_users.username,
        system_users.user_type,
        system_users.hash,
        system_users.name,
        system_users.image_profile,
        system_users_type.title AS user_type_name,
        system_users.mobile_token,
        system_users.token AS token,
        emp_information.partment,
        system_users.vication_agreement,
        emp_information.fingerprint_required,
        emp_information.is_work,

        GROUP_CONCAT(UserRoles.role_id) as role_ids
    ');
        $this->db->select("
    case
        when user_type = '9876533' then 1
        else 0
    end as is_admin
");
        $this->db->from('system_users');
        $this->db->join('emp_information', 'system_users.hash = emp_information.user_hash');
        $this->db->join('UserRoles', 'system_users.hash = UserRoles.user_hash', 'left'); // Use 'left' join to include users without roles
        $this->db->join('system_users_type', 'system_users_type.hash = system_users.user_type', 'left');
        $this->db->where('system_users.username', $username);
        $this->db->where('system_users.password', $password);
        $this->db->group_by('system_users.hash'); // Group by user hash to avoid duplicate rows
        $query = $this->db->get();
        return $query->row_array();
    }

    public function checkPermission()
    {
        $access = true;
        $token = $this->tokenmiddleware->main();
        $user_hash = $this->tokenmiddleware->checkToken($token)['hash'];

        $headers = apache_request_headers();
        try {
            // check if headers has location
            if (!isset($headers['location'])) {
                $access = true;
                return true;
            }
            $location = $headers['location'];
            $page = $this->db->query("SELECT id FROM pages WHERE location = '" . $location . "'")->row_array()['id'];
            if (!$page) {
                // insert page
                $this->db->query("INSERT INTO pages (name,location) VALUES ('" . $location . "','" . $location . "')");
            }
            $page_id = $this->db->query("SELECT id FROM pages WHERE location = '" . $location . "'")->row_array()['id'];
            // check if user has permission
            $permission = $this->db->query("SELECT access FROM permission WHERE user_hash = '" . $user_hash . "' and page_id = '" . $page_id . "'")->row_array();
            if (!$permission) {
                // insert permission
                $this->db->query("INSERT INTO permission (access,user_hash,page_id) VALUES ('0','" . $user_hash . "','" . $page_id . "')");
                $access = false;
            } else {
                if ($permission['access'] == 0) {
                    $access = false;
                }
            }
        } catch (Exception $e) {
            $access = true;
        }
        if ($access) {
            return true;
        } else {
            echo json_encode(
                array(
                    'status' => 403,
                    'message' => 'ليس لديك صلاحية للدخول لهذه الصفحة',
                    'data' => null,
                    'isAuth' => true,
                    'action' => true,
                    'needPermission' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
    }

    public function updateUserPermission($user_hash, $data, $senderHash)
    {
        if (!$this->checkUserPermissionChange($senderHash)) {
            return [
                'status' => 400,
                'message' => 'لا تمتلك الصلاحية للقيام بهذه العملية',
                'data' => null,
                'isAuth' => true,
                'action' => false,
                'hash' => $user_hash,
            ];
        }

        // loop through data
        foreach ($data as $key => $value) {
            // check if row exist
            $row = $this->db->query("SELECT id FROM permission WHERE user_hash = '" . $user_hash . "' and page_id = '" . $key . "'")->row_array();
            if (!$row) {
                // insert permission
                $this->db->query("INSERT INTO permission (access,user_hash,page_id) VALUES ('" . $value . "','" . $user_hash . "','" . $key . "')");
            } else {
                // update permission
                $this->db->query("UPDATE permission SET access = '" . $value . "' WHERE user_hash = '" . $user_hash . "' and page_id = '" . $key . "'");
            }
        }
        return [
            'status' => 200,
            'message' => 'تم تحديث الصلاحيات بنجاح',
            'data' => null,
            'isAuth' => true,
            'action' => true,
            'hash' => $user_hash,
        ];
    }

    public function getUserPermission($user_hash, $senderHash)
    {
        if ($this->checkUserPermissionRead($senderHash)) {
            $this->db->select('permission.page_id,pages.id as id, pages.name, pages.category, permission.access');
            $this->db->from('permission');
            $this->db->join('blocked_pages', 'permission.page_id = blocked_pages.pageId', 'left');
            $this->db->join('emp_information', 'permission.user_hash = emp_information.user_hash', 'left');
            $this->db->join('pages', 'permission.page_id = pages.id', 'left');
            $this->db->where('(blocked_pages.pageId IS NULL OR emp_information.pagesPermissions IS NULL)');
            $this->db->where('permission.user_hash', $user_hash);
            $this->db->group_by('permission.page_id');
            $query = $this->db->get();
            $userPermission = $query->result_array();
            $allPermission = $this->db->query("SELECT id,name, category FROM pages")->result_array();

            $data = [];

            // Initialize all pages with access = 0
            foreach ($allPermission as $page) {
                $data[$page['id']] = [
                    'name' => $page['name'],
                    'category' => $page['category'],
                    'access' => 0,
                    'id' => $page['id'],
                ];
            }

            // foreach ($userPermission as $key => $value) {
            //     $data[$value['id']]['name'] = $value['name'];
            //     $data[$value['id']]['category'] = $value['category'];
            //     $data[$value['id']]['access'] = $value['access'];
            //     $data[$value['id']]['id'] = $value['page_id'];
            //     $data[$value['page_id']]['access'] = $value['access'];
            // }
            foreach ($userPermission as $value) {
                $data[$value['page_id']]['access'] = $value['access'];
            }
            return $data;

        } else if ($this->ReadHisPages($senderHash)) {
            if ($user_hash !== $senderHash) {
                return [];
            }
            $this->db->select('p.page_id, pages.id as id, pages.name, pages.category, p.access, rp.role_id');
            $this->db->from('permission p');
            $this->db->join('pages', 'p.page_id = pages.id');
            $this->db->join('RolesPages rp', 'p.page_id = rp.page_id'); // This join makes sure we only get pages related to roles
            $this->db->join('UserRoles ur', 'ur.role_id = rp.role_id'); // Ensure you're joining on the correct field here
            $this->db->where('ur.user_hash', $senderHash); // Filter by the user's hash
            $query = $this->db->get();
            $rolePages = $query->result_array();
            $allPages = $this->db->query("SELECT id,name, category FROM pages")->result_array();

            $data = [];

            // Initialize all pages with access = 0
            foreach ($rolePages as $page) {
                $data[$page['page_id']] = [
                    'name' => $page['name'],
                    'category' => $page['category'],
                    'access' => 0,
                    'id' => $page['page_id'],
                ];
            }

            foreach ($rolePages as $value) {
                $data[$value['page_id']]['access'] = $value['access'];
            }
            return $data;
        } else {
            return [];
        }

    }

    private function checkUserPermissionRead($userHash)
    {
        $this->db->select('Roles.id')
            ->from('UserRoles')
            ->join('Roles', 'UserRoles.role_id = Roles.id')
            ->join('RolesPermissions', 'Roles.id = RolesPermissions.role_id')
            ->where('UserRoles.user_hash', $userHash)
            ->where('RolesPermissions.permission_id', 1); // 1 is the permission ID for read

        $query = $this->db->get();
        return $query->num_rows() > 0;
    }

    private function checkUserPermissionWrite($userHash)
    {
        $this->db->select('id')
            ->from('UserRoles')
            ->join('Roles', 'UserRoles.role_id = Roles.id')
            ->join('RolesPermissions', 'Roles.id = RolesPermissions.role_id')
            ->where('UserRoles.user_hash', $userHash)
            ->where('RolesPermissions.permission_id', 3); // 3 is the permission ID for write

        $query = $this->db->get();
        return $query->num_rows() > 0;
    }

    private function checkUserPermissionChange($userHash)
    {
        $this->db->select('Roles.id')
            ->from('UserRoles')
            ->join('Roles', 'UserRoles.role_id = Roles.id')
            ->join('RolesPermissions', 'Roles.id = RolesPermissions.role_id')
            ->where('UserRoles.user_hash', $userHash)
            ->where('RolesPermissions.permission_id', 2); // 2 is the permission ID for change

        $query = $this->db->get();
        return $query->num_rows() > 0;
    }

    private function ReadHisPages($userHash)
    {
        $this->db->select('Roles.id')
            ->from('UserRoles')
            ->join('Roles', 'UserRoles.role_id = Roles.id')
            ->join('RolesPermissions', 'Roles.id = RolesPermissions.role_id')
            ->where('UserRoles.user_hash', $userHash)
            ->where('RolesPermissions.permission_id', 4); // 4 is the permission ID for read his pages

        $query = $this->db->get();
        return $query->num_rows() > 0;
    }



    public function getUserActivePermission($user_hash)
    {
        $pages = $this->db->query("SELECT location FROM permission INNER JOIN pages ON permission.page_id = pages.id WHERE user_hash = '" . $user_hash . "' and access = '1'")->result_array();
        $data = [];
        foreach ($pages as $key => $value) {
            array_push($data, $value['location']);
        }
        return $data;
    }

    public function summary($user_hash)
    {
        $queries = [
            "tasks_uncompleted" => "SELECT  * FROM task WHERE receiver = '" . $user_hash . "' AND status = '111' ORDER BY id DESC LIMIT 3",
            "emergency_tasks" => "SELECT * FROM task WHERE emergency = '1' AND receiver = '" . $user_hash . "' ORDER BY id DESC LIMIT 3",
            "overdue_tasks" => "SELECT * FROM task WHERE CURDATE() > DATE(deadline) AND receiver = '" . $user_hash . "' AND status = '111' ORDER BY id DESC LIMIT 3",
            "completed_task_count" => "SELECT COUNT(id) AS complete_task FROM task WHERE MONTH(CURDATE()) = MONTH(DATE(deadline)) AND receiver = '" . $user_hash . "' AND status = '222'",
            "incomplete_task_count" => "SELECT COUNT(id) AS not_complete_task FROM task WHERE MONTH(CURDATE()) = MONTH(DATE(deadline)) AND receiver = '" . $user_hash . "' AND status = '111'",
            "arrive_time" => "SELECT IFNULL(TIME_FORMAT(TIME(fingerprint_time),'%r'),'Ù„Ø§ØªÙˆØ¬Ø¯ Ø¨ØµÙ…Ø© Ù„Ù‡Ø°Ø§ Ø§Ù„ÙŠÙˆÙ…') AS arrive_time FROM emp_fingerprint WHERE DATE(fingerprint_time) = CURDATE() AND user_hash='" . $user_hash . "' AND in_out='in'",
            "out_time" => "SELECT IFNULL(TIME_FORMAT(TIME(fingerprint_time),'%r'),'Ù„Ø§ØªÙˆØ¬Ø¯ Ø¨ØµÙ…Ø© Ù„Ù‡Ø°Ø§ Ø§Ù„ÙŠÙˆÙ…') AS out_time FROM emp_fingerprint WHERE DATE(fingerprint_time) = CURDATE() AND user_hash='" . $user_hash . "' AND in_out='out'",
            "normal_vacation_month" => "SELECT get_vacations_current_month('222','" . $user_hash . "')*8 AS normal_vacation_month, hour_day FROM vacations_type LIMIT 1",
            "normal_vacation_year" => "SELECT get_vacations_current_year('222','" . $user_hash . "')*8 AS normal_vacation_year, hour_day FROM vacations_type LIMIT 1",
            "half_vacation_month" => "SELECT get_vacations_current_month('333','" . $user_hash . "') AS half_vacation_month, hour_day FROM vacations_type LIMIT 1",
            "half_vacation_year" => "SELECT get_vacations_current_year('333','" . $user_hash . "') AS half_vacation_year, hour_day FROM vacations_type LIMIT 1",
            "balance_vacations_month" => "SELECT (CEIL((vacations_limit/12))*8-get_vacations_current_month('222',user_hash)*8-get_vacations_current_month('333',user_hash)) AS balance_vacations_month FROM emp_information WHERE user_hash='" . $user_hash . "'",
            "balance_vacations_year" => "SELECT (vacations_limit*8-get_vacations_current_year('222',user_hash)*8-get_vacations_current_year('333',user_hash)) AS balance_vacations_year FROM emp_information WHERE user_hash='" . $user_hash . "'",
            "work_in_check_distance" => "SELECT work_in_check_distance, work_out_check_distance FROM emp_information WHERE user_hash='" . $user_hash . "'",
        ];

        $results = [];
        foreach ($queries as $resultName => $query) {
            $result = $this->db->query($query);
            // $results[$resultName] in ['tasks_completed', 'emergency_tasks', 'overdue_tasks']
            if (in_array($resultName, ['tasks_uncompleted', 'emergency_tasks', 'overdue_tasks'])) {
                $results[$resultName] = $result->result_array();
            } else {
                if ($resultName == 'arrive_time') {
                    $results[$resultName] = $result->row_array()['arrive_time'] ?? 'لم يتم تسجيل الحضور';
                } else if ($resultName == 'out_time') {
                    $results[$resultName] = $result->row_array()['out_time'] ?? 'لم يتم تسجيل الانصراف';
                } else {
                    $results[$resultName] = $result->row_array();
                }
            }
        }

        return $results;
    }

    public function getVacationsSummary($hash)
    {
        $query = $this->db->query("
            SELECT
                (get_vacations_current_month('222', '$hash') * 8) AS normal_vacation_month,
                hour_day AS normal_vacation_hour_day
            FROM
                vacations_type
            LIMIT 1;

            SELECT
                (get_vacations_current_year('222', '$hash') * 8) AS normal_vacation_year,
                hour_day AS normal_vacation_hour_day
            FROM
                vacations_type
            LIMIT 1;

            SELECT
                get_vacations_current_month('333', '$hash') AS half_vacation_month,
                hour_day AS half_vacation_hour_day
            FROM
                vacations_type
            LIMIT 1;

            SELECT
                get_vacations_current_year('333', '$hash') AS half_vacation_year,
                hour_day AS half_vacation_hour_day
            FROM
                vacations_type
            LIMIT 1;

            SELECT
                (CEIL((vacations_limit / 12)) * 8 - get_vacations_current_month('222', user_hash) * 8 - get_vacations_current_month('333', user_hash)) AS balance_vacations_month
            FROM
                emp_information
            WHERE
                user_hash = '$hash';

            SELECT
                (vacations_limit * 8 - get_vacations_current_year('222', user_hash) * 8 - get_vacations_current_year('333', user_hash)) AS balance_vacations_year
            FROM
                emp_information
            WHERE
                user_hash = '$hash';
    ");

        return $query->row_array();
    }

    public function getPreviousFIngerIn($user_hash)
    {
        $today = date('Y-m-d');

        $query = $this->db->select('*')
            ->from('emp_fingerprint')
            ->where('in_out', 'in')
            ->where('user_hash', $user_hash)
            ->where('fingerprint_time >=', "$today 00:00:00")
            ->where('fingerprint_time <=', "$today 23:59:59")
            ->order_by('fingerprint_time', 'desc')
            ->limit(1)
            ->get();

        if ($query->num_rows() > 0) {
            return $query->row(); // Fetch the result
        } else {
            return [];
        }
    }

    private function convertDecimalToDatetime($decimalTime)
    {
        // Extract hours and minutes from the decimal time
        $hours = floor($decimalTime);
        $minutes = ($decimalTime - $hours) * 60;

        // Format the hours and minutes into H:i:s format
        $time = sprintf('%02d:%02d:00', $hours, $minutes);

        // Assume the date is the current date
        $currentDate = date('Y-m-d');

        // Combine the date with the end time
        $fingerprintTime = $currentDate . ' ' . $time;

        return $fingerprintTime;
    }

    public function hasFingerprint($data, $in_out)
    {
        $today = date('Y-m-d');
        $user_hash = $data['user_hash'];
        $this->db->where('user_hash', $user_hash)
            ->where('fingerprint_time >=', $today . ' 00:00:00')
            ->where('fingerprint_time <=', $today . ' 23:59:59')
            ->where('in_out', $in_out)
            ->where('data_entry !=', '');

        $query = $this->db->get('emp_fingerprint');
        return $query->num_rows() > 0;
    }

    public function fingerPrint($passedData)
    {

        $hash = $passedData['hash'];
        $x = $passedData['x'];
        $y = $passedData['y'];
        $type = $passedData['type'];
        $mobileInfo = $passedData['mobileInfo'];
        $fingerprint_time = $passedData['fingerprint_time'];


        $validationError = $this->validateInputs($hash, $x, $y, $type, $mobileInfo, $fingerprint_time);

        if ($validationError) {
            return $this->createErrorResponse($validationError);
        }


        $this->validateDistanceIfRequired($hash, $x, $y, $type);
        $this->checkMobileInfo($hash, $mobileInfo);
        $data = $this->prepareFingerPrintData($passedData);


        $isFlexibleSchedule = $this->isFlexibleSchedule($hash, $fingerprint_time);
        if ($isFlexibleSchedule) {
            return $this->handleFlexibleScheduleFingerPrint($hash, $x, $y, $type, $mobileInfo, $fingerprint_time, $data);
        }




        if ($type === self::IN_TYPE) {
            return $this->handleInFingerPrint($data);
        } else {
            return $this->handleOutFingerPrint($data);
        }
    }

    private function validateInputs($hash, $x, $y, $type, $mobileInfo, $fingerprint_time)
    {
        if (empty($hash) || !is_string($hash)) {
            return "Invalid hash.";
        }

        if (!is_numeric($x) || !is_numeric($y)) {
            return "Invalid coordinates.";
        }

        if (!in_array($type, [self::IN_TYPE, self::OUT_TYPE])) {
            return "Invalid type. Must be 'in' or 'out'.";
        }

        if (empty($mobileInfo) || !is_string($mobileInfo)) {
            return "Invalid mobile info.";
        }

        if (!strtotime($fingerprint_time)) {
            return "Invalid fingerprint time.";
        }

        return null;
    }

    private function validateDistanceIfRequired($hash, $x, $y, $type)
    {
        $check = $this->checkDistanceAllowed($hash, $type);
        $check = filter_var($check, FILTER_VALIDATE_BOOLEAN);
        if ($check) {
            $this->checkDistance($hash, $x, $y);
        }
    }

    private function prepareFingerPrintData($data)
    {
        return [
            'user_hash' => $data['hash'],
            'fingerprint_time' => date('Y-m-d H:i:s', strtotime($data['fingerprint_time'])),
            'in_out' => $data['type'],
            'x' => $data['x'],
            'y' => $data['y'],
            'location' => "POINT('{$data['x']}', '{$data['y']}')",
            'mobile_info' => $data['mobileInfo'],
            'hash' => $this->generateNewHash(),
            'fingerprint_type' => $data['fingerprint_type'],
            'islos' => $data['islos'] ?? NULL,
            'app_version' => $data['app_version'] ?? NULL,
        ];
    }


    private function isFlexibleSchedule($userHash, $fingerprint_time)
    {

        $date = date('Y-m-d', strtotime($fingerprint_time));

        // if employee has flexible_shceudle return true
        $this->db->select('id')->from('daily_flexible_schedule_targets')->where('user_hash', $userHash)->where('date', $date);
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return true;
        } else {
            $query2 = $this->db->select('id')->from('actual_schedules')->where('user_hash', $userHash)->where('date', $date)->get();
            if ($query2->num_rows() > 0) {
                return false;
            }
            $is_flexible = $this->db->select('flexible_schedule')
                ->from('emp_information')
                ->where('user_hash', $userHash)
                ->get()
                ->row(); // Use row() instead of row_object() for standard object access

            return $is_flexible->flexible_schedule == '1'; // Check if the result exists and compare
        }
    }

    private function handleFlexibleScheduleFingerPrint($hash, $x, $y, $type, $mobileInfo, $fingerprint_time, $data)
    {
        // $this->validateDistanceIfRequired($hash, $x, $y, $type);
        // $this->checkMobileInfo($hash, $mobileInfo);

        // Prepare data specific to flexible schedules
        // $data = $this->prepareFingerPrintData($hash, $x, $y, $type, $mobileInfo, $fingerprint_time);

        // Handle IN/OUT types with flexible schedule logic
        if ($type === self::IN_TYPE) {
            return $this->handleFlexibleInFingerPrint($data);

        } else {
            return $this->handleFlexibleOutFingerPrint($data);
        }
    }

    private function handleFlexibleInFingerPrint($data)
    {

        $lastFingerprint = $this->getLastFingerprint($data);

        // Check if the last entry exists and is of "in" type
        if ($lastFingerprint && $lastFingerprint['in_out'] == self::IN_TYPE) {
            return "لقد قمت بتسجيل البصمة مسبقاً";
        }

        $hasVacation = $this->isEmployeeHasVacation($data['user_hash'], $data['fingerprint_time']);
        if ($hasVacation) {
            $vacationDetails = $this->getEmployeeVacationDetails($data['user_hash'], $data['fingerprint_time']);
            if ($vacationDetails) {
                $dateStart = (new DateTime($vacationDetails['date_start']))->format('Y-m-d');
                $dateEnd = (new DateTime($vacationDetails['date_end']))->format('Y-m-d');

                return 'أنت في إجازة ' . $vacationDetails['type'] . ' من ' . $dateStart . ' إلى ' . $dateEnd;
            }
        }

        if (!$this->isHasWorkToday($data['user_hash'], $data['fingerprint_time'])) {

            $data['message'] = "ليس لديك عمل اليوم";
            $this->insertFingerPrintErrorLog($data);
            return $data['message'];
        }
        ;

        $data['is_flexible'] = 1;
        $this->insertFlexibleFingerPrint($data);
        $this->insertLocationToMovement($data);
        $this->notifyFirebase($data['user_hash'], self::IN_TYPE);
        return null;
    }

    private function getLastFingerprint($data)
    {
        $userHash = $data['user_hash']; // Extracting the user_hash from the input data

        // Get the current date and time
        $currentDateTime = date('Y-m-d H:i:s');

        // Get the time exactly 20 hours ago
        $previous20Hours = date('Y-m-d H:i:s', strtotime('-20 hours'));

        // Use CodeIgniter's Query Builder to execute the equivalent SQL query
        $this->db->select('*')
            ->from('emp_fingerprint')
            ->where('user_hash', $userHash)
            ->where('fingerprint_time >=', $previous20Hours) // Check from 20 hours ago
            ->where('fingerprint_time <=', $currentDateTime) // Up to the current time
            ->order_by('fingerprint_time', 'DESC') // Order by the fingerprint_time column (latest first)
            ->limit(1); // Limit to one result

        $query = $this->db->get(); // Execute the query

        if ($query->num_rows() > 0) {
            return $query->row_array(); // Return the result as an associative array
        }

        return null; // Return null if no rows found
    }

    private function isHasWorkToday($user_hash, $fingerprint_time)
    {
        $today = date('Y-m-d', strtotime($fingerprint_time));
        $maxAttempts = 2; // Maximum number of attempts to avoid infinite loop
        $attempts = 0;

        while ($attempts < $maxAttempts) {
            $this->db->select('id')->from('daily_flexible_schedule_targets')->where('user_hash', $user_hash)->where('date', $today)->where('is_deleted', '0');

            if ($this->db->get()->num_rows() > 0) {
                return true; // Record exists
            } else {
                $result = $this->GroupScheduleModel->insertOrUpdateDailyFleixbleScheduleIfNeeded($user_hash, $fingerprint_time);
                if (!$result) {
                    return false; // Insert or update failed
                }
            }

            $attempts++;
        }

        return false; // Maximum attempts reached without success
    }

    private function handleFlexibleOutFingerPrint($data)
    {

        $hasFingerPrintedInToday = $this->hasFingerPrintedInToday($data['user_hash'], date('Y-m-d'));
        if (!$hasFingerPrintedInToday) {
            $message = "لا يمكنك تسجيل بصمة الخروج دون تسجيل بصمة الدخول أولاً.";
            $data['message'] = $message;
            $this->insertFingerPrintErrorLog($data);
            return $message;
        }

        $lastFingerprintType = $this->getLastFingerprintType($data['user_hash']);
        if ($lastFingerprintType === self::OUT_TYPE) {
            $message = "لا يمكنك تسجيل بصمة الخروج مرتين دون تسجيل بصمة الدخول بينهما.";
            $data['message'] = $message;
            $this->insertFingerPrintErrorLog($data);
            return $message;
        }

        $data['is_flexible'] = 1;
        $this->insertFlexibleFingerPrint($data);
        $this->insertLocationToMovement($data);
        $this->notifyFirebase($data['user_hash'], self::OUT_TYPE);
    }

    private function generateNewHash()
    {
        return (string) rand(1000000000000000, 9999999999999999);
    }

    private function handleInFingerPrint($data)
    {


        $lastFingerprint = $this->getLastFingerprint($data);

        // Check if the last entry exists and is of "in" type
        if ($lastFingerprint && $lastFingerprint['in_out'] == self::IN_TYPE) {
            return "لقد قمت بتسجيل البصمة مسبقاً";
        }



        // $countFingerIn = $this->countFingerprint($data, self::IN_TYPE);
        // if ($countFingerIn > 0) {
        //     $countFingerOut = $this->countFingerprint($data, self::OUT_TYPE);
        //     if ($countFingerIn > $countFingerOut) {
        //         return $this->handleForgetToFingerOut($data);
        //     }
        // }

        $currentSchedule = $this->getCurrentSchedule($data['user_hash'], date('Y-m-d'), $data['fingerprint_time']);
        if (is_array($currentSchedule) && isset($currentSchedule['status']) && $currentSchedule['status'] === 'error') {
            $data['message'] = $currentSchedule['message'];
            $this->insertFingerPrintErrorLog($data);
            return $currentSchedule['message'];
        }

        // Continue if schedule is found
        $schedule = $currentSchedule['data'];

        if ($this->isWithinSchedule($data['fingerprint_time'], $schedule)) {
            $isCurrentScheduleLoggedBefore = $this->getIsCurrentScheduleLoggedBefore($schedule, $data['user_hash'], date('y-m-d'));
            if ($isCurrentScheduleLoggedBefore) {
                $data['message'] = "لقد قمت بتسجيل دخول وخروج من قبل لهذا الجدول لا يمكن الدخول مجددا , لتعديل بصمتك الرجاء التواصل مع أدمن النظام";
                $this->insertFingerPrintErrorLog($data);
                return $data['message'];
            }

            $data['schedule_id'] = $currentSchedule['data']['id'];
            $this->insertFingerPrint($data);
            $this->insertLocationToMovement($data);
            $this->notifyFirebase($data['user_hash'], self::IN_TYPE);
            return null;
        }

        // Constructing a detailed message
        $startTime = date('H:i A', strtotime($schedule['start']));
        $endTime = date('H:i A', strtotime($schedule['end']));
        $allowedEarlyCheckIn = date('H:i A', strtotime('-1 hour', strtotime($schedule['start'])));

        $message = "عفوا، جدولك اليومي يحدد أنه يجب عليك تسجيل بصمة دخولك بين الساعة $allowedEarlyCheckIn والساعة $startTime. الوقت الحالي هو $endTime وهو خارج الأوقات المحددة. يرجى التأكد من تسجيل بصمة الدخول ضمن الأوقات المقررة.";
        $data['message'] = $message;
        $this->insertFingerPrintErrorLog($data);
        return $message;
    }

    private function insertFingerPrintErrorLog($data)
    {
        $this->db->query("INSERT INTO emp_fingerprint_error_log (user_hash, fingerprint_time, fingerprint_type, in_out, location, mobile_info, hash , message) VALUES (?, ?, ?, ?, POINT(?, ?), ?, ?, ?)", [
            $data['user_hash'],
            $data['fingerprint_time'],
            $data['fingerprint_type'],
            $data['in_out'],
            $data['x'],
            $data['y'],
            $data['mobile_info'],
            $data['hash'],
            $data['message'],
        ]);
        // Insert into emp_fingerprint_approval table with status 'pending'
        $this->insertFingerprintApproval($data, $data['message']);
    }

    private function insertFingerprintApproval($data, $reason)
    {
        // Prepare the data to insert
        $insertData = [
            'user_hash' => $data['user_hash'],
            'fingerprint_type' => $data['fingerprint_type'],
            'fingerprint_time' => $data['fingerprint_time'],
            'in_out' => $data['in_out'],
            'mobile_info' => $data['mobile_info'],
            'schedule_id' => $data['schedule_id'] ?? null,
            'is_flexible' => $data['is_flexible'] ?? 0,
            'reason_for_approval' => $reason,
            'status' => 'pending', // Default status
        ];

        // Set the location using ST_PointFromText
        $this->db->set('location', "ST_PointFromText('POINT(" . floatval($data['y']) . " " . floatval($data['x']) . ")')", false);

        // Insert the data into the emp_fingerprint_approval table
        $this->db->insert('emp_fingerprint_approval', $insertData);
    }

    private function isWithinSchedule($fingerprint_time, $currentSchedule)
    {

        $fingerprintTime = strtotime($fingerprint_time);
        $scheduleStart = strtotime($currentSchedule['start']);
        $scheduleEnd = strtotime($currentSchedule['end']);

        // Allow check-in up to 1 hour before schedule start
        $allowedEarlyCheckIn = strtotime('-1 hour', $scheduleStart);

        return ($fingerprintTime >= $allowedEarlyCheckIn && $fingerprintTime <= $scheduleEnd);
    }

    private function handleOutFingerPrint($data)
    {
        $hasFingerPrintedInToday = $this->hasFingerPrintedInToday($data['user_hash'], date('Y-m-d'));
        if (!$hasFingerPrintedInToday) {
            $message = "لا يمكنك تسجيل بصمة الخروج دون تسجيل بصمة الدخول أولاً.";
            $data['message'] = $message;
            $this->insertFingerPrintErrorLog($data);
            return $message;
        }

        $lastFingerprintType = $this->getLastFingerprintType($data['user_hash']);
        if ($lastFingerprintType === self::OUT_TYPE) {
            $message = "لا يمكنك تسجيل بصمة الخروج مرتين دون تسجيل بصمة الدخول بينهما.";
            $data['message'] = $message;
            $this->insertFingerPrintErrorLog($data);
            return $message;
        }

        $this->insertFingerPrint($data);
        $this->insertLocationToMovement($data);
        $this->notifyFirebase($data['user_hash'], self::OUT_TYPE);
    }

    private function hasFingerPrintedInToday($userHash, $date)
    {
        // Check if the user has fingerprinted in today
        $this->db->select('id')
            ->from('emp_fingerprint')
            ->where('user_hash', $userHash)
            ->where('DATE(fingerprint_time)', $date)
            ->where('in_out', self::IN_TYPE);
        $query = $this->db->get();

        if ($query->num_rows() > 0) {
            return true;
        }

        // If not fingerprinted in today, check if fingerprinted in yesterday without fingerprinting out
        $yesterday = date('Y-m-d', strtotime($date . ' -1 day'));

        $this->db->select('id, in_out')
            ->from('emp_fingerprint')
            ->where('user_hash', $userHash)
            ->where('DATE(fingerprint_time)', $yesterday)
            ->order_by('fingerprint_time', 'DESC')
            ->limit(1);
        $queryLastFingerprintYesterday = $this->db->get();

        if ($queryLastFingerprintYesterday->num_rows() > 0) {
            $lastFingerprint = $queryLastFingerprintYesterday->row();
            if ($lastFingerprint->in_out == self::IN_TYPE) {
                return true;
            }
        }

        return false;
    }

    private function getLastFingerprintType($userHash)
    {
        $this->db->select('in_out')
            ->from('emp_fingerprint')
            ->where('user_hash', $userHash)
            ->order_by('fingerprint_time', 'DESC')
            ->limit(1);
        $query = $this->db->get();
        if ($query->num_rows() > 0) {
            return $query->row()->in_out;
        }
        return null;
    }
    private function insertFingerPrint($data)
    {
        $sql = "INSERT INTO emp_fingerprint (user_hash, fingerprint_time, fingerprint_type, in_out, location, mobile_info, hash, schedule_id, islos, app_version)
            VALUES (?, ?, ?, ?, POINT(?, ?), ?, ?, ?, ? , ?)";
        $this->db->query($sql, [
            $data['user_hash'],
            $data['fingerprint_time'],
            $data['fingerprint_type'],
            $data['in_out'],
            $data['x'],
            $data['y'],
            $data['mobile_info'],
            $data['hash'],
            $data['schedule_id'] ?? null,
            $data['islos'] ?? NULL,
            $data['app_version'] ?? NULL,
        ]);

        $data['fingerprint_id'] = $this->db->insert_id();

        $this->handleFingerprintScheduling($data);

    }

    public function handleFingerprintScheduling($data)
    {
        $fingerPrintTypeKey = $data['in_out'] == self::IN_TYPE ? 'fingerprint_in_id' : 'fingerprint_out_id';

        $deadline = date('Y-m-d H:i:s', strtotime($data['fingerprint_time'] . ' + 24 hours'));

        $fingerPrintScheduleData = array(
            'user_hash' => $data['user_hash'],
            $fingerPrintTypeKey => $data['fingerprint_id'],
            'date' => date('Y-m-d'),
            'dead_line' => $deadline,
            'is_flexible' => $data['is_flexible'] ?? 0,
        );

        if ($data['in_out'] == self::IN_TYPE) {
            $this->FingerPrintSchedulerModel->insert_in_fingerprint($fingerPrintScheduleData);
        } else {
            $this->FingerPrintSchedulerModel->update_out_fingerprint($data['user_hash'], $data['fingerprint_id']);
            return $data['fingerprint_id'];
        }
    }

    private function insertFlexibleFingerPrint($data)
    {
        $sql = "INSERT INTO emp_fingerprint (user_hash, fingerprint_time, fingerprint_type, in_out, location, mobile_info, hash, is_flexible, islos, app_version)
        VALUES (?, ?, ?, ?, POINT(?, ?), ?, ?, ?, ? , ?)";

        $this->db->query($sql, [
            $data['user_hash'],
            $data['fingerprint_time'],
            $data['fingerprint_type'],
            $data['in_out'],
            $data['x'],
            $data['y'],
            $data['mobile_info'],
            $data['hash'],
            $data['is_flexible'],
            $data['islos'] ?? NULL,
            $data['app_version'] ?? NULL,
        ]);

        $data['fingerprint_id'] = $this->db->insert_id();

        $this->handleFingerprintScheduling($data);
    }

    private function notifyFirebase($userHash, $status)
    {
        $this->jwtmiddleware->insertNotToFirebase([
            "type" => "fingerPrint",
            "status" => $status,
            "user" => $userHash,
        ]);
    }

    public function checkDistanceAllowed($hash, $type)
    {
        if ($type == "in") {
            $query = "select work_in_check_distance as allow from emp_information where user_hash='$hash'";
        } else if ($type == "out") {
            $query = "select work_out_check_distance as allow from emp_information where user_hash='$hash'";
        }
        $query = $this->db->query($query);
        $result = $query->row_array();
        return $result['allow'];
    }

    private function checkDistance($hash, $x, $y)
    {
        $query = $this->db->query(
            "
            SELECT
                ROUND(ST_Distance_Sphere(location, POINT('$x', '$y'))) AS distance_from_location,
                allow_merters
            FROM
                office_to_user
            INNER JOIN
                office_location ON office_to_user.office_hash = office_location.hash
            WHERE
                office_to_user.user_hash = '$hash'
            ORDER BY
                distance_from_location ASC;
            "
        );
        $results = $query->result_array();

        if (!empty($results)) {
            foreach ($results as $result) {
                $allow_merters = $result['allow_merters'];
                $distance_from_location = $result['distance_from_location'];

                // If at least one location is within the allowed distance, return true
                if ($distance_from_location <= $allow_merters) {
                    return true;
                }
            }

            // If no location is within the allowed distance, return false with the closest location's details
            $closestLocation = $results[0]; // The first row is the closest due to ORDER BY
            $allow_merters = $closestLocation['allow_merters'];
            $distance_from_location = $closestLocation['distance_from_location'];

            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لقد تجاوزت المسافة المسموح بها للدخول وهي ' . $allow_merters . ' متر وانت على بعد ' . $distance_from_location . ' متر من مكان العمل',
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        }
        return true;
    }
    private function checkMobileInfo($userHash, $mobileInfo)
    {
        // get mobile info from system_users
        $query = $this->db->query(
            "
            select
                mobile_info
            from
                system_users
            where
                hash='$userHash'
            limit 1;
            "
        );
        $mobile = $query->row_array()['mobile_info'];
        // if null set mobile info
        $query = $this->db->query(
            "
            update
                system_users
            set
                mobile_info='$mobileInfo'
            where
                hash='$userHash'
            limit 1;
            "
        );
        if ($mobile != $mobileInfo) {
            // insert device_log id, name, user_hash, old_mobile_info, new_mobile_info, status, hash, record_date
            $hash = $this->jwtmiddleware->createHash();
            $query = $this->db->query(
                "
                insert into
                    device_log(user_hash,old_mobile_info,new_mobile_info,status,hash,record_date)
                    values('$userHash','$mobile','$mobileInfo','1','$hash',now())
                "
            );
        }
    }

    private function handleForgetToFingerOut($data)
    {
        $hash = $data['hash'];
        $checkIn = $data['fingerprint_time'];
        unset($data['hash']);

        $prevoiusFingerIn = $this->getPreviousFIngerIn($data['user_hash']);

        $currentSchedule = $this->getCurrentSchedule($data['user_hash'], date('Y-m-d'), date('H:i:s', strtotime($prevoiusFingerIn->fingerprint_time)));

        if ($currentSchedule['status'] === 'success') {
            $fingerprint_time = date('H:i:s', strtotime($data['fingerprint_time']));
            list($hours, $minutes) = explode(':', $fingerprint_time);
            $fingerprintTimeFloat = (float) $hours + ((float) $minutes / 60.0);

            $scheduleStartFloat = (float) date('H:i', strtotime($currentSchedule['data']['start'])) + ((float) date('i', strtotime($currentSchedule['data']['start'])) / 60.0);
            $scheduleEndFloat = (float) date('H:i', strtotime($currentSchedule['data']['end'])) + ((float) date('i', strtotime($currentSchedule['data']['end'])) / 60.0);

            // Check if the current time falls within the schedule
            if ($fingerprintTimeFloat >= $scheduleStartFloat - 1 && $fingerprintTimeFloat <= $scheduleEndFloat) {
                return "لقد بصمت دخول لا يمكن بصم دخول مرتين لنفس الشيفت";
            } else {
                // Handle forget to finger out (clock out)
                $data['fingerprint_time'] = $this->convertDecimalToDatetime($scheduleEndFloat);
                $data['in_out'] = 'out';
                $this->fingerPrint($data['user_hash'], $data['x'], $data['y'], 'out', $data['mobile_info'], $data['fingerprint_time']);

                $data['hash'] = $hash;
                $data['fingerprint_time'] = $checkIn;
                $data['in_out'] = 'in';
                $data['fingerprint_type'] = '111';
                $this->fingerPrint($data['user_hash'], $data['y'], $data['y'], 'in', $data['mobile_info'], $data['fingerprint_time']);
            }
        } else {
            return $currentSchedule['message'];
        }
    }

    public function countFingerprint($data, $in_out)
    {
        $today = date('Y-m-d');
        $user_hash = $data['user_hash'];
        $this->db->where('user_hash', $user_hash)
            ->where('fingerprint_time >=', $today . ' 00:00:00')
            ->where('fingerprint_time <=', $today . ' 23:59:59')
            ->where('in_out', $in_out);

        $query = $this->db->get('emp_fingerprint');
        return $query->num_rows();
    }

    private function getCurrentSchedule($user_hash, $date, $fingerprint_time)
    {
        // Query the database to get the schedule data for the given user and date
        $this->db->select('schedule_data')
            ->from('actual_schedules')
            ->where('user_hash', $user_hash)
            ->where('date', $date);
        $data = $this->db->get()->row();



        if ($data && !empty($data->schedule_data)) {
            // Convert fingerprint time to Unix timestamp
            $fingerprint_time = strtotime($fingerprint_time);

            // Decode schedule data
            $schedule_array = json_decode($data->schedule_data, true);
            if (empty($schedule_array) || (is_array($schedule_array) && count(array_filter($schedule_array, 'count')) === 0)) {
                return [
                    'status' => 'error',
                    'message' => 'ليس لك مواعيد عمل اليوم',
                ];
            }

            // Iterate over the schedule data to find a matching schedule
            foreach ($schedule_array as $schedule) {
                // Handle hour-only format by appending ':00' for minutes
                $scheduleStart = strtotime($this->convertDecimalToTime($schedule['start']));
                $scheduleEnd = strtotime($this->convertDecimalToTime($schedule['end']));
                $scheduleId = $schedule['id'];
                if (isset($schedule['lastAllowanceTimeToCheckIn']) && $schedule['lastAllowanceTimeToCheckIn'] > 0) {
                    $lastAllowanceTimeToCheckIn = $scheduleStart + ($schedule['lastAllowanceTimeToCheckIn'] * 3600); // Add late allowance in seconds

                    if ($fingerprint_time > $lastAllowanceTimeToCheckIn) {
                        return [
                            'status' => 'error',
                            'message' => 'لا يمكنك تسجيل الدخول بعد الوقت المسموح به.',
                        ];
                    }
                }

                // Allow check-in up to 1 hour before the schedule start
                $allowedEarlyCheckIn = strtotime('-1 hour', $scheduleStart);

                if ($fingerprint_time >= $allowedEarlyCheckIn && $fingerprint_time <= $scheduleEnd) {
                    return [
                        'status' => 'success',
                        'data' => [
                            'start' => date('H:i', $scheduleStart),
                            'end' => date('H:i', $scheduleEnd),
                            'id' => $scheduleId,
                        ],
                    ];
                }
            }

            // No matching schedule found, construct a detailed message
            $startTimes = array_map(fn($s) => strtotime($this->convertDecimalToTime($s['start'])), $schedule_array);
            $endTimes = array_map(fn($s) => strtotime($this->convertDecimalToTime($s['end'])), $schedule_array);

            $earliestStart = min($startTimes);
            $latestEnd = max($endTimes);

            $startTime = date('h:i A', $earliestStart);
            $endTime = date('h:i A', $latestEnd);

            return [
                'status' => 'error',
                'message' => "عفوا، لا توجد فترة في جدولك تتطابق مع الوقت الحالي. يجب تسجيل بصمة دخولك بين الساعة $startTime والساعة $endTime حسب جدولك اليومي. يرجى مراجعة جدولك وضمان تسجيل بصمة الدخول ضمن الأوقات المقررة.",
            ];
        }

        $hasVacation = $this->isEmployeeHasVacation($user_hash, $fingerprint_time);
        if ($hasVacation) {
            $vacationDetails = $this->getEmployeeVacationDetails($user_hash, $fingerprint_time);
            if ($vacationDetails) {
                $dateStart = (new DateTime($vacationDetails['date_start']))->format('Y-m-d');
                $dateEnd = (new DateTime($vacationDetails['date_end']))->format('Y-m-d');

                return [
                    'status' => 'error',
                    'message' => 'أنت في إجازة ' . $vacationDetails['type'] . ' من ' . $dateStart . ' إلى ' . $dateEnd,
                ];
            }
        }

        $maxIterations = 3; // Set a limit to avoid infinite loop
        $iterations = 0;

        do {
            $result = $this->GroupScheduleModel->insertOrUpdateActualScheduleIfNeeded($user_hash, $fingerprint_time);
            $iterations++;

            // Break if max iterations reached
            if ($iterations >= $maxIterations) {
                break;
            }

        } while ($result);

        return [
            'status' => 'error',
            'message' => 'لم يتم العثور على بيانات الجدول لهذا المستخدم في التاريخ المحدد.',
        ];
    }

    private function getIsCurrentScheduleLoggedBefore($schedule, $user_hash, $current_date)
    {
        $this->db->select('COUNT(*) as count');
        $this->db->from('emp_fingerprint');
        $this->db->where('schedule_id', $schedule['id'])
            ->where('user_hash', $user_hash)
            ->where('in_out', 'in')
            ->where('DATE(fingerprint_time)', $current_date); // Corrected date comparison

        $query = $this->db->get();
        $result = $query->row();

        return $result->count > 0; // Return true if an entry exists, otherwise false
    }

    private function isEmployeeHasVacation($user_hash, $date)
    {
        $query = $this->db->select('id')->from('vacations')->where('user_hash', $user_hash)->where('date_start <=', $date)->where('date_end >=', $date)->get();
        if ($query->num_rows() > 0) {
            return true;
        }
        return false;
    }

    private function getEmployeeVacationDetails($user_hash, $date)
    {
        $query = $this->db->select('v.date_start, v.date_end, vt.type')
            ->from('vacations v')
            ->join('vacations_type vt', 'v.vication_type_hash = vt.hash')
            ->where('v.user_hash', $user_hash)
            ->where('v.date_start <=', $date)
            ->where('v.date_end >=', $date)
            ->get();
        if ($query->num_rows() > 0) {
            return $query->row_array();
        }
        return null;
    }

    private function convertDecimalToTime($decimalTime)
    {
        $hours = floor($decimalTime);
        $minutes = floor(($decimalTime - $hours) * 60);
        $seconds = round((($decimalTime - $hours) * 60 - $minutes) * 60);
        return sprintf('%02d:%02d:%02d', $hours, $minutes, $seconds);
    }

    private function convertHourToTime($hour, $currentDate)
    {
        // Ensure $hour is a well-formed numeric value
        $hour = (float) $hour; // Convert to float to ensure numeric operations

        // Extract hours and minutes from the decimal representation
        $hours = floor($hour); // Get the integer part (hours)
        $minutes = ($hour - $hours) * 60; // Get the decimal part and convert to minutes

        // Return the full datetime in 'Y-m-d H:i:s' format
        return sprintf('%s %02d:%02d:00', $currentDate, $hours, $minutes);
    }

    private function insertLocationToMovement($locationData)
    {

        $data = [
            'user_hash' => $locationData['user_hash'],
            'lat' => $locationData['x'],
            'lng' => $locationData['y'],

            'finger_print' => $locationData['in_out'],
            'timestamp' => $locationData['fingerprint_time'],
        ];

        if ($locationData['mobile_info'] != null) {
            $data['mobile_info'] = $locationData['mobile_info'];
        }

        $this->db->insert('employee_movements', $data);
    }

    public function checkIfPrint($hash, $type)
    {
        $query = $this->db->query(
            "
            select check_in_fingerprint('$hash','in') as count;
            "
        );
        $result = $query->row_array();
        $queryOut = $this->db->query(
            "
            select check_in_fingerprint('$hash','out') as count;
            "
        );
        $resultOut = $queryOut->row_array();
        if ($type == "out" && $result['count'] == 0) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لم تقم بتسجيل بصمة الدخول',
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else if ($type == "out" && $resultOut['count'] > 0) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لقد قمت بتسجيل البصمة مسبقا',
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else if ($type == "in" && $result['count'] > 0) {
            echo json_encode(
                array(
                    'status' => 400,
                    'message' => 'لقد قمت بتسجيل البصمة مسبقا',
                    'data' => null,
                    'isAuth' => true,
                ),
                JSON_UNESCAPED_UNICODE
            );
            exit();
        } else {
            return true;
        }
    }

    public function updateProfile($data, $hash)
    {
        $this->db->set('name', $data['name']);
        $this->db->set('username', $data['username']);
        $this->db->set('password', $data['password']);
        // $this->db->set('image_profile', $data['image_profile']);
        $this->db->where('hash', $hash);
        $this->db->update('system_users');
        return $this->db->affected_rows();
    }

    public function getUserRole($rold_id)
    {
        $query = $this->db->query(
            "
            select
                name
            from
                Roles
            where
                id = '$rold_id'
            limit 1;
            "
        );
        return $query->row_array()['name'];
    }

    private function createErrorResponse($message)
    {
        return [
            'status' => 400,
            'message' => $message,
            'data' => null,
            'isAuth' => true,
        ];
    }

    private function createSuccessResponse($message, $data = null)
    {
        return [
            'status' => 200,
            'message' => $message,
            'data' => $data,
            'isAuth' => true,
        ];
    }

    public function error_logs_fingerPrint($error_message, $error_code)
    {
        $data = array(
            'error_message' => $error_message, // The long error message is handled here
            'error_code' => $error_code,
        );
        return $this->db->insert('error_logs_fingerPrint', $data);
    }

    public function updateAuthToken($token, $id)
    {
        $this->db->where('id', $id); // Apply WHERE condition first
        $this->db->update('ftth_auth', ['token' => $token]); // Then update the table
        return $this->db->affected_rows(); // Return the number of affected rows
    }

    public function getAuthToken($id)
    {
        return $this->db->select('token')->from('ftth_auth')->where('id', $id)->get()->row_array();
    }

}
