<?php
class WhatsAppModel extends CI_Model
{
    public function __construct()
    {
        parent::__construct();
        $this->load->database();
    }

    public function counts_records()
    {
        return $this->db->select('COUNT(*) as count')->from('whatsapp')->get()->result_array()[0]['count'];
    }
    public function getMessages()
    {
        $page = $this->input->get('page') ?? 1;
        $perPage = $this->input->get('perPage') ?? 10;
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;
        return $this->db->select("
            id,
      CASE
                WHEN sender LIKE '964%' THEN CONCAT('0', SUBSTRING(SUBSTRING_INDEX(sender, '@', 1), 4))

                ELSE sender
            END AS sender,
            name,
            type,
            media,
            message,
            created_at
        ")
            ->from('whatsapp')
            ->order_by('id', 'desc')
            ->limit($perPage, $offset)
            ->like('message', $search)
            ->get()
            ->result_array();
    }


    // whats app template

    public function createTemplate($data)
    {
        $this->db->insert('whatsAppTemplates', $data);
        return $this->db->insert_id();
    }

    public function getTemplates()
    {
        $page = (int) ($this->input->get('page') ?? 1);
        $perPage = (int) ($this->input->get('perPage') ?? 10);
        $search = $this->input->get('q') ?? '';
        $offset = ($page - 1) * $perPage;

        // Build the query
        $this->db->select('id, name, message, notify_whatsapp_type')
            ->from('whatsAppTemplates');

        // Apply search if provided
        if (!empty($search)) {
            $this->db->like('name', $search);
        }

        // Get total count for pagination
        $totalCount = $this->db->count_all_results('', false);

        // Apply limit and offset for pagination
        $this->db->limit($perPage, $offset);
        $data = $this->db->get()->result_array();

        return [
            'data' => $data,
            'total' => $totalCount
        ];
    }
    public function getAllTemplatesWithNotifiy()
    {
        $this->db->select('whatsAppTemplates.id, notify_whatsapp_type.name, message, notify_whatsapp_type.ar_label, notify_whatsapp_type.id as notify_type_id, notify_whatsapp_type.hash_type as hash_type')
            ->from('whatsAppTemplates');
        $this->db->join('notify_whatsapp_type', 'whatsAppTemplates.notify_whatsapp_type = notify_whatsapp_type.id');
        $data = $this->db->get()->result_array();
        return $data;
    }


    public function getTemplate($id)
    {
        return $this->db->select('*')->from('whatsAppTemplates')->where('id', $id)->get()->row_array();
    }



    public function updateTemplate($id, $data)
    {
        $this->db->where('id', $id);
        $this->db->update('whatsAppTemplates', $data);
        return true;
    }
    public function deleteTemplate($id)
    {
        $this->db->where('id', $id);
        $this->db->delete('whatsAppTemplates');

        if ($this->db->affected_rows() > 0) {
            return true;
        }

        return false;
    }

    public function insertWhatsAppMessageLog($logData)
    {
        $this->db->insert('subscribers_whatsapp_message_send_log', $logData);
    }


    public function getWhatsAppNotifiesTypeOptions()
    {
        return $this->db->select('id as value , ar_label as label')->from('notify_whatsapp_type')->get()->result_array();
    }

    public function getWhatsAppNotifiesAvailable()
    {
        return $this->db->select('notify_whatsapp_type.id as value, notify_whatsapp_type.ar_label as label')
            ->from('notify_whatsapp_type')
            ->join('whatsAppTemplates', 'whatsAppTemplates.notify_whatsapp_type = notify_whatsapp_type.id', 'left')
            ->where('whatsAppTemplates.notify_whatsapp_type IS NULL')
            ->get()
            ->result_array();
    }





}
