<?php
date_default_timezone_set('Asia/Baghdad');

class ScheduledTaskProcessor
{

    private $db;

    public function __construct()
    {
        // Initialize the database connection (replace with your DB credentials)
        $this->db = new mysqli('localhost', 'redha', 'redhaRedha@1redha', 'newCompany');
        if ($this->db->connect_error) {
            die('Connection failed: ' . $this->db->connect_error);
        }
    }

    public function processTasks()
    {
        $now = date('Y-m-d H:i:s');
        // Fetch pending tasks scheduled for execution
        $query = "SELECT * FROM scheduled_tasks WHERE status = 'pending' AND execute_at <= ? LIMIT 10";
        $stmt = $this->db->prepare($query);
        $stmt->bind_param('s', $now);
        $stmt->execute();
        $result = $stmt->get_result();

        while ($task = $result->fetch_assoc()) {
            $this->processTask($task);
        }

        $stmt->close();
    }

    private function processTask($task)
    {
        // Mark task as in progress and set the executed_at timestamp
        $now = date('Y-m-d H:i:s');
        $updateQuery = "UPDATE scheduled_tasks SET status = 'in_progress', executed_at = ? WHERE id = ?";
        $stmt = $this->db->prepare($updateQuery);
        $stmt->bind_param('si', $now, $task['id']);
        $stmt->execute();
        $stmt->close();

        try {
            // Decode task data (assuming JSON format)
            $taskData = json_decode($task['task_data'], true);

            // Task-specific processing logic
            switch ($task['task_type']) {
                // case 'partial_vacation_OUT':
                //     $this->handlePartialVacation($taskData);
                //     break;
                // case 'partial_vacation_IN':
                //     $this->handlePartialVacationPrintIn($taskData);
                //     break;
                case 'calculate_all_employees':
                    $this->handleCalcAllEmployees($taskData);
                    $this->handleCalcAllFlexibleEmployees($taskData);
                    break;
                case 'calculate_specific_employees':
                    $this->handleCalcEmployees($taskData);
                    $this->handleCalcFlexibleEmployees($taskData);
                    break;
                // Add other task types here
            }

            // Mark task as completed
            $this->updateTaskStatus($task['id'], 'completed');
        } catch (Exception $e) {
            // Handle any errors and increment retry count
            $this->handleTaskFailure($task['id'], $e->getMessage());
        }
    }

    public function handleCalcAllFlexibleEmployees($taskData)
    {
        $date_start = $taskData['date_start'];
        $date_end = $taskData['date_end'];

        $this->processFlexibleEmployeesData($date_start, $date_end);

    }

    public function handleCalcAllEmployees($taskData)
    {
        $date_start = $taskData['date_start'];
        $date_end = $taskData['date_end'];
        $configId = $taskData['configId'];

        $user_hashes_array = $this->getAllEmployeeUserHashes();

        $this->processEmployeeData($date_start, $date_end, $user_hashes_array, $configId);
    }

    public function handleCalcEmployees($taskData)
    {
        // Check if $taskData is already an array
        if (is_array($taskData)) {
            $data = $taskData;
        } else {
            // Ensure $taskData is a string
            if (!is_string($taskData)) {
                $this->log("Invalid task data: expected a JSON string or an array.", 'error');
                return ['status' => 'error', 'message' => 'Invalid task data'];
            }

            $data = json_decode($taskData, true);

            // Check if json_decode was successful
            if (json_last_error() !== JSON_ERROR_NONE) {
                $this->log("JSON decode error: " . json_last_error_msg(), 'error');
                return ['status' => 'error', 'message' => 'JSON decode error'];
            }
        }

        // Extract date_start, date_end, configId, and employeesArr from taskData
        $date_start = $data['date_start'] ?? null;
        $date_end = $data['date_end'] ?? null;
        $configId = $data['configId'] ?? null;
        $employeesString = $data['employees_user_hash_array'] ?? null;

        // Check if required fields are present
        if (!$date_start || !$date_end || !$employeesString) {
            $this->log("Missing required fields in task data.", 'error');
            return ['status' => 'error', 'message' => 'Missing required fields'];
        }

        // Decode the string into an array if it's a string
        if (is_string($employeesString)) {
            $employeesArray = json_decode($employeesString, true);

            // Check if json_decode was successful
            if (json_last_error() !== JSON_ERROR_NONE) {
                $this->log("JSON decode error for employees array: " . json_last_error_msg(), 'error');
                return ['status' => 'error', 'message' => 'JSON decode error for employees array'];
            }
        } else {
            $employeesArray = $employeesString;
        }

        // Process the employee data
        return $this->processEmployeeData($date_start, $date_end, $employeesArray, $configId);
    }
    public function handleCalcFlexibleEmployees($taskData)
    {
        // Check if $taskData is already an array
        if (is_array($taskData)) {
            $data = $taskData;
        } else {
            // Ensure $taskData is a string
            if (!is_string($taskData)) {
                $this->log("Invalid task data: expected a JSON string or an array.", 'error');
                return ['status' => 'error', 'message' => 'Invalid task data'];
            }

            $data = json_decode($taskData, true);

            // Check if json_decode was successful
            if (json_last_error() !== JSON_ERROR_NONE) {
                $this->log("JSON decode error: " . json_last_error_msg(), 'error');
                return ['status' => 'error', 'message' => 'JSON decode error'];
            }
        }

        // Extract date_start, date_end, configId, and employeesArr from taskData
        $date_start = $data['date_start'] ?? null;
        $date_end = $data['date_end'] ?? null;
        $employeesString = $data['employees_user_hash_array'] ?? null;

        // Check if required fields are present
        if (!$date_start || !$date_end || !$employeesString) {
            $this->log("Missing required fields in task data.", 'error');
            return ['status' => 'error', 'message' => 'Missing required fields'];
        }

        // Decode the string into an array if it's a string
        if (is_string($employeesString)) {
            $employeesArray = json_decode($employeesString, true);

            // Check if json_decode was successful
            if (json_last_error() !== JSON_ERROR_NONE) {
                $this->log("JSON decode error for employees array: " . json_last_error_msg(), 'error');
                return ['status' => 'error', 'message' => 'JSON decode error for employees array'];
            }
        } else {
            $employeesArray = $employeesString;
        }

        // Process the employee data
        return $this->processFlexbileEmployeeData($date_start, $date_end, $employeesArray);
    }

    public function processEmployeeData($date_start, $date_end, $user_hashes_array, $configId = null)
    {
        foreach ($user_hashes_array as $user_hash) {
            $results = $this->getEmployeeData($date_start, $date_end, $user_hash);

            if ($configId) {
                $configData = $this->getConfigDataById($configId);
                foreach ($results as &$result) {
                    $calculation = $this->calculateLateArrivalAndEarlyDeparture($result, $configData);
                    $result = array_merge($result, $calculation);
                    echo "Insert the row " . $result['user_hash'] . "\n";
                    $this->insertCalculatedData($result);
                    $this->updateOutReference($this->db, $result);

                }
            } else {
                foreach ($results as &$result) {
                    $calculation = $this->calculateLateArrivalAndEarlyDeparture($result);
                    $result = array_merge($result, $calculation);
                    echo "Insert the row " . $result['user_hash'] . "\n";
                    $this->insertCalculatedData($result);
                    $this->updateOutReference($this->db, $result);
                }
            }
        }

        return ['status' => 'success'];
    }
    public function processFlexibleEmployeesData($dateStart, $dateEnd)
    {

        $mysqli = $this->db;
        $mysqli->begin_transaction();

        // SQL query to retrieve fingerprint data
        $fingerprints = $this->getFingerPrints($mysqli, $dateStart, $dateEnd);

        $fingerprintGroups = [];
        $skippedIns = [];
        $skippedOuts = [];
        $ins = [];
        $outs = [];

        // Filter 'in' and 'out' records
        foreach ($fingerprints as $record) {
            if ($record['in_out'] === 'in') {
                $ins[] = $record;
            } elseif ($record['in_out'] === 'out') {
                $outs[] = $record;
            }
        }

        // Match 'in' and 'out' records
        foreach ($ins as $inRecord) {
            $nearestOut = $this->findNearestOut($inRecord, $outs); // Function to find the nearest 'out'
            if ($nearestOut) {

                $updateSql = "UPDATE emp_fingerprint SET in_out_reference = ? WHERE id = ? AND in_out_reference IS NULL";
                $updateStmt = $mysqli->prepare($updateSql);
                $updateStmt->bind_param('ii', $inRecord['id'], $nearestOut['id']); // Set in_out_reference to the 'in' record's id
                $updateStmt->execute();

                if (!isset($fingerprintGroups[$inRecord['user_hash']])) {
                    $fingerprintGroups[$inRecord['user_hash']] = [];
                }

                if (!isset($fingerprintGroups[$inRecord['user_hash']][$inRecord['date']])) {
                    $fingerprintGroups[$inRecord['user_hash']][$inRecord['date']] = [
                        'ins' => [],
                        'outs' => [],
                        'target_hours' => floatval($inRecord['target_hours']),
                    ];
                }

                $fingerprintGroups[$inRecord['user_hash']][$inRecord['date']]['ins'][] = $inRecord['fingerprint_time'];
                $fingerprintGroups[$inRecord['user_hash']][$inRecord['date']]['outs'][] = $nearestOut['fingerprint_time'];
            } else {
                $skippedIns[] = $inRecord;
            }
        }

        $skippedOuts = $outs;

        // Handle skipped 'in' records
        if (!empty($skippedIns)) {
            foreach ($skippedIns as $skipIn) {
                $updateSql = "UPDATE emp_fingerprint SET status = 'skipped_in' WHERE id = ?";
                $updateStmt = $mysqli->prepare($updateSql);
                $updateStmt->bind_param('i', $skipIn['id']);
                $updateStmt->execute();
            }
        }

        // Handle skipped 'out' records
        if (!empty($skippedOuts)) {
            foreach ($skippedOuts as $skipOut) {
                $updateSql = "UPDATE emp_fingerprint SET status = 'skipped_out' WHERE id = ?";
                $updateStmt = $mysqli->prepare($updateSql);
                $updateStmt->bind_param('i', $skipOut['id']);
                $updateStmt->execute();
            }
        }

        // Process valid fingerprint records
        foreach ($fingerprintGroups as $userHash => $dates) {
            foreach ($dates as $date => $data) {
                $inFingerprintTimes = $data['ins'];
                $outFingerprintTimes = $data['outs'];
                $targetHours = $data['target_hours'];
                $totalWorkedHours = 0;
                $usedOutTimes = [];

                foreach ($inFingerprintTimes as $inTime) {
                    $closestOutTime = null;
                    $minDiff = PHP_INT_MAX;

                    foreach ($outFingerprintTimes as $outIndex => $outTime) {
                        if (in_array($outIndex, $usedOutTimes)) {
                            continue;
                        }

                        $diff = strtotime($outTime) - strtotime($inTime);
                        if ($diff >= 0 && $diff < $minDiff) {
                            $minDiff = $diff;
                            $closestOutTime = $outTime;
                            $closestOutIndex = $outIndex;
                        }
                    }

                    if ($closestOutTime !== null) {
                        $workedHours = (strtotime($closestOutTime) - strtotime($inTime)) / 3600;
                        $totalWorkedHours += $workedHours;
                        $usedOutTimes[] = $closestOutIndex;
                    }
                }

                // Calculate missed hours and overtime
                $missedHours = max(0, $targetHours - $totalWorkedHours);
                $overTimeHoursDuration = max(0, $totalWorkedHours - $targetHours);
                $overTimeHours = 0;

                // Check if overtime hours is greater than or equal to allowed overtime
                if ($overTimeHours >= $this->getAllowanceOvertime() ?? 0) {
                    $overTimeHours = $overTimeHoursDuration;
                }

                // Insert into employees_worked_hours
                $insertSql = "INSERT INTO employees_worked_hours
                (user_hash, date, total_missed_hours, total_worked_hours, target_hours, overtime_hours, is_flexible)
                VALUES (?, ?, ?, ?, ?, ?, 1)
                ON DUPLICATE KEY UPDATE
                total_missed_hours = VALUES(total_missed_hours),
                total_worked_hours = VALUES(total_worked_hours),
                target_hours = VALUES(target_hours),
                overtime_hours = VALUES(overtime_hours),
                updated_at = NOW()";

                $insertStmt = $mysqli->prepare($insertSql);
                $insertStmt->bind_param('ssdddd', $userHash, $date, $missedHours, $totalWorkedHours, $targetHours, $overTimeHours);
                if ($insertStmt->execute()) {
                    echo "Data for User Hash: $userHash on Date: $date inserted successfully.\n";
                } else {
                    echo "Error inserting data for User Hash: $userHash on Date: $date.\n";
                }
            }
        }

        // Commit the transaction
        $mysqli->commit();
    }

    public function processFlexbileEmployeeData($dateStart, $dateEnd, $user_hashes_array)
    {
        foreach ($user_hashes_array as $user_hash) {
            $mysqli = $this->db;
            $mysqli->begin_transaction();

            // SQL query to retrieve fingerprint data
            $fingerprints = $this->getFingerPrints($mysqli, $dateStart, $dateEnd, $user_hash);

            $fingerprintGroups = [];
            $skippedIns = [];
            $skippedOuts = [];
            $ins = [];
            $outs = [];

            // Filter 'in' and 'out' records
            foreach ($fingerprints as $record) {
                if ($record['in_out'] === 'in') {
                    $ins[] = $record;
                } elseif ($record['in_out'] === 'out') {
                    $outs[] = $record;
                }
            }

            // Match 'in' and 'out' records
            foreach ($ins as $inRecord) {
                $nearestOut = $this->findNearestOut($inRecord, $outs); // Function to find the nearest 'out'
                if ($nearestOut) {
                    if (!isset($fingerprintGroups[$inRecord['user_hash']])) {
                        $fingerprintGroups[$inRecord['user_hash']] = [];
                    }

                    if (!isset($fingerprintGroups[$inRecord['user_hash']][$inRecord['date']])) {
                        $fingerprintGroups[$inRecord['user_hash']][$inRecord['date']] = [
                            'ins' => [],
                            'outs' => [],
                            'target_hours' => floatval($inRecord['target_hours']),
                        ];
                    }

                    $fingerprintGroups[$inRecord['user_hash']][$inRecord['date']]['ins'][] = $inRecord['fingerprint_time'];
                    $fingerprintGroups[$inRecord['user_hash']][$inRecord['date']]['outs'][] = $nearestOut['fingerprint_time'];
                } else {
                    $skippedIns[] = $inRecord;
                }
            }

            $skippedOuts = $outs;

            // Handle skipped 'in' records
            if (!empty($skippedIns)) {
                foreach ($skippedIns as $skipIn) {
                    $updateSql = "UPDATE emp_fingerprint SET status = 'skipped_in' WHERE id = ?";
                    $updateStmt = $mysqli->prepare($updateSql);
                    $updateStmt->bind_param('i', $skipIn['id']);
                    $updateStmt->execute();
                }
            }

            // Handle skipped 'out' records
            if (!empty($skippedOuts)) {
                foreach ($skippedOuts as $skipOut) {
                    $updateSql = "UPDATE emp_fingerprint SET status = 'skipped_out' WHERE id = ?";
                    $updateStmt = $mysqli->prepare($updateSql);
                    $updateStmt->bind_param('i', $skipOut['id']);
                    $updateStmt->execute();
                }
            }

            // Process valid fingerprint records
            foreach ($fingerprintGroups as $userHash => $dates) {
                foreach ($dates as $date => $data) {
                    $inFingerprintTimes = $data['ins'];
                    $outFingerprintTimes = $data['outs'];
                    $targetHours = $data['target_hours'];
                    $totalWorkedHours = 0;
                    $usedOutTimes = [];

                    foreach ($inFingerprintTimes as $inTime) {
                        $closestOutTime = null;
                        $minDiff = PHP_INT_MAX;

                        foreach ($outFingerprintTimes as $outIndex => $outTime) {
                            if (in_array($outIndex, $usedOutTimes)) {
                                continue;
                            }

                            $diff = strtotime($outTime) - strtotime($inTime);
                            if ($diff >= 0 && $diff < $minDiff) {
                                $minDiff = $diff;
                                $closestOutTime = $outTime;
                                $closestOutIndex = $outIndex;
                            }
                        }

                        if ($closestOutTime !== null) {
                            $workedHours = (strtotime($closestOutTime) - strtotime($inTime)) / 3600;
                            $totalWorkedHours += $workedHours;
                            $usedOutTimes[] = $closestOutIndex;
                        }
                    }

                    // Calculate missed hours and overtime
                    $missedHours = max(0, $targetHours - $totalWorkedHours);
                    $overTimeHoursDuration = max(0, $totalWorkedHours - $targetHours);
                    $overTimeHours = 0;

                    // Check if overtime hours is greater than or equal to allowed overtime
                    if ($overTimeHours >= $this->getAllowanceOvertime() ?? 0) {
                        $overTimeHours = $overTimeHoursDuration;
                    }
                    // Insert into employees_worked_hours
                    $insertSql = "INSERT INTO employees_worked_hours
                    (user_hash, date, total_missed_hours, total_worked_hours, target_hours, overtime_hours, is_flexible)
                    VALUES (?, ?, ?, ?, ?, ?, 1)
                    ON DUPLICATE KEY UPDATE
                    total_missed_hours = VALUES(total_missed_hours),
                    total_worked_hours = VALUES(total_worked_hours),
                    target_hours = VALUES(target_hours),
                    overtime_hours = VALUES(overtime_hours),
                    updated_at = NOW()";

                    $insertStmt = $mysqli->prepare($insertSql);
                    $insertStmt->bind_param('ssddddd', $userHash, $date, $missedHours, $totalWorkedHours, $targetHours, $overTimeHours);
                    if ($insertStmt->execute()) {
                        echo "Data for User Hash: $userHash on Date: $date inserted successfully.\n";
                    } else {
                        echo "Error inserting data for User Hash: $userHash on Date: $date.\n";
                    }
                }
            }
            // Commit the transaction
            $mysqli->commit();
        }
    }
    private function getFingerPrints($mysqli, $dateStart, $dateEnd, $user_hash = null)
    {
        $fingerPrintDataSql = "SELECT
        emp_fingerprint.id,
        emp_fingerprint.user_hash,
        DATE(fingerprint_time) AS date,
        fingerprint_time,
        in_out,
        target_hours,
        fingerprint_type
    FROM
        emp_fingerprint
        LEFT JOIN daily_flexible_schedule_targets
            ON daily_flexible_schedule_targets.user_hash = emp_fingerprint.user_hash
            AND DATE(emp_fingerprint.fingerprint_time) = daily_flexible_schedule_targets.date
        LEFT JOIN actual_schedules ach
            ON ach.user_hash = emp_fingerprint.user_hash
            AND DATE(emp_fingerprint.fingerprint_time) = ach.date
    WHERE
        is_flexible = '1'
        AND (daily_flexible_schedule_targets.user_hash IS NOT NULL OR in_out = 'out')
        AND ach.user_hash IS NULL
        AND DATE(fingerprint_time) >= ?
        AND (
            (DATE(fingerprint_time) < ? AND in_out != 'out') OR
            (DATE(fingerprint_time) <= ? AND in_out = 'out')
        )";

        if ($user_hash !== null) {
            $fingerPrintDataSql .= " AND emp_fingerprint.user_hash = ?";
        }

        $fingerPrintDataSql .= " ORDER BY fingerprint_time DESC";

        // Prepare and execute the query
        $stmt = $mysqli->prepare($fingerPrintDataSql);
        if ($user_hash !== null) {
            $stmt->bind_param('ssss', $dateStart, $dateEnd, $dateEnd, $user_hash);
        } else {
            $stmt->bind_param('sss', $dateStart, $dateEnd, $dateEnd);
        }
        $stmt->execute();
        $result = $stmt->get_result();
        $fingerprints = $result->fetch_all(MYSQLI_ASSOC);
        return $fingerprints;
    }
    private function log(string $message, string $level = 'info'): void
    {
        // Implement your logging mechanism here
        // For example, you can use Monolog or a simple file-based logger
        echo strtoupper($level) . ": " . $message . "\n"; // Replace this with actual logging
    }

    public function findNearestOut($inRecord, &$outs)
    {
        $nearestOut = null;
        $minDifference = 24 * 3600; // 24 hours in seconds

        // First pass: prioritize fingerprint_type = '111'
        foreach ($outs as $key => $outRecord) {
            if ($inRecord['user_hash'] === $outRecord['user_hash'] && $outRecord['fingerprint_type'] == '111') {
                $inTime = strtotime($inRecord['fingerprint_time']);
                $outTime = strtotime($outRecord['fingerprint_time']);
                $timeDifference = $outTime - $inTime;

                // Ensure the 'out' time is after 'in' and within 24 hours
                if ($timeDifference > 0 && $timeDifference <= $minDifference) {
                    $nearestOut = $outRecord;
                    $minDifference = $timeDifference;
                    unset($outs[$key]); // Remove the matched 'out' entry to avoid reuse
                    break; // Exit loop as we found the highest priority match
                }
            }
        }

        // Second pass: if no match found with fingerprint_type = '111', check for fingerprint_type = '0'
        if (!$nearestOut) {
            foreach ($outs as $key => $outRecord) {
                if ($inRecord['user_hash'] === $outRecord['user_hash'] && $outRecord['fingerprint_type'] == '0') {
                    $inTime = strtotime($inRecord['fingerprint_time']);
                    $outTime = strtotime($outRecord['fingerprint_time']);
                    $timeDifference = $outTime - $inTime;

                    // Ensure the 'out' time is after 'in' and within 24 hours
                    if ($timeDifference > 0 && $timeDifference <= $minDifference) {
                        $nearestOut = $outRecord;
                        $minDifference = $timeDifference;
                        unset($outs[$key]); // Remove the matched 'out' entry to avoid reuse
                        break; // Exit loop as we found a match
                    }
                }
            }
        }

        return $nearestOut;
    }

    public function getEmployeeData($date_start, $date_end, $user_hash)
    {

        $query = "
                        SELECT
                JSON_OBJECT(
                    'user_hash', ei.user_hash,
                    'schedule_for_day', asch.schedule_data,
                    'allowance_time', asch.allowance_time,
                    'in_fingerprint_time', CONCAT(
                        '[',
                        GROUP_CONCAT(
                            CASE
                                WHEN ef.in_out = 'in' AND DATE(ef.fingerprint_time) = asch.date
                                THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"')
                            END
                            ORDER BY ef.fingerprint_time ASC
                            SEPARATOR ','
                        ),
                        ']'
                    ),
                    'out_fingerprint_time', CONCAT(
                        '[',
                        GROUP_CONCAT(
                            CASE
                                WHEN ef.in_out = 'out' AND ef.in_out_reference IS NOT NULL AND DATE(ef.fingerprint_time) = asch.date
                                THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"')
                            END
                            ORDER BY ef.fingerprint_time ASC
                            SEPARATOR ','
                        ),
                        ']'
                    ),
                        'in_fingerprint_time_schedule_id', CONCAT(
                        '[',
                        GROUP_CONCAT(
                            CASE
                                WHEN ef.in_out = 'in' AND DATE(ef.fingerprint_time) = asch.date
                                THEN CONCAT('\"', ef.schedule_id, '\"')
                            END
                            ORDER BY ef.fingerprint_time ASC
                            SEPARATOR ','
                        ),
                        ']'
                    ),
                    'out_fingerprint_time_schedule_id', CONCAT(
                        '[',
                        GROUP_CONCAT(
                            CASE
                                WHEN ef.in_out = 'out' AND ef.in_out_reference IS NOT NULL AND DATE(ef.fingerprint_time) = asch.date
                                THEN CONCAT('\"', ef.schedule_id, '\"')
                            END
                            ORDER BY ef.fingerprint_time ASC
                            SEPARATOR ','
                        ),
                        ']'
                    ),
                    'date', asch.date
                ) AS employee_data
            FROM emp_information ei
            JOIN actual_schedules asch ON ei.user_hash = asch.user_hash
            LEFT JOIN emp_fingerprint ef ON ei.user_hash = ef.user_hash
            WHERE asch.date >= ? AND asch.date <= ? AND ei.user_hash = ?
            GROUP BY ei.user_hash, asch.date
        ";

        $stmt = $this->db->prepare($query);
        $stmt->bind_param('sss', $date_start, $date_end, $user_hash);
        $stmt->execute();
        $result = $stmt->get_result();

        $results = [];
        while ($row = $result->fetch_assoc()) {
            $results[] = json_decode($row['employee_data'], true);
        }

        $stmt->close();
        return $results;
    }

    // public function getEmployeeFlexibleData($date_start, $date_end, $user_hash)
    // {

    //     $query = "SELECT
    //     emp_fingerprint.id,
    //     emp_fingerprint.user_hash,
    //     DATE(fingerprint_time) AS date,
    //     fingerprint_time,
    //     in_out,
    //     target_hours,
    //     fingerprint_type
    // FROM
    //     emp_fingerprint
    //         LEFT JOIN
    //     daily_flexible_schedule_targets ON daily_flexible_schedule_targets.user_hash = emp_fingerprint.user_hash
    //         AND DATE(emp_fingerprint.fingerprint_time) = daily_flexible_schedule_targets.date
    //         LEFT JOIN
    //     actual_schedules ach ON ach.user_hash = emp_fingerprint.user_hash AND
    //     DATE(emp_fingerprint.fingerprint_time) = ach.date
    // WHERE
    //     is_flexible = '1'
    //     AND (daily_flexible_schedule_targets.user_hash IS NOT NULL OR in_out = 'out')
    //     AND ach.user_hash IS NULL
    //     AND DATE(fingerprint_time) >= '$date_start'
    //     AND (
    //         (DATE(fingerprint_time) < '$date_end' AND in_out != 'out') OR
    //         (DATE(fingerprint_time) <= '$date_end' AND in_out = 'out')
    //     )
    //     AND
    // ORDER BY fingerprint_time DESC";
    // }

    private function getConfigDataById($configId)
    {
        $query = "SELECT * FROM emp_worked_hours_flags WHERE id = ?";
        $stmt = $this->db->prepare($query);
        $stmt->bind_param('i', $configId);
        $stmt->execute();
        $result = $stmt->get_result();
        return $result->num_rows > 0 ? $result->fetch_assoc() : null;
    }

    public function calculateLateArrivalAndEarlyDeparture($data, $configData = null)
    {
        $totalLateArrivalMinutes = 0;
        $totalEarlyDepartureMinutes = 0;
        $totalScheduledMinutes = 0;
        $totalWorkedMinutes = 0;
        $totalOvertimeMinutes = 0;

        if ($configData == null) {
            $calc_full_late = $this->getAllowanceTimeFlag();
            $is_calc_by_missing = $this->getIsPenaltyBasedOnMissingFlag();
            $over_time_allowance = $this->getAllowanceOvertime() ?? 0;
        } else {
            $calc_full_late = $configData['calculate_full_late'];
            $is_calc_by_missing = $configData['calc_by_missing'];
            $over_time_allowance = $configData['over_time_allowance'] ?? 0;
        }
        $over_time_allowance = ($over_time_allowance ?? 0) * 60;

        if (!isset($data['schedule_for_day']) || !is_array($data['schedule_for_day']) || empty($data['schedule_for_day'])) {
            return [
                'error' => 'Missing or invalid schedule_for_day data.',
                'status' => 'error',
            ];
        }

        $allowanceTime = isset($data['allowance_time']) ? (int) $data['allowance_time'] : null;

        $inFingerprintTimes = isset($data['in_fingerprint_time']) ? json_decode($data['in_fingerprint_time'], true) : [];
        $outFingerprintTimes = isset($data['out_fingerprint_time']) ? json_decode($data['out_fingerprint_time'], true) : [];
        $ins = isset($data['in_fingerprint_time']) ? json_decode($data['in_fingerprint_time'], true) : [];
        $outs = isset($data['out_fingerprint_time']) ? json_decode($data['out_fingerprint_time'], true) : [];

        $in_fingerprint_time_schedule_id = isset($data['in_fingerprint_time_schedule_id']) ? json_decode($data['in_fingerprint_time_schedule_id'], true) : [];
        $out_fingerprint_time_schedule_id = isset($data['out_fingerprint_time_schedule_id']) ? json_decode($data['out_fingerprint_time_schedule_id'], true) : [];

        sort($inFingerprintTimes);
        sort($outFingerprintTimes);

        sort($in_fingerprint_time_schedule_id);
        sort($out_fingerprint_time_schedule_id);

        // Combine in_fingerprint_time with schedule_id
        $inFingerprintWithSchedule = [];
        foreach ($inFingerprintTimes as $index => $time) {
            $inFingerprintWithSchedule[] = [
                'time' => $time,
                'schedule_id' => $in_fingerprint_time_schedule_id[$index] ?? null // Use null if schedule_id is missing
            ];
        }

        // Combine out_fingerprint_time with schedule_id
        $outFingerprintWithSchedule = [];
        foreach ($outFingerprintTimes as $index => $time) {
            $outFingerprintWithSchedule[] = [
                'time' => $time,
                'schedule_id' => $out_fingerprint_time_schedule_id[$index] ?? null // Use null if schedule_id is missing
            ];
        }

        // Format the final object
        $formattedObject = [
            'in_fingerprint_time' => $inFingerprintWithSchedule,
            'out_fingerprint_time' => $outFingerprintWithSchedule
        ];




        $firstScheduleStartTime = null;
        $lastScheduleEndTime = null;
        // Get the first in_fingerprint_time and last out_fingerprint_time
        $firstScheduleStartTime = new DateTime($data['date'] . ' ' . $this->decimalToTime($data['schedule_for_day'][0]['start']));
        $lastScheduleEndTime = new DateTime($data['date'] . ' ' . $this->decimalToTime(end($data['schedule_for_day'])['end']));
        // Create DateTime objects using $data['date'] and the formatted times
        // $firstScheduleStartTime = new DateTime($data['date'] . ' ' . $firstInTime);
        // $lastScheduleEndTime = new DateTime($data['date'] . ' ' . $lastOutTime);


        $details = [];

        foreach ($data['schedule_for_day'] as $schedule) {
            $scheduleId = $schedule['id'];
            $startWork = $schedule['start'];
            $endWork = $schedule['end'];
            $earlyDepartureMinutes = 0;
            $lateArrivalMinutes = 0;


            $startWorkFormatted = $this->decimalToTime($startWork);
            $endWorkFormatted = $this->decimalToTime($endWork);
            $scheduledStartTime = new DateTime($data['date'] . ' ' . $startWorkFormatted);
            $scheduledEndTime = new DateTime($data['date'] . ' ' . $endWorkFormatted);


            $scheduledDuration = $scheduledStartTime->diff($scheduledEndTime);
            $scheduledMinutes = $scheduledDuration->h * 60 + $scheduledDuration->i;
            $totalScheduledMinutes += $scheduledMinutes;

            $closestInTime = null;
            $closestOutTime = null;
            $inTime = null;
            $outTime = null;
            // Search in_fingerprint_time for the current schedule_id
            foreach ($formattedObject['in_fingerprint_time'] as $inEntry) {


                if ($inEntry['schedule_id'] == $scheduleId) {
                    $inTime = (new DateTime($inEntry['time']))->format('H:i:s');
                    $closestInTime = new DateTime($data['date'] . ' ' . $inTime);
                    break;
                }
            }

            // Search out_fingerprint_time for the current schedule_id
            foreach ($formattedObject['out_fingerprint_time'] as $outEntry) {
                if ($outEntry['schedule_id'] == $scheduleId) {
                    $outTime = (new DateTime($outEntry['time']))->format('H:i:s');
                    $closestOutTime = new DateTime($data['date'] . ' ' . $outTime);
                    break;
                }
            }

            // this means absence
            if ($closestInTime === null && $closestOutTime === null) {
                $penaltyDuration = $endWork - $startWork;
                $shouldInAt = $this->decimalToTime($startWork);
                $shouldOutAt = $this->decimalToTime($endWork);
                $details[] = [
                    'id' => $scheduleId,
                    'shouldInAt' => $shouldInAt,
                    'shouldOutAt' => $shouldOutAt,
                    'actual' => null,
                    'penalty_duration_with_hours' => $penaltyDuration,
                    'allowance_with_minutes' => $data['allowance_time'],
                    'status' => 'absent',
                ];
                continue;
            }

            if ($closestInTime !== null && $closestInTime > $scheduledStartTime) {
                $lateArrival = $closestInTime->diff($scheduledStartTime);
                $lateArrivalMinutes = $lateArrival->h * 60 + $lateArrival->i;
                if (is_numeric($allowanceTime)) {
                    if ($lateArrivalMinutes <= $allowanceTime) {
                        $lateArrivalMinutes = 0;
                    } else if (!$calc_full_late) {
                        $lateArrivalMinutes -= $allowanceTime;
                    }
                }
                $totalLateArrivalMinutes += $lateArrivalMinutes;
            }

            // Calculate early departure
            if ($closestOutTime !== null && $closestOutTime < $scheduledEndTime) {
                $earlyDeparture = $scheduledEndTime->diff($closestOutTime);
                $earlyDepartureMinutes = $earlyDeparture->h * 60 + $earlyDeparture->i;
                $totalEarlyDepartureMinutes += $earlyDepartureMinutes;
            }

            // Calculate actual worked time within this scheduled time
            if ($closestInTime !== null && $closestOutTime !== null) {
                if ($is_calc_by_missing) {
                    // Count all worked hours, ignoring the scheduled times
                    $workedDuration = $closestInTime->diff($closestOutTime);
                    $workedMinutes = $workedDuration->h * 60 + $workedDuration->i;
                    $totalWorkedMinutes += $workedMinutes;
                } else {
                    // Ensure that the actual worked time is within the scheduled time
                    $actualStartTime = max($closestInTime, $scheduledStartTime);
                    $actualEndTime = min($closestOutTime, $scheduledEndTime);

                    // Calculate worked time if there is a valid period
                    if ($actualStartTime < $actualEndTime) {
                        $workedDuration = $actualStartTime->diff($actualEndTime);
                        $workedMinutes = $workedDuration->h * 60 + $workedDuration->i;
                        $totalWorkedMinutes += $workedMinutes;
                    }
                }
            }

            $shouldInAt = $this->decimalToTime($startWork);
            $shouldOutAt = $this->decimalToTime($endWork);
            $details[] = [
                'id' => $scheduleId,
                'shouldInAt' => $shouldInAt,
                'shouldOutAt' => $shouldOutAt,
                'actualIn' => $inTime,
                'actualOut' => $outTime,
                'penatlyLateInMinutes' => $lateArrivalMinutes,
                'penatlyDepartureOutMinutes' => $earlyDepartureMinutes,
                'penatlyLateInHours' => $lateArrivalMinutes / 60,
                'penatlyDepartureOutHours' => $earlyDepartureMinutes / 60,
                'allowanceWithMinutes' => $data['allowance_time'],
                'status' => 'attended',
            ];
        }

        // Calculate overtime outside scheduled hours
        if (!empty($ins)) {
            $earliestInTime = new DateTime(min($ins));
            if ($earliestInTime < $firstScheduleStartTime) {
                $earlyOvertimeDuration = $earliestInTime->diff($firstScheduleStartTime);
                $earlyOvertimeMinutes = $earlyOvertimeDuration->h * 60 + $earlyOvertimeDuration->i;

                // Apply the overtime allowance
                if ($earlyOvertimeMinutes > $over_time_allowance) {
                    // Only add the minutes that exceed the allowance
                    $totalOvertimeMinutes += $earlyOvertimeMinutes;
                }
            }
        }

        // Late overtime outside scheduled hours (after the schedule ends)
        if (!empty($outs)) {
            $latestOutTime = new DateTime(max($outs));
            if ($latestOutTime > $lastScheduleEndTime) {
                $lateOvertimeDuration = $latestOutTime->diff($lastScheduleEndTime);
                $lateOvertimeMinutes = $lateOvertimeDuration->h * 60 + $lateOvertimeDuration->i;

                // Apply the overtime allowance
                if ($lateOvertimeMinutes > $over_time_allowance) {
                    $totalOvertimeMinutes += $lateOvertimeMinutes;
                }
            }
        }





        $totalMissedMinutes = max($totalScheduledMinutes - $totalWorkedMinutes, 0);
        $overtimeHours = round($totalOvertimeMinutes / 60, 2);

        if ($is_calc_by_missing) {
            $overtimeHours = round(($totalWorkedMinutes / 60) - $totalScheduledMinutes / 60, 2);
            if ($overtimeHours < 0) {
                $overtimeHours = 0;
            }
        } else {
            $overtimeHours = round($totalOvertimeMinutes / 60, 2);
        }

        $targetHours = round($totalScheduledMinutes / 60, 2);
        $workedHours = round($totalWorkedMinutes / 60, 2);

        return [
            'totalLateArrivalMinutes' => $totalLateArrivalMinutes,
            'totalEarlyDepartureMinutes' => $totalEarlyDepartureMinutes,
            'totalMissedHours' => round($totalMissedMinutes / 60, 2),
            'totalWorkedHours' => $workedHours,
            'targetHours' => $targetHours,
            'overtimeHours' => $overtimeHours,
            'date' => $data['date'],
            'status' => "success",
            'details' => json_encode($details)
        ];
    }


    public function getAllowanceTimeFlag()
    {
        $query = "SELECT calculate_full_late FROM emp_worked_hours_flags WHERE is_main = 1 LIMIT 1";
        $stmt = $this->db->prepare($query);
        $stmt->execute();
        $result = $stmt->get_result();

        if ($result->num_rows > 0) {
            $row = $result->fetch_assoc();
            return $row['calculate_full_late'] == 1 ? true : false;
        }
        return false;
    }

    public function getIsPenaltyBasedOnMissingFlag()
    {
        $query = "SELECT calc_by_missing FROM emp_worked_hours_flags WHERE is_main = 1 LIMIT 1";
        $stmt = $this->db->prepare($query);
        $stmt->execute();
        $result = $stmt->get_result();

        if ($result->num_rows > 0) {
            $row = $result->fetch_assoc();
            return $row['calc_by_missing'] == 1 ? true : false;
        }
        return false;
    }

    public function getAllowanceOvertime()
    {
        $query = "SELECT over_time_allowance FROM emp_worked_hours_flags WHERE is_main = 1 LIMIT 1";
        $stmt = $this->db->prepare($query);
        $stmt->execute();
        $result = $stmt->get_result();

        if ($result->num_rows > 0) {
            $row = $result->fetch_assoc();
            return $row['over_time_allowance'];
        }
        return 0;
    }

    private function insertCalculatedData($data)
    {
        $user_hash = $data['user_hash'];
        $date = $data['date'];
        $total_late_arrival_minutes = $data['totalLateArrivalMinutes'];
        $total_early_departure_minutes = $data['totalEarlyDepartureMinutes'];
        $total_missed_hours = $data['totalMissedHours'];
        $total_worked_hours = $data['totalWorkedHours'];
        $target_hours = $data['targetHours'];
        $overtime_hours = $data['overtimeHours'];
        $details = $data['details'];

        $query = "INSERT INTO employees_worked_hours
          (user_hash, date, total_late_arrival_minutes, total_early_departure_minutes,
           total_missed_hours, total_worked_hours, target_hours, overtime_hours, details)
          VALUES
          (?, ?, ?, ?, ?, ?, ?, ?, ?)
          ON DUPLICATE KEY UPDATE
          total_late_arrival_minutes = VALUES(total_late_arrival_minutes),
          total_early_departure_minutes = VALUES(total_early_departure_minutes),
          total_missed_hours = VALUES(total_missed_hours),
          total_worked_hours = VALUES(total_worked_hours),
          target_hours = VALUES(target_hours),
          overtime_hours = VALUES(overtime_hours),
          details = VALUES(details),
          updated_at = NOW()";
        $stmt = $this->db->prepare($query);
        $stmt->bind_param(
            "ssdddddds", // Add the correct data types for the parameters
            $user_hash,
            $date,
            $total_late_arrival_minutes,
            $total_early_departure_minutes,
            $total_missed_hours,
            $total_worked_hours,
            $target_hours,
            $overtime_hours,
            $details // Now this parameter is accounted for
        );
        return $stmt->execute();
    }

    public function updateOutReference($conn, $data)
    {
        // Ensure the in and out times are available
        if (!isset($data['in_fingerprint_time']) || !isset($data['out_fingerprint_time'])) {
            return;
        }

        // Convert the JSON strings to arrays
        $inFingerprintTimes = json_decode($data['in_fingerprint_time'], true);
        $outFingerprintTimes = json_decode($data['out_fingerprint_time'], true);

        foreach ($inFingerprintTimes as $index => $inTime) {
            if (isset($outFingerprintTimes[$index])) {
                $inTime = $inFingerprintTimes[$index];
                $outTime = $outFingerprintTimes[$index];

                $inId = null;
                // Fetch the in entry ID
                $sqlIn = "SELECT id FROM emp_fingerprint
                          WHERE user_hash = ?
                          AND fingerprint_time = ?
                          AND in_out = 'in' LIMIT 1";
                $stmtIn = $conn->prepare($sqlIn);
                $stmtIn->bind_param("ss", $data['user_hash'], $inTime);
                $stmtIn->execute();
                $stmtIn->bind_result($inId);
                $stmtIn->fetch();
                $stmtIn->close();

                $outId = null;
                // Fetch the out entry ID
                $sqlOut = "SELECT id FROM emp_fingerprint
                           WHERE user_hash = ?
                           AND fingerprint_time = ?
                           AND in_out = 'out' LIMIT 1";
                $stmtOut = $conn->prepare($sqlOut);
                $stmtOut->bind_param("ss", $data['user_hash'], $outTime);
                $stmtOut->execute();
                $stmtOut->bind_result($outId);
                $stmtOut->fetch();
                $stmtOut->close();

                // Update the out entry to reference the in entry ID
                if ($inId !== null && $outId !== null) {
                    $updateSql = "UPDATE emp_fingerprint SET in_out_reference = ? WHERE id = ? AND in_out_reference IS NULL";
                    $updateStmt = $conn->prepare($updateSql);
                    $updateStmt->bind_param("ii", $inId, $outId);
                    $updateStmt->execute();
                    $updateStmt->close();
                }
            }
        }
    }
    public function decimalToTime($decimal)
    {
        $hours = floor($decimal);
        $minutes = ($decimal - $hours) * 60;
        return sprintf('%02d:%02d', $hours, $minutes);
    }
    public function getAllEmployeeUserHashes()
    {
        $query = "SELECT user_hash FROM emp_information";
        $result = $this->db->query($query);

        $userHashes = [];
        if ($result->num_rows > 0) {
            while ($row = $result->fetch_assoc()) {
                $userHashes[] = $row['user_hash'];
            }
        }

        return $userHashes;
    }

    // private function handlePartialVacation($taskData)
    // {
    //     // Example logic to automatically clock out a user at the start of a partial vacation
    //     $userId = $taskData['user_id']; // Assuming taskData contains user_id
    //     $vacationStartTime = $taskData['start_time'];
    //     print_r($vacationStartTime);
    //     // Perform the fingerprint out action
    //     $this->fingerprintOut($userId, $vacationStartTime);
    // }

    // private function handlePartialVacationPrintIn($taskData)
    // {
    //     $userId = $taskData['user_id'];
    //     $vacationEndTime = $taskData['end_time'];

    //     $this->fingerPrintIn($userId, $vacationEndTime);
    // }

    private function fingerPrintIn($userId, $time)
    {
        // Define the query
        $checkQuery = "SELECT
            SUM(CASE WHEN in_out = 'in' THEN 1 ELSE 0 END) as in_count,
            SUM(CASE WHEN in_out = 'out' THEN 1 ELSE 0 END) as out_count
        FROM emp_fingerprint
        WHERE user_hash = ?
        AND DATE(fingerprint_time) = CURDATE()
        AND fingerprint_time > ?";

        // Prepare and execute the SQL statement
        $stmt = $this->db->prepare($checkQuery);
        if ($stmt === false) {
            throw new Exception('Prepare failed: ' . $this->db->error);
        }

        // Bind parameters and execute
        $stmt->bind_param('is', $userId, $time);
        if (!$stmt->execute()) {
            throw new Exception('Execute failed: ' . $stmt->error);
        }

        // Get the result
        $result = $stmt->get_result()->fetch_assoc();

        // Close the statement
        $stmt->close();

        // Check the counts
        if ($result['in_count'] > 0 && $result['out_count'] == 0) {
            return ['status' => 'error', 'message' => 'لا يمكن تسجيل الدخول مرة أخرى قبل تسجيل الخروج.'];
        } elseif ($result['in_count'] == 0 && $result['out_count'] > 0) {
            return ['status' => 'error', 'message' => 'لا يمكن تسجيل الخروج قبل تسجيل الدخول.'];
        } else {
            // Insert the new fingerprint record
            $data = array(
                'user_hash' => $userId,
                'fingerprint_time' => $time,
                'in_out' => 'in',
            );
            $insertQuery = "INSERT INTO emp_fingerprint (user_hash, fingerprint_time, in_out) VALUES (?, ?, 'in')";
            $stmt = $this->db->prepare($insertQuery);
            $stmt->bind_param('is', $data['user_hash'], $data['fingerprint_time']);
            $stmt->execute();
            $stmt->close();

            return ['status' => 'success', 'message' => 'تم تسجيل الدخول بنجاح.'];
        }
    }

    private function fingerprintOut($userId, $time)
    {
        $checkQuery = "SELECT
                            SUM(CASE WHEN in_out = 'in' THEN 1 ELSE 0 END) as in_count,
                            SUM(CASE WHEN in_out = 'out' THEN 1 ELSE 0 END) as out_count
                       FROM emp_fingerprint
                       WHERE user_hash = ?
                       AND DATE(fingerprint_time) = CURDATE()
                       AND fingerprint_time < ?";

        // Prepare and execute the SQL statement
        $stmt = $this->db->prepare($checkQuery);
        if ($stmt === false) {
            throw new Exception('Prepare failed: ' . $this->db->error);
        }

        // Bind parameters and execute
        $stmt->bind_param('is', $userId, $time);
        if (!$stmt->execute()) {
            throw new Exception('Execute failed: ' . $stmt->error);
        }

        // Get the result
        $result = $stmt->get_result();
        $row = $result->fetch_assoc();

        print_r($time);

        // Close the statement
        $stmt->close();

        // Check if there is an "in" record for today and if the number of "out" records is less than "in" records
        if ($row['in_count'] == 0) {
            throw new Exception("No 'in' record found for today or 'in' record is after the 'out' time.");
        }
        if ($row['out_count'] >= $row['in_count']) {
            throw new Exception("Cannot insert 'out' record without a corresponding 'in' record.");
        }

        // Prepare the SQL query to insert the fingerprint log
        $insertQuery = "INSERT INTO emp_fingerprint (user_hash, fingerprint_time, in_out) VALUES (?, ?, 'out')";

        // Prepare and execute the SQL statement
        $stmt = $this->db->prepare($insertQuery);
        if ($stmt === false) {
            throw new Exception('Prepare failed: ' . $this->db->error);
        }

        // Bind parameters and execute
        $stmt->bind_param('is', $userId, $time);
        if (!$stmt->execute()) {
            throw new Exception('Execute failed: ' . $stmt->error);
        }

        // Close the statement
        $stmt->close();

        return ['status' => 'success', 'message' => "Fingerprint 'out' record inserted successfully."];
    }

    private function updateTaskStatus($taskId, $status)
    {
        $updateQuery = "UPDATE scheduled_tasks SET status = ? WHERE id = ?";
        $stmt = $this->db->prepare($updateQuery);
        $stmt->bind_param('si', $status, $taskId);
        $stmt->execute();
        $stmt->close();
    }

    private function handleTaskFailure($taskId, $errorMessage)
    {
        // Increment retry count and log the error
        $updateQuery = "UPDATE scheduled_tasks SET status = 'failed', retry_count = retry_count + 1 WHERE id = ?";
        $stmt = $this->db->prepare($updateQuery);
        $stmt->bind_param('i', $taskId);
        $stmt->execute();
        $stmt->close();

        // Log the error
        error_log("Task ID $taskId failed: $errorMessage");
    }

    public function __destruct()
    {
        $this->db->close();
    }
}

// Instantiate the processor and start processing tasks
$taskProcessor = new ScheduledTaskProcessor();
$taskProcessor->processTasks();
