<?php



// Declare user_hash and date variables at the beginning of the file
$specific_user_hash = '5245663516015139';
$specific_date = '2024-08-04'; 

$config_path = '/var/www/html/newCompany/app/application/config/database.php';


// Check if the CodeIgniter database configuration file exists
if (file_exists($config_path)) {
    // Include the configuration file
    include $config_path;

    // Debugging: Check if $db is actually an array
    if (!is_array($db)) {
        die("Error: \$db is not an array. The config file might not be loaded correctly.");
    }

    // Ensure active group is set
    $active_group = isset($active_group) ? $active_group : 'default';

    // Check if the active group exists in $db
    if (!isset($db[$active_group])) {
        die("Error: Active group '$active_group' does not exist in \$db.");
    }

    // Dynamically set these variables from the CodeIgniter config
    $servername = $db[$active_group]['hostname'];
    $username = $db[$active_group]['username'];
    $password = $db[$active_group]['password'];
    $dbname = $db[$active_group]['database'];

} else {
    die('Error: Configuration file not found.');
}

// Create connection using MySQLi
$conn = new mysqli($servername, $username, $password, $dbname);

// Check connection
if ($conn->connect_error) {
    die("Connection failed: " . $conn->connect_error);
}


function insertCalculatedData($conn, $data)
{
    $user_hash = $data['user_hash'];
    $date = $data['date'];
    $total_late_arrival_minutes = $data['totalLateArrivalMinutes'];
    $total_early_departure_minutes = $data['totalEarlyDepartureMinutes'];
    $total_missed_hours = $data['totalMissedHours'];
    $total_worked_hours = $data['totalWorkedHours'];
    $target_hours = $data['targetHours'];
    $overtime_hours = $data['overtimeHours'];

    $sql = "INSERT INTO employees_worked_hours
          (user_hash, date, total_late_arrival_minutes, total_early_departure_minutes,
           total_missed_hours, total_worked_hours, target_hours, overtime_hours)
          VALUES
          (?, ?, ?, ?, ?, ?, ?, ?)
          ON DUPLICATE KEY UPDATE
          total_late_arrival_minutes = VALUES(total_late_arrival_minutes),
          total_early_departure_minutes = VALUES(total_early_departure_minutes),
          total_missed_hours = VALUES(total_missed_hours),
          total_worked_hours = VALUES(total_worked_hours),
          target_hours = VALUES(target_hours),
          overtime_hours = VALUES(overtime_hours)";

    $stmt = $conn->prepare($sql);
    if ($stmt === false) {
        die('Prepare Error: ' . $conn->error);
    }
    $stmt->bind_param(
        "isdddddd",
        $user_hash,
        $date,
        $total_late_arrival_minutes,
        $total_early_departure_minutes,
        $total_missed_hours,
        $total_worked_hours,
        $target_hours,
        $overtime_hours
    );

    $stmt->execute();

    if ($stmt->error) {
        die("Insertion error: " . $stmt->error);
    }

    $stmt->close();
}

// Modified SQL Query
$sql = "SELECT *
FROM (
    SELECT
        JSON_OBJECT(
            'user_hash', ei.user_hash,
            'schedule_for_day', asch.schedule_data,
            'allowance_time', asch.allowance_time,
            'in_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE 
                WHEN ef.in_out = 'in' THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"') 
            END ORDER BY ef.fingerprint_time SEPARATOR ','), ']'),
            'out_fingerprint_time', CONCAT('[', GROUP_CONCAT(CASE 
                WHEN ef.in_out = 'out' 
                AND (
                    DATE(ef.fingerprint_time) = ?
                    OR (
                        DATE(ef.fingerprint_time) = DATE_ADD(?, INTERVAL 1 DAY)
                        AND TIME(ef.fingerprint_time) < '03:57:00'
                    )
                )
                THEN CONCAT('\"', DATE_FORMAT(ef.fingerprint_time, '%Y-%m-%d %H:%i:%s'), '\"') 
            END ORDER BY ef.fingerprint_time DESC SEPARATOR ','), ']'),
            'date', asch.date
        ) AS employee_data
    FROM
        emp_information ei
    JOIN
        actual_schedules asch ON ei.user_hash = asch.user_hash
        AND asch.date = ?
    LEFT JOIN
        emp_fingerprint ef ON ei.user_hash = ef.user_hash
        AND (
            (DATE(ef.fingerprint_time) = ?)
            OR (
                DATE(ef.fingerprint_time) = DATE_ADD(?, INTERVAL 1 DAY)
                AND TIME(ef.fingerprint_time) < '03:57:00'
            )
        )
    WHERE
        ei.user_hash = ?
        AND asch.schedule_data IS NOT NULL
        AND TRIM(asch.schedule_data) <> ''
    GROUP BY
        ei.user_hash, asch.date
) AS subquery;";

// Prepare and execute the query
$stmt = $conn->prepare($sql);
if ($stmt === false) {
    die('Prepare Error: ' . $conn->error);
}

$stmt->bind_param("ssssss", $specific_date, $specific_date, $specific_date, $specific_date, $specific_date, $specific_user_hash);
$stmt->execute();

$result = $stmt->get_result();

// Check if the query was successful
if ($result === false) {
    die("Error: " . $conn->error);
}

// Initialize an array to hold the results
$results = [];

// Fetch and store the results in the array
while ($row = $result->fetch_assoc()) {
    $results[] = json_decode($row['employee_data'], true);
}

// Function to calculate late arrival and early departure
function calculateLateArrivalAndEarlyDeparture($data)
{
    $totalLateArrivalMinutes = 0;
    $totalEarlyDepartureMinutes = 0;
    $totalScheduledMinutes = 0;
    $totalWorkedMinutes = 0;
    $totalTargetMinutes = 0;

    // Check if schedules are provided
    if (!isset($data['schedule_for_day']) || !is_array($data['schedule_for_day']) || empty($data['schedule_for_day'])) {
        return [
            'error' => 'Missing or invalid schedule_for_day data.',
            'status' => 'error',
        ];
    }

    $allowanceTime = isset($data['allowance_time']) ? (int) $data['allowance_time'] : null;


    foreach ($data['schedule_for_day'] as $index => $schedule) {
        $startWork = $schedule['start'];
        $endWork = $schedule['end'];
        // $allowanceTime = isset($schedule['allowance_time']) ? (int) $schedule['allowance_time'] : null;

        // Calculate scheduled work duration
        $startWorkFormatted = decimalToTime($startWork);
        $endWorkFormatted = decimalToTime($endWork);
        $scheduledStartTime = new DateTime($data['date'] . ' ' . $startWorkFormatted);
        $scheduledEndTime = new DateTime($data['date'] . ' ' . $endWorkFormatted);
        $scheduledDuration = $scheduledStartTime->diff($scheduledEndTime);
        $scheduledMinutes = $scheduledDuration->h * 60 + $scheduledDuration->i;

        $totalScheduledMinutes += $scheduledMinutes;
        $totalTargetMinutes += $scheduledMinutes;

        // Check if in_fingerprint_time exists for this schedule index
        if (isset($data['in_fingerprint_time'])) {
            $inFingerprintTimes = json_decode($data['in_fingerprint_time'], true);
            if (isset($inFingerprintTimes[$index])) {
                $inFingerprintTime = new DateTime($inFingerprintTimes[$index]);

                // Calculate late arrival
                if ($inFingerprintTime > $scheduledStartTime) {
                    $lateArrival = $scheduledStartTime->diff($inFingerprintTime);
                    $lateArrivalMinutes = $lateArrival->h * 60 + $lateArrival->i;
                    if (is_numeric($allowanceTime)) {
                        if ($lateArrivalMinutes <= $allowanceTime) {
                            $lateArrivalMinutes = 0;
                        } else {

                            // $lateArrivalMinutes -= $allowanceTime;
                            $lateArrivalMinutes = $lateArrivalMinutes;
                        }
                    }
                    $totalLateArrivalMinutes += $lateArrivalMinutes;
                }
            }
        }

        // Check if out_fingerprint_time exists for this schedule index
        if (isset($data['out_fingerprint_time'])) {
            $outFingerprintTimes = json_decode($data['out_fingerprint_time'], true);
            if (isset($outFingerprintTimes[$index])) {
                $outFingerprintTime = new DateTime($outFingerprintTimes[$index]);

                // Calculate early departure
                if ($outFingerprintTime < $scheduledEndTime) {
                    $earlyDeparture = $outFingerprintTime->diff($scheduledEndTime);
                    $earlyDepartureMinutes = $earlyDeparture->h * 60 + $earlyDeparture->i;
                    $totalEarlyDepartureMinutes += $earlyDepartureMinutes;
                }
            }
        }

        // Check if both in_fingerprint_time and out_fingerprint_time exist for this schedule index
        if (isset($data['in_fingerprint_time'], $data['out_fingerprint_time'])) {
            $inFingerprintTimes = json_decode($data['in_fingerprint_time'], true);
            $outFingerprintTimes = json_decode($data['out_fingerprint_time'], true);
            if (isset($inFingerprintTimes[$index], $outFingerprintTimes[$index])) {
                $inFingerprintTime = new DateTime($inFingerprintTimes[$index]);
                $outFingerprintTime = new DateTime($outFingerprintTimes[$index]);

                // Calculate worked duration
                $workedDuration = $inFingerprintTime->diff($outFingerprintTime);
                $workedMinutes = $workedDuration->h * 60 + $workedDuration->i;
                $totalWorkedMinutes += $workedMinutes;
            }
        }
    }

    $totalMissedMinutes = max(round($totalScheduledMinutes - $totalWorkedMinutes), 0);
    $overtimeMinutes = $totalWorkedMinutes - $totalScheduledMinutes;
    $overtimeHours = max(round($overtimeMinutes / 60, 2), 0);
    $targetHours = round($totalTargetMinutes / 60, 2);
    $workedHours = round($totalWorkedMinutes / 60, 2);

    return [
        'totalLateArrivalMinutes' => $totalLateArrivalMinutes,
        'totalEarlyDepartureMinutes' => $totalEarlyDepartureMinutes,
        'totalMissedHours' => round($totalMissedMinutes / 60, 2),
        'totalWorkedHours' => $workedHours,
        'targetHours' => $targetHours,
        'overtimeHours' => $overtimeHours,
        'date' => $data['date'],
        'status' => "success",
    ];
}

// Function to convert decimal hours to time format
function decimalToTime($decimal)
{
    $hours = floor($decimal);
    $minutes = ($decimal - $hours) * 60;
    return sprintf('%02d:%02d', $hours, $minutes);
}

// Process each result and calculate late arrival and early departure
foreach ($results as &$result) {
    $calculation = calculateLateArrivalAndEarlyDeparture($result);
    $result = array_merge($result, $calculation);

    insertCalculatedData($conn, $result);
}

// Output the results array with calculations
print_r($results);

$stmt->close();
$conn->close();